@@ -32,6 +32,10 @@
  *       * good contrast but fast varying background
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
 int main(int    argc,
@@ -125,6 +129,7 @@ L_REGPARAMS  *rp;
     pix8 = pixApplyInvBackgroundGrayMap(pixs, pix7, 7, 7);
     pixaAddPix(pixa1, pix8, L_INSERT);
     regTestWritePixAndCheck(rp, pix8, IFF_JFIF_JPEG);  /* 13 */
+    pixDestroy(&pix6);
     pixDestroy(&pix7);
 
         /* Process the result for gray and binary output */
