@@ -49,6 +49,10 @@
  *   images, such as w91frag.jpg.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
 #define  ALL    1
@@ -69,14 +73,15 @@ static char  mainName[] = "binarize_set";
         return ERROR_INT(" Syntax: binarize_set infile", mainName, 1);
     infile = argv[1];
 
+    setLeptDebugOK(1);
+    lept_mkdir("lept/binar");
+
     pixa = pixaCreate(5);
     pixs = pixRead(infile);
     pixGetDimensions(pixs, &w, NULL, &d);
-    pixSaveTiled(pixs, pixa, 1.0, 1, 50, 32);
+    pixaAddPix(pixa, pixs, L_INSERT);
     pixDisplay(pixs, 100, 0);
 
-    lept_mkdir("lept/binar");
-
 #if ALL
     /* 1. Standard background normalization with a global threshold.  */
     pixg = pixConvertTo8(pixs, 0);
@@ -84,10 +89,9 @@ static char  mainName[] = "binarize_set";
     pix2 = pixThresholdToBinary(pix1, 160);
     pixWrite("/tmp/lept/binar/binar1.png", pix2, IFF_PNG);
     pixDisplay(pix2, 100, 0);
-    pixSaveTiled(pix2, pixa, 1.0, 1, 50, 32);
+    pixaAddPix(pixa, pix2, L_INSERT);
     pixDestroy(&pixg);
     pixDestroy(&pix1);
-    pixDestroy(&pix2);
 #endif
 
 #if ALL
@@ -102,11 +106,10 @@ static char  mainName[] = "binarize_set";
     pix1 = pixOtsuThreshOnBackgroundNorm(pixg, NULL, 10, 15, 100,
                                     50, 255, 2, 2, 0.10, &threshval);
     fprintf(stderr, "thresh val = %d\n", threshval);
-    pixSaveTiled(pix1, pixa, 1.0, 1, 50, 32);
+    pixaAddPix(pixa, pix1, L_INSERT);
     pixWrite("/tmp/lept/binar/binar2.png", pix1, IFF_PNG);
     pixDisplay(pix1, 100, 200);
     pixDestroy(&pixg);
-    pixDestroy(&pix1);
 #endif
 
 #if ALL
@@ -116,11 +119,10 @@ static char  mainName[] = "binarize_set";
     pix1 = pixMaskedThreshOnBackgroundNorm(pixg, NULL, 10, 15, 100,
                                            50, 2, 2, 0.10, &threshval);
     fprintf(stderr, "thresh val = %d\n", threshval);
-    pixSaveTiled(pix1, pixa, 1.0, 1, 50, 32);
+    pixaAddPix(pixa, pix1, L_INSERT);
     pixWrite("/tmp/lept/binar/binar3.png", pix1, IFF_PNG);
     pixDisplay(pix1, 100, 400);
     pixDestroy(&pixg);
-    pixDestroy(&pix1);
 #endif
 
 #if ALL
@@ -131,12 +133,11 @@ static char  mainName[] = "binarize_set";
         pixg = pixConvertTo8(pixs, 0);
     pixg2 = pixContrastNorm(NULL, pixg, 20, 20, 130, 2, 2);
     pixSauvolaBinarizeTiled(pixg2, 25, 0.40, 1, 1, NULL, &pix1);
-    pixSaveTiled(pix1, pixa, 1.0, 1, 50, 32);
+    pixaAddPix(pixa, pix1, L_INSERT);
     pixWrite("/tmp/lept/binar/binar4.png", pix1, IFF_PNG);
     pixDisplay(pix1, 100, 600);
     pixDestroy(&pixg);
     pixDestroy(&pixg2);
-    pixDestroy(&pix1);
 #endif
 
 #if ALL
@@ -159,22 +160,18 @@ static char  mainName[] = "binarize_set";
 
     ival = L_MIN(ival, 110);
     pix1 = pixThresholdToBinary(pixg2, ival);
-    pixSaveTiled(pix1, pixa, 1.0, 1, 50, 32);
+    pixaAddPix(pixa, pix1, L_INSERT);
     pixWrite("/tmp/lept/binar/binar5.png", pix1, IFF_PNG);
     pixDisplay(pix1, 100, 800);
     pixDestroy(&pixg);
     pixDestroy(&pixg2);
-    pixDestroy(&pix1);
 #endif
 
-    pix1 = pixaDisplayTiledInRows(pixa, 32, w + 100, 1.0, 0, 30, 2);
+    pix1 = pixaDisplayTiledInColumns(pixa, 2, 1.0, 30, 2);
     pixWrite("/tmp/lept/binar/binar6.png", pix1, IFF_PNG);
     pixDisplay(pix1, 1000, 0);
     pixDestroy(&pix1);
     pixaDestroy(&pixa);
 
-    pixDestroy(&pixs);
     return 0;
 }
-
-
