@@ -32,6 +32,10 @@
  *    The resolution is preserved.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "string.h"
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -59,7 +63,6 @@ static char  mainName[] = "binarizefiles.c";
             "      subdirout: subdirectory of /tmp for output files\n");
         return 1;
     }
-
     dirin = argv[1];
     pattern = argv[2];
     thresh = atoi(argv[3]);
@@ -72,6 +75,8 @@ static char  mainName[] = "binarizefiles.c";
         scalefactor = 1.0;
     }
 
+    setLeptDebugOK(1);
+
         /* Get the input filenames */
     sa = getSortedPathnamesInDirectory(dirin, pattern, 0, 0);
     sarrayWriteStream(stderr, sa);
