@@ -37,6 +37,10 @@
  *    The brick functions all have a pre-allocated pix as the dest.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
 void  TestAll(L_REGPARAMS *rp, PIX *pixs, l_int32 symmetric);
