@@ -30,6 +30,10 @@
  *   Tests the bbuffer operations
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
 #define   NBLOCKS     11
