@@ -34,6 +34,10 @@
  *  The input image is rgb.  Other examples are breviar.32 and amoris.2.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
 PIX *TestForRedColor(L_REGPARAMS *rp, const char *fname,
@@ -53,6 +57,11 @@ PIX          *pix8, *pix9, *pix10, *pix11, *pix12, *pix13, *pix14, *pix15;
 PIXA         *pixa;
 L_REGPARAMS  *rp;
 
+#if !defined(HAVE_LIBPNG)
+    L_ERROR("This test requires libpng to run.\n", "colorize_reg");
+    exit(77);
+#endif
+
     if (regTestSetup(argc, argv, &rp))
               return 1;
 
