@@ -30,6 +30,10 @@
  *  Regression test for simple color morphological operations
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
 static const l_int32  SIZE = 7;
