@@ -32,6 +32,10 @@
  *       - global linear color mapping and extraction of color magnitude
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
 int main(int    argc,
@@ -49,6 +53,11 @@ PIXA         *pixa, *pixat;
 PIXCMAP      *cmap;
 L_REGPARAMS  *rp;
 
+#if !defined(HAVE_LIBPNG)
+    L_ERROR("This test requires libpng to run.\n", "colorspace_reg");
+    exit(77);
+#endif
+
     if (regTestSetup(argc, argv, &rp))
         return 1;
 
@@ -161,7 +170,7 @@ L_REGPARAMS  *rp;
                          "white point space for red", "amount of color");
     for (j = 0; j < 6; j++) {
         na = numaaGetNuma(naa1, j, L_CLONE);
-        sprintf(label, "thresh %d", 30 + 10 * j);
+        snprintf(label, sizeof(label), "thresh %d", 30 + 10 * j);
         gplotAddPlot(gplot1, naseq, na, GPLOT_LINES, label);
         numaDestroy(&na);
         na = numaaGetNuma(naa2, j, L_CLONE);
