@@ -31,6 +31,10 @@
  *     based on size, using logical combinations of indicator arrays.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
 static void Count_pieces(L_REGPARAMS *rp, PIX  *pix, l_int32 nexp);
@@ -226,8 +230,7 @@ L_REGPARAMS  *rp;
 
             /* Remove band successively from full image */
         pixRemoveWithIndicator(pix1, pixa1, na4);
-        pixSaveTiled(pix1, pixa3, 0.25, 1 - i % 2, 25, 8);
-
+        pixaAddPix(pixa3, pix1, L_COPY);
         numaDestroy(&na2);
         numaDestroy(&na3);
         numaDestroy(&na4);
@@ -268,8 +271,7 @@ L_REGPARAMS  *rp;
     numaInvert(na2, na2);  /* get components to be removed */
     pixRemoveWithIndicator(pixs, pixa1, na2);
     regTestWritePixAndCheck(rp, pixs, IFF_PNG);  /* 86 */
-    pixSaveTiled(pixs, pixa3, 0.25, 1, 25, 8);
-    pixDestroy(&pixs);
+    pixaAddPix(pixa3, pixs, L_INSERT);
     boxaDestroy(&boxa1);
     pixaDestroy(&pixa1);
     numaDestroy(&naw);
@@ -281,7 +283,7 @@ L_REGPARAMS  *rp;
     numaDestroy(&na5);
 
     if (rp->display) {
-        pix1 = pixaDisplay(pixa3, 0, 0);
+        pix1 = pixaDisplayTiledInColumns(pixa3, 2, 0.25, 25, 2);
         pixDisplay(pix1, 100, 100);
         pixWrite("/tmp/lept/filter/result.png", pix1, IFF_PNG);
         pixDestroy(&pix1);
