@@ -45,15 +45,18 @@
  *    at the bottom.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include "allheaders.h"
 
 int main(int    argc,
          char **argv)
 {
-char           *dirin, *substr, *fileout;
-l_int32         res;
-
+char    *dirin, *substr, *fileout;
+l_int32  res;
 
     if (argc != 5) {
         fprintf(stderr,
@@ -67,11 +70,12 @@ l_int32         res;
             "         fileout:  Output ps file.\n");
         return 1;
     }
-
     dirin = argv[1];
     substr = argv[2];
     res = atoi(argv[3]);
     fileout = argv[4];
+
+    setLeptDebugOK(1);
     if (!strcmp(substr, "allfiles"))
         substr = NULL;
     if (res != 0)
