@@ -32,6 +32,10 @@
  *   in the process.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
 #define   DO_QUAD     1
@@ -46,6 +50,9 @@ L_DEWARPA  *dewa;
 PIX        *pixs, *pixn, *pixg, *pixb, *pixd, *pixt1, *pixt2;
 PIX        *pixs2, *pixn2, *pixg2, *pixb2, *pixd2;
 
+    setLeptDebugOK(1);
+    lept_mkdir("lept/model");
+
 /*    pixs = pixRead("1555.007.jpg"); */
     pixs = pixRead("cat.035.jpg");
 /*    pixs = pixRead("cat.010.jpg"); */
@@ -60,9 +67,9 @@ PIX        *pixs2, *pixn2, *pixg2, *pixb2, *pixd2;
     dewarpaUseBothArrays(dewa, 1);
     dew1 = dewarpCreate(pixb, 35);
     dewarpaInsertDewarp(dewa, dew1);
-    dewarpBuildPageModel(dew1, "/tmp/lept/dewarp_model1.pdf");
+    dewarpBuildPageModel(dew1, "/tmp/lept/model/dewarp_model1.pdf");
     dewarpaApplyDisparity(dewa, 35, pixg, 200, 0, 0, &pixd,
-                          "/tmp/lept/dewarp_apply1.pdf");
+                          "/tmp/lept/model/dewarp_apply1.pdf");
 
          /* Write out some of the files to be imaged */
     lept_rmdir("lept/dewtest");
@@ -121,7 +128,7 @@ PIX        *pixs2, *pixn2, *pixg2, *pixb2, *pixd2;
     dewarpaInsertRefModels(dewa, 0, 1);
     dewarpaInfo(stderr, dewa);
     dewarpaApplyDisparity(dewa, 7, pixg2, 200, 0, 0, &pixd2,
-                          "/tmp/lept/dewarp_apply2.pdf");
+                          "/tmp/lept/model/dewarp_apply2.pdf");
     dewarpaDestroy(&dewa);
 
         /* Write out files for the second image */
@@ -151,7 +158,7 @@ PIX        *pixs2, *pixn2, *pixg2, *pixb2, *pixd2;
         /* Generate the big pdf file */
     convertFilesToPdf("/tmp/lept/dewtest", NULL, 135, 1.0, 0, 0, "Dewarp Test",
                       "/tmp/lept/dewarptest1.pdf");
-    fprintf(stderr, "pdf file made: /tmp/lept/dewarptest1.pdf\n");
+    fprintf(stderr, "pdf file made: /tmp/lept/model/dewarptest1.pdf\n");
 
     lept_rmdir("lept/dewmod");
     lept_rmdir("lept/dewtest");
