@@ -41,6 +41,10 @@
  *   No scaling is done if @scalefact == 0.0 or @scalefact == 1.0.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include "allheaders.h"
 
@@ -48,7 +52,7 @@ int main(int    argc,
          char **argv)
 {
 char         buf[32];
-char        *fileout, *fontdir, *textstr;
+char        *fileout, *textstr;
 l_int32      n, i, same, maxd, ntext, border, lossless, display, showtext;
 l_float32    scalefact;
 L_BMF       *bmf;
@@ -63,6 +67,7 @@ static char  mainName[] = "displaypixa";
                  " lossless disp fileout showtext\n");
          return 1;
     }
+    setLeptDebugOK(1);
 
         /* Input file can be either pixa or pixacomp */
     pixa = pixaReadBoth(argv[1]);
