@@ -30,6 +30,10 @@
  *     Compare the timings of various binary morphological implementations.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #ifndef  _WIN32
 #include <unistd.h>
 #else
@@ -60,6 +64,7 @@ static char  mainName[] = "dwamorph2_reg";
 
     if (argc != 1)
         return ERROR_INT(" Syntax: dwamorph2_reg", mainName, 1);
+    setLeptDebugOK(1);
 
     pixs = pixRead("feyn-fract.tif");
     pixt = pixCreateTemplate(pixs);
