@@ -38,6 +38,10 @@
  *    (2) We do the same with the word "Caltech".
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
     /* for pixDisplayHitMissSel() */
