@@ -44,6 +44,10 @@
  *    in the Document Image Applications chapter.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
     /* for pixDisplayHitMissSel() */
@@ -130,7 +134,7 @@ SEL     *selhm;
         pixr = pixClone(pix);
     else if (red == 8)
         pixr = pixReduceRankBinaryCascade(pix, 2, 0, 0, 0);
-    else if (red == 16)
+    else  /* red == 16 */
         pixr = pixReduceRankBinaryCascade(pix, 2, 2, 0, 0);
     pixDestroy(&pix);
 
