@@ -32,6 +32,10 @@
  *    convolution, with conversions to and from Pix.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <math.h>
 #include "allheaders.h"
 
@@ -76,16 +80,15 @@ L_REGPARAMS  *rp;
     if (regTestSetup(argc, argv, &rp))
         return 1;
 
-    pixa = pixaCreate(0);
 
         /* Gaussian kernel */
+    pixa = pixaCreate(0);
     kel = makeGaussianKernel(5, 5, 3.0, 4.0);
     kernelGetSum(kel, &sum);
     if (rp->display) fprintf(stderr, "Sum for 2d gaussian kernel = %f\n", sum);
     pix0 = kernelDisplayInPix(kel, 41, 2);
     regTestWritePixAndCheck(rp, pix0, IFF_PNG);  /* 0 */
-    pixSaveTiled(pix0, pixa, 1.0, 1, 20, 8);
-    pixDestroy(&pix0);
+    pixaAddPix(pixa, pix0, L_INSERT);
 
         /* Separable gaussian kernel */
     makeGaussianKernelSep(5, 5, 3.0, 4.0, &kelx, &kely);
@@ -97,26 +100,29 @@ L_REGPARAMS  *rp;
                          sumx * sumy);
     pix0 = kernelDisplayInPix(kelx, 41, 2);
     regTestWritePixAndCheck(rp, pix0, IFF_PNG);  /* 1 */
-    pixSaveTiled(pix0, pixa, 1.0, 0, 20, 8);
-    pixDestroy(&pix0);
+    pixaAddPix(pixa, pix0, L_INSERT);
     pix0 = kernelDisplayInPix(kely, 41, 2);
     regTestWritePixAndCheck(rp, pix0, IFF_PNG);  /* 2 */
-    pixSaveTiled(pix0, pixa, 1.0, 0, 20, 8);
+    pixaAddPix(pixa, pix0, L_INSERT);
+    pix0 = pixaDisplayTiledInColumns(pixa, 4, 1.0, 20, 2);
+    regTestWritePixAndCheck(rp, pix0, IFF_PNG);  /* 3 */
+    pixaDestroy(&pixa);
     pixDestroy(&pix0);
 
         /* Use pixRasterop() to generate source image */
+    pixa = pixaCreate(0);
     pixs = pixRead("test8.jpg");
     pixs2 = pixRead("karen8.jpg");
     pixRasterop(pixs, 150, 125, 150, 100, PIX_SRC, pixs2, 75, 100);
-    regTestWritePixAndCheck(rp, pixs, IFF_JFIF_JPEG);  /* 3 */
+    regTestWritePixAndCheck(rp, pixs, IFF_JFIF_JPEG);  /* 4 */
 
         /* Convolution directly with pix */
     pix1 = pixConvolve(pixs, kel, 8, 1);
-    regTestWritePixAndCheck(rp, pix1, IFF_JFIF_JPEG);  /* 4 */
-    pixSaveTiled(pix1, pixa, 1.0, 1, 20, 8);
+    regTestWritePixAndCheck(rp, pix1, IFF_JFIF_JPEG);  /* 5 */
+    pixaAddPix(pixa, pix1, L_INSERT);
     pix2 = pixConvolveSep(pixs, kelx, kely, 8, 1);
-    regTestWritePixAndCheck(rp, pix2, IFF_JFIF_JPEG);  /* 5 */
-    pixSaveTiled(pix2, pixa, 1.0, 0, 20, 8);
+    regTestWritePixAndCheck(rp, pix2, IFF_JFIF_JPEG);  /* 6 */
+    pixaAddPix(pixa, pix2, L_INSERT);
 
         /* Convolution indirectly with fpix, using fpixRasterop()
          * to generate the source image. */
@@ -125,12 +131,12 @@ L_REGPARAMS  *rp;
     fpixRasterop(fpixs, 150, 125, 150, 100, fpixs2, 75, 100);
     fpix1 = fpixConvolve(fpixs, kel, 1);
     pix3 = fpixConvertToPix(fpix1, 8, L_CLIP_TO_ZERO, 1);
-    regTestWritePixAndCheck(rp, pix3, IFF_JFIF_JPEG);  /* 6 */
-    pixSaveTiled(pix3, pixa, 1.0, 1, 20, 8);
+    regTestWritePixAndCheck(rp, pix3, IFF_JFIF_JPEG);  /* 7 */
+    pixaAddPix(pixa, pix3, L_INSERT);
     fpix2 = fpixConvolveSep(fpixs, kelx, kely, 1);
     pix4 = fpixConvertToPix(fpix2, 8, L_CLIP_TO_ZERO, 1);
-    regTestWritePixAndCheck(rp, pix4, IFF_JFIF_JPEG);  /* 7 */
-    pixSaveTiled(pix4, pixa, 1.0, 0, 20, 8);
+    regTestWritePixAndCheck(rp, pix4, IFF_JFIF_JPEG);  /* 8 */
+    pixaAddPix(pixa, pix4, L_INSERT);
     pixDestroy(&pixs2);
     fpixDestroy(&fpixs2);
     fpixDestroy(&fpix1);
@@ -154,37 +160,33 @@ L_REGPARAMS  *rp;
                    &diff, NULL, NULL);
     fprintf(stderr, "Ave diff of pixConvolveSep and fpixConvolveSep: %f\n",
             diff);
+    pix5 = pixRead("/tmp/lept/comp/compare_gray0.png");
+    regTestWritePixAndCheck(rp, pix5, IFF_PNG);  /* 9 */
+    pixaAddPix(pixa, pix5, L_INSERT);
+    pix1 = pixaDisplayTiledInColumns(pixa, 2, 1.0, 20, 2);
+    regTestWritePixAndCheck(rp, pix1, IFF_PNG);  /* 10 */
+    pixaDestroy(&pixa);
     pixDestroy(&pix1);
-    pixDestroy(&pix2);
-    pixDestroy(&pix3);
-    pixDestroy(&pix4);
 
         /* Test arithmetic operations; add in a fraction rotated by 180 */
+    pixa = pixaCreate(0);
     pixs3 = pixRotate180(NULL, pixs);
-    regTestWritePixAndCheck(rp, pixs3, IFF_JFIF_JPEG);  /* 8 */
-    pixSaveTiled(pixs3, pixa, 1.0, 1, 20, 8);
+    regTestWritePixAndCheck(rp, pixs3, IFF_JFIF_JPEG);  /* 11 */
+    pixaAddPix(pixa, pixs3, L_INSERT);
     fpixs3 = pixConvertToFPix(pixs3, 3);
     fpixd = fpixLinearCombination(NULL, fpixs, fpixs3, 20.0, 5.0);
     fpixAddMultConstant(fpixd, 0.0, 23.174);   /* multiply up in magnitude */
     pixd = fpixDisplayMaxDynamicRange(fpixd);  /* bring back to 8 bpp */
-    regTestWritePixAndCheck(rp, pixd, IFF_JFIF_JPEG);  /* 9 */
-    pixSaveTiled(pixd, pixa, 1.0, 0, 20, 8);
-    pixDestroy(&pixs3);
+    regTestWritePixAndCheck(rp, pixd, IFF_JFIF_JPEG);  /* 12 */
+    pixaAddPix(pixa, pixd, L_INSERT);
     fpixDestroy(&fpixs3);
     fpixDestroy(&fpixd);
-    pixDestroy(&pixd);
     pixDestroy(&pixs);
     fpixDestroy(&fpixs);
 
-        /* Save the comparison graph; gnuplot should have made it by now! */
-    pix5 = pixRead("/tmp/lept/comp/compare_gray0.png");
-    regTestWritePixAndCheck(rp, pix5, IFF_PNG);  /* 10 */
-    pixSaveTiled(pix5, pixa, 1.0, 1, 20, 8);
-    pixDestroy(&pix5);
-
         /* Display results */
-    pixd = pixaDisplay(pixa, 0, 0);
-    regTestWritePixAndCheck(rp, pixd, IFF_JFIF_JPEG);  /* 11 */
+    pixd = pixaDisplayTiledInColumns(pixa, 2, 1.0, 20, 2);
+    regTestWritePixAndCheck(rp, pixd, IFF_JFIF_JPEG);  /* 13 */
     pixDisplayWithTitle(pixd, 100, 100, NULL, rp->display);
     pixDestroy(&pixd);
     pixaDestroy(&pixa);
@@ -195,42 +197,36 @@ L_REGPARAMS  *rp;
     pixg = pixConvertTo8(pixs, 0);
     l_setConvolveSampling(5, 5);
     pix1 = pixConvolve(pixg, kel, 8, 1);
-    regTestWritePixAndCheck(rp, pix1, IFF_JFIF_JPEG);  /* 12 */
-    pixSaveTiled(pix1, pixa, 1.0, 1, 20, 32);
+    regTestWritePixAndCheck(rp, pix1, IFF_JFIF_JPEG);  /* 14 */
+    pixaAddPix(pixa, pix1, L_INSERT);
     pix2 = pixConvolveSep(pixg, kelx, kely, 8, 1);
-    regTestWritePixAndCheck(rp, pix2, IFF_JFIF_JPEG);  /* 13 */
-    pixSaveTiled(pix2, pixa, 1.0, 0, 20, 32);
+    regTestWritePixAndCheck(rp, pix2, IFF_JFIF_JPEG);  /* 15 */
+    pixaAddPix(pixa, pix2, L_INSERT);
     pix3 = pixConvolveRGB(pixs, kel);
-    regTestWritePixAndCheck(rp, pix3, IFF_JFIF_JPEG);  /* 14 */
-    pixSaveTiled(pix3, pixa, 1.0, 0, 20, 32);
+    regTestWritePixAndCheck(rp, pix3, IFF_JFIF_JPEG);  /* 16 */
+    pixaAddPix(pixa, pix3, L_INSERT);
     pix4 = pixConvolveRGBSep(pixs, kelx, kely);
-    regTestWritePixAndCheck(rp, pix4, IFF_JFIF_JPEG);  /* 15 */
-    pixSaveTiled(pix4, pixa, 1.0, 0, 20, 32);
+    regTestWritePixAndCheck(rp, pix4, IFF_JFIF_JPEG);  /* 17 */
+    pixaAddPix(pixa, pix4, L_INSERT);
 
         /* Then on fpix */
     fpixg = pixConvertToFPix(pixg, 1);
     fpix1 = fpixConvolve(fpixg, kel, 1);
     pix5 = fpixConvertToPix(fpix1, 8, L_CLIP_TO_ZERO, 0);
-    regTestWritePixAndCheck(rp, pix5, IFF_JFIF_JPEG);  /* 16 */
-    pixSaveTiled(pix5, pixa, 1.0, 1, 20, 32);
+    regTestWritePixAndCheck(rp, pix5, IFF_JFIF_JPEG);  /* 18 */
+    pixaAddPix(pixa, pix5, L_INSERT);
     fpix2 = fpixConvolveSep(fpixg, kelx, kely, 1);
     pix6 = fpixConvertToPix(fpix2, 8, L_CLIP_TO_ZERO, 0);
-    regTestWritePixAndCheck(rp, pix6, IFF_JFIF_JPEG);  /* 17 */
-    pixSaveTiled(pix2, pixa, 1.0, 0, 20, 32);
-    regTestCompareSimilarPix(rp, pix1, pix5, 2, 0.00, 0);  /* 18 */
-    regTestCompareSimilarPix(rp, pix2, pix6, 2, 0.00, 0);  /* 19 */
-    pixDestroy(&pix1);
-    pixDestroy(&pix2);
-    pixDestroy(&pix3);
-    pixDestroy(&pix4);
-    pixDestroy(&pix5);
-    pixDestroy(&pix6);
+    regTestWritePixAndCheck(rp, pix6, IFF_JFIF_JPEG);  /* 19 */
+    pixaAddPix(pixa, pix6, L_INSERT);
+    regTestCompareSimilarPix(rp, pix1, pix5, 2, 0.00, 0);  /* 20 */
+    regTestCompareSimilarPix(rp, pix2, pix6, 2, 0.00, 0);  /* 21 */
     fpixDestroy(&fpixg);
     fpixDestroy(&fpix1);
     fpixDestroy(&fpix2);
 
-    pixd = pixaDisplay(pixa, 0, 0);
-    regTestWritePixAndCheck(rp, pixd, IFF_JFIF_JPEG);  /* 20 */
+    pixd = pixaDisplayTiledInColumns(pixa, 2, 1.0, 20, 2);
+    regTestWritePixAndCheck(rp, pixd, IFF_JFIF_JPEG);  /* 22 */
     pixDisplayWithTitle(pixd, 600, 100, NULL, rp->display);
     pixDestroy(&pixs);
     pixDestroy(&pixg);
@@ -251,7 +247,7 @@ L_REGPARAMS  *rp;
         return regTestCleanup(rp);
     }
     dewarpaInsertDewarp(dewa, dew);
-    dewarpBuildPageModel(dew, NULL);
+    dewarpBuildPageModel(dew, NULL);  /* two invalid indices in ptaGetPt() */
     dewarpPopulateFullRes(dew, NULL, 0, 0);
     fpixs = dew->fullvdispar;
     fpixs2 = fpixAddContinuedBorder(fpixs, 200, 200, 100, 300);
@@ -265,14 +261,14 @@ L_REGPARAMS  *rp;
     pix5 = pixRead("karen8.jpg");
     dpix2 = pixConvertToDPix(pix5, 1);
     pix6 = dpixConvertToPix(dpix2, 8, L_CLIP_TO_ZERO, 0);
-    regTestWritePixAndCheck(rp, pix1, IFF_PNG);  /* 21 */
+    regTestWritePixAndCheck(rp, pix1, IFF_PNG);  /* 23 */
     pixDisplayWithTitle(pix1, 0, 100, NULL, rp->display);
-    regTestWritePixAndCheck(rp, pix2, IFF_PNG);  /* 22 */
+    regTestWritePixAndCheck(rp, pix2, IFF_PNG);  /* 24 */
     pixDisplayWithTitle(pix2, 470, 100, NULL, rp->display);
-    regTestWritePixAndCheck(rp, pix3, IFF_PNG);  /* 23 */
+    regTestWritePixAndCheck(rp, pix3, IFF_PNG);  /* 25 */
     pixDisplayWithTitle(pix3, 1035, 100, NULL, rp->display);
-    regTestComparePix(rp, pix3, pix4);  /* 24 */
-    regTestComparePix(rp, pix5, pix6);  /* 25 */
+    regTestComparePix(rp, pix3, pix4);  /* 26 */
+    regTestComparePix(rp, pix5, pix6);  /* 27 */
     pixDestroy(&pixs);
     pixDestroy(&pixn);
     pixDestroy(&pixg);
@@ -293,14 +289,14 @@ L_REGPARAMS  *rp;
     fpixWrite("/tmp/lept/regout/fpix1.fp", dew->fullvdispar);
     fpix1 = fpixRead("/tmp/lept/regout/fpix1.fp");
     pix1 = fpixAutoRenderContours(fpix1, 40);
-    regTestWritePixAndCheck(rp, pix1, IFF_PNG);  /* 26 */
+    regTestWritePixAndCheck(rp, pix1, IFF_PNG);  /* 28 */
     pixDisplayWithTitle(pix1, 0, 500, NULL, rp->display);
     pixDestroy(&pix1);
 
     MakePtasAffine(1, &ptas, &ptad);
     fpix2 = fpixAffinePta(fpix1, ptad, ptas, 200, 0.0);
     pix2 = fpixAutoRenderContours(fpix2, 40);
-    regTestWritePixAndCheck(rp, pix2, IFF_PNG);  /* 27 */
+    regTestWritePixAndCheck(rp, pix2, IFF_PNG);  /* 29 */
     pixDisplayWithTitle(pix2, 400, 500, NULL, rp->display);
     fpixDestroy(&fpix2);
     pixDestroy(&pix2);
@@ -310,7 +306,7 @@ L_REGPARAMS  *rp;
     MakePtas(1, &ptas, &ptad);
     fpix2 = fpixProjectivePta(fpix1, ptad, ptas, 200, 0.0);
     pix3 = fpixAutoRenderContours(fpix2, 40);
-    regTestWritePixAndCheck(rp, pix3, IFF_PNG);  /* 28 */
+    regTestWritePixAndCheck(rp, pix3, IFF_PNG);  /* 30 */
     pixDisplayWithTitle(pix3, 400, 500, NULL, rp->display);
     fpixDestroy(&fpix1);
     fpixDestroy(&fpix2);
