@@ -57,29 +57,30 @@
  *        libwebp, libopenjp2, libgif
  */
 
-#include "allheaders.h"
-
-    /* Needed for checking libraries */
 #ifdef HAVE_CONFIG_H
 #include <config_auto.h>
 #endif /* HAVE_CONFIG_H */
 
-#define   BMP_FILE            "test1.bmp"
-#define   FILE_1BPP           "feyn.tif"
-#define   FILE_2BPP           "speckle2.png"
-#define   FILE_2BPP_C         "weasel2.4g.png"
-#define   FILE_4BPP           "speckle4.png"
-#define   FILE_4BPP_C         "weasel4.16c.png"
-#define   FILE_8BPP_1         "dreyfus8.png"
-#define   FILE_8BPP_2         "weasel8.240c.png"
-#define   FILE_8BPP_3         "test8.jpg"
-#define   FILE_16BPP          "test16.tif"
-#define   FILE_32BPP          "marge.jpg"
-#define   FILE_32BPP_ALPHA    "test32-alpha.png"
-#define   FILE_1BIT_ALPHA     "test-1bit-alpha.png"
-#define   FILE_CMAP_ALPHA     "test-cmap-alpha.png"
-#define   FILE_TRANS_ALPHA    "test-fulltrans-alpha.png"
-#define   FILE_GRAY_ALPHA     "test-gray-alpha.png"
+#include "allheaders.h"
+
+#define   BMP_FILE             "test1.bmp"
+#define   FILE_1BPP            "feyn.tif"
+#define   FILE_2BPP            "speckle2.png"
+#define   FILE_2BPP_C          "weasel2.4g.png"
+#define   FILE_4BPP            "speckle4.png"
+#define   FILE_4BPP_C          "weasel4.16c.png"
+#define   FILE_8BPP_1          "dreyfus8.png"
+#define   FILE_8BPP_2          "weasel8.240c.png"
+#define   FILE_8BPP_3          "test8.jpg"
+#define   FILE_16BPP           "test16.tif"
+#define   FILE_32BPP           "marge.jpg"
+#define   FILE_32BPP_ALPHA     "test32-alpha.png"
+#define   FILE_1BIT_ALPHA      "test-1bit-alpha.png"
+#define   FILE_CMAP_ALPHA      "test-cmap-alpha.png"
+#define   FILE_TRANS_ALPHA     "test-fulltrans-alpha.png"
+#define   FILE_GRAY_ALPHA      "test-gray-alpha.png"
+#define   FILE_GRAY_ALPHA_TIF  "gray-alpha.tif"
+#define   FILE_RGB16_TIF       "rgb16.tif"
 
 static l_int32 testcomp(const char *filename, PIX *pix, l_int32 comptype);
 static l_int32 testcomp_mem(PIX *pixs, PIX **ppixt, l_int32 index,
@@ -98,11 +99,12 @@ char          psname[256];
 char         *tempname;
 l_uint8      *data;
 l_int32       i, d, n, success, failure, same;
-l_int32       w, h, bps, spp;
+l_int32       w, h, bps, spp, iscmap;
 size_t        size, nbytes;
-PIX          *pix1, *pix2, *pix4, *pix8, *pix16, *pix32;
+PIX          *pix1, *pix2, *pix3, *pix4, *pix8, *pix16, *pix32;
 PIX          *pix, *pixt, *pixd;
 PIXA         *pixa;
+PIXCMAP      *cmap;
 L_REGPARAMS  *rp;
 
 #if  !HAVE_LIBJPEG
@@ -257,6 +259,102 @@ L_REGPARAMS  *rp;
             success = FALSE;
         pixDestroy(&pix);
     }
+
+        /* Test writing and reading tiff colormaps */
+    pix1 = pixRead(FILE_8BPP_2);
+    pixWrite("/tmp/lept/regout/weas8.tif", pix1, IFF_TIFF);
+    readHeaderTiff("/tmp/lept/regout/weas8.tif", 0, &w, &h, &bps, &spp,
+                   NULL, &iscmap, NULL);
+    if (w != 82 || h != 73 || bps != 8 || spp != 1 || iscmap != 1) {
+        fprintf(stderr, "Header error testing tiff cmaps\n");
+        success = FALSE;
+    }
+    pix2 = pixRead("/tmp/lept/regout/weas8.tif");
+    pixWrite("/tmp/lept/regout/weas8a.tif", pix2, IFF_TIFF);
+    pix3 = pixRead("/tmp/lept/regout/weas8a.tif");
+    pixEqual(pix1, pix3, &same);
+    if (!same) {
+        fprintf(stderr, "Tiff read/write failed for cmaps\n");
+        success = FALSE;
+    }
+    pixDestroy(&pix1);
+    pixDestroy(&pix2);
+    pixDestroy(&pix3);
+
+        /* Test writing and reading 1 bpp tiff with colormap */
+    pix1 = pixRead("feyn-fract2.tif");
+    cmap = pixcmapCreate(1);
+    pixcmapAddColor(cmap, 0, 0, 0);  /* inverted b/w */
+    pixcmapAddColor(cmap, 255, 255, 255);
+    pixSetColormap(pix1, cmap);
+    pixWrite("/tmp/lept/regout/fract1.tif", pix1, IFF_TIFF_ZIP);
+    pix2 = pixRead("/tmp/lept/regout/fract1.tif");
+    pixEqual(pix1, pix2, &same);
+    if (!same) {
+        fprintf(stderr, "Tiff read/write failed for 1 bpp cmap\n");
+        success = FALSE;
+    }
+    cmap = pixcmapCreate(1);
+    pixcmapAddColor(cmap, 255, 255, 255);
+    pixcmapAddColor(cmap, 100, 200, 50);  /* with color */
+    pixSetColormap(pix1, cmap);  /* replace the colormap */
+    pixWrite("/tmp/lept/regout/fract2.tif", pix1, IFF_TIFF_ZIP);
+    pix3 = pixRead("/tmp/lept/regout/fract2.tif");
+    pixEqual(pix1, pix3, &same);
+    if (!same) {
+        fprintf(stderr, "Tiff read/write failed for 1 bpp color cmap\n");
+        success = FALSE;
+    }
+    pixDestroy(&pix1);
+    pixDestroy(&pix2);
+    pixDestroy(&pix3);
+
+        /* Test writing and reading tiff with alpha */
+    pix1 = pixRead(FILE_GRAY_ALPHA_TIF);  /* converts to RGBA */
+    pixWrite("/tmp/lept/regout/graya.tif", pix1, IFF_TIFF);
+    readHeaderTiff("/tmp/lept/regout/graya.tif", 0, &w, &h, &bps, &spp,
+                   NULL, &iscmap, NULL);
+    if (w != 100 || h != 100 || bps != 8 || spp != 4 || iscmap != 0) {
+        fprintf(stderr, "Header error testing tiff with alpha\n");
+        success = FALSE;
+    }
+    pix2 = pixRead("/tmp/lept/regout/graya.tif");
+    pixEqual(pix1, pix2, &same);
+    if (!same) {
+        fprintf(stderr, "Tiff read/write failed for graya.tif\n");
+        success = FALSE;
+    }
+    pixDestroy(&pix1);
+    pixDestroy(&pix2);
+    pix1 = pixRead(FILE_GRAY_ALPHA);  /* converts to RGBA */
+    pixWriteTiff("/tmp/lept/regout/graya2.tif", pix1, IFF_TIFF_ZIP, "w");
+    pix2 = pixRead("/tmp/lept/regout/graya2.tif");
+    pixEqual(pix1, pix2, &same);
+    if (!same) {
+        fprintf(stderr, "Tiff read/write failed for graya2.tif\n");
+        success = FALSE;
+    }
+    pixDestroy(&pix1);
+    pixDestroy(&pix2);
+
+        /* Test reading 16 bit sampled rgb tiff */
+    pix1 = pixRead(FILE_RGB16_TIF);  /* converts 16 to 8 bits RGB */
+    pixWrite("/tmp/lept/regout/rgb16.tif", pix1, IFF_TIFF_ZIP);
+    readHeaderTiff("/tmp/lept/regout/rgb16.tif", 0, &w, &h, &bps, &spp,
+                   NULL, &iscmap, NULL);
+    if (w != 129 || h != 90 || bps != 8 || spp != 3 || iscmap != 0) {
+        fprintf(stderr, "Header error testing tiff with alpha\n");
+        success = FALSE;
+    }
+    pix2 = pixRead("/tmp/lept/regout/rgb16.tif");
+    pixEqual(pix1, pix2, &same);
+    if (!same) {
+        fprintf(stderr, "Tiff read/write failed for rgb16.tif\n");
+        success = FALSE;
+    }
+    pixDestroy(&pix1);
+    pixDestroy(&pix2);
+
     if (success)
         fprintf(stderr,
             "\n  ********** Success on tiff r/w to file *********\n\n");
@@ -488,6 +586,8 @@ part6:
     pixWrite(tempname, pix, IFF_PNM);
     if (get_header_data(tempname, IFF_PNM)) success = FALSE;
     pixDestroy(&pix);
+
+        /* These tiff formats work on 1 bpp images */
     pix = pixRead(FILE_1BPP);
     pixWrite(tempname, pix, IFF_TIFF_G3);
     if (get_header_data(tempname, IFF_TIFF_G3)) success = FALSE;
@@ -636,7 +736,8 @@ PIX       *pixd = NULL;
         return 1;
     }
 
-    if (format == IFF_JFIF_JPEG || format == IFF_JP2 || format == IFF_WEBP) {
+    if (format == IFF_JFIF_JPEG || format == IFF_JP2 ||
+        format == IFF_WEBP || format == IFF_TIFF_JPEG) {
         ds = pixGetDepth(pixs);
         dd = pixGetDepth(pixd);
         if (dd == 8) {
@@ -721,10 +822,7 @@ size_t      size1, size2;
         return 0;
 #endif  /* !HAVE_LIBPNG */
 #if !HAVE_LIBTIFF
-    if (true_format == IFF_TIFF_G3 || true_format == IFF_TIFF_G4 ||
-        true_format == IFF_TIFF_ZIP || true_format == IFF_TIFF_LZW ||
-        true_format == IFF_TIFF_PACKBITS || true_format == IFF_TIFF_RLE ||
-        true_format == IFF_TIFF)
+    if (L_FORMAT_IS_TIFF(true_format))
         return 0;
 #endif  /* !HAVE_LIBTIFF */
 
@@ -736,7 +834,7 @@ size_t      size1, size2;
     if (ret1)
         fprintf(stderr, "Error: couldn't read header data: %s\n", filename);
     else {
-        if (format1 > IFF_PNG && format1 < IFF_PNM) {
+        if (L_FORMAT_IS_TIFF(format1)) {
             tiff_compression_name = get_tiff_compression_name(format1);
             fprintf(stderr, "Format data for image %s with format %s:\n"
                 "  nbytes = %lu, size (w, h, d) = (%d, %d, %d)\n"
@@ -796,6 +894,8 @@ get_tiff_compression_name(l_int32  format)
         tiff_compression_name =  "tiff_rle";
     else if (format == IFF_TIFF_PACKBITS)
         tiff_compression_name =  "tiff_packbits";
+    else if (format == IFF_TIFF_JPEG)
+        tiff_compression_name =  "tiff_jpeg";
     else if (format == IFF_TIFF)
         tiff_compression_name = "tiff_uncompressed";
     else
