@@ -42,6 +42,10 @@
  *               /tmp/lept/jb/result
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
     /* Choose one of these */
@@ -57,11 +61,6 @@
 #define   DISPLAY_DIFFERENCE        1
 #define   DISPLAY_ALL_INSTANCES     0
 
-    /* For display output of all instances, sorted by class */
-#define   X_SPACING                10
-#define   Y_SPACING                15
-#define   MAX_OUTPUT_WIDTH         400
-
 static const char  rootname[] = "/tmp/lept/jb/result";
 
 int main(int    argc,
@@ -82,7 +81,6 @@ static char  mainName[] = "jbcorrelation";
         return ERROR_INT(
              " Syntax: jbcorrelation dirin thresh weight [firstpage, npages]",
              mainName, 1);
-
     dirin = argv[1];
     thresh = atof(argv[2]);
     weight = atof(argv[3]);
@@ -96,6 +94,7 @@ static char  mainName[] = "jbcorrelation";
         npages = atoi(argv[5]);
     }
 
+    setLeptDebugOK(1);
     lept_mkdir("lept/jb");
 
 #if 0
@@ -206,9 +205,9 @@ static char  mainName[] = "jbcorrelation";
 #endif  /* RENDER_DEBUG */
 
 #if  DISPLAY_ALL_INSTANCES
-        /* display all instances, organized by template */
-    pix = pixaaDisplayByPixa(classer->pixaa,
-                             X_SPACING, Y_SPACING, MAX_OUTPUT_WIDTH);
+        /* Display all instances, organized by template.
+         * The display programs have a lot of trouble with these. */
+    pix = pixaaDisplayByPixa(classer->pixaa, 5, 1.0, 10, 0, 0);
     pixWrite("/tmp/lept/jb/output_instances", pix, IFF_PNG);
     pixDestroy(&pix);
 #endif  /* DISPLAY_ALL_INSTANCES */
