@@ -36,10 +36,16 @@
  *
  *    * jp2k supports 8 bpp gray, rgb, and rgba.
  *    * This makes calls into the jpeg2000 library libopenjp2.
- *    * Compared to reading and writing jpeg, reading jp2k is slow
- *      and writing jp2k is very slow.
+ *    * Compared to reading and writing jpeg, reading jp2k is
+ *      very slow, and writing jp2k is miserably slow.
+ *    * If we try to run this starting with image half the size,
+ *      the library gives opj_start_compress() encoding errors!
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <math.h>
 #include "allheaders.h"
 
@@ -74,12 +80,10 @@ L_REGPARAMS  *rp;
     if (regTestSetup(argc, argv, &rp))
         return 1;
 
-    lept_mkdir("lept/jp2");
     DoJp2kTest1(rp, "karen8.jpg");
     DoJp2kTest1(rp, "test24.jpg");
-    DoJp2kTest2(rp, "karen8.jpg");
+/*    DoJp2kTest2(rp, "karen8.jpg");  */  /* encode fails on smallest image */
     DoJp2kTest2(rp, "test24.jpg");
-
     return regTestCleanup(rp);
 }
 
@@ -87,35 +91,43 @@ L_REGPARAMS  *rp;
 void DoJp2kTest1(L_REGPARAMS  *rp,
                  const char   *fname)
 {
-char  buf[256];
+char     buf[64];
+char    *name;
 l_int32  w, h;
 BOX     *box;
-PIX     *pix1, *pix2, *pix3;
+PIX     *pix0, *pix1, *pix2, *pix3;
 
-    pix1 = pixRead(fname);
+        /* Read, write, read back and write again */
+    pix0 = pixRead(fname);
+    pix1 = pixScale(pix0, 0.5, 0.5);
     pixGetDimensions(pix1, &w, &h, NULL);
-    box = boxCreate(w / 4, h / 4, w / 2, h / 2);
-    snprintf(buf, sizeof(buf), "/tmp/lept/jp2/jp2kio.%03d.jp2", rp->index + 1);
-    pixWrite(buf, pix1, IFF_JP2);
-    regTestCheckFile(rp, buf);
-    pix2 = pixRead(buf);
+    regTestWritePixAndCheck(rp, pix1, IFF_JP2);
+    name = regTestGenLocalFilename(rp, -1, IFF_JP2);
+    pix2 = pixRead(name);
+    regTestWritePixAndCheck(rp, pix2, IFF_JP2);
     pixDisplayWithTitle(pix2, 0, 100, "1", rp->display);
     pixDestroy(&pix1);
     pixDestroy(&pix2);
 
-    pix1 = pixReadJp2k(buf, 1, box, 0, 0);  /* just read the box region */
-    pixWriteJp2k(buf, pix1, 38, 0, 0, 0);
+        /* Test cropping and scaling in the jp2 interface */
+    box = boxCreate(w / 4, h / 4, w / 2, h / 2);
+    pix1 = pixReadJp2k(name, 1, box, 0, 0);  /* read cropped to the box */
+    snprintf(buf, sizeof(buf), "/tmp/lept/regout/jp2kio.%02d.jp2",
+             rp->index + 1);
+    pixWriteJp2k(buf, pix1, 38, 0, 0, 0);  /* write cropped to the box */
     regTestCheckFile(rp, buf);
-    pix2 = pixRead(buf);
+    pix2 = pixRead(buf);  /* read the cropped image */
     regTestWritePixAndCheck(rp, pix2, IFF_JP2);
     pixDisplayWithTitle(pix2, 500, 100, "2", rp->display);
-    pix3 = pixReadJp2k(buf, 2, NULL, 0, 0);  /* read image at 2x reduction */
+    pix3 = pixReadJp2k(buf, 2, NULL, 0, 0);  /* read cropped image at 2x red */
     regTestWritePixAndCheck(rp, pix3, IFF_JP2);
     pixDisplayWithTitle(pix3, 1000, 100, "3", rp->display);
+    pixDestroy(&pix0);
     pixDestroy(&pix1);
     pixDestroy(&pix2);
     pixDestroy(&pix3);
     boxDestroy(&box);
+    lept_free(name);
     return;
 }
 
@@ -123,43 +135,50 @@ void DoJp2kTest2(L_REGPARAMS  *rp,
                  const char   *fname)
 {
 char      buf[256];
+char     *name;
 l_uint8  *data;
 l_int32   w, h;
 size_t    nbytes;
 BOX      *box;
-PIX      *pix1, *pix2, *pix3;
+PIX      *pix0, *pix1, *pix2, *pix3;
 
-    pix1 = pixRead(fname);
+        /* Test the memory interface */
+    pix0 = pixRead(fname);
+    pix1 = pixScale(pix0, 0.5, 0.5);
     pixGetDimensions(pix1, &w, &h, NULL);
-    box = boxCreate(w / 3, h / 3, w / 3, h / 3);
-    snprintf(buf, sizeof(buf), "/tmp/lept/jp2/jp2kio.%03d.jp2", rp->index + 1);
-    pixWrite(buf, pix1, IFF_JP2);
-    regTestCheckFile(rp, buf);
-    pix2 = pixRead(buf);
-    data = l_binaryRead(buf, &nbytes);
+    regTestWritePixAndCheck(rp, pix1, IFF_JP2);
+    name = regTestGenLocalFilename(rp, -1, IFF_JP2);
+    pix2 = pixRead(name);
+    regTestWritePixAndCheck(rp, pix2, IFF_JP2);
+    data = l_binaryRead(name, &nbytes);
     pix3 = pixReadMemJp2k(data, nbytes, 1, NULL, 0, 0);
+    regTestWritePixAndCheck(rp, pix3, IFF_JP2);
     pixDisplayWithTitle(pix3, 0, 100, "1", rp->display);
     pixDestroy(&pix1);
     pixDestroy(&pix2);
     pixDestroy(&pix3);
     lept_free(data);
 
-    pix1 = pixReadJp2k(buf, 1, box, 0, 0);  /* just read the box region */
-    pixWriteJp2k(buf, pix1, 38, 0, 0, 0);
+        /* Test scaling on read with the memory interface */
+    box = boxCreate(w / 3, h / 3, w / 3, h / 3);
+    pix1 = pixReadJp2k(name, 1, box, 0, 0);  /* just read the box region */
+    snprintf(buf, sizeof(buf), "/tmp/lept/regout/jp2kio.%02d.jp2",
+             rp->index + 1);
+    pixWriteJp2k(buf, pix1, 38, 0, 0, 0);  /* write cropped to the box */
     regTestCheckFile(rp, buf);
     data = l_binaryRead(buf, &nbytes);
-    pix2 = pixReadMemJp2k(data, nbytes, 1, NULL, 0, 0);
+    pix2 = pixReadMemJp2k(data, nbytes, 1, NULL, 0, 0);  /* read it again */
     regTestWritePixAndCheck(rp, pix2, IFF_JP2);
     pixDisplayWithTitle(pix2, 500, 100, "2", rp->display);
     pix3 = pixReadMemJp2k(data, nbytes, 2, NULL, 0, 0);  /* read at 2x red */
     regTestWritePixAndCheck(rp, pix3, IFF_JP2);
     pixDisplayWithTitle(pix3, 1000, 100, "3", rp->display);
     boxDestroy(&box);
+    pixDestroy(&pix0);
     pixDestroy(&pix1);
     pixDestroy(&pix2);
     pixDestroy(&pix3);
     lept_free(data);
+    lept_free(name);
     return;
 }
-
-
