@@ -42,6 +42,10 @@
  *    Use pageseg*.tif input images.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
     /* Control the display output */
@@ -60,8 +64,9 @@ static char  mainName[] = "livre_pageseg";
 
     if (argc != 2)
         return ERROR_INT(" Syntax:  livre_pageseg filein", mainName, 1);
-
     filein = argv[1];
+    setLeptDebugOK(1);
+
     if ((pixs = pixRead(filein)) == NULL)
         return ERROR_INT("pix not made", mainName, 1);
 
@@ -76,32 +81,32 @@ l_int32
 DoPageSegmentation(PIX     *pixs,   /* should be at least 300 ppi */
                    l_int32  which)  /* 1, 2, 3, 4 */
 {
-char         buf[256];
-l_int32      zero;
-BOXA        *boxatm, *boxahm;
-PIX         *pixr;   /* image reduced to 150 ppi */
-PIX         *pixhs;  /* image of halftone seed, 150 ppi */
-PIX         *pixm;   /* image of mask of components, 150 ppi */
-PIX         *pixhm1; /* image of halftone mask, 150 ppi */
-PIX         *pixhm2; /* image of halftone mask, 300 ppi */
-PIX         *pixht;  /* image of halftone components, 150 ppi */
-PIX         *pixnht; /* image without halftone components, 150 ppi */
-PIX         *pixi;   /* inverted image, 150 ppi */
-PIX         *pixvws; /* image of vertical whitespace, 150 ppi */
-PIX         *pixm1;  /* image of closed textlines, 150 ppi */
-PIX         *pixm2;  /* image of refined text line mask, 150 ppi */
-PIX         *pixm3;  /* image of refined text line mask, 300 ppi */
-PIX         *pixb1;  /* image of text block mask, 150 ppi */
-PIX         *pixb2;  /* image of text block mask, 300 ppi */
-PIX         *pixnon; /* image of non-text or halftone, 150 ppi */
-PIX         *pix1, *pix2, *pix3, *pix4;
-PIXA        *pixa;
-PIXCMAP     *cmap;
-PTAA        *ptaa;
-l_int32      ht_flag = 0;
-l_int32      ws_flag = 0;
-l_int32      text_flag = 0;
-l_int32      block_flag = 0;
+char      buf[256];
+l_int32   zero;
+BOXA     *boxatm, *boxahm;
+PIX      *pixr;   /* image reduced to 150 ppi */
+PIX      *pixhs;  /* image of halftone seed, 150 ppi */
+PIX      *pixm;   /* image of mask of components, 150 ppi */
+PIX      *pixhm1; /* image of halftone mask, 150 ppi */
+PIX      *pixhm2; /* image of halftone mask, 300 ppi */
+PIX      *pixht;  /* image of halftone components, 150 ppi */
+PIX      *pixnht; /* image without halftone components, 150 ppi */
+PIX      *pixi;   /* inverted image, 150 ppi */
+PIX      *pixvws; /* image of vertical whitespace, 150 ppi */
+PIX      *pixm1;  /* image of closed textlines, 150 ppi */
+PIX      *pixm2;  /* image of refined text line mask, 150 ppi */
+PIX      *pixm3;  /* image of refined text line mask, 300 ppi */
+PIX      *pixb1;  /* image of text block mask, 150 ppi */
+PIX      *pixb2;  /* image of text block mask, 300 ppi */
+PIX      *pixnon; /* image of non-text or halftone, 150 ppi */
+PIX      *pix1, *pix2, *pix3;
+PIXA     *pixa;
+PIXCMAP  *cmap;
+PTAA     *ptaa;
+l_int32   ht_flag = 0;
+l_int32   ws_flag = 0;
+l_int32   text_flag = 0;
+l_int32   block_flag = 0;
 
     PROCNAME("DoPageSegmentation");
 
