@@ -37,6 +37,10 @@
  *     Example: modifyhuesat test24.jpg 5 0.2 5 0.2 /tmp/junkout.jpg
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
 int main(int    argc,
