@@ -31,6 +31,10 @@
  *    showing display mode and rejection of invalid sequence components.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
 #define  SEQUENCE1    "O1.3 + C3.1 + R22 + D2.2 + X4"
@@ -51,8 +55,9 @@ static char  mainName[] = "morphseq_reg";
     if (argc != 1)
         return ERROR_INT(" Syntax:  morphseq_reg", mainName, 1);
 
-    pixs = pixRead("feyn.tif");
+    setLeptDebugOK(1);
     lept_mkdir("lept");
+    pixs = pixRead("feyn.tif");
 
         /* 1 bpp */
     pixd = pixMorphSequence(pixs, SEQUENCE1, -1);
