@@ -33,6 +33,10 @@
  *       - writing special tiff tags to file [not tested here]
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 #include <string.h>
 
@@ -48,14 +52,18 @@ char         *fname, *filename;
 const char   *str;
 char          buf[512];
 l_int32       i, n, npages, equal, success;
-size_t        length, offset, size;
+size_t        offset, size;
 FILE         *fp;
-NUMA         *naflags, *nasizes;
-PIX          *pix, *pix1, *pix2;
+PIX          *pix1, *pix2;
 PIXA         *pixa, *pixa1, *pixa2, *pixa3;
-SARRAY       *savals, *satypes, *sa;
+SARRAY       *sa;
 L_REGPARAMS  *rp;
 
+#if !defined(HAVE_LIBPNG)
+    L_ERROR("This test requires libpng to run.\n", "mtiff_reg");
+    exit(77);
+#endif
+
    if (regTestSetup(argc, argv, &rp))
         return 1;
 
@@ -266,7 +274,7 @@ L_REGPARAMS  *rp;
     regTestCompareValues(rp, TRUE, success, 0);  /* 19 */
     if (success) {
         tiffGetCount(fp, &npages);
-        regTestCompareValues(rp, 4, npages, 0);  /* 20 */
+        regTestCompareValues(rp, 5, npages, 0);  /* 20 */
         fprintf(stderr, " Tiff: %d page\n", npages);
     }
     lept_fclose(fp);
@@ -312,8 +320,13 @@ L_REGPARAMS  *rp;
     pixaDestroy(&pixa);
 
 
-#if 0    /* -----   test adding custom public tags to a tiff header ----- */
-    pix = pixRead("feyn.tif");
+#if 1    /* -----   test adding custom public tags to a tiff header ----- */
+{
+    size_t        length;
+    NUMA         *naflags, *nasizes;
+    SARRAY       *savals, *satypes;
+
+    pix1 = pixRead("feyn.tif");
     naflags = numaCreate(10);
     savals = sarrayCreate(10);
     satypes = sarrayCreate(10);
@@ -323,32 +336,32 @@ L_REGPARAMS  *rp;
     numaAddNumber(naflags, 700);
     str = "<xmp>This is a Fake XMP packet</xmp>\n<text>Guess what ...?</text>";
     length = strlen(str);
-    sarrayAddString(savals, (char *)str, L_COPY);
-    sarrayAddString(satypes, (char *)"char*", L_COPY);
+    sarrayAddString(savals, str, L_COPY);
+    sarrayAddString(satypes, "char*", L_COPY);
     numaAddNumber(nasizes, length);  /* get it all */
 
     numaAddNumber(naflags, 269);  /* DOCUMENTNAME */
-    sarrayAddString(savals, (char *)"One silly title", L_COPY);
-    sarrayAddString(satypes, (char *)"const char*", L_COPY);
+    sarrayAddString(savals, "One silly title", L_COPY);
+    sarrayAddString(satypes, "const char*", L_COPY);
     numaAddNumber(naflags, 270);  /* IMAGEDESCRIPTION */
-    sarrayAddString(savals, (char *)"One page of text", L_COPY);
-    sarrayAddString(satypes, (char *)"const char*", L_COPY);
+    sarrayAddString(savals, "One page of text", L_COPY);
+    sarrayAddString(satypes, "const char*", L_COPY);
         /* the max sample is used by rendering programs
          * to scale the dynamic range */
     numaAddNumber(naflags, 281);  /* MAXSAMPLEVALUE */
-    sarrayAddString(savals, (char *)"4", L_COPY);
-    sarrayAddString(satypes, (char *)"l_uint16", L_COPY);
+    sarrayAddString(savals, "4", L_COPY);
+    sarrayAddString(satypes, "l_uint16", L_COPY);
         /* note that date is required to be a 20 byte string */
     numaAddNumber(naflags, 306);  /* DATETIME */
-    sarrayAddString(savals, (char *)"2004:10:11 09:35:15", L_COPY);
-    sarrayAddString(satypes, (char *)"const char*", L_COPY);
+    sarrayAddString(savals, "2004:10:11 09:35:15", L_COPY);
+    sarrayAddString(satypes, "const char*", L_COPY);
         /* note that page number requires 2 l_uint16 input */
     numaAddNumber(naflags, 297);  /* PAGENUMBER */
-    sarrayAddString(savals, (char *)"1-412", L_COPY);
-    sarrayAddString(satypes, (char *)"l_uint16-l_uint16", L_COPY);
-    pixWriteTiffCustom("/tmp/lept/tiff/tags.tif", pix, IFF_TIFF_G4, "w", naflags,
+    sarrayAddString(savals, "1-412", L_COPY);
+    sarrayAddString(satypes, "l_uint16-l_uint16", L_COPY);
+    pixWriteTiffCustom("/tmp/lept/tiff/tags.tif", pix1, IFF_TIFF_G4, "w", naflags,
                        savals, satypes, nasizes);
-    fprintTiffInfo(stderr, (char *)"/tmp/lept/tiff/tags.tif");
+    fprintTiffInfo(stderr, "/tmp/lept/tiff/tags.tif");
     fprintf(stderr, "num flags = %d\n", numaGetCount(naflags));
     fprintf(stderr, "num sizes = %d\n", numaGetCount(nasizes));
     fprintf(stderr, "num vals = %d\n", sarrayGetCount(savals));
@@ -357,7 +370,8 @@ L_REGPARAMS  *rp;
     numaDestroy(&nasizes);
     sarrayDestroy(&savals);
     sarrayDestroy(&satypes);
-    pixDestroy(&pix);
+    pixDestroy(&pix1);
+}
 #endif
 
     return regTestCleanup(rp);
