@@ -35,6 +35,10 @@
  *   Uses 6 images, all segmented and scaled to a fixed width
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
     /* All images scaled to this width  */
