@@ -31,6 +31,10 @@
  *  pixels are added (randomly, here) to a pix.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
 static PIX *PixDisplayWithColormap(PIX *pixs, l_int32 repl);
