@@ -30,6 +30,10 @@
  *     Regression test exercising various pixaDisplay*() functions.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
 int main(int    argc,
