@@ -33,6 +33,10 @@
  *    We also show some other ways to accumulate and display pixa.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <math.h>
 #include "allheaders.h"
 
@@ -47,7 +51,7 @@ int main(int    argc,
          char **argv)
 {
 l_uint8      *data1, *data2;
-l_int32       i, n;
+l_int32       i;
 size_t        size1, size2;
 BOX          *box;
 PIX          *pix, *pix1, *pix2, *pix3;
@@ -72,11 +76,10 @@ SARRAY       *sa;
     pixc2 = pixcompCreateFromPix(pix2, IFF_JFIF_JPEG);
     pix3 = pixCreateFromPixcomp(pixc2);
     regTestWritePixAndCheck(rp, pix3, IFF_JFIF_JPEG);  /* 0 */
-    pixSaveTiledOutline(pix3, pixa, 1.0, 1, 30, 2, 32);
+    pixaAddPix(pixa, pix3, L_INSERT);
     pixacompAddPix(pixac, pix1, IFF_DEFAULT);
     pixDestroy(&pix1);
     pixDestroy(&pix2);
-    pixDestroy(&pix3);
     pixcompDestroy(&pixc1);
     pixcompDestroy(&pixc2);
 
@@ -87,11 +90,10 @@ SARRAY       *sa;
     pixc2 = pixcompCreateFromPix(pix2, IFF_JFIF_JPEG);
     pix3 = pixCreateFromPixcomp(pixc2);
     regTestWritePixAndCheck(rp, pix3, IFF_JFIF_JPEG);  /* 1 */
-    pixSaveTiledOutline(pix3, pixa, 1.0, 1, 30, 2, 32);
+    pixaAddPix(pixa, pix3, L_INSERT);
     pixacompAddPix(pixac, pix1, IFF_DEFAULT);
     pixDestroy(&pix1);
     pixDestroy(&pix2);
-    pixDestroy(&pix3);
     pixcompDestroy(&pixc1);
     pixcompDestroy(&pixc2);
 
@@ -102,13 +104,12 @@ SARRAY       *sa;
     pixc2 = pixcompCreateFromPix(pix2, IFF_TIFF_G4);
     pix3 = pixCreateFromPixcomp(pixc2);
     regTestWritePixAndCheck(rp, pix3, IFF_TIFF_G4);  /* 2 */
-    pixSaveTiledOutline(pix3, pixa, 1.0, 0, 30, 2, 32);
+    pixaAddPix(pixa, pix3, L_INSERT);
     pixacompAddPix(pixac, pix1, IFF_DEFAULT);
     boxDestroy(&box);
     pixDestroy(&pix);
     pixDestroy(&pix1);
     pixDestroy(&pix2);
-    pixDestroy(&pix3);
     pixcompDestroy(&pixc1);
     pixcompDestroy(&pixc2);
 
@@ -118,11 +119,10 @@ SARRAY       *sa;
     pixc2 = pixcompCreateFromPix(pix2, IFF_PNG);
     pix3 = pixCreateFromPixcomp(pixc2);
     regTestWritePixAndCheck(rp, pix3, IFF_PNG);  /* 3 */
-    pixSaveTiledOutline(pix3, pixa, 1.0, 0, 30, 2, 32);
+    pixaAddPix(pixa, pix3, L_INSERT);
     pixacompAddPix(pixac, pix1, IFF_DEFAULT);
     pixDestroy(&pix1);
     pixDestroy(&pix2);
-    pixDestroy(&pix3);
     pixcompDestroy(&pixc1);
     pixcompDestroy(&pixc2);
 
