@@ -32,6 +32,10 @@
  *     the plot commands and data required for input to gnuplot.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include <math.h>
 
