@@ -33,6 +33,10 @@
  *       - boxaEqual() and pixaEqual()
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
 void BoxaSortTest(L_REGPARAMS *rp, const char *fname, l_int32 index,
@@ -45,6 +49,11 @@ int main(int    argc,
 {
 L_REGPARAMS  *rp;
 
+#if !defined(HAVE_LIBPNG)
+    L_ERROR("This test requires libpng to run.\n", "ptra2_reg");
+    exit(77);
+#endif
+
     if (regTestSetup(argc, argv, &rp))
         return 1;
 
