@@ -56,6 +56,10 @@
  *     that is used to identify unknown (scaled) samples.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "string.h"
 #include "allheaders.h"
 
