@@ -37,6 +37,10 @@
  *      Works on palette images of 2, 4 and 8 bpp
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
 int main(int    argc,
@@ -51,10 +55,10 @@ static char  mainName[] = "removecmap";
     if (argc != 4)
         return ERROR_INT("Syntax:  removecmap filein type fileout",
                          mainName, 1);
-
     filein = argv[1];
     type = atoi(argv[2]);
     fileout = argv[3];
+    setLeptDebugOK(1);
 
     if ((pixs = pixRead(filein)) == NULL)
         return ERROR_INT("pixs not made", mainName, 1);
