@@ -30,6 +30,10 @@
  *     This tests the font rendering functions
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
 #define   DIRECTORY    "./fonts"
