@@ -84,10 +84,14 @@
  *   Cygwin needs to prevent it from appending ".exe" to the filename.)
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include "allheaders.h"
 
-static const l_int32  L_BUF_SIZE = 512;
+static const l_int32  L_BUFSIZE = 512;  /* hardcoded below in sscanf() */
 static const char *version = "1.5";
 
 
@@ -95,13 +99,11 @@ int main(int    argc,
          char **argv)
 {
 char        *filein, *str, *tempfile, *prestring, *outprotos, *protostr;
-const char  *spacestr = " ";
-char         buf[L_BUF_SIZE];
-l_uint8     *allheaders;
+char         buf[L_BUFSIZE];
 l_int32      i, maxindex, in_line, nflags, protos_added, firstfile, len, ret;
 size_t       nbytes;
 L_BYTEA     *ba, *ba2;
-SARRAY      *sa, *safirst;
+SARRAY      *sa;
 static char  mainName[] = "xtractprotos";
 
     if (argc == 1) {
@@ -114,6 +116,8 @@ static char  mainName[] = "xtractprotos";
         return 1;
     }
 
+    setLeptDebugOK(1);
+
     /* ---------------------------------------------------------------- */
     /* Parse input flags and find prestring and outprotos, if requested */
     /* ---------------------------------------------------------------- */
@@ -125,12 +129,12 @@ static char  mainName[] = "xtractprotos";
         if (argv[i][0] == '-') {
             if (!strncmp(argv[i], "-prestring", 10)) {
                 nflags++;
-                ret = sscanf(argv[i] + 1, "prestring=%s", buf);
+                ret = sscanf(argv[i] + 1, "prestring=%490s", buf);
                 if (ret != 1) {
                     fprintf(stderr, "parse failure for prestring\n");
                     return 1;
                 }
-                if ((len = strlen(buf)) > L_BUF_SIZE - 3) {
+                if ((len = strlen(buf)) > L_BUFSIZE - 3) {
                     L_WARNING("prestring too large; omitting!\n", mainName);
                 } else {
                     buf[len] = ' ';
@@ -139,7 +143,7 @@ static char  mainName[] = "xtractprotos";
                 }
             } else if (!strncmp(argv[i], "-protos", 7)) {
                 nflags++;
-                ret = sscanf(argv[i] + 1, "protos=%s", buf);
+                ret = sscanf(argv[i] + 1, "protos=%490s", buf);
                 if (ret != 1) {
                     fprintf(stderr, "parse failure for protos\n");
                     return 1;
@@ -164,15 +168,15 @@ static char  mainName[] = "xtractprotos";
 
         /* First the extern C head */
     sa = sarrayCreate(0);
-    sarrayAddString(sa, (char *)"/*", L_COPY);
-    snprintf(buf, L_BUF_SIZE,
+    sarrayAddString(sa, "/*", L_COPY);
+    snprintf(buf, L_BUFSIZE,
              " *  These prototypes were autogen'd by xtractprotos, v. %s",
              version);
     sarrayAddString(sa, buf, L_COPY);
-    sarrayAddString(sa, (char *)" */", L_COPY);
-    sarrayAddString(sa, (char *)"#ifdef __cplusplus", L_COPY);
-    sarrayAddString(sa, (char *)"extern \"C\" {", L_COPY);
-    sarrayAddString(sa, (char *)"#endif  /* __cplusplus */\n", L_COPY);
+    sarrayAddString(sa, " */", L_COPY);
+    sarrayAddString(sa, "#ifdef __cplusplus", L_COPY);
+    sarrayAddString(sa, "extern \"C\" {", L_COPY);
+    sarrayAddString(sa, "#endif  /* __cplusplus */\n", L_COPY);
     str = sarrayToString(sa, 1);
     l_byteaAppendString(ba, str);
     lept_free(str);
@@ -190,7 +194,7 @@ static char  mainName[] = "xtractprotos";
         len = strlen(filein);
         if (filein[len - 1] == 'h')  /* skip .h files */
             continue;
-        snprintf(buf, L_BUF_SIZE, "cpp -ansi -DNO_PROTOS %s %s",
+        snprintf(buf, L_BUFSIZE, "cpp -ansi -DNO_PROTOS %s %s",
                  filein, tempfile);
         ret = system(buf);  /* cpp */
         if (ret) {
@@ -213,9 +217,9 @@ static char  mainName[] = "xtractprotos";
 
         /* Lastly the extern C tail */
     sa = sarrayCreate(0);
-    sarrayAddString(sa, (char *)"\n#ifdef __cplusplus", L_COPY);
-    sarrayAddString(sa, (char *)"}", L_COPY);
-    sarrayAddString(sa, (char *)"#endif  /* __cplusplus */", L_COPY);
+    sarrayAddString(sa, "\n#ifdef __cplusplus", L_COPY);
+    sarrayAddString(sa, "}", L_COPY);
+    sarrayAddString(sa, "#endif  /* __cplusplus */", L_COPY);
     str = sarrayToString(sa, 1);
     l_byteaAppendString(ba, str);
     lept_free(str);
