@@ -38,22 +38,25 @@
  *  ===================================================================
  *
  *      Adaptive Otsu-based thresholding
- *          l_int32    pixOtsuAdaptiveThreshold()       8 bpp
+ *          l_int32       pixOtsuAdaptiveThreshold()       8 bpp
  *
  *      Otsu thresholding on adaptive background normalization
- *          PIX       *pixOtsuThreshOnBackgroundNorm()  8 bpp
+ *          PIX          *pixOtsuThreshOnBackgroundNorm()  8 bpp
  *
  *      Masking and Otsu estimate on adaptive background normalization
- *          PIX       *pixMaskedThreshOnBackgroundNorm()  8 bpp
+ *          PIX          *pixMaskedThreshOnBackgroundNorm()  8 bpp
  *
  *      Sauvola local thresholding
- *          l_int32    pixSauvolaBinarizeTiled()
- *          l_int32    pixSauvolaBinarize()
- *          PIX       *pixSauvolaGetThreshold()
- *          PIX       *pixApplyLocalThreshold();
+ *          l_int32       pixSauvolaBinarizeTiled()
+ *          l_int32       pixSauvolaBinarize()
+ *          static PIX   *pixSauvolaGetThreshold()
+ *          static PIX   *pixApplyLocalThreshold();
  *
- *      Thresholding using connected components
- *          PIX       *pixThresholdByConnComp()
+ *      Global thresholding using connected components
+ *          PIX          *pixThresholdByConnComp()
+ *
+ *      Global thresholding by histogram
+ *          PIX          *pixThresholdByHisto()
  *
  *  Notes:
  *      (1) pixOtsuAdaptiveThreshold() computes a global threshold over each
@@ -65,35 +68,43 @@
  *          that use background normalization with other techniques.
  *      (3) Sauvola binarization computes a local threshold based on
  *          the local average and square average.  It takes two constants:
- *          the window size for the measurment at each pixel and a
+ *          the window size for the measurement at each pixel and a
  *          parameter that determines the amount of normalized local
  *          standard deviation to subtract from the local average value.
- *      (4) pixThresholdByCC() uses the numbers of 4 and 8 connected
+ *      (4) pixThresholdByConnComp() uses the numbers of 4 and 8 connected
  *          components at different thresholding to determine if a
  *          global threshold can be used (for text or line-art) and the
  *          value it should have.
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <math.h>
 #include "allheaders.h"
 
+static PIX *pixSauvolaGetThreshold(PIX *pixm, PIX *pixms, l_float32 factor,
+                                   PIX **ppixsd);
+static PIX *pixApplyLocalThreshold(PIX *pixs, PIX *pixth);
+
 /*------------------------------------------------------------------*
  *                 Adaptive Otsu-based thresholding                 *
  *------------------------------------------------------------------*/
 /*!
  * \brief   pixOtsuAdaptiveThreshold()
  *
- * \param[in]    pixs 8 bpp
- * \param[in]    sx, sy desired tile dimensions; actual size may vary
- * \param[in]    smoothx, smoothy half-width of convolution kernel applied to
- *                                threshold array: use 0 for no smoothing
- * \param[in]    scorefract fraction of the max Otsu score; typ. 0.1;
- *                          use 0.0 for standard Otsu
- * \param[out]   ppixth [optional] array of threshold values
- *                      found for each tile
- * \param[out]   ppixd [optional] thresholded input pixs, based on
- *                     the threshold array
+ * \param[in]    pixs              8 bpp
+ * \param[in]    sx, sy            desired tile dimensions; actual size may vary
+ * \param[in]    smoothx, smoothy  half-width of convolution kernel applied to
+ *                                 threshold array: use 0 for no smoothing
+ * \param[in]    scorefract        fraction of the max Otsu score; typ. 0.1;
+ *                                 use 0.0 for standard Otsu
+ * \param[out]   ppixth            [optional] array of threshold values
+ *                                 found for each tile
+ * \param[out]   ppixd             [optional] thresholded input pixs,
+ *                                 based on the threshold array
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -136,7 +147,7 @@
  *          of the problem noted in (3) above for tiling.  Use Sauvola.
  * </pre>
  */
-l_int32
+l_ok
 pixOtsuAdaptiveThreshold(PIX       *pixs,
                          l_int32    sx,
                          l_int32    sy,
@@ -220,17 +231,17 @@ PIXTILING  *pt;
 /*!
  * \brief   pixOtsuThreshOnBackgroundNorm()
  *
- * \param[in]    pixs 8 bpp grayscale; not colormapped
- * \param[in]    pixim [optional] 1 bpp 'image' mask; can be null
- * \param[in]    sx, sy tile size in pixels
- * \param[in]    thresh threshold for determining foreground
- * \param[in]    mincount min threshold on counts in a tile
- * \param[in]    bgval target bg val; typ. > 128
- * \param[in]    smoothx half-width of block convolution kernel width
- * \param[in]    smoothy half-width of block convolution kernel height
- * \param[in]    scorefract fraction of the max Otsu score; typ. 0.1
- * \param[out]   pthresh [optional] threshold value that was
- *                       used on the normalized image
+ * \param[in]    pixs         8 bpp grayscale; not colormapped
+ * \param[in]    pixim        [optional] 1 bpp 'image' mask; can be null
+ * \param[in]    sx, sy       tile size in pixels
+ * \param[in]    thresh       threshold for determining foreground
+ * \param[in]    mincount     min threshold on counts in a tile
+ * \param[in]    bgval        target bg val; typ. > 128
+ * \param[in]    smoothx      half-width of block convolution kernel width
+ * \param[in]    smoothy      half-width of block convolution kernel height
+ * \param[in]    scorefract   fraction of the max Otsu score; typ. 0.1
+ * \param[out]   pthresh      [optional] threshold value that was
+ *                            used on the normalized image
  * \return  pixd 1 bpp thresholded image, or NULL on error
  *
  * <pre>
@@ -314,16 +325,16 @@ PIX      *pixn, *pixt, *pixd;
 /*!
  * \brief   pixMaskedThreshOnBackgroundNorm()
  *
- * \param[in]    pixs 8 bpp grayscale; not colormapped
- * \param[in]    pixim [optional] 1 bpp 'image' mask; can be null
- * \param[in]    sx, sy tile size in pixels
- * \param[in]    thresh threshold for determining foreground
- * \param[in]    mincount min threshold on counts in a tile
- * \param[in]    smoothx half-width of block convolution kernel width
- * \param[in]    smoothy half-width of block convolution kernel height
- * \param[in]    scorefract fraction of the max Otsu score; typ. ~ 0.1
- * \param[out]   pthresh [optional] threshold value that was
- *                       used on the normalized image
+ * \param[in]    pixs         8 bpp grayscale; not colormapped
+ * \param[in]    pixim        [optional] 1 bpp 'image' mask; can be null
+ * \param[in]    sx, sy       tile size in pixels
+ * \param[in]    thresh       threshold for determining foreground
+ * \param[in]    mincount     min threshold on counts in a tile
+ * \param[in]    smoothx      half-width of block convolution kernel width
+ * \param[in]    smoothy      half-width of block convolution kernel height
+ * \param[in]    scorefract   fraction of the max Otsu score; typ. ~ 0.1
+ * \param[out]   pthresh      [optional] threshold value that was
+ *                            used on the normalized image
  * \return  pixd 1 bpp thresholded image, or NULL on error
  *
  * <pre>
@@ -437,12 +448,12 @@ PIX      *pixn, *pixm, *pixd, *pix1, *pix2, *pix3, *pix4;
 /*!
  * \brief   pixSauvolaBinarizeTiled()
  *
- * \param[in]    pixs 8 bpp grayscale, not colormapped
- * \param[in]    whsize window half-width for measuring local statistics
- * \param[in]    factor factor for reducing threshold due to variance; >= 0
- * \param[in]    nx, ny subdivision into tiles; >= 1
- * \param[out]   ppixth [optional] Sauvola threshold values
- * \param[out]   ppixd [optional] thresholded image
+ * \param[in]    pixs      8 bpp grayscale, not colormapped
+ * \param[in]    whsize    window half-width for measuring local statistics
+ * \param[in]    factor    factor for reducing threshold due to variance; >= 0
+ * \param[in]    nx, ny    subdivision into tiles; >= 1
+ * \param[out]   ppixth    [optional] Sauvola threshold values
+ * \param[out]   ppixd     [optional] thresholded image
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -463,7 +474,7 @@ PIX      *pixn, *pixm, *pixd, *pix1, *pix2, *pix3, *pix4;
  *          See pixSauvolaBinarize() for details.
  * </pre>
  */
-l_int32
+l_ok
 pixSauvolaBinarizeTiled(PIX       *pixs,
                         l_int32    whsize,
                         l_float32  factor,
@@ -555,14 +566,14 @@ PIXTILING  *pt;
 /*!
  * \brief   pixSauvolaBinarize()
  *
- * \param[in]    pixs 8 bpp grayscale; not colormapped
- * \param[in]    whsize window half-width for measuring local statistics
- * \param[in]    factor factor for reducing threshold due to variance; >= 0
- * \param[in]    addborder 1 to add border of width (%whsize + 1) on all sides
- * \param[out]   ppixm [optional] local mean values
- * \param[out]   ppixsd [optional] local standard deviation values
- * \param[out]   ppixth [optional] threshold values
- * \param[out]   ppixd [optional] thresholded image
+ * \param[in]    pixs       8 bpp grayscale; not colormapped
+ * \param[in]    whsize     window half-width for measuring local statistics
+ * \param[in]    factor     factor for reducing threshold due to variance; >= 0
+ * \param[in]    addborder  1 to add border of width (%whsize + 1) on all sides
+ * \param[out]   ppixm      [optional] local mean values
+ * \param[out]   ppixsd     [optional] local standard deviation values
+ * \param[out]   ppixth     [optional] threshold values
+ * \param[out]   ppixd      [optional] thresholded image
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -590,7 +601,7 @@ PIXTILING  *pt;
  *          0.2 and 0.5.
  * </pre>
  */
-l_int32
+l_ok
 pixSauvolaBinarize(PIX       *pixs,
                    l_int32    whsize,
                    l_float32  factor,
@@ -642,7 +653,7 @@ PIX     *pixg, *pixsc, *pixm, *pixms, *pixth, *pixd;
     if (ppixth || ppixd)
         pixth = pixSauvolaGetThreshold(pixm, pixms, factor, ppixsd);
     if (ppixd) {
-        pixd = pixApplyLocalThreshold(pixsc, pixth, 1);
+        pixd = pixApplyLocalThreshold(pixsc, pixth);
         pixCopyResolution(pixd, pixs);
     }
 
@@ -666,11 +677,11 @@ PIX     *pixg, *pixsc, *pixm, *pixms, *pixth, *pixd;
 /*!
  * \brief   pixSauvolaGetThreshold()
  *
- * \param[in]    pixm 8 bpp grayscale; not colormapped
- * \param[in]    pixms 32 bpp
- * \param[in]    factor factor for reducing threshold due to variance; >= 0
- * \param[out]   ppixsd [optional] local standard deviation
- * \return  pixd 8 bpp, sauvola threshold values, or NULL on error
+ * \param[in]    pixm     8 bpp grayscale; not colormapped
+ * \param[in]    pixms    32 bpp
+ * \param[in]    factor   factor for reducing threshold due to variance; >= 0
+ * \param[out]   ppixsd   [optional] local standard deviation
+ * \return  pixd   8 bpp, sauvola threshold values, or NULL on error
  *
  * <pre>
  * Notes:
@@ -700,7 +711,7 @@ PIX     *pixg, *pixsc, *pixm, *pixms, *pixth, *pixd;
  *            s = sqrt(ms - mv * mv)
  * </pre>
  */
-PIX *
+static PIX *
 pixSauvolaGetThreshold(PIX       *pixm,
                        PIX       *pixms,
                        l_float32  factor,
@@ -778,15 +789,13 @@ PIX        *pixsd, *pixd;
 /*!
  * \brief   pixApplyLocalThreshold()
  *
- * \param[in]    pixs 8 bpp grayscale; not colormapped
- * \param[in]    pixth 8 bpp array of local thresholds
- * \param[in]    redfactor  ...
- * \return  pixd 1 bpp, thresholded image, or NULL on error
+ * \param[in]    pixs     8 bpp grayscale; not colormapped
+ * \param[in]    pixth    8 bpp array of local thresholds
+ * \return  pixd   1 bpp, thresholded image, or NULL on error
  */
-PIX *
+static PIX *
 pixApplyLocalThreshold(PIX     *pixs,
-                       PIX     *pixth,
-                       l_int32  redfactor)
+                       PIX     *pixth)
 {
 l_int32    i, j, w, h, wpls, wplt, wpld, vals, valt;
 l_uint32  *datas, *datat, *datad, *lines, *linet, *lined;
@@ -826,24 +835,24 @@ PIX       *pixd;
 
 
 /*----------------------------------------------------------------------*
- *                  Thresholding using connected components             *
+ *            Global thresholding using connected components            *
  *----------------------------------------------------------------------*/
 /*!
  * \brief   pixThresholdByConnComp()
  *
- * \param[in]    pixs depth > 1, colormap OK
- * \param[in]    pixm [optional] 1 bpp mask giving region to ignore by setting
- *                    pixels to white; use NULL if no mask
- * \param[in]    start, end, incr binarization threshold levels to test
- * \param[in]    thresh48 threshold on normalized difference between the
- *                        numbers of 4 and 8 connected components
- * \param[in]    threshdiff threshold on normalized difference between the
- *                          number of 4 cc at successive iterations
- * \param[out]   pglobthresh [optional] best global threshold; 0
- *                           if no threshold is found
- * \param[out]   ppixd [optional] image thresholded to binary, or
- *                     null if no threshold is found
- * \param[in]    debugflag 1 for plotted results
+ * \param[in]    pixs          depth > 1, colormap OK
+ * \param[in]    pixm          [optional] 1 bpp mask giving region to ignore
+ *                             by setting pixels to white; use NULL if no mask
+ * \param[in]    start, end, incr   binarization threshold levels to test
+ * \param[in]    thresh48      threshold on normalized difference between the
+ *                             numbers of 4 and 8 connected components
+ * \param[in]    threshdiff    threshold on normalized difference between the
+ *                             number of 4 cc at successive iterations
+ * \param[out]   pglobthresh   [optional] best global threshold; 0
+ *                             if no threshold is found
+ * \param[out]   ppixd         [optional] image thresholded to binary, or
+ *                             null if no threshold is found
+ * \param[in]    debugflag     1 for plotted results
  * \return  0 if OK, 1 on error or if no threshold is found
  *
  * <pre>
@@ -882,7 +891,7 @@ PIX       *pixd;
  *          which the caller should check.
  * </pre>
  */
-l_int32
+l_ok
 pixThresholdByConnComp(PIX       *pixs,
                        PIX       *pixm,
                        l_int32    start,
@@ -959,7 +968,8 @@ PIX       *pix1, *pix2, *pix3;
         pixDestroy(&pix3);
     }
     if (debugflag) {
-        gplot = gplotCreate("/tmp/threshroot", GPLOT_PNG,
+        lept_mkdir("lept/binarize");
+        gplot = gplotCreate("/tmp/lept/binarize", GPLOT_PNG,
                             "number of cc vs. threshold",
                             "threshold", "number of cc");
         gplotAddPlot(gplot, NULL, na4, GPLOT_LINES, "plot 4cc");
@@ -980,8 +990,8 @@ PIX       *pix1, *pix2, *pix3;
             diff48 = (count4 - count8) / firstcount4;
             diff4 = L_ABS(prevcount4 - count4) / firstcount4;
             if (debugflag) {
-                fprintf(stderr, "diff48 = %7.3f, diff4 = %7.3f\n",
-                        diff48, diff4);
+                lept_stderr("diff48 = %7.3f, diff4 = %7.3f\n",
+                            diff48, diff4);
             }
             if (diff48 < thresh48 && diff4 < threshdiff) {
                 found = TRUE;
@@ -1000,12 +1010,94 @@ PIX       *pix1, *pix2, *pix3;
             *ppixd = pixConvertTo1(pix2, globthresh);
             pixCopyResolution(*ppixd, pixs);
         }
-        if (debugflag) fprintf(stderr, "global threshold = %d\n", globthresh);
+        if (debugflag) lept_stderr("global threshold = %d\n", globthresh);
         pixDestroy(&pix2);
         return 0;
     }
 
-    if (debugflag) fprintf(stderr, "no global threshold found\n");
+    if (debugflag) lept_stderr("no global threshold found\n");
     pixDestroy(&pix2);
     return 1;
 }
+
+/*----------------------------------------------------------------------*
+ *                   Global thresholding by histogram                   *
+ *----------------------------------------------------------------------*/
+/*!
+ * \brief   pixThresholdByHisto()
+ *
+ * \param[in]    pixs          gray 8 bpp, no colormap
+ * \param[in]    factor        subsampling factor >= 1
+ * \param[in]    halfw         half of window width for smoothing;
+ *                             use 0 for default
+ * \param[in]    delta         relative amount to resolve peaks and valleys;
+ *                             in (0 ... 1], use 0 for default
+ * \param[out]   pthresh       best global threshold; 0 if no threshold is found
+ * \param[out]   ppixd         [optional] thresholded 1 bpp pix
+ * \param[out]   ppixhisto     [optional] rescaled histogram of gray values
+ * \return  0 if OK, 1 on error or if no threshold is found
+ *
+ * <pre>
+ * Notes:
+ *      (1) This finds a global threshold.  It is best for an image that
+ *          has a fairly well-defined fg and bg.
+ *      (2) If it finds a good threshold and %ppixd is defined, the binarized
+ *          image is returned in &pixd; otherwise it return null.
+ *      (3) Suggest using default values for %half and %delta.
+ *      (4) Returns 0 in %pthresh if it can't find a good threshold.
+ * </pre>
+ */
+l_ok
+pixThresholdByHisto(PIX       *pixs,
+                    l_int32    factor,
+                    l_int32    halfw,
+                    l_float32  delta,
+                    l_int32   *pthresh,
+                    PIX      **ppixd,
+                    PIX      **ppixhisto)
+{
+l_int32    i, n;
+l_float32  maxval, val1, val2, fract;
+NUMA      *na1, *na2, *na3, *naloc, *nav;
+PIX       *pix1;
+
+    PROCNAME("pixThresholdByHisto");
+
+    if (ppixhisto) *ppixhisto = NULL;
+    if (ppixd) *ppixd = NULL;
+    if (!pthresh)
+        return ERROR_INT("&thresh not defined", procName, 1);
+    *pthresh = 0;
+    if (!pixs || pixGetDepth(pixs) != 8)
+        return ERROR_INT("pixs undefined or not 8 bpp", procName, 1);
+    if (pixGetColormap(pixs))
+        return ERROR_INT("pixs has colormap", procName, 1);
+    if (factor < 1)
+        return ERROR_INT("sampling must be >= 1", procName, 1);
+    if (halfw <= 0) halfw = 20;
+    if (delta <= 0.0) delta = 0.1;
+
+        /* Make a histogram of pixel values where the largest peak
+         * is normalized to a value of 1.0. */
+    na1 = pixGetGrayHistogram(pixs, factor);
+    na2 = numaWindowedMean(na1, halfw);  /* smoothing */
+    numaGetMax(na2, &maxval, NULL);
+    na3 = numaTransform(na2, 0.0, 1.0 / maxval);  /* rescale to max of 1.0 */
+    numaDestroy(&na1);
+    numaDestroy(&na2);
+
+    numaFindLocForThreshold(na3, 0, pthresh, &fract);
+    L_INFO("fractional area under first peak: %5.3f\n", procName, fract);
+
+    if (ppixhisto) {
+        lept_mkdir("lept/histo");
+        gplotSimple1(na3, GPLOT_PNG, "/tmp/lept/histo/histo", NULL);
+        *ppixhisto = pixRead("/tmp/lept/histo/histo.png");
+    }
+    numaDestroy(&na3);
+
+    if (*pthresh > 0 && ppixd)
+        *ppixd = pixThresholdToBinary(pixs, *pthresh);
+    return 0;
+}
+
