@@ -88,6 +88,10 @@
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
 /*!
@@ -267,7 +271,7 @@ L_STACK  *stack, *auxstack;
 
 #if  DEBUG
     pixCountPixels(pix1, &iszero, NULL);
-    fprintf(stderr, "Number of remaining pixels = %d\n", iszero);
+    lept_stderr("Number of remaining pixels = %d\n", iszero);
     lept_mkdir("lept/cc");
     pixWriteDebug("/tmp/lept/cc/remain.png", pix1, IFF_PNG);
 #endif  /* DEBUG */
@@ -359,7 +363,7 @@ L_STACK  *stack, *auxstack;
 
 #if  DEBUG
     pixCountPixels(pix1, &iszero, NULL);
-    fprintf(stderr, "Number of remaining pixels = %d\n", iszero);
+    lept_stderr("Number of remaining pixels = %d\n", iszero);
     lept_mkdir("lept/cc");
     pixWriteDebug("/tmp/lept/cc/remain.png", pix1, IFF_PNG);
 #endif  /* DEBUG */
@@ -1119,15 +1123,10 @@ L_STACK  *auxstack;
         }
 
             /* Get a fillseg to use */
-        if (lstackGetCount(auxstack) > 0) {
+        if (lstackGetCount(auxstack) > 0)
             fseg = (FILLSEG *)lstackRemove(auxstack);
-        } else {
-            if ((fseg = (FILLSEG *)LEPT_CALLOC(1, sizeof(FILLSEG))) == NULL) {
-                L_ERROR("fillseg not made\n", procName);
-                return;
-            }
-        }
-
+        else
+            fseg = (FILLSEG *)LEPT_CALLOC(1, sizeof(FILLSEG));
         fseg->xleft = xleft;
         fseg->xright = xright;
         fseg->y = y;
@@ -1181,15 +1180,10 @@ L_STACK  *auxstack;
         }
 
             /* Get a fillseg to use */
-        if (lstackGetCount(auxstack) > 0) {
+        if (lstackGetCount(auxstack) > 0)
             fseg = (FILLSEG *)lstackRemove(auxstack);
-        } else {
-            if ((fseg = (FILLSEG *)LEPT_CALLOC(1, sizeof(FILLSEG))) == NULL) {
-                L_ERROR("fillseg not made\n", procName);
-                return;
-            }
-        }
-
+        else
+            fseg = (FILLSEG *)LEPT_CALLOC(1, sizeof(FILLSEG));
         fseg->xleft = xleft;
         fseg->xright = xright;
         fseg->y = y;
