@@ -399,30 +399,33 @@
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <math.h>
 #include "allheaders.h"
 
 static l_int32 dewarpaExtendArraysToSize(L_DEWARPA *dewa, l_int32 size);
 
     /* Parameter values used in dewarpaCreate() */
-static const l_int32     INITIAL_PTR_ARRAYSIZE = 20;   /* n'import quoi */
-static const l_int32     MAX_PTR_ARRAYSIZE = 10000;
-static const l_int32     DEFAULT_ARRAY_SAMPLING = 30;
-static const l_int32     MIN_ARRAY_SAMPLING = 8;
-static const l_int32     DEFAULT_MIN_LINES = 15;
-static const l_int32     MIN_MIN_LINES = 4;
-static const l_int32     DEFAULT_MAX_REF_DIST = 16;
-static const l_int32     DEFAULT_USE_BOTH = TRUE;
-static const l_int32     DEFAULT_CHECK_COLUMNS = FALSE;
+static const l_int32     InitialPtrArraySize = 20;   /* n'import quoi */
+static const l_int32     MaxPtrArraySize = 10000;
+static const l_int32     DefaultArraySampling = 30;
+static const l_int32     MinArraySampling = 8;
+static const l_int32     DefaultMinLines = 15;
+static const l_int32     MinMinLines = 4;
+static const l_int32     DefaultMaxRefDist = 16;
+static const l_int32     DefaultUseBoth = TRUE;
+static const l_int32     DefaultCheckColumns = TRUE;
 
     /* Parameter values used in dewarpaSetCurvatures() */
-static const l_int32     DEFAULT_MAX_LINECURV = 180;
-static const l_int32     DEFAULT_MIN_DIFF_LINECURV = 0;
-static const l_int32     DEFAULT_MAX_DIFF_LINECURV = 200;
-static const l_int32     DEFAULT_MAX_EDGECURV = 50;
-static const l_int32     DEFAULT_MAX_DIFF_EDGECURV = 40;
-static const l_int32     DEFAULT_MAX_EDGESLOPE = 80;
-
+static const l_int32     DefaultMaxLineCurv = 150;
+static const l_int32     DefaultMinDiffLineCurv = 0;
+static const l_int32     DefaultMaxDiffLineCurv = 170;
+static const l_int32     DefaultMaxEdgeCurv = 50;
+static const l_int32     DefaultMaxDiffEdgeCurv = 40;
+static const l_int32     DefaultMaxEdgeSlope = 80;
 
 /*----------------------------------------------------------------------*
  *                           Create/destroy Dewarp                      *
@@ -430,8 +433,8 @@ static const l_int32     DEFAULT_MAX_EDGESLOPE = 80;
 /*!
  * \brief   dewarpCreate()
  *
- * \param[in]   pixs 1 bpp
- * \param[in]   pageno page number
+ * \param[in]   pixs     1 bpp
+ * \param[in]   pageno   page number
  * \return  dew or NULL on error
  *
  * <pre>
@@ -455,8 +458,7 @@ L_DEWARP  *dew;
     if (pixGetDepth(pixs) != 1)
         return (L_DEWARP *)ERROR_PTR("pixs not 1 bpp", procName, NULL);
 
-    if ((dew = (L_DEWARP *)LEPT_CALLOC(1, sizeof(L_DEWARP))) == NULL)
-        return (L_DEWARP *)ERROR_PTR("dew not made", procName, NULL);
+    dew = (L_DEWARP *)LEPT_CALLOC(1, sizeof(L_DEWARP));
     dew->pixs = pixClone(pixs);
     dew->pageno = pageno;
     dew->w = pixGetWidth(pixs);
@@ -468,8 +470,8 @@ L_DEWARP  *dew;
 /*!
  * \brief   dewarpCreateRef()
  *
- * \param[in]    pageno this page number
- * \param[in]    refpage page number of dewarp disparity arrays to be used
+ * \param[in]    pageno    this page number
+ * \param[in]    refpage   page number of dewarp disparity arrays to be used
  * \return  dew or NULL on error
  *
  * <pre>
@@ -490,8 +492,7 @@ L_DEWARP  *dew;
 
     PROCNAME("dewarpCreateRef");
 
-    if ((dew = (L_DEWARP *)LEPT_CALLOC(1, sizeof(L_DEWARP))) == NULL)
-        return (L_DEWARP *)ERROR_PTR("dew not made", procName, NULL);
+    dew = (L_DEWARP *)LEPT_CALLOC(1, sizeof(L_DEWARP));
     dew->pageno = pageno;
     dew->hasref = 1;
     dew->refpage = refpage;
@@ -502,7 +503,7 @@ L_DEWARP  *dew;
 /*!
  * \brief   dewarpDestroy()
  *
- * \param[in,out]   pdew will be set to null before returning
+ * \param[in,out]   pdew    will be set to null before returning
  * \return  void
  */
 void
@@ -540,11 +541,11 @@ L_DEWARP  *dew;
 /*!
  * \brief   dewarpaCreate()
  *
- * \param[in]   nptrs number of dewarp page ptrs; typically the number of pages
- * \param[in]   sampling use 0 for default value; the minimum allowed is 8
- * \param[in]   redfactor of input images: 1 is full resolution; 2 is 2x reduced
- * \param[in]   minlines minimum number of lines to accept; use 0 for default
- * \param[in]   maxdist for locating reference disparity; use -1 for default
+ * \param[in]   nptrs       number of dewarp page ptrs; typ. the number of pages
+ * \param[in]   sampling    use 0 for default value; the minimum allowed is 8
+ * \param[in]   redfactor   of input images: 1 is full res; 2 is 2x reduced
+ * \param[in]   minlines    minimum number of lines to accept; use 0 for default
+ * \param[in]   maxdist     for locating reference disparity; use -1 for default
  * \return  dewa or NULL on error
  *
  * <pre>
@@ -580,28 +581,28 @@ L_DEWARPA  *dewa;
     PROCNAME("dewarpaCreate");
 
     if (nptrs <= 0)
-        nptrs = INITIAL_PTR_ARRAYSIZE;
-    if (nptrs > MAX_PTR_ARRAYSIZE)
+        nptrs = InitialPtrArraySize;
+    if (nptrs > MaxPtrArraySize)
         return (L_DEWARPA *)ERROR_PTR("too many pages", procName, NULL);
     if (redfactor != 1 && redfactor != 2)
         return (L_DEWARPA *)ERROR_PTR("redfactor not in {1,2}",
                                       procName, NULL);
     if (sampling == 0) {
-         sampling = DEFAULT_ARRAY_SAMPLING;
-    } else if (sampling < MIN_ARRAY_SAMPLING) {
+         sampling = DefaultArraySampling;
+    } else if (sampling < MinArraySampling) {
          L_WARNING("sampling too small; setting to %d\n", procName,
-                   MIN_ARRAY_SAMPLING);
-         sampling = MIN_ARRAY_SAMPLING;
+                   MinArraySampling);
+         sampling = MinArraySampling;
     }
     if (minlines == 0) {
-        minlines = DEFAULT_MIN_LINES;
-    } else if (minlines < MIN_MIN_LINES) {
+        minlines = DefaultMinLines;
+    } else if (minlines < MinMinLines) {
         L_WARNING("minlines too small; setting to %d\n", procName,
-                  MIN_MIN_LINES);
-        minlines = DEFAULT_MIN_LINES;
+                  MinMinLines);
+        minlines = DefaultMinLines;
     }
     if (maxdist < 0)
-         maxdist = DEFAULT_MAX_REF_DIST;
+         maxdist = DefaultMaxRefDist;
 
     dewa = (L_DEWARPA *)LEPT_CALLOC(1, sizeof(L_DEWARPA));
     dewa->dewarp = (L_DEWARP **)LEPT_CALLOC(nptrs, sizeof(L_DEWARPA *));
@@ -615,14 +616,14 @@ L_DEWARPA  *dewa;
     dewa->redfactor = redfactor;
     dewa->minlines = minlines;
     dewa->maxdist = maxdist;
-    dewa->max_linecurv = DEFAULT_MAX_LINECURV;
-    dewa->min_diff_linecurv = DEFAULT_MIN_DIFF_LINECURV;
-    dewa->max_diff_linecurv = DEFAULT_MAX_DIFF_LINECURV;
-    dewa->max_edgeslope = DEFAULT_MAX_EDGESLOPE;
-    dewa->max_edgecurv = DEFAULT_MAX_EDGECURV;
-    dewa->max_diff_edgecurv = DEFAULT_MAX_DIFF_EDGECURV;
-    dewa->check_columns = DEFAULT_CHECK_COLUMNS;
-    dewa->useboth = DEFAULT_USE_BOTH;
+    dewa->max_linecurv = DefaultMaxLineCurv;
+    dewa->min_diff_linecurv = DefaultMinDiffLineCurv;
+    dewa->max_diff_linecurv = DefaultMaxDiffLineCurv;
+    dewa->max_edgeslope = DefaultMaxEdgeSlope;
+    dewa->max_edgecurv = DefaultMaxEdgeCurv;
+    dewa->max_diff_edgecurv = DefaultMaxDiffEdgeCurv;
+    dewa->check_columns = DefaultCheckColumns;
+    dewa->useboth = DefaultUseBoth;
     return dewa;
 }
 
@@ -630,11 +631,11 @@ L_DEWARPA  *dewa;
 /*!
  * \brief   dewarpaCreateFromPixacomp()
  *
- * \param[in]   pixac pixacomp of G4, 1 bpp images; with 1x1x1 placeholders
- * \param[in]   useboth 0 for only vert disparity; 1 for both vert and horiz
- * \param[in]   sampling use -1 or 0 for default value; otherwise minimum of 5
- * \param[in]   minlines minimum number of lines to accept; e.g., 10
- * \param[in]   maxdist for locating reference disparity; use -1 for default
+ * \param[in]   pixac      pixacomp of G4, 1 bpp images; with 1x1x1 placeholders
+ * \param[in]   useboth    0 for only vert disparity; 1 for both vert and horiz
+ * \param[in]   sampling   use -1 or 0 for default value; otherwise minimum of 5
+ * \param[in]   minlines   minimum number of lines to accept; e.g., 10
+ * \param[in]   maxdist    for locating reference disparity; use -1 for default
  * \return  dewa or NULL on error
  *
  * <pre>
@@ -715,7 +716,7 @@ PIX        *pixt;
 /*!
  * \brief   dewarpaDestroy()
  *
- * \param[in,out]   pdewa will be set to null before returning
+ * \param[in,out]   pdewa    will be set to null before returning
  * \return  void
  */
 void
@@ -755,10 +756,10 @@ L_DEWARPA  *dewa;
  * \brief   dewarpaDestroyDewarp()
  *
  * \param[in]    dewa
- * \param[in]    pageno of dew to be destroyed
+ * \param[in]    pageno    of dew to be destroyed
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 dewarpaDestroyDewarp(L_DEWARPA  *dewa,
                      l_int32     pageno)
 {
@@ -786,7 +787,7 @@ L_DEWARP   *dew;
  * \brief   dewarpaInsertDewarp()
  *
  * \param[in]    dewa
- * \param[in]    dew  to be added
+ * \param[in]    dew     to be added
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -801,7 +802,7 @@ L_DEWARP   *dew;
  *          allocation.
  * </pre>
  */
-l_int32
+l_ok
 dewarpaInsertDewarp(L_DEWARPA  *dewa,
                     L_DEWARP   *dew)
 {
@@ -817,7 +818,7 @@ L_DEWARP  *prevdew;
 
     dew->dewa = dewa;
     pageno = dew->pageno;
-    if (pageno > MAX_PTR_ARRAYSIZE)
+    if (pageno > MaxPtrArraySize)
         return ERROR_INT("too many pages", procName, 1);
     if (pageno > dewa->maxpage)
         dewa->maxpage = pageno;
@@ -858,7 +859,7 @@ L_DEWARP  *prevdew;
  * \brief   dewarpaExtendArraysToSize()
  *
  * \param[in]    dewa
- * \param[in]    size new size of dewarpa array
+ * \param[in]    size    new size of dewarpa array
  * \return  0 if OK; 1 on error
  *
  * <pre>
@@ -894,9 +895,9 @@ dewarpaExtendArraysToSize(L_DEWARPA  *dewa,
 /*!
  * \brief   dewarpaGetDewarp()
  *
- * \param[in]    dewa populated with dewarp structs for pages
- * \param[in]    index into dewa: this is the pageno
- * \return  dew handle; still owned by dewa, or NULL on error
+ * \param[in]    dewa    populated with dewarp structs for pages
+ * \param[in]    index   into dewa: this is the pageno
+ * \return  dew   handle; still owned by dewa, or NULL on error
  */
 L_DEWARP *
 dewarpaGetDewarp(L_DEWARPA  *dewa,
@@ -923,17 +924,17 @@ dewarpaGetDewarp(L_DEWARPA  *dewa,
  * \brief   dewarpaSetCurvatures()
  *
  * \param[in]    dewa
- * \param[in]    max_linecurv -1 for default
- * \param[in]    min_diff_linecurv -1 for default; 0 to accept all models
- * \param[in]    max_diff_linecurv -1 for default
- * \param[in]    max_edgecurv -1 for default
- * \param[in]    max_diff_edgecurv -1 for default
- * \param[in]    max_edgeslope -1 for default
+ * \param[in]    max_linecurv        -1 for default
+ * \param[in]    min_diff_linecurv   -1 for default; 0 to accept all models
+ * \param[in]    max_diff_linecurv   -1 for default
+ * \param[in]    max_edgecurv        -1 for default
+ * \param[in]    max_diff_edgecurv   -1 for default
+ * \param[in]    max_edgeslope       -1 for default
  * \return  0 if OK, 1 on error
  *
  * <pre>
  * Notes:
- *      (1) Approximating the line by a quadratic, the coefficent
+ *      (1) Approximating the line by a quadratic, the coefficient
  *          of the quadratic term is the curvature, and distance
  *          units are in pixels (of course).  The curvature is very
  *          small, so we multiply by 10^6 and express the constraints
@@ -959,10 +960,10 @@ dewarpaGetDewarp(L_DEWARPA  *dewa,
  *          A model having maximum line curvature larger than about 150
  *          micro-units should probably not be used.
  *      (5) A model having a left or right edge curvature larger than
- *          about 100 micro-units should probably not be used.
+ *          about 50 micro-units should probably not be used.
  * </pre>
  */
-l_int32
+l_ok
 dewarpaSetCurvatures(L_DEWARPA  *dewa,
                      l_int32     max_linecurv,
                      l_int32     min_diff_linecurv,
@@ -977,32 +978,32 @@ dewarpaSetCurvatures(L_DEWARPA  *dewa,
         return ERROR_INT("dewa not defined", procName, 1);
 
     if (max_linecurv == -1)
-        dewa->max_linecurv = DEFAULT_MAX_LINECURV;
+        dewa->max_linecurv = DefaultMaxLineCurv;
     else
         dewa->max_linecurv = L_ABS(max_linecurv);
 
     if (min_diff_linecurv == -1)
-        dewa->min_diff_linecurv = DEFAULT_MIN_DIFF_LINECURV;
+        dewa->min_diff_linecurv = DefaultMinDiffLineCurv;
     else
         dewa->min_diff_linecurv = L_ABS(min_diff_linecurv);
 
     if (max_diff_linecurv == -1)
-        dewa->max_diff_linecurv = DEFAULT_MAX_DIFF_LINECURV;
+        dewa->max_diff_linecurv = DefaultMaxDiffLineCurv;
     else
         dewa->max_diff_linecurv = L_ABS(max_diff_linecurv);
 
     if (max_edgecurv == -1)
-        dewa->max_edgecurv = DEFAULT_MAX_EDGECURV;
+        dewa->max_edgecurv = DefaultMaxEdgeCurv;
     else
         dewa->max_edgecurv = L_ABS(max_edgecurv);
 
     if (max_diff_edgecurv == -1)
-        dewa->max_diff_edgecurv = DEFAULT_MAX_DIFF_EDGECURV;
+        dewa->max_diff_edgecurv = DefaultMaxDiffEdgeCurv;
     else
         dewa->max_diff_edgecurv = L_ABS(max_diff_edgecurv);
 
     if (max_edgeslope == -1)
-        dewa->max_edgeslope = DEFAULT_MAX_EDGESLOPE;
+        dewa->max_edgeslope = DefaultMaxEdgeSlope;
     else
         dewa->max_edgeslope = L_ABS(max_edgeslope);
 
@@ -1026,7 +1027,7 @@ dewarpaSetCurvatures(L_DEWARPA  *dewa,
  *          always be valid.
  * </pre>
  */
-l_int32
+l_ok
 dewarpaUseBothArrays(L_DEWARPA  *dewa,
                      l_int32     useboth)
 {
@@ -1045,7 +1046,7 @@ dewarpaUseBothArrays(L_DEWARPA  *dewa,
  * \brief   dewarpaSetCheckColumns()
  *
  * \param[in]    dewa
- * \param[in]    check_columns 0 for false, 1 for true
+ * \param[in]    check_columns    0 for false, 1 for true
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1062,9 +1063,10 @@ dewarpaUseBothArrays(L_DEWARPA  *dewa,
  *           - useboth = 1 (TRUE)
  *          If there are multiple columns, additionally
  *           - check_columns = 0 (FALSE)
+ *
  * </pre>
  */
-l_int32
+l_ok
 dewarpaSetCheckColumns(L_DEWARPA  *dewa,
                        l_int32     check_columns)
 {
@@ -1082,7 +1084,7 @@ dewarpaSetCheckColumns(L_DEWARPA  *dewa,
  * \brief   dewarpaSetMaxDistance()
  *
  * \param[in]    dewa
- * \param[in]    maxdist for using ref models
+ * \param[in]    maxdist    for using ref models
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1090,7 +1092,7 @@ dewarpaSetCheckColumns(L_DEWARPA  *dewa,
  *      (1) This sets the maxdist field.
  * </pre>
  */
-l_int32
+l_ok
 dewarpaSetMaxDistance(L_DEWARPA  *dewa,
                       l_int32     maxdist)
 {
@@ -1140,8 +1142,8 @@ L_DEWARP  *dew;
 /*!
  * \brief   dewarpReadStream()
  *
- * \param[in]    fp file stream
- * \return  dew, or NULL on error
+ * \param[in]    fp    file stream
+ * \return  dew  dewarp, or NULL on error
  *
  * <pre>
  * Notes:
@@ -1255,8 +1257,8 @@ FPIX      *fpixv, *fpixh;
 /*!
  * \brief   dewarpReadMem()
  *
- * \param[in]    data  serialization of dewarp
- * \param[in]    size  of data in bytes
+ * \param[in]    data     serialization of dewarp
+ * \param[in]    size     of data in bytes
  * \return  dew  dewarp, or NULL on error
  */
 L_DEWARP  *
@@ -1287,7 +1289,7 @@ L_DEWARP  *dew;
  * \param[in]    dew
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 dewarpWrite(const char  *filename,
             L_DEWARP    *dew)
 {
@@ -1314,7 +1316,7 @@ FILE    *fp;
 /*!
  * \brief   dewarpWriteStream()
  *
- * \param[in]    fp file stream opened for "wb"
+ * \param[in]    fp     file stream opened for "wb"
  * \param[in]    dew
  * \return  0 if OK, 1 on error
  *
@@ -1325,7 +1327,7 @@ FILE    *fp;
  *          been built for this page.
  * </pre>
  */
-l_int32
+l_ok
 dewarpWriteStream(FILE      *fp,
                   L_DEWARP  *dew)
 {
@@ -1371,8 +1373,8 @@ l_int32  vdispar, hdispar;
 /*!
  * \brief   dewarpWriteMem()
  *
- * \param[out]   pdata data of serialized dewarp (not ascii)
- * \param[out]   psize size of returned data
+ * \param[out]   pdata    data of serialized dewarp (not ascii)
+ * \param[out]   psize    size of returned data
  * \param[in]    dew
  * \return  0 if OK, 1 on error
  *
@@ -1381,7 +1383,7 @@ l_int32  vdispar, hdispar;
  *      (1) Serializes a dewarp in memory and puts the result in a buffer.
  * </pre>
  */
-l_int32
+l_ok
 dewarpWriteMem(l_uint8  **pdata,
                size_t    *psize,
                L_DEWARP  *dew)
@@ -1457,7 +1459,7 @@ L_DEWARPA  *dewa;
 /*!
  * \brief   dewarpaReadStream()
  *
- * \param[in]    fp file stream
+ * \param[in]    fp    file stream
  * \return  dewa, or NULL on error
  *
  * <pre>
@@ -1506,6 +1508,9 @@ NUMA       *namodels;
     if (fscanf(fp, "fullmodel = %d\n", &useboth) != 1)
         return (L_DEWARPA *)ERROR_PTR("read fail for useboth", procName, NULL);
 
+    if (ndewarp > MaxPtrArraySize)
+        return (L_DEWARPA *)ERROR_PTR("too many pages", procName, NULL);
+
     dewa = dewarpaCreate(maxpage + 1, sampling, redfactor, minlines, maxdist);
     dewa->maxpage = maxpage;
     dewa->max_linecurv = max_linecurv;
@@ -1520,6 +1525,7 @@ NUMA       *namodels;
     for (i = 0; i < ndewarp; i++) {
         if ((dew = dewarpReadStream(fp)) == NULL) {
             L_ERROR("read fail for dew[%d]\n", procName, i);
+            dewarpaDestroy(&dewa);
             return NULL;
         }
         dewarpaInsertDewarp(dewa, dew);
@@ -1528,7 +1534,6 @@ NUMA       *namodels;
 
         /* Validate the models and insert reference models */
     dewarpaInsertRefModels(dewa, 0, 0);
-
     return dewa;
 }
 
@@ -1568,7 +1573,7 @@ L_DEWARPA  *dewa;
  * \param[in]    dewa
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 dewarpaWrite(const char  *filename,
              L_DEWARPA   *dewa)
 {
@@ -1595,11 +1600,11 @@ FILE    *fp;
 /*!
  * \brief   dewarpaWriteStream()
  *
- * \param[in]    fp file stream opened for "wb"
+ * \param[in]    fp     file stream opened for "wb"
  * \param[in]    dewa
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 dewarpaWriteStream(FILE       *fp,
                    L_DEWARPA  *dewa)
 {
@@ -1645,8 +1650,8 @@ l_int32  ndewarp, i, pageno;
 /*!
  * \brief   dewarpaWriteMem()
  *
- * \param[out]   pdata data of serialized dewarpa (not ascii)
- * \param[out]   psize size of returned data
+ * \param[out]   pdata    data of serialized dewarpa (not ascii)
+ * \param[out]   psize    size of returned data
  * \param[in]    dewa
  * \return  0 if OK, 1 on error
  *
@@ -1655,7 +1660,7 @@ l_int32  ndewarp, i, pageno;
  *      (1) Serializes a dewarpa in memory and puts the result in a buffer.
  * </pre>
  */
-l_int32
+l_ok
 dewarpaWriteMem(l_uint8   **pdata,
                 size_t     *psize,
                 L_DEWARPA  *dewa)
