@@ -58,6 +58,10 @@
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <math.h>
 #include "allheaders.h"
 
@@ -68,7 +72,8 @@ static l_int32 dewarpFilterLineEndPoints(L_DEWARP  *dew, PTA *ptal1, PTA *ptar1,
                                          PTA **pptal2, PTA **pptar2);
 static PTA *dewarpRemoveBadEndPoints(l_int32 w, PTA *ptas);
 static l_int32 dewarpIsLineCoverageValid(PTAA *ptaa2, l_int32 h,
-                                         l_int32 *ptopline, l_int32 *pbotline);
+                                         l_int32 *pntop, l_int32 *pnbot,
+                                         l_int32 *pytop, l_int32 *pybot);
 static l_int32 dewarpQuadraticLSF(PTA *ptad, l_float32 *pa, l_float32 *pb,
                                   l_float32 *pc, l_float32 *pmederr);
 static l_int32 pixRenderMidYs(PIX *pixs, NUMA *namidys, l_int32 linew);
@@ -77,7 +82,7 @@ static l_int32 pixRenderHorizEndPoints(PIX *pixs, PTA *ptal, PTA *ptar,
 
 
 #ifndef  NO_CONSOLE_IO
-#define  DEBUG_TEXTLINE_CENTERS    0   /* set this to 1 for debuging */
+#define  DEBUG_TEXTLINE_CENTERS    0   /* set this to 1 for debugging */
 #define  DEBUG_SHORT_LINES         0   /* ditto */
 #else
 #define  DEBUG_TEXTLINE_CENTERS    0   /* always must be 0 */
@@ -85,10 +90,10 @@ static l_int32 pixRenderHorizEndPoints(PIX *pixs, PTA *ptal, PTA *ptar,
 #endif  /* !NO_CONSOLE_IO */
 
     /* Special parameter values for reducing horizontal disparity */
-static const l_float32   L_MIN_RATIO_LINES_TO_HEIGHT = 0.45;
-static const l_int32     L_MIN_LINES_FOR_HORIZ_1 = 10; /* initially */
-static const l_int32     L_MIN_LINES_FOR_HORIZ_2 = 3;  /* after, in each half */
-static const l_float32   L_ALLOWED_W_FRACT = 0.05;  /* no bigger */
+static const l_float32   MinRatioLinesToHeight = 0.45;
+static const l_int32     MinLinesForHoriz1 = 10; /* initially */
+static const l_int32     MinLinesForHoriz2 = 3;  /* after, in each half */
+static const l_float32   AllowedWidthFract = 0.05;  /* no bigger */
 
 
 /*----------------------------------------------------------------------*
@@ -98,7 +103,7 @@ static const l_float32   L_ALLOWED_W_FRACT = 0.05;  /* no bigger */
  * \brief   dewarpBuildPageModel()
  *
  * \param[in]    dew
- * \param[in]    debugfile use NULL to skip writing this
+ * \param[in]    debugfile    use NULL to skip writing this
  * \return  0 if OK, 1 if unable to build the model or on error
  *
  * <pre>
@@ -144,11 +149,11 @@ static const l_float32   L_ALLOWED_W_FRACT = 0.05;  /* no bigger */
  *          longest textlines.
  * </pre>
  */
-l_int32
+l_ok
 dewarpBuildPageModel(L_DEWARP    *dew,
                      const char  *debugfile)
 {
-l_int32  linecount, topline, botline, ret;
+l_int32  linecount, ntop, nbot, ytop, ybot, ret;
 PIX     *pixs, *pix1, *pix2, *pix3;
 PTA     *pta;
 PTAA    *ptaa1, *ptaa2;
@@ -213,12 +218,13 @@ PTAA    *ptaa1, *ptaa2;
     }
 
         /* Verify that the lines have a reasonable coverage of the
-         * vertical extent of the image foreground. */
+         * vertical extent of the page. */
     if (dewarpIsLineCoverageValid(ptaa2, pixGetHeight(pixs),
-                                  &topline, &botline) == FALSE) {
+                                  &ntop, &nbot, &ytop, &ybot) == FALSE) {
         ptaaDestroy(&ptaa2);
-        L_WARNING("invalid line coverage: [%d ... %d] in height %d\n",
-                  procName, topline, botline, pixGetHeight(pixs));
+        L_WARNING("invalid line coverage: ntop = %d, nbot = %d;"
+                  " spanning [%d ... %d] in height %d\n", procName,
+                  ntop, nbot, ytop, ybot, pixGetHeight(pixs));
         return 1;
     }
 
@@ -254,7 +260,7 @@ PTAA    *ptaa1, *ptaa2;
         }
         convertFilesToPdf("/tmp/lept/dewmod", NULL, 135, 1.0, 0, 0,
                           "Dewarp Build Model", debugfile);
-        fprintf(stderr, "pdf file: %s\n", debugfile);
+        lept_stderr("pdf file: %s\n", debugfile);
     }
 
     ptaaDestroy(&ptaa2);
@@ -266,8 +272,8 @@ PTAA    *ptaa1, *ptaa2;
  * \brief   dewarpFindVertDisparity()
  *
  * \param[in]    dew
- * \param[in]    ptaa unsmoothed lines, not vertically ordered
- * \param[in]    rotflag 0 if using dew->pixs; 1 if rotated by 90 degrees cw
+ * \param[in]    ptaa       unsmoothed lines, not vertically ordered
+ * \param[in]    rotflag    0 if using dew->pixs; 1 if rotated by 90 degrees cw
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -290,13 +296,13 @@ PTAA    *ptaa1, *ptaa2;
  *          a pdf.  Non-pix debug output goes to /tmp.
  * </pre>
  */
-l_int32
+l_ok
 dewarpFindVertDisparity(L_DEWARP  *dew,
                         PTAA      *ptaa,
                         l_int32    rotflag)
 {
 l_int32     i, j, nlines, npts, nx, ny, sampling;
-l_float32   c0, c1, c2, x, y, midy, val, medval, medvar, minval, maxval;
+l_float32   c0, c1, c2, x, y, midy, val, medval, meddev, minval, maxval;
 l_float32  *famidys;
 NUMA       *nax, *nafit, *nacurve0, *nacurve1, *nacurves;
 NUMA       *namidy, *namidys, *namidysi;
@@ -374,15 +380,15 @@ FPIX       *fpix;
          * the line curvatures.  It is not rejecting lines based on
          * the magnitude of the curvature.  That is done when constraints
          * are applied for valid models. */
-    numaGetMedianVariation(nacurve0, &medval, &medvar);
+    numaGetMedianDevFromMedian(nacurve0, &medval, &meddev);
     L_INFO("\nPage %d\n", procName, dew->pageno);
-    L_INFO("Pass 1: Curvature: medval = %f, medvar = %f\n",
-           procName, medval, medvar);
+    L_INFO("Pass 1: Curvature: medval = %f, meddev = %f\n",
+           procName, medval, meddev);
     ptaa1 = ptaaCreate(nlines);
     nacurve1 = numaCreate(nlines);
     for (i = 0; i < nlines; i++) {  /* for each line */
         numaGetFValue(nacurve0, i, &val);
-        if (L_ABS(val - medval) > 7.0 * medvar)  /* TODO: reduce to ~ 3.0 */
+        if (L_ABS(val - medval) > 7.0 * meddev)  /* TODO: reduce to ~ 3.0 */
             continue;
         pta = ptaaGetPta(ptaa0, i, L_CLONE);
         ptaaAddPta(ptaa1, pta, L_INSERT);
@@ -509,7 +515,7 @@ FPIX       *fpix;
         convertFilesToPdf("/tmp/lept/dewmod", "004", 135, 1.0, 0, 0,
                           "Dewarp Vert Disparity",
                           "/tmp/lept/dewarp/vert_disparity.pdf");
-        fprintf(stderr, "pdf file: /tmp/lept/dewarp/vert_disparity.pdf\n");
+        lept_stderr("pdf file: /tmp/lept/dewarp/vert_disparity.pdf\n");
     }
 
         /* Save the result in a fpix at the specified subsampling  */
@@ -537,7 +543,7 @@ FPIX       *fpix;
  * \brief   dewarpFindHorizDisparity()
  *
  * \param[in]    dew
- * \param[in]    ptaa unsmoothed lines, not vertically ordered
+ * \param[in]    ptaa     unsmoothed lines, not vertically ordered
  * \return  0 if OK, 1 if horizontal disparity array is not built, or on error
  *
  * <pre>
@@ -554,7 +560,7 @@ FPIX       *fpix;
  *      (5) Debug output goes to /tmp/lept/dewmod/ for collection into a pdf.
  * </pre>
  */
-l_int32
+l_ok
 dewarpFindHorizDisparity(L_DEWARP  *dew,
                          PTAA      *ptaa)
 {
@@ -606,6 +612,10 @@ FPIX      *fpix;
     ret = dewarpFilterLineEndPoints(dew, ptal1, ptar1, &ptal2, &ptar2);
     ptaDestroy(&ptal1);
     ptaDestroy(&ptar1);
+    if (ret) {
+        L_INFO("Not enough filtered end points\n", procName);
+        return 1;
+    }
 
         /* Do a quadratic fit to the left and right endpoints of the
          * longest lines.  Each line is represented by 3 coefficients:
@@ -676,7 +686,7 @@ FPIX      *fpix;
         convertFilesToPdf("/tmp/lept/dewmod", "005", 135, 1.0, 0, 0,
                           "Dewarp Horiz Disparity",
                           "/tmp/lept/dewarp/horiz_disparity.pdf");
-        fprintf(stderr, "pdf file: /tmp/lept/dewarp/horiz_disparity.pdf\n");
+        lept_stderr("pdf file: /tmp/lept/dewarp/horiz_disparity.pdf\n");
         pixDestroy(&pix1);
         ptaDestroy(&pta1);
         ptaDestroy(&pta2);
@@ -746,8 +756,8 @@ FPIX      *fpix;
 /*!
  * \brief   dewarpGetTextlineCenters()
  *
- * \param[in]    pixs 1 bpp
- * \param[in]    debugflag 1 for debug output
+ * \param[in]    pixs        1 bpp
+ * \param[in]    debugflag   1 for debug output
  * \return  ptaa of center values of textlines
  *
  * <pre>
@@ -861,8 +871,8 @@ PTAA     *ptaa;
 /*!
  * \brief   dewarpGetMeanVerticals()
  *
- * \param[in]    pixs 1 bpp, single c.c.
- * \param[in]    x,y location of UL corner of pixs with respect to page image
+ * \param[in]    pixs     1 bpp, single c.c.
+ * \param[in]    x,y      location of UL corner of pixs, relative to page image
  * \return  pta (mean y-values in component for each x-value,
  *                   both translated by (x,y
  */
@@ -905,9 +915,9 @@ PTA       *pta;
 /*!
  * \brief   dewarpRemoveShortLines()
  *
- * \param[in]    pixs 1 bpp
- * \param[in]    ptaas input lines
- * \param[in]    fract minimum fraction of longest line to keep
+ * \param[in]    pixs       1 bpp
+ * \param[in]    ptaas      input lines
+ * \param[in]    fract      minimum fraction of longest line to keep
  * \param[in]    debugflag
  * \return  ptaad containing only lines of sufficient length,
  *                     or NULL on error
@@ -1015,7 +1025,7 @@ PTA       *pta, *ptal1, *ptar1;
 
         /* Are there at least 10 lines? */
     n = ptaaGetCount(ptaa);
-    if (n < L_MIN_LINES_FOR_HORIZ_1) {
+    if (n < MinLinesForHoriz1) {
         L_INFO("only %d lines; too few\n", procName, n);
         return 1;
     }
@@ -1036,7 +1046,7 @@ PTA       *pta, *ptal1, *ptar1;
         /* Use the min and max of the y value on the left side. */
     ptaGetRange(ptal1, &miny, &maxy, NULL, NULL);
     ratio = (maxy - miny) / (l_float32)h;
-    if (ratio < L_MIN_RATIO_LINES_TO_HEIGHT) {
+    if (ratio < MinRatioLinesToHeight) {
         L_INFO("ratio lines to height, %f, too small\n", procName, ratio);
         ptaDestroy(&ptal1);
         ptaDestroy(&ptar1);
@@ -1112,7 +1122,7 @@ PTA       *ptal1, *ptar1, *ptal2, *ptar2;
     }
 
     n = L_MIN(ptaGetCount(ptal1), ptaGetCount(ptar1));
-    if (n < L_MIN_LINES_FOR_HORIZ_1 - 2) {
+    if (n < MinLinesForHoriz1 - 2) {
         ptaDestroy(&ptal1);
         ptaDestroy(&ptar1);
         L_INFO("First filter: only %d endpoints; needed 8\n", procName, n);
@@ -1173,7 +1183,7 @@ PTA       *ptau1, *ptau2, *ptad1, *ptad2;
     if (!ptas)
         return (PTA *)ERROR_PTR("ptas not defined", procName, NULL);
 
-    delta = w * L_ALLOWED_W_FRACT;
+    delta = AllowedWidthFract * w;
     n = ptaGetCount(ptas);  /* will be at least 8 */
 
         /* Check the upper half */
@@ -1187,7 +1197,7 @@ PTA       *ptau1, *ptau2, *ptad1, *ptad2;
             ptaAddPt(ptau2, xval, yval);
     }
     ptaDestroy(&ptau1);
-    if (ptaGetCount(ptau2) < L_MIN_LINES_FOR_HORIZ_2) {
+    if (ptaGetCount(ptau2) < MinLinesForHoriz2) {
         ptaDestroy(&ptau2);
         L_INFO("Second filter: upper set is too small after outliers removed\n",
                procName);
@@ -1195,7 +1205,7 @@ PTA       *ptau1, *ptau2, *ptad1, *ptad2;
     }
 
         /* Check the lower half */
-    ptad1 = ptaSelectRange(ptas, n / 2 + 1, 0);
+    ptad1 = ptaSelectRange(ptas, n / 2 + 1, -1);
     ptaGetRankValue(ptad1, 0.5, NULL, L_SORT_BY_Y, &rval);
     nd = ptaGetCount(ptad1);
     ptad2 = ptaCreate(nd);
@@ -1205,7 +1215,7 @@ PTA       *ptau1, *ptau2, *ptad1, *ptad2;
             ptaAddPt(ptad2, xval, yval);
     }
     ptaDestroy(&ptad1);
-    if (ptaGetCount(ptad2) < L_MIN_LINES_FOR_HORIZ_2) {
+    if (ptaGetCount(ptad2) < MinLinesForHoriz2) {
         ptaDestroy(&ptau2);
         ptaDestroy(&ptad2);
         L_INFO("Second filter: lower set is too small after outliers removed\n",
@@ -1222,28 +1232,33 @@ PTA       *ptau1, *ptau2, *ptad1, *ptad2;
 /*!
  * \brief   dewarpIsLineCoverageValid()
  *
- * \param[in]    ptaa of validated lines
- * \param[in]    h height of pix
- * \param[out]   ptopline location of top line
- * \param[out]   pbotline location of bottom line
+ * \param[in]    ptaa       of validated lines
+ * \param[in]    h          height of pix
+ * \param[out]   pntop      number of lines in top half
+ * \param[out]   pnbot      number of lines in bottom half
+ * \param[out]   pytop      location of top line
+ * \param[out]   pybot      location of bottom line
  * \return  1 if coverage is valid, 0 if not or on error.
  *
  * <pre>
  * Notes:
  *      (1) The criterion for valid coverage is:
- *          (a) there must be lines in both halves (top and bottom)
+ *          (a) there must be at least 4 lines in each half (top and bottom)
  *              of the image.
- *          (b) the coverage must be at least 40% of the image height
+ *          (b) the coverage must be at least 50% of the image height
  * </pre>
  */
 static l_int32
 dewarpIsLineCoverageValid(PTAA     *ptaa,
                           l_int32   h,
-                          l_int32  *ptopline,
-                          l_int32  *pbotline)
+                          l_int32  *pntop,
+                          l_int32  *pnbot,
+                          l_int32  *pytop,
+                          l_int32  *pybot)
 {
-l_int32    i, n, both_halves;
-l_float32  top, bot, y, fraction;
+l_int32    i, n, iy, both_halves, ntop, nbot, ytop, ybot, nmin;
+l_float32  y, fraction;
+NUMA      *na;
 
     PROCNAME("dewarpIsLineCoverageValid");
 
@@ -1253,21 +1268,34 @@ l_float32  top, bot, y, fraction;
         return ERROR_INT("ptaa empty", procName, 0);
     if (h <= 0)
         return ERROR_INT("invalid h", procName, 0);
-    if (!ptopline || !pbotline)
-        return ERROR_INT("&topline and &botline not defined", procName, 0);
+    if (!pntop || !pnbot)
+        return ERROR_INT("&ntop and &nbot not defined", procName, 0);
+    if (!pytop || !pybot)
+        return ERROR_INT("&ytop and &ybot not defined", procName, 0);
 
-    top = 100000.0;
-    bot = 0.0;
+    na = numaCreate(n);
     for (i = 0; i < n; i++) {
         ptaaGetPt(ptaa, i, 0, NULL, &y);
-        if (y < top) top = y;
-        if (y > bot) bot = y;
+        numaAddNumber(na, y);
     }
-    *ptopline = (l_int32)top;
-    *pbotline = (l_int32)bot;
-    both_halves = top < 0.5 * h && bot > 0.5 * h;
-    fraction = (bot - top) / h;
-    if (both_halves && fraction > 0.40)
+    numaSort(na, na, L_SORT_INCREASING);
+    for (i = 0, ntop = 0; i < n; i++) {
+        numaGetIValue(na, i, &iy);
+        if (i == 0) ytop = iy;
+        if (i == n - 1) ybot = iy;
+        if (iy < 0.5 * h)
+            ntop++;
+    }
+    numaDestroy(&na);
+    nbot = n - ntop;
+    *pntop = ntop;
+    *pnbot = nbot;
+    *pytop = ytop;
+    *pybot = ybot;
+    nmin = 4;  /* minimum number of lines required in each half */
+    both_halves = (ntop >= nmin) && (nbot >= nmin);
+    fraction = (l_float32)(ybot - ytop) / (l_float32)h;
+    if (both_halves && fraction > 0.50)
         return 1;
     return 0;
 }
@@ -1276,11 +1304,11 @@ l_float32  top, bot, y, fraction;
 /*!
  * \brief   dewarpQuadraticLSF()
  *
- * \param[in]    ptad left or right end points of longest lines
- * \param[out]   pa  coeff a of LSF: y = ax^2 + bx + c
- * \param[out]   pb  coeff b of LSF: y = ax^2 + bx + c
- * \param[out]   pc  coeff c of LSF: y = ax^2 + bx + c
- * \param[out]   pmederr [optional] median error
+ * \param[in]    ptad      left or right end points of longest lines
+ * \param[out]   pa        coeff a of LSF: y = ax^2 + bx + c
+ * \param[out]   pb        coeff b of LSF: y = ax^2 + bx + c
+ * \param[out]   pc        coeff c of LSF: y = ax^2 + bx + c
+ * \param[out]   pmederr   [optional] median error
  * \return  0 if OK, 1 on error.
  *
  * <pre>
@@ -1340,9 +1368,9 @@ NUMA      *naerr;
  * \brief   dewarpFindHorizSlopeDisparity()
  *
  * \param[in]    dew
- * \param[in]    pixb (1 bpp, with vertical and horizontal disparity removed)
- * \param[in]    fractthresh (threshold fractional difference in density)
- * \param[in]    parity (0 if even page, 1 if odd page)
+ * \param[in]    pixb         1 bpp, with vert and horiz disparity removed
+ * \param[in]    fractthresh  threshold fractional difference in density
+ * \param[in]    parity       0 if even page, 1 if odd page
  * \return       0 if OK, 1 on error
  *
  * <pre>
@@ -1352,7 +1380,7 @@ NUMA      *naerr;
  *          disparity only if the absolute value of the fractional
  *          difference equals or exceeds this threshold.
  *      (2) %parity indicates where the binding is: on the left for
- *          %parity == 0 and on the right for @parity == 1.
+ *          %parity == 0 and on the right for %parity == 1.
  *      (3) This takes a 1 bpp %pixb where both vertical and horizontal
  *          disparity have been applied, so the text lines are straight and,
  *          more importantly, the line end points are vertically aligned.
@@ -1374,7 +1402,7 @@ NUMA      *naerr;
  *      (6) Debug output goes to /tmp/lept/dewmod/ for collection into a pdf.
  * </pre>
  */
-l_int32
+l_ok
 dewarpFindHorizSlopeDisparity(L_DEWARP  *dew,
                               PIX       *pixb,
                               l_float32  fractthresh,
@@ -1386,6 +1414,7 @@ l_float32  fract, delta, sum, aveval, fval, del, denom;
 l_float32  ca, cb, cc, cd, ce, y;
 BOX       *box;
 BOXA      *boxa1, *boxa2;
+GPLOT     *gplot;
 NUMA      *na1, *na2, *na3, *na4, *nasum;
 PIX       *pix1;
 PTA       *pta1;
@@ -1409,7 +1438,7 @@ FPIX      *fpix;
     boxa2 = boxaSelectBySize(boxa1, 0, 5, L_SELECT_HEIGHT, L_SELECT_IF_GT,
                              NULL);
     nb = boxaGetCount(boxa2);
-    fprintf(stderr, "number of components: %d\n", nb);
+    lept_stderr("number of components: %d\n", nb);
     boxaDestroy(&boxa1);
 
         /* Estimate the horizontal density of vertical strokes */
@@ -1505,8 +1534,9 @@ FPIX      *fpix;
     ptaGetQuarticLSF(pta1, &ca, &cb, &cc, &cd, &ce, &na3);
     ptaGetArrays(pta1, &na4, NULL);
     if (dew->debug) {
-        gplotSimpleXY1(na4, na3, GPLOT_LINES, GPLOT_PNG,
-                       "/tmp/lept/dew/0094", NULL);
+        gplot = gplotSimpleXY1(na4, na3, GPLOT_LINES, GPLOT_PNG,
+                              "/tmp/lept/dew/0094", NULL);
+        gplotDestroy(&gplot);
         lept_mv("/tmp/lept/dew/0094.png", "lept/dewmod", NULL, NULL);
     }
     ptaDestroy(&pta1);
@@ -1578,8 +1608,8 @@ FPIX      *fpix;
  * \brief   dewarpBuildLineModel()
  *
  * \param[in]    dew
- * \param[in]    opensize size of opening to remove perpendicular lines
- * \param[in]    debugfile use NULL to skip writing this
+ * \param[in]    opensize    size of opening to remove perpendicular lines
+ * \param[in]    debugfile   use NULL to skip writing this
  * \return  0 if OK, 1 if unable to build the model or on error
  *
  * <pre>
@@ -1602,7 +1632,7 @@ FPIX      *fpix;
  *          See notes there.
  * </pre>
  */
-l_int32
+l_ok
 dewarpBuildLineModel(L_DEWARP    *dew,
                      l_int32      opensize,
                      const char  *debugfile)
@@ -1762,7 +1792,7 @@ PTAA    *ptaa1, *ptaa2;
         }
         convertFilesToPdf("/tmp/lept/dewline", NULL, 135, 1.0, 0, 0,
                           "Dewarp Build Line Model", debugfile);
-        fprintf(stderr, "pdf file: %s\n", debugfile);
+        lept_stderr("pdf file: %s\n", debugfile);
     }
 
     return 0;
@@ -1777,8 +1807,8 @@ PTAA    *ptaa1, *ptaa2;
  *
  * \param[in]    dewa
  * \param[in]    pageno
- * \param[out]   pvsuccess [optional] 1 on success
- * \param[out]   phsuccess [optional] 1 on success
+ * \param[out]   pvsuccess    [optional] 1 on success
+ * \param[out]   phsuccess    [optional] 1 on success
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1786,7 +1816,7 @@ PTAA    *ptaa1, *ptaa2;
  *      (1) This tests if a model has been built, not if it is valid.
  * </pre>
  */
-l_int32
+l_ok
 dewarpaModelStatus(L_DEWARPA  *dewa,
                    l_int32     pageno,
                    l_int32    *pvsuccess,
@@ -1815,9 +1845,9 @@ L_DEWARP  *dew;
 /*!
  * \brief   pixRenderMidYs()
  *
- * \param[in]    pixs 32 bpp
- * \param[in]    namidys y location of reference lines for vertical disparity
- * \param[in]    linew width of rendered line; typ 2
+ * \param[in]    pixs      32 bpp
+ * \param[in]    namidys   y location of reference lines for vertical disparity
+ * \param[in]    linew     width of rendered line; typ 2
  * \return  0 if OK, 1 on error
  */
 static l_int32
@@ -1851,10 +1881,10 @@ PIXCMAP  *cmap;
 /*!
  * \brief   pixRenderHorizEndPoints()
  *
- * \param[in]    pixs 32 bpp
- * \param[in]    ptal left side line end points
- * \param[in]    ptar right side line end points
- * \param[in]    color 0xrrggbb00
+ * \param[in]    pixs     32 bpp
+ * \param[in]    ptal     left side line end points
+ * \param[in]    ptar     right side line end points
+ * \param[in]    color    0xrrggbb00
  * \return  0 if OK, 1 on error
  */
 static l_int32
