@@ -100,6 +100,10 @@
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
 /*--------------------------------------------------------------------------*
@@ -108,9 +112,9 @@
 /*!
  * \brief   l_dnaHashCreate()
  *
- * \param[in]   nbuckets the number of buckets in the hash table,
- *                       which should be prime.
- * \param[in]   initsize initial size of each allocated dna; 0 for default
+ * \param[in]   nbuckets   the number of buckets in the hash table,
+ *                         which should be prime.
+ * \param[in]   initsize   initial size of each allocated dna; 0 for default
  * \return  ptr to new dnahash, or NULL on error
  *
  * <pre>
@@ -128,8 +132,7 @@ L_DNAHASH  *dahash;
 
     if (nbuckets <= 0)
         return (L_DNAHASH *)ERROR_PTR("negative hash size", procName, NULL);
-    if ((dahash = (L_DNAHASH *)LEPT_CALLOC(1, sizeof(L_DNAHASH))) == NULL)
-        return (L_DNAHASH *)ERROR_PTR("dahash not made", procName, NULL);
+    dahash = (L_DNAHASH *)LEPT_CALLOC(1, sizeof(L_DNAHASH));
     if ((dahash->dna = (L_DNA **)LEPT_CALLOC(nbuckets, sizeof(L_DNA *)))
         == NULL) {
         LEPT_FREE(dahash);
@@ -145,7 +148,7 @@ L_DNAHASH  *dahash;
 /*!
  * \brief   l_dnaHashDestroy()
  *
- * \param[in,out]   pdahash to be nulled, if it exists
+ * \param[in,out]   pdahash   will be set to null before returning
  * \return  void
  */
 void
@@ -224,8 +227,8 @@ L_DNA   *da;
  * \brief   l_dnaHashGetDna()
  *
  * \param[in]    dahash
- * \param[in]    key  key to be hashed into a bucket number
- * \param[in]    copyflag L_NOCOPY, L_COPY, L_CLONE
+ * \param[in]    key        key to be hashed into a bucket number
+ * \param[in]    copyflag   L_NOCOPY, L_COPY, L_CLONE
  * \return  ptr to dna
  */
 L_DNA *
@@ -259,11 +262,11 @@ L_DNA   *da;
  * \brief   l_dnaHashAdd()
  *
  * \param[in]    dahash
- * \param[in]    key  key to be hashed into a bucket number
- * \param[in]    value  float value to be appended to the specific dna
+ * \param[in]    key      key to be hashed into a bucket number
+ * \param[in]    value    float value to be appended to the specific dna
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 l_dnaHashAdd(L_DNAHASH  *dahash,
              l_uint64    key,
              l_float64   value)
@@ -334,8 +337,8 @@ L_DNAHASH  *dahash;
  * \brief   l_dnaRemoveDupsByHash()
  *
  * \param[in]    das
- * \param[out]   pdad hash set
- * \param[out]   pdahash [optional] dnahash used for lookup
+ * \param[out]   pdad      hash set
+ * \param[out]   pdahash   [optional] dnahash used for lookup
  * \return  0 if OK; 1 on error
  *
  * <pre>
@@ -343,10 +346,10 @@ L_DNAHASH  *dahash;
  *      (1) Generates a dna with unique values.
  *      (2) The dnahash is built up with dad to assure uniqueness.
  *          It can be used to find if an element is in the set:
- *              l_dnaFindValByHash(dad, dahash, val, \&index)
+ *              l_dnaFindValByHash(dad, dahash, val, &index)
  * </pre>
  */
-l_int32
+l_ok
 l_dnaRemoveDupsByHash(L_DNA       *das,
                       L_DNA      **pdad,
                       L_DNAHASH  **pdahash)
@@ -395,9 +398,9 @@ L_DNAHASH  *dahash;
  * \brief   l_dnaMakeHistoByHash()
  *
  * \param[in]    das
- * \param[out]   pdahash hash map: val --> index
- * \param[out]   pdav array of values: index --> val
- * \param[out]   pdac histo array of counts: index --> count
+ * \param[out]   pdahash    hash map: val --> index
+ * \param[out]   pdav       array of values: index --> val
+ * \param[out]   pdac       histo array of counts: index --> count
  * \return  0 if OK; 1 on error
  *
  * <pre>
@@ -410,14 +413,14 @@ L_DNAHASH  *dahash;
  *          and is needed for fast lookup.  It is a hash set, because
  *          the values are unique.
  *      (3) Lookup is simple:
- *              l_dnaFindValByHash(dav, dahash, val, \&index);
- *              if (index \>= 0)
- *                  l_dnaGetIValue(dac, index, \&icount);
+ *              l_dnaFindValByHash(dav, dahash, val, &index);
+ *              if (index >= 0)
+ *                  l_dnaGetIValue(dac, index, &icount);
  *              else
  *                  icount = 0;
  * </pre>
  */
-l_int32
+l_ok
 l_dnaMakeHistoByHash(L_DNA       *das,
                      L_DNAHASH  **pdahash,
                      L_DNA      **pdav,
@@ -473,7 +476,7 @@ L_DNAHASH  *dahash;
  * \brief   l_dnaIntersectionByHash()
  *
  * \param[in]    da1, da2
- * \return  dad intersection of the number arrays, or NULL on error
+ * \return  dad   intersection of the number arrays, or NULL on error
  *
  * <pre>
  * Notes:
@@ -537,9 +540,9 @@ L_DNA      *da_small, *da_big, *dad;
  * \brief   l_dnaFindValByHash()
  *
  * \param[in]    da
- * \param[in]    dahash containing indices into %da
- * \param[in]    val  searching for this number in %da
- * \param[out]   pindex index into da if found; -1 otherwise
+ * \param[in]    dahash    containing indices into %da
+ * \param[in]    val       searching for this number in %da
+ * \param[out]   pindex    index into da if found; -1 otherwise
  * \return  0 if OK; 1 on error
  *
  * <pre>
@@ -549,7 +552,7 @@ L_DNA      *da_small, *da_big, *dad;
  *                the dna of indices looking for %val in %da.
  * </pre>
  */
-l_int32
+l_ok
 l_dnaFindValByHash(L_DNA      *da,
                    L_DNAHASH  *dahash,
                    l_float64   val,
