@@ -45,6 +45,7 @@
  *
  *      Generic TRC mapper
  *           PIX     *pixTRCMap()
+ *           PIX     *pixTRCMapGeneral()
  *
  *      Unsharp-masking
  *           PIX     *pixUnsharpMasking()
@@ -114,17 +115,16 @@
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
 
 #include <math.h>
 #include "allheaders.h"
 
     /* Scales contrast enhancement factor to have a useful range
      * between 0.0 and 1.0 */
-static const l_float32  ENHANCE_SCALE_FACTOR = 5.;
-
-    /* Default number of pixels sampled to determine histogram */
-static const l_int32  DEFAULT_HISTO_SAMPLES = 100000;
-
+static const l_float32  EnhanceScaleFactor = 5.0;
 
 /*-------------------------------------------------------------*
  *         Gamma TRC (tone reproduction curve) mapping         *
@@ -168,6 +168,8 @@ static const l_int32  DEFAULT_HISTO_SAMPLES = 100000;
  *           will darken the image and make the colors more intense;
  *           e.g., minval = 50, maxval = 200.
  *      (11) See numaGammaTRC() for further examples of use.
+ *      (12) Use pixTRCMapGeneral() if applying different mappings
+ *           to each channel in an RGB image.
  * </pre>
  */
 PIX *
@@ -570,7 +572,7 @@ NUMA      *na;
     if (factor == 0.0)
         return numaMakeSequence(0, 1, 256);  /* linear map */
 
-    scale = ENHANCE_SCALE_FACTOR;
+    scale = EnhanceScaleFactor;
     ymax = atan((l_float64)(1.0 * factor * scale));
     ymin = atan((l_float64)(-127. * factor * scale / 128.));
     dely = ymax - ymin;
@@ -783,9 +785,8 @@ pixTRCMap(PIX   *pixs,
           NUMA  *na)
 {
 l_int32    w, h, d, wm, hm, wpl, wplm, i, j, sval8, dval8;
-l_int32   *tab;
 l_uint32   sval32, dval32;
-l_uint32  *data, *datam, *line, *linem;
+l_uint32  *data, *datam, *line, *linem, *tab;
 
     PROCNAME("pixTRCMap");
 
@@ -805,7 +806,7 @@ l_uint32  *data, *datam, *line, *linem;
             return ERROR_INT("pixm not 1 bpp", procName, 1);
     }
 
-    tab = numaGetIArray(na);  /* get the array for efficiency */
+    tab = (l_uint32 *)numaGetIArray(na);  /* get the array for efficiency */
     wpl = pixGetWpl(pixs);
     data = pixGetData(pixs);
     if (!pixm) {
@@ -878,6 +879,101 @@ l_uint32  *data, *datam, *line, *linem;
 }
 
 
+/*!
+ * \brief   pixTRCMapGeneral()
+ *
+ * \param[in]    pixs             32 bpp rgb; not colormapped
+ * \param[in]    pixm             [optional] 1 bpp mask
+ * \param[in]    nar, nag, nab    mapping arrays
+ * \return  pixd, or NULL on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) This operation is in-place on %pixs.
+ *      (2) Each of the r,g,b mapping arrays is of size 256. They map the
+ *          input value for that color component into values in the
+ *          range [0, 255].
+ *      (3) In the special case where the r, g and b mapping arrays are
+ *          all the same, call pixTRCMap() instead.
+ *      (4) If defined, the optional 1 bpp mask %pixm has its origin
+ *          aligned with %pixs, and the map function is applied only
+ *          to pixels in %pixs under the fg of pixm.
+ *      (5) The alpha channel is not saved.
+ * </pre>
+ */
+l_int32
+pixTRCMapGeneral(PIX   *pixs,
+                 PIX   *pixm,
+                 NUMA  *nar,
+                 NUMA  *nag,
+                 NUMA  *nab)
+{
+l_int32    w, h, wm, hm, wpl, wplm, i, j;
+l_uint32   sval32, dval32;
+l_uint32  *data, *datam, *line, *linem, *tabr, *tabg, *tabb;
+
+    PROCNAME("pixTRCMapGeneral");
+
+    if (!pixs || pixGetDepth(pixs) != 32)
+        return ERROR_INT("pixs not defined or not 32 bpp", procName, 1);
+    if (pixm && pixGetDepth(pixm) != 1)
+        return ERROR_INT("pixm defined and not 1 bpp", procName, 1);
+    if (!nar || !nag || !nab)
+        return ERROR_INT("na{r,g,b} not all defined", procName, 1);
+    if (numaGetCount(nar) != 256 || numaGetCount(nag) != 256 ||
+        numaGetCount(nab) != 256)
+        return ERROR_INT("na{r,g,b} not all of size 256", procName, 1);
+
+        /* Get the arrays for efficiency */
+    tabr = (l_uint32 *)numaGetIArray(nar);
+    tabg = (l_uint32 *)numaGetIArray(nag);
+    tabb = (l_uint32 *)numaGetIArray(nab);
+    pixGetDimensions(pixs, &w, &h, NULL);
+    wpl = pixGetWpl(pixs);
+    data = pixGetData(pixs);
+    if (!pixm) {
+        for (i = 0; i < h; i++) {
+            line = data + i * wpl;
+            for (j = 0; j < w; j++) {
+                sval32 = *(line + j);
+                dval32 =
+                    tabr[(sval32 >> L_RED_SHIFT) & 0xff] << L_RED_SHIFT |
+                    tabg[(sval32 >> L_GREEN_SHIFT) & 0xff] << L_GREEN_SHIFT |
+                    tabb[(sval32 >> L_BLUE_SHIFT) & 0xff] << L_BLUE_SHIFT;
+                *(line + j) = dval32;
+            }
+        }
+    } else {
+        datam = pixGetData(pixm);
+        wplm = pixGetWpl(pixm);
+        pixGetDimensions(pixm, &wm, &hm, NULL);
+        for (i = 0; i < h; i++) {
+            if (i >= hm)
+                break;
+            line = data + i * wpl;
+            linem = datam + i * wplm;
+            for (j = 0; j < w; j++) {
+                if (j >= wm)
+                    break;
+                if (GET_DATA_BIT(linem, j) == 0)
+                    continue;
+                sval32 = *(line + j);
+                dval32 =
+                    tabr[(sval32 >> L_RED_SHIFT) & 0xff] << L_RED_SHIFT |
+                    tabg[(sval32 >> L_GREEN_SHIFT) & 0xff] << L_GREEN_SHIFT |
+                    tabb[(sval32 >> L_BLUE_SHIFT) & 0xff] << L_BLUE_SHIFT;
+                *(line + j) = dval32;
+            }
+        }
+    }
+
+    LEPT_FREE(tabr);
+    LEPT_FREE(tabg);
+    LEPT_FREE(tabb);
+    return 0;
+}
+
+
 
 /*-----------------------------------------------------------------------*
  *                             Unsharp masking                           *
@@ -1591,7 +1687,7 @@ l_uint32  *data, *line;
  * \param[in]    pixs     32 bpp rgb
  * \param[in]    factor   subsampling factor; integer >= 1
  * \param[out]   psat     average saturation
- * \return  pixd, or NULL on error
+ * \return  0 if OK, 1 on error
  */
 l_int32
 pixMeasureSaturation(PIX        *pixs,
@@ -1730,7 +1826,7 @@ l_uint32  *data, *line;
  *          and calibration of the printer.  This function can be used
  *          to iteratively match a color print to the original.  On each
  *          iteration, the center offsets are set to the best match so
- *          far, and the @delta increments are typically reduced.
+ *          far, and the %delta increments are typically reduced.
  * </pre>
  */
 PIX *
@@ -1742,7 +1838,7 @@ pixMosaicColorShiftRGB(PIX       *pixs,
                        l_int32    nincr)
 {
 char       buf[64];
-l_int32    i, j;
+l_int32    i;
 l_float32  del;
 L_BMF     *bmf;
 PIX       *pix1, *pix2, *pix3;
@@ -1905,7 +2001,7 @@ PIX       *pixd;
 /*!
  * \brief   pixDarkenGray()
  *
- * \param[in]    pixd     [optional] can be null or equal to pixs
+ * \param[in]    pixd      [optional] can be null or equal to pixs
  * \param[in]    pixs      32 bpp rgb
  * \param[in]    thresh    pixels with max component >= %thresh are unchanged
  * \param[in]    satlimit  pixels with saturation >= %satlimit are unchanged
@@ -1914,9 +2010,9 @@ PIX       *pixd;
  * <pre>
  * Notes:
  *      (1) This darkens gray pixels, by a fraction (sat/%satlimit), where
- *          the sat, the saturation, is the component difference (max - min).
+ *          the saturation, sat, is the component difference (max - min).
  *          The pixel value is unchanged if sat >= %satlimit.  A typical
- *          value of %satlimit might be 50; the larger the value, the
+ *          value of %satlimit might be 40; the larger the value, the
  *          more that pixels with a smaller saturation will be darkened.
  *      (2) Pixels with max component >= %thresh are unchanged. This can be
  *          used to prevent bright pixels with low saturation from being
@@ -1924,6 +2020,8 @@ PIX       *pixd;
  *          causes the darkening to be applied to all pixels.
  *      (3) This function is useful to enhance pixels relative to a
  *          gray background.
+ *      (4) A related function that builds a 1 bpp mask over the gray
+ *          pixels is pixMaskOverGrayPixels().
  * </pre>
  */
 PIX *
