@@ -82,7 +82,6 @@ typedef unsigned int uintptr_t;
 
 typedef intptr_t l_intptr_t;
 typedef uintptr_t l_uintptr_t;
-typedef void *L_TIMER;
 
 
 /*--------------------------------------------------------------------*
@@ -97,37 +96,46 @@ typedef void *L_TIMER;
  *  I/O libraries, plus zlib.  Setting any of these to 0 here causes
  *  non-functioning stubs to be linked.
  */
-#if !defined(HAVE_CONFIG_H) && !defined(ANDROID_BUILD)
-#define  HAVE_LIBJPEG     1
-#define  HAVE_LIBTIFF     1
-#define  HAVE_LIBPNG      1
-#define  HAVE_LIBZ        1
-#define  HAVE_LIBGIF      0
-#define  HAVE_LIBUNGIF    0
-#define  HAVE_LIBWEBP     0
-#define  HAVE_LIBJP2K     0
+#if !defined(HAVE_CONFIG_H) && !defined(ANDROID_BUILD) && !defined(OS_IOS)
+
+  #if !defined(HAVE_LIBJPEG)
+  #define  HAVE_LIBJPEG       1
+  #endif
+  #if !defined(HAVE_LIBTIFF)
+  #define  HAVE_LIBTIFF       1
+  #endif
+  #if !defined(HAVE_LIBPNG)
+  #define  HAVE_LIBPNG        1
+  #endif
+  #if !defined(HAVE_LIBZ)
+  #define  HAVE_LIBZ          1
+  #endif
+  #if !defined(HAVE_LIBGIF)
+  #define  HAVE_LIBGIF        0
+  #endif
+  #if !defined(HAVE_LIBUNGIF)
+  #define  HAVE_LIBUNGIF      0
+  #endif
+  #if !defined(HAVE_LIBWEBP)
+  #define  HAVE_LIBWEBP       0
+  #endif
+  #if !defined(HAVE_LIBWEBP_ANIM)
+  #define  HAVE_LIBWEBP_ANIM  0
+  #endif
+  #if !defined(HAVE_LIBJP2K)
+  #define  HAVE_LIBJP2K       0
+  #endif
+
+
+  /*-----------------------------------------------------------------------*
+   * Leptonica supports OpenJPEG 2.0+.  If you have a version of openjpeg  *
+   * (HAVE_LIBJP2K == 1) that is >= 2.0, set the path to the openjpeg.h    *
+   * header in angle brackets here.                                        *
+   *-----------------------------------------------------------------------*/
+  #define  LIBJP2K_HEADER   <openjpeg-2.3/openjpeg.h>
 
-    /* Leptonica supports both OpenJPEG 2.0 and 2.1.  If you have a
-     * version of openjpeg (HAVE_LIBJP2K) that is not 2.1, set the
-     * path to the openjpeg.h header in angle brackets here. */
-#define  LIBJP2K_HEADER   <openjpeg-2.1/openjpeg.h>
 #endif  /* ! HAVE_CONFIG_H etc. */
 
-/*
- * On linux systems, you can do I/O between Pix and memory.  Specifically,
- * you can compress (write compressed data to memory from a Pix) and
- * uncompress (read from compressed data in memory to a Pix).
- * For jpeg, png, jp2k, gif, pnm and bmp, these use the non-posix GNU
- * functions fmemopen() and open_memstream().  These functions are not
- * available on other systems.
- * To use these functions in linux, you must define HAVE_FMEMOPEN to 1.
- * To use them on MacOS, which does not support these functions, set it to 0.
- */
-#if !defined(HAVE_CONFIG_H) && !defined(ANDROID_BUILD) && !defined(_MSC_VER)
-#define  HAVE_FMEMOPEN    1
-#endif  /* ! HAVE_CONFIG_H etc. */
-
-
 /*--------------------------------------------------------------------*
  * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*
  *                          USER CONFIGURABLE                         *
@@ -148,85 +156,115 @@ typedef void *L_TIMER;
 #define  USE_PSIO         1
 
 
+/*-------------------------------------------------------------------------*
+ * On linux systems, you can do I/O between Pix and memory.  Specifically,
+ * you can compress (write compressed data to memory from a Pix) and
+ * uncompress (read from compressed data in memory to a Pix).
+ * For jpeg, png, jp2k, gif, pnm and bmp, these use the non-posix GNU
+ * functions fmemopen() and open_memstream().  These functions are not
+ * available on other systems.
+ * To use these functions in linux, you must define HAVE_FMEMOPEN to 1.
+ * To use them on MacOS, which does not support these functions, set it to 0.
+ *-------------------------------------------------------------------------*/
+#if !defined(HAVE_CONFIG_H) && !defined(ANDROID_BUILD) && !defined(OS_IOS) && \
+    !defined(_WIN32)
+#define  HAVE_FMEMOPEN    1
+#endif  /* ! HAVE_CONFIG_H etc. */
+
+/*-------------------------------------------------------------------------*
+ * fstatat() is defined by POSIX, but some systems do not support it.      *
+ * One example is older macOS systems (pre-10.10).                         *
+ * Play it safe and set the default value to 0.                            *
+ *-------------------------------------------------------------------------*/
+#if !defined(HAVE_CONFIG_H)
+#define  HAVE_FSTATAT     0
+#endif /* ! HAVE_CONFIG_H */
+
 /*--------------------------------------------------------------------*
- * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*
- *                          USER CONFIGURABLE                         *
- * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*
- *     Optional subdirectory translation for read/write to /tmp       *
- *--------------------------------------------------------------------*/
-/*
  * It is desirable on Windows to have all temp files written to the same
  * subdirectory of the Windows <Temp> directory, because files under <Temp>
  * persist after reboot, and the regression tests write a lot of files.
- * Consequently, all temp files on Windows are written to <Temp>/leptonica/
- * or subdirectories of it, with the translation:
- *        /tmp/xxx  -->   <Temp>/leptonica/xxx
- *
- * This is not the case for Unix, but we provide an option for reading
- * and writing on Unix with this translation:
- *        /tmp/xxx  -->   /tmp/leptonica/xxx
- * By default, leptonica is distributed for Unix without this translation
- * (except on Cygwin, which runs on Windows).
- */
-#if defined (__CYGWIN__)
-  #define  ADD_LEPTONICA_SUBDIR    1
-#else
-  #define  ADD_LEPTONICA_SUBDIR    0
-#endif
+ * We write all test files to /tmp/lept or subdirectories of /tmp/lept.
+ * Windows temp files are specified as in unix, but have the translation
+ *        /tmp/lept/xxx  -->   <Temp>/lept/xxx
+ *--------------------------------------------------------------------*/
 
 
 /*--------------------------------------------------------------------*
  *                          Built-in types                            *
  *--------------------------------------------------------------------*/
-typedef signed char             l_int8;
-typedef unsigned char           l_uint8;
-typedef short                   l_int16;
-typedef unsigned short          l_uint16;
-typedef int                     l_int32;
-typedef unsigned int            l_uint32;
-typedef float                   l_float32;
-typedef double                  l_float64;
+typedef int                     l_ok;    /*!< return type 0 if OK, 1 on error */
+typedef signed char             l_int8;     /*!< signed 8-bit value */
+typedef unsigned char           l_uint8;    /*!< unsigned 8-bit value */
+typedef short                   l_int16;    /*!< signed 16-bit value */
+typedef unsigned short          l_uint16;   /*!< unsigned 16-bit value */
+typedef int                     l_int32;    /*!< signed 32-bit value */
+typedef unsigned int            l_uint32;   /*!< unsigned 32-bit value */
+typedef float                   l_float32;  /*!< 32-bit floating point value */
+typedef double                  l_float64;  /*!< 64-bit floating point value */
 #ifdef COMPILER_MSVC
-typedef __int64                 l_int64;
-typedef unsigned __int64        l_uint64;
+typedef __int64                 l_int64;    /*!< signed 64-bit value */
+typedef unsigned __int64        l_uint64;   /*!< unsigned 64-bit value */
 #else
-typedef long long               l_int64;
-typedef unsigned long long      l_uint64;
+typedef long long               l_int64;    /*!< signed 64-bit value */
+typedef unsigned long long      l_uint64;   /*!< unsigned 64-bit value */
 #endif  /* COMPILER_MSVC */
 
 
+/*-------------------------------------------------------------------------*
+ * For security, the library is distributed in a configuration that does   *
+ * not permit (1) forking with 'system', which is used for displaying      *
+ * images and generating gnuplots, and (2) writing files with specified    *
+ * compiled-in file names.  All such writes are with functions such as     *
+ * pixWriteDebug() where the "Debug" is appended to the usual name.        *
+ * Whether the "Debug" version defaults to the standard version or is a    *
+ * no-op depends on the value of this global variable.  The default value  *
+ * of LeptDebugOK is 0, and it is set in writefile.c.  This value can be   *
+ * over-ridden, for development and debugging, by setLeptDebugOK().        *
+ *-------------------------------------------------------------------------*/
+LEPT_DLL extern l_int32  LeptDebugOK;  /* default is 0 */
+
+
 /*------------------------------------------------------------------------*
  *                            Standard macros                             *
  *------------------------------------------------------------------------*/
 #ifndef L_MIN
-#define L_MIN(x,y)   (((x) < (y)) ? (x) : (y))
+/*! Minimum of %x and %y */
+#define L_MIN(x, y)   (((x) < (y)) ? (x) : (y))
 #endif
 
 #ifndef L_MAX
-#define L_MAX(x,y)   (((x) > (y)) ? (x) : (y))
+/*! Maximum of %x and %y */
+#define L_MAX(x, y)   (((x) > (y)) ? (x) : (y))
 #endif
 
 #ifndef L_ABS
+/*! Absolute value of %x */
 #define L_ABS(x)     (((x) < 0) ? (-1 * (x)) : (x))
 #endif
 
 #ifndef L_SIGN
+/*! Sign of %x */
 #define L_SIGN(x)    (((x) < 0) ? -1 : 1)
 #endif
 
 #ifndef UNDEF
+/*! Undefined value */
 #define UNDEF        -1
 #endif
 
 #ifndef NULL
+/*! NULL value */
 #define NULL          0
 #endif
 
 #ifndef TRUE
+/*! True value */
 #define TRUE          1
 #endif
 
 #ifndef FALSE
+/*! False value */
 #define FALSE         0
 #endif
 
@@ -248,6 +286,7 @@ typedef unsigned long long      l_uint64;
 /*------------------------------------------------------------------------*
  *                    Simple search state variables                       *
  *------------------------------------------------------------------------*/
+/*! Search State */
 enum {
     L_NOT_FOUND = 0,
     L_FOUND = 1
@@ -257,12 +296,28 @@ enum {
 /*------------------------------------------------------------------------*
  *                     Path separator conversion                          *
  *------------------------------------------------------------------------*/
+/*! Path Separators */
 enum {
     UNIX_PATH_SEPCHAR = 0,
     WIN_PATH_SEPCHAR = 1
 };
 
 
+/*------------------------------------------------------------------------*
+ *                          Timing structs                                *
+ *------------------------------------------------------------------------*/
+typedef void *L_TIMER;
+
+/*! Timing struct */
+struct L_WallTimer {
+    l_int32  start_sec;
+    l_int32  start_usec;
+    l_int32  stop_sec;
+    l_int32  stop_usec;
+};
+typedef struct L_WallTimer  L_WALLTIMER;
+
+
 /*------------------------------------------------------------------------*
  *                      Standard memory allocation                        *
  *                                                                        *
@@ -270,15 +325,20 @@ enum {
  *  on all heap data except for Pix.  Memory management for Pix           *
  *  also defaults to malloc and free.  See pix1.c for details.            *
  *------------------------------------------------------------------------*/
-#define MALLOC(blocksize)           malloc(blocksize)
-#define CALLOC(numelem, elemsize)   calloc(numelem, elemsize)
-#define REALLOC(ptr, blocksize)     realloc(ptr, blocksize)
-#define FREE(ptr)                   free(ptr)
+#define LEPT_MALLOC(blocksize)           malloc(blocksize)
+#define LEPT_CALLOC(numelem, elemsize)   calloc(numelem, elemsize)
+#define LEPT_REALLOC(ptr, blocksize)     realloc(ptr, blocksize)
+#define LEPT_FREE(ptr)                   free(ptr)
 
 
 /*------------------------------------------------------------------------*
  *         Control printing of error, warning, and info messages          *
  *                                                                        *
+ *  Leptonica never sends output to stdout.  By default, all messages     *
+ *  go to stderr.  However, we provide a mechanism for runtime            *
+ *  redirection of output, using a custom stderr handler defined          *
+ *  by the user.  See utils1.c for details and examples.                  *
+ *                                                                        *
  *  To omit all messages to stderr, simply define NO_CONSOLE_IO on the    *
  *  command line.  For finer grained control, we have a mechanism         *
  *  based on the message severity level.  The following assumes that      *
@@ -313,6 +373,9 @@ enum {
  *  printed, if desired," whereas the run-time threshold setting says,    *
  *  "Print messages that have an equal or greater severity than this."    *
  *------------------------------------------------------------------------*/
+
+    /*! Control printing of error, warning and info messages */
+/*! Message Control */
 enum {
     L_SEVERITY_EXTERNAL = 0,   /* Get the severity from the environment   */
     L_SEVERITY_ALL      = 1,   /* Lowest severity: print all messages     */
@@ -338,34 +401,47 @@ enum {
  *  message threshold checking, because code for messages whose severity
  *  is lower than MINIMUM_SEVERITY won't be generated.
  *
- *  A production library might typically permit WARNING and higher
- *  messages to be generated, and a development library might permit
- *  DEBUG and higher.  The actual messages printed (as opposed to
- *  generated) would depend on the current run-time severity threshold.
+ *  A production library might typically permit ERROR messages to be
+ *  generated, and a development library might permit DEBUG and higher.
+ *  The actual messages printed (as opposed to generated) would depend
+ *  on the current run-time severity threshold.
+ *
+ *  This is a complex mechanism and a few examples may help.
+ *  (1) No output permitted under any circumstances.
+ *      Use:  -DNO_CONSOLE_IO  or  -DMINIMUM_SEVERITY=6
+ *  (2) Suppose you want to only allow error messages, and you don't
+ *      want to permit info or warning messages at runtime.
+ *      Use:  -DMINIMUM_SEVERITY=5
+ *  (3) Suppose you want to only allow error messages by default,
+ *      but you will permit this to be over-ridden at runtime.
+ *      Use:  -DDEFAULT_SEVERITY=5
+ *            and to allow info and warning override:
+ *                 setMsgSeverity(L_SEVERITY_INFO);
  */
 
 #ifdef  NO_CONSOLE_IO
   #undef MINIMUM_SEVERITY
   #undef DEFAULT_SEVERITY
 
-  #define MINIMUM_SEVERITY      L_SEVERITY_NONE
-  #define DEFAULT_SEVERITY      L_SEVERITY_NONE
+  #define MINIMUM_SEVERITY      L_SEVERITY_NONE    /*!< Compile-time default */
+  #define DEFAULT_SEVERITY      L_SEVERITY_NONE    /*!< Run-time default */
 
 #else
   #ifndef MINIMUM_SEVERITY
-    #define MINIMUM_SEVERITY    L_SEVERITY_INFO    /* Compile-time default */
+    #define MINIMUM_SEVERITY    L_SEVERITY_INFO    /*!< Compile-time default */
   #endif
 
   #ifndef DEFAULT_SEVERITY
-    #define DEFAULT_SEVERITY    MINIMUM_SEVERITY   /* Run-time default */
+    #define DEFAULT_SEVERITY    MINIMUM_SEVERITY   /*!< Run-time default */
   #endif
 #endif
 
 
-/*  The run-time message severity threshold is defined in utils.c.  */
+/*!  The run-time message severity threshold is defined in utils1.c.  */
 LEPT_DLL extern l_int32  LeptMsgSeverity;
 
 /*
+ * <pre>
  *  Usage
  *  =====
  *  Messages are of two types.
@@ -413,59 +489,60 @@ LEPT_DLL extern l_int32  LeptMsgSeverity;
  *  The L_nnn() macros below do not return a value, but because the
  *  conditional operator requires one for the false condition, we
  *  specify a void expression.
+ * </pre>
  */
 
 #ifdef  NO_CONSOLE_IO
 
   #define PROCNAME(name)
-  #define ERROR_INT(a,b,c)            ((l_int32)(c))
-  #define ERROR_FLOAT(a,b,c)          ((l_float32)(c))
-  #define ERROR_PTR(a,b,c)            ((void *)(c))
-  #define L_ERROR(a,...)
-  #define L_WARNING(a,...)
-  #define L_INFO(a,...)
+  #define ERROR_INT(a, b, c)            ((l_int32)(c))
+  #define ERROR_FLOAT(a, b, c)          ((l_float32)(c))
+  #define ERROR_PTR(a, b, c)            ((void *)(c))
+  #define L_ERROR(a, ...)
+  #define L_WARNING(a, ...)
+  #define L_INFO(a, ...)
 
 #else
 
   #define PROCNAME(name)              static const char procName[] = name
-  #define IF_SEV(l,t,f) \
+  #define IF_SEV(l, t, f) \
       ((l) >= MINIMUM_SEVERITY && (l) >= LeptMsgSeverity ? (t) : (f))
 
-  #define ERROR_INT(a,b,c) \
-      IF_SEV(L_SEVERITY_ERROR, returnErrorInt((a),(b),(c)), (l_int32)(c))
-  #define ERROR_FLOAT(a,b,c) \
-      IF_SEV(L_SEVERITY_ERROR, returnErrorFloat((a),(b),(c)), (l_float32)(c))
-  #define ERROR_PTR(a,b,c) \
-      IF_SEV(L_SEVERITY_ERROR, returnErrorPtr((a),(b),(c)), (void *)(c))
+  #define ERROR_INT(a, b, c) \
+      IF_SEV(L_SEVERITY_ERROR, returnErrorInt((a), (b), (c)), (l_int32)(c))
+  #define ERROR_FLOAT(a, b, c) \
+      IF_SEV(L_SEVERITY_ERROR, returnErrorFloat((a), (b), (c)), (l_float32)(c))
+  #define ERROR_PTR(a, b, c) \
+      IF_SEV(L_SEVERITY_ERROR, returnErrorPtr((a), (b), (c)), (void *)(c))
 
-  #define L_ERROR(a,...) \
+  #define L_ERROR(a, ...) \
       IF_SEV(L_SEVERITY_ERROR, \
-             (void)fprintf(stderr, "Error in %s: " a, __VA_ARGS__), \
+             (void)lept_stderr("Error in %s: " a, __VA_ARGS__), \
              (void)0)
-  #define L_WARNING(a,...) \
+  #define L_WARNING(a, ...) \
       IF_SEV(L_SEVERITY_WARNING, \
-             (void)fprintf(stderr, "Warning in %s: " a, __VA_ARGS__), \
+             (void)lept_stderr("Warning in %s: " a, __VA_ARGS__), \
              (void)0)
-  #define L_INFO(a,...) \
+  #define L_INFO(a, ...) \
       IF_SEV(L_SEVERITY_INFO, \
-             (void)fprintf(stderr, "Info in %s: " a, __VA_ARGS__), \
+             (void)lept_stderr("Info in %s: " a, __VA_ARGS__), \
              (void)0)
 
 #if 0  /* Alternative method for controlling L_* message output */
-  #define L_ERROR(a,...) \
+  #define L_ERROR(a, ...) \
     { if (L_SEVERITY_ERROR >= MINIMUM_SEVERITY && \
           L_SEVERITY_ERROR >= LeptMsgSeverity) \
-          fprintf(stderr, "Error in %s: " a, __VA_ARGS__) \
+          lept_stderr("Error in %s: " a, __VA_ARGS__) \
     }
-  #define L_WARNING(a,...) \
+  #define L_WARNING(a, ...) \
     { if (L_SEVERITY_WARNING >= MINIMUM_SEVERITY && \
           L_SEVERITY_WARNING >= LeptMsgSeverity) \
-          fprintf(stderr, "Warning in %s: " a, __VA_ARGS__) \
+          lept_stderr("Warning in %s: " a, __VA_ARGS__) \
     }
-  #define L_INFO(a,...) \
+  #define L_INFO(a, ...) \
     { if (L_SEVERITY_INFO >= MINIMUM_SEVERITY && \
           L_SEVERITY_INFO >= LeptMsgSeverity) \
-             fprintf(stderr, "Info in %s: " a, __VA_ARGS__) \
+          lept_stderr("Info in %s: " a, __VA_ARGS__) \
     }
 #endif
 
@@ -473,9 +550,9 @@ LEPT_DLL extern l_int32  LeptMsgSeverity;
 
 
 /*------------------------------------------------------------------------*
- *                        snprintf() renamed in MSVC                      *
+ *              snprintf() renamed in MSVC (pre-VS2015)                   *
  *------------------------------------------------------------------------*/
-#ifdef _MSC_VER
+#if defined _MSC_VER && _MSC_VER < 1900
 #define snprintf(buf, size, ...)  _snprintf_s(buf, size, _TRUNCATE, __VA_ARGS__)
 #endif
 
