@@ -111,8 +111,8 @@
  *
  *  The ascender/descender signal is useful for determining text
  *  orientation in Roman alphabets because the incidence of letters
- *  with straight-line ascenders (b, d, h, k, l, <t>) outnumber
- *  those with descenders (<g>, p, q).  The letters <t> and <g>
+ *  with straight-line ascenders (b, d, h, k, l, 't') outnumber
+ *  those with descenders ('g', p, q).  The letters 't' and 'g'
  *  will respond variably to the filter, depending on the type face.
  *
  *  What about the mirror image situations?  These aren't common
@@ -167,9 +167,46 @@
  *
  *  A high-level interface, pixOrientCorrect() combines the detection
  *  of the orientation with the rotation decision and the rotation itself.
+ *
+ *  Finally, use can be made of programs such as exiftool and convert to
+ *  read exif camera orientation data in jpeg files and conditionally rotate.
+ *  Here is an example shell script, made by Dan9er:
+ *  ==================================================================
+ *  #!/bin/sh
+ *  #   orientByExif.sh
+ *  #   Dependencies: exiftool (exiflib) and convert (ImageMagick)
+ *  #   Note: if there is no exif orientation data in the jpeg file,
+ *  #         this simply copies the input file.
+ *  #
+ *  if [[ -z $(command -v exiftool) || -z $(command -v convert) ]]; then
+ *      echo "You need to install dependencies; e.g.:"
+ *      echo "   sudo apt install libimage-exiftool-perl"
+ *      echo "   sudo apt install imagemagick"
+ *      exit 1
+ *  fi
+ *  if [[ $# != 2 ]]; then
+ *      echo "Syntax: orientByExif infile outfile"
+ *      exit 2
+ *  fi
+ *  if [[ ${1: -4} != ".jpg" ]]; then
+ *      echo "File is not a jpeg"
+ *      exit 3
+ *  fi
+ *  if [[ $(exiftool -s3 -n -Orientation "$1") = 1 ]]; then
+ *      echo "Image is already upright"
+ *      exit 0
+ *  fi
+ *  convert "$1" -auto-orient "$2"
+ *  echo "Done"
+ *  exit 0
+ *  ==================================================================
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <math.h>
 #include "allheaders.h"
 
@@ -199,13 +236,13 @@ static const char *textsel4 = "xxxxxx"
                               " oo  x";
 
     /* Parameters for determining orientation */
-static const l_int32  DEFAULT_MIN_UP_DOWN_COUNT = 70;
-static const l_float32  DEFAULT_MIN_UP_DOWN_CONF = 8.0;
-static const l_float32  DEFAULT_MIN_UP_DOWN_RATIO = 2.5;
+static const l_int32  DefaultMinUpDownCount = 70;
+static const l_float32  DefaultMinUpDownConf = 8.0;
+static const l_float32  DefaultMinUpDownRatio = 2.5;
 
     /* Parameters for determining mirror flip */
-static const l_int32  DEFAULT_MIN_MIRROR_FLIP_COUNT = 100;
-static const l_float32  DEFAULT_MIN_MIRROR_FLIP_CONF = 5.0;
+static const l_int32  DefaultMinMirrorFlipCount = 100;
+static const l_float32  DefaultMinMirrorFlipConf = 5.0;
 
     /* Static debug function */
 static void pixDebugFlipDetect(const char *filename, PIX *pixs,
@@ -218,13 +255,13 @@ static void pixDebugFlipDetect(const char *filename, PIX *pixs,
 /*!
  * \brief   pixOrientCorrect()
  *
- * \param[in]    pixs 1 bpp, deskewed, English text, 150 - 300 ppi
- * \param[in]    minupconf minimum value for which a decision can be made
- * \param[in]    minratio minimum conf ratio required for a decision
- * \param[out]   pupconf [optional] ; use NULL to skip
- * \param[out]   pleftconf [optional] ; use NULL to skip
- * \param[out]   protation [optional] ; use NULL to skip
- * \param[in]    debug 1 for debug output; 0 otherwise
+ * \param[in]    pixs        1 bpp, deskewed, English text, 150 - 300 ppi
+ * \param[in]    minupconf   minimum value for which a decision can be made
+ * \param[in]    minratio    minimum conf ratio required for a decision
+ * \param[out]   pupconf     [optional] ; use NULL to skip
+ * \param[out]   pleftconf   [optional] ; use NULL to skip
+ * \param[out]   protation   [optional] ; use NULL to skip
+ * \param[in]    debug       1 for debug output; 0 otherwise
  * \return  pixd  may be rotated by 90, 180 or 270; null on error
  *
  * <pre>
@@ -256,8 +293,6 @@ PIX       *pix1;
     if (!pixs || pixGetDepth(pixs) != 1)
         return (PIX *)ERROR_PTR("pixs undefined or not 1 bpp", procName, NULL);
 
-    lept_mkdir("lept/orient");
-
         /* Get confidences for orientation */
     pixUpDownDetectDwa(pixs, &upconf, 0, debug);
     pix1 = pixRotate90(pixs, 1);
@@ -311,11 +346,11 @@ PIX       *pix1;
 /*!
  * \brief   pixOrientDetect()
  *
- * \param[in]    pixs 1 bpp, deskewed, English text, 150 - 300 ppi
- * \param[out]   pupconf [optional] ; may be NULL
- * \param[out]   pleftconf [optional] ; may be NULL
- * \param[in]    mincount min number of up + down; use 0 for default
- * \param[in]    debug 1 for debug output; 0 otherwise
+ * \param[in]    pixs       1 bpp, deskewed, English text, 150 - 300 ppi
+ * \param[out]   pupconf    [optional] ; may be NULL
+ * \param[out]   pleftconf  [optional] ; may be NULL
+ * \param[in]    mincount   min number of up + down; use 0 for default
+ * \param[in]    debug      1 for debug output; 0 otherwise
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -363,10 +398,16 @@ PIX       *pix1;
  *      (6) One should probably not interpret the direction unless
  *          there are a sufficient number of counts for both orientations,
  *          in which case neither upconf nor leftconf will be 0.0.
- *      (7) Uses rasterop implementation of HMT.
+ *      (7) This algorithm will fail on some images, such as tables,
+ *          where most of the characters are numbers and appear as
+ *          uppercase, but there are some repeated words that give a
+ *          biased signal.  It may be advisable to run a table detector
+ *          first (e.g., pixDecideIfTable()), and not run the orientation
+ *          detector if it is a table.
+ *      (8) Uses rasterop implementation of HMT.
  * </pre>
  */
-l_int32
+l_ok
 pixOrientDetect(PIX        *pixs,
                 l_float32  *pupconf,
                 l_float32  *pleftconf,
@@ -382,9 +423,7 @@ PIX  *pix1;
     if (!pupconf && !pleftconf)
         return ERROR_INT("nothing to do", procName, 1);
     if (mincount == 0)
-        mincount = DEFAULT_MIN_UP_DOWN_COUNT;
-
-    lept_mkdir("lept/orient");
+        mincount = DefaultMinUpDownCount;
 
     if (pupconf)
         pixUpDownDetect(pixs, pupconf, mincount, debug);
@@ -401,12 +440,12 @@ PIX  *pix1;
 /*!
  * \brief   makeOrientDecision()
  *
- * \param[in]    upconf nonzero
- * \param[in]    leftconf nonzero
- * \param[in]    minupconf minimum value for which a decision can be made
- * \param[in]    minratio minimum conf ratio required for a decision
- * \param[out]   porient text orientation enum {0,1,2,3,4}
- * \param[in]    debug 1 for debug output; 0 otherwise
+ * \param[in]    upconf      nonzero
+ * \param[in]    leftconf    nonzero
+ * \param[in]    minupconf   minimum value for which a decision can be made
+ * \param[in]    minratio    minimum conf ratio required for a decision
+ * \param[out]   porient     text orientation enum {0,1,2,3,4}
+ * \param[in]    debug       1 for debug output; 0 otherwise
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -427,7 +466,7 @@ PIX  *pix1;
  *            L_TEXT_ORIENT_RIGHT:    landscape, text up facing right
  * </pre>
  */
-l_int32
+l_ok
 makeOrientDecision(l_float32  upconf,
                    l_float32  leftconf,
                    l_float32  minupconf,
@@ -447,12 +486,10 @@ l_float32  absupconf, absleftconf;
         return 0;
     }
 
-    lept_mkdir("lept/orient");
-
     if (minupconf == 0.0)
-        minupconf = DEFAULT_MIN_UP_DOWN_CONF;
+        minupconf = DefaultMinUpDownConf;
     if (minratio == 0.0)
-        minratio = DEFAULT_MIN_UP_DOWN_RATIO;
+        minratio = DefaultMinUpDownRatio;
     absupconf = L_ABS(upconf);
     absleftconf = L_ABS(leftconf);
 
@@ -468,17 +505,17 @@ l_float32  absupconf, absleftconf;
         *porient = L_TEXT_ORIENT_RIGHT;
 
     if (debug) {
-        fprintf(stderr, "upconf = %7.3f, leftconf = %7.3f\n", upconf, leftconf);
+        lept_stderr("upconf = %7.3f, leftconf = %7.3f\n", upconf, leftconf);
         if (*porient == L_TEXT_ORIENT_UNKNOWN)
-            fprintf(stderr, "Confidence is low; no determination is made\n");
+            lept_stderr("Confidence is low; no determination is made\n");
         else if (*porient == L_TEXT_ORIENT_UP)
-            fprintf(stderr, "Text is rightside-up\n");
+            lept_stderr("Text is rightside-up\n");
         else if (*porient == L_TEXT_ORIENT_LEFT)
-            fprintf(stderr, "Text is rotated 90 deg ccw\n");
+            lept_stderr("Text is rotated 90 deg ccw\n");
         else if (*porient == L_TEXT_ORIENT_DOWN)
-            fprintf(stderr, "Text is upside-down\n");
+            lept_stderr("Text is upside-down\n");
         else   /* *porient == L_TEXT_ORIENT_RIGHT */
-            fprintf(stderr, "Text is rotated 90 deg cw\n");
+            lept_stderr("Text is rotated 90 deg cw\n");
     }
 
     return 0;
@@ -488,10 +525,10 @@ l_float32  absupconf, absleftconf;
 /*!
  * \brief   pixUpDownDetect()
  *
- * \param[in]    pixs 1 bpp, deskewed, English text, 150 - 300 ppi
- * \param[out]   pconf confidence that text is rightside-up
- * \param[in]    mincount min number of up + down; use 0 for default
- * \param[in]    debug 1 for debug output; 0 otherwise
+ * \param[in]    pixs       1 bpp, deskewed, English text, 150 - 300 ppi
+ * \param[out]   pconf      confidence that text is rightside-up
+ * \param[in]    mincount   min number of up + down; use 0 for default
+ * \param[in]    debug      1 for debug output; 0 otherwise
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -506,7 +543,7 @@ l_float32  absupconf, absleftconf;
  *          and not rotated at a 90 degree angle.
  * </pre>
  */
-l_int32
+l_ok
 pixUpDownDetect(PIX        *pixs,
                 l_float32  *pconf,
                 l_int32     mincount,
@@ -519,11 +556,11 @@ pixUpDownDetect(PIX        *pixs,
 /*!
  * \brief   pixUpDownDetectGeneral()
  *
- * \param[in]    pixs 1 bpp, deskewed, English text, 150 - 300 ppi
- * \param[out]   pconf confidence that text is rightside-up
- * \param[in]    mincount min number of up + down; use 0 for default
- * \param[in]    npixels number of pixels removed from each side of word box
- * \param[in]    debug 1 for debug output; 0 otherwise
+ * \param[in]    pixs       1 bpp, deskewed, English text, 150 - 300 ppi
+ * \param[out]   pconf      confidence that text is rightside-up
+ * \param[in]    mincount   min number of up + down; use 0 for default
+ * \param[in]    npixels    number of pixels removed from each side of word box
+ * \param[in]    debug      1 for debug output; 0 otherwise
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -554,7 +591,7 @@ pixUpDownDetect(PIX        *pixs,
  *          is going too far -- components will get merged.
  * </pre>
  */
-l_int32
+l_ok
 pixUpDownDetectGeneral(PIX        *pixs,
                        l_float32  *pconf,
                        l_int32     mincount,
@@ -574,11 +611,13 @@ SEL       *sel1, *sel2, *sel3, *sel4;
     if (!pixs || pixGetDepth(pixs) != 1)
         return ERROR_INT("pixs not defined or not 1 bpp", procName, 1);
     if (mincount == 0)
-        mincount = DEFAULT_MIN_UP_DOWN_COUNT;
+        mincount = DefaultMinUpDownCount;
     if (npixels < 0)
         npixels = 0;
 
-    lept_mkdir("lept/orient");
+    if (debug) {
+        lept_mkdir("lept/orient");
+    }
 
     sel1 = selCreateFromString(textsel1, 5, 6, NULL);
     sel2 = selCreateFromString(textsel2, 5, 6, NULL);
@@ -652,12 +691,12 @@ SEL       *sel1, *sel2, *sel3, *sel4;
 
     if (debug) {
         if (pixm) pixWriteDebug("/tmp/lept/orient/pixm1.png", pixm, IFF_PNG);
-        fprintf(stderr, "nup = %7.3f, ndown = %7.3f, conf = %7.3f\n",
+        lept_stderr("nup = %7.3f, ndown = %7.3f, conf = %7.3f\n",
                 nup, ndown, *pconf);
-        if (*pconf > DEFAULT_MIN_UP_DOWN_CONF)
-            fprintf(stderr, "Text is rightside-up\n");
-        if (*pconf < -DEFAULT_MIN_UP_DOWN_CONF)
-            fprintf(stderr, "Text is upside-down\n");
+        if (*pconf > DefaultMinUpDownConf)
+            lept_stderr("Text is rightside-up\n");
+        if (*pconf < -DefaultMinUpDownConf)
+            lept_stderr("Text is upside-down\n");
     }
 
     pixDestroy(&pix0);
@@ -677,11 +716,11 @@ SEL       *sel1, *sel2, *sel3, *sel4;
 /*!
  * \brief   pixOrientDetectDwa()
  *
- * \param[in]    pixs 1 bpp, deskewed, English text
- * \param[out]   pupconf [optional] ; may be NULL
- * \param[out]   pleftconf [optional] ; may be NULL
- * \param[in]    mincount min number of up + down; use 0 for default
- * \param[in]    debug 1 for debug output; 0 otherwise
+ * \param[in]    pixs        1 bpp, deskewed, English text
+ * \param[out]   pupconf     [optional] ; may be NULL
+ * \param[out]   pleftconf   [optional] ; may be NULL
+ * \param[in]    mincount    min number of up + down; use 0 for default
+ * \param[in]    debug       1 for debug output; 0 otherwise
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -695,7 +734,7 @@ SEL       *sel1, *sel2, *sel3, *sel4;
  *      (3) This runs about 2.5 times faster than the pixOrientDetect().
  * </pre>
  */
-l_int32
+l_ok
 pixOrientDetectDwa(PIX        *pixs,
                    l_float32  *pupconf,
                    l_float32  *pleftconf,
@@ -711,7 +750,7 @@ PIX  *pix1;
     if (!pupconf && !pleftconf)
         return ERROR_INT("nothing to do", procName, 1);
     if (mincount == 0)
-        mincount = DEFAULT_MIN_UP_DOWN_COUNT;
+        mincount = DefaultMinUpDownCount;
 
     if (pupconf)
         pixUpDownDetectDwa(pixs, pupconf, mincount, debug);
@@ -728,10 +767,10 @@ PIX  *pix1;
 /*!
  * \brief   pixUpDownDetectDwa()
  *
- * \param[in]    pixs 1 bpp, deskewed, English text, 150 - 300 ppi
- * \param[out]   pconf confidence that text is rightside-up
- * \param[in]    mincount min number of up + down; use 0 for default
- * \param[in]    debug 1 for debug output; 0 otherwise
+ * \param[in]    pixs       1 bpp, deskewed, English text, 150 - 300 ppi
+ * \param[out]   pconf      confidence that text is rightside-up
+ * \param[in]    mincount   min number of up + down; use 0 for default
+ * \param[in]    debug      1 for debug output; 0 otherwise
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -748,7 +787,7 @@ PIX  *pix1;
  *          and not rotated at a 90 degree angle.
  * </pre>
  */
-l_int32
+l_ok
 pixUpDownDetectDwa(PIX       *pixs,
                   l_float32  *pconf,
                   l_int32     mincount,
@@ -761,11 +800,11 @@ pixUpDownDetectDwa(PIX       *pixs,
 /*!
  * \brief   pixUpDownDetectGeneralDwa()
  *
- * \param[in]    pixs 1 bpp, deskewed, English text
- * \param[out]   pconf confidence that text is rightside-up
- * \param[in]    mincount min number of up + down; use 0 for default
- * \param[in]    npixels number of pixels removed from each side of word box
- * \param[in]    debug 1 for debug output; 0 otherwise
+ * \param[in]    pixs       1 bpp, deskewed, English text
+ * \param[out]   pconf      confidence that text is rightside-up
+ * \param[in]    mincount   min number of up + down; use 0 for default
+ * \param[in]    npixels    number of pixels removed from each side of word box
+ * \param[in]    debug      1 for debug output; 0 otherwise
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -773,7 +812,7 @@ pixUpDownDetectDwa(PIX       *pixs,
  *      (1) See the notes in pixUpDownDetectGeneral() for usage.
  * </pre>
  */
-l_int32
+l_ok
 pixUpDownDetectGeneralDwa(PIX        *pixs,
                           l_float32  *pconf,
                           l_int32     mincount,
@@ -796,12 +835,10 @@ PIX       *pixt, *pix0, *pix1, *pix2, *pix3, *pixm;
     if (!pixs || pixGetDepth(pixs) != 1)
         return ERROR_INT("pixs not defined or not 1 bpp", procName, 1);
     if (mincount == 0)
-        mincount = DEFAULT_MIN_UP_DOWN_COUNT;
+        mincount = DefaultMinUpDownCount;
     if (npixels < 0)
         npixels = 0;
 
-    lept_mkdir("lept/orient");
-
         /* One of many reasonable pre-filtering sequences: (1, 8) and (30, 1).
          * This closes holes in x-height characters and joins them at
          * the x-height.  There is more noise in the descender detection
@@ -871,13 +908,16 @@ PIX       *pixt, *pix0, *pix1, *pix2, *pix3, *pixm;
         *pconf = 2. * ((nup - ndown) / sqrt(nup + ndown));
 
     if (debug) {
-        if (pixm) pixWriteDebug("/tmp/lept/orient/pixm2.png", pixm, IFF_PNG);
-        fprintf(stderr, "nup = %7.3f, ndown = %7.3f, conf = %7.3f\n",
+        if (pixm) {
+            lept_mkdir("lept/orient");
+            pixWriteDebug("/tmp/lept/orient/pixm2.png", pixm, IFF_PNG);
+        }
+        lept_stderr("nup = %7.3f, ndown = %7.3f, conf = %7.3f\n",
                 nup, ndown, *pconf);
-        if (*pconf > DEFAULT_MIN_UP_DOWN_CONF)
-            fprintf(stderr, "Text is rightside-up\n");
-        if (*pconf < -DEFAULT_MIN_UP_DOWN_CONF)
-            fprintf(stderr, "Text is upside-down\n");
+        if (*pconf > DefaultMinUpDownConf)
+            lept_stderr("Text is rightside-up\n");
+        if (*pconf < -DefaultMinUpDownConf)
+            lept_stderr("Text is upside-down\n");
     }
 
     pixDestroy(&pix0);
@@ -894,10 +934,10 @@ PIX       *pixt, *pix0, *pix1, *pix2, *pix3, *pixm;
 /*!
  * \brief   pixMirrorDetect()
  *
- * \param[in]    pixs 1 bpp, deskewed, English text
- * \param[out]   pconf confidence that text is not LR mirror reversed
- * \param[in]    mincount min number of left + right; use 0 for default
- * \param[in]    debug 1 for debug output; 0 otherwise
+ * \param[in]    pixs       1 bpp, deskewed, English text
+ * \param[out]   pconf      confidence that text is not LR mirror reversed
+ * \param[in]    mincount   min number of left + right; use 0 for default
+ * \param[in]    debug      1 for debug output; 0 otherwise
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -930,7 +970,7 @@ PIX       *pixt, *pix0, *pix1, *pix2, *pix3, *pixm;
  *          of morphology!
  * </pre>
  */
-l_int32
+l_ok
 pixMirrorDetect(PIX        *pixs,
                 l_float32  *pconf,
                 l_int32     mincount,
@@ -949,7 +989,11 @@ SEL       *sel1, *sel2;
     if (!pixs || pixGetDepth(pixs) != 1)
         return ERROR_INT("pixs not defined or not 1 bpp", procName, 1);
     if (mincount == 0)
-        mincount = DEFAULT_MIN_MIRROR_FLIP_COUNT;
+        mincount = DefaultMinMirrorFlipCount;
+
+    if (debug) {
+        lept_mkdir("lept/orient");
+    }
 
     sel1 = selCreateFromString(textsel1, 5, 6, NULL);
     sel2 = selCreateFromString(textsel2, 5, 6, NULL);
@@ -990,11 +1034,11 @@ SEL       *sel1, *sel2;
         *pconf = 2. * ((nright - nleft) / sqrt(nright + nleft));
 
     if (debug) {
-        fprintf(stderr, "nright = %f, nleft = %f\n", nright, nleft);
-        if (*pconf > DEFAULT_MIN_MIRROR_FLIP_CONF)
-            fprintf(stderr, "Text is not mirror reversed\n");
-        if (*pconf < -DEFAULT_MIN_MIRROR_FLIP_CONF)
-            fprintf(stderr, "Text is mirror reversed\n");
+        lept_stderr("nright = %f, nleft = %f\n", nright, nleft);
+        if (*pconf > DefaultMinMirrorFlipConf)
+            lept_stderr("Text is not mirror reversed\n");
+        if (*pconf < -DefaultMinMirrorFlipConf)
+            lept_stderr("Text is mirror reversed\n");
     }
 
     return 0;
@@ -1008,10 +1052,10 @@ SEL       *sel1, *sel2;
 /*!
  * \brief   pixMirrorDetectDwa()
  *
- * \param[in]    pixs 1 bpp, deskewed, English text
- * \param[out]   pconf confidence that text is not LR mirror reversed
- * \param[in]    mincount min number of left + right; use 0 for default
- * \param[in]    debug 1 for debug output; 0 otherwise
+ * \param[in]    pixs       1 bpp, deskewed, English text
+ * \param[out]   pconf      confidence that text is not LR mirror reversed
+ * \param[in]    mincount   min number of left + right; use 0 for default
+ * \param[in]    debug      1 for debug output; 0 otherwise
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1021,7 +1065,7 @@ SEL       *sel1, *sel2;
  *      (2) See notes in pixMirrorDetect().
  * </pre>
  */
-l_int32
+l_ok
 pixMirrorDetectDwa(PIX        *pixs,
                    l_float32  *pconf,
                    l_int32     mincount,
@@ -1041,7 +1085,7 @@ PIX       *pix0, *pix1, *pix2, *pix3;
     if (!pixs || pixGetDepth(pixs) != 1)
         return ERROR_INT("pixs not defined or not 1 bpp", procName, 1);
     if (mincount == 0)
-        mincount = DEFAULT_MIN_MIRROR_FLIP_COUNT;
+        mincount = DefaultMinMirrorFlipCount;
 
         /* Fill x-height characters but not space between them, sort of. */
     pix3 = pixMorphSequenceDwa(pixs, "d1.30", 0);
@@ -1078,11 +1122,11 @@ PIX       *pix0, *pix1, *pix2, *pix3;
         *pconf = 2. * ((nright - nleft) / sqrt(nright + nleft));
 
     if (debug) {
-        fprintf(stderr, "nright = %f, nleft = %f\n", nright, nleft);
-        if (*pconf > DEFAULT_MIN_MIRROR_FLIP_CONF)
-            fprintf(stderr, "Text is not mirror reversed\n");
-        if (*pconf < -DEFAULT_MIN_MIRROR_FLIP_CONF)
-            fprintf(stderr, "Text is mirror reversed\n");
+        lept_stderr("nright = %f, nleft = %f\n", nright, nleft);
+        if (*pconf > DefaultMinMirrorFlipConf)
+            lept_stderr("Text is not mirror reversed\n");
+        if (*pconf < -DefaultMinMirrorFlipConf)
+            lept_stderr("Text is mirror reversed\n");
     }
 
     return 0;
@@ -1093,13 +1137,13 @@ PIX       *pix0, *pix1, *pix2, *pix3;
  *                        Static debug helper                     *
  *----------------------------------------------------------------*/
 /*
- *  pixDebugFlipDetect()
+ * \brief   pixDebugFlipDetect()
  *
- *      Input:  filename (for output debug file)
- *              pixs (input to pix*Detect)
- *              pixhm (hit-miss result from ascenders or descenders)
- *              enable (1 to enable this function; 0 to disable)
- *      Return: void
+ * \param[in]    filename   for output debug file
+ * \param[in]    pixs       input to pix*Detect
+ * \param[in]    pixhm      hit-miss result from ascenders or descenders
+ * \param[in]    enable     1 to enable this function; 0 to disable
+ * \return   void
  */
 static void
 pixDebugFlipDetect(const char *filename,
