@@ -94,6 +94,10 @@
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include "allheaders.h"
 
@@ -239,7 +243,7 @@ static char wplstrm[][10] = {"- wpls", "- wpls2", "- wpls3", "- wpls4",
  *      (2) See fmorphautogen1() and fmorphautogen2() for details.
  * </pre>
  */
-l_int32
+l_ok
 fmorphautogen(SELA        *sela,
               l_int32      fileindex,
               const char  *filename)
@@ -274,12 +278,12 @@ l_int32  ret1, ret2;
  *          opening or closing for any of the sels in the input sela.
  *      (2) The fileindex parameter is inserted into the output
  *          filename, as described below.
- *      (3) If filename == NULL, the output file is fmorphgen.<n>.c,
- *          where <n> is equal to the 'fileindex' parameter.
- *      (4) If filename != NULL, the output file is <filename>.<n>.c.
+ *      (3) If filename == NULL, the output file is fmorphgen.[n].c,
+ *          where [n] is equal to the %fileindex parameter.
+ *      (4) If filename != NULL, the output file is [%filename].[n].c.
  * </pre>
  */
-l_int32
+l_ok
 fmorphautogen1(SELA        *sela,
                l_int32      fileindex,
                const char  *filename)
@@ -331,9 +335,9 @@ SARRAY  *sa1, *sa2, *sa3;
     str_doc1 = stringNew(bigbuf);
     sprintf(bigbuf, " *             PIX     *pixFMorphopGen_%d()", fileindex);
     str_doc2 = stringNew(bigbuf);
-    sprintf(bigbuf, " *  pixMorphDwa_%d()", fileindex);
+    sprintf(bigbuf, " * \\brief   pixMorphDwa_%d()", fileindex);
     str_doc3 = stringNew(bigbuf);
-    sprintf(bigbuf, " *  pixFMorphopGen_%d()", fileindex);
+    sprintf(bigbuf, " * \\brief   pixFMorphopGen_%d()", fileindex);
     str_doc4 = stringNew(bigbuf);
     sprintf(bigbuf, "pixMorphDwa_%d(PIX     *pixd,", fileindex);
     str_def1 = stringNew(bigbuf);
@@ -482,9 +486,9 @@ SARRAY  *sa1, *sa2, *sa3;
  *          in the input sela.
  *      (2) The fileindex parameter is inserted into the output
  *          filename, as described below.
- *      (3) If filename == NULL, the output file is fmorphgenlow.<n>.c,
- *          where <n> is equal to the 'fileindex' parameter.
- *      (4) If filename != NULL, the output file is <filename>low.<n>.c.
+ *      (3) If filename == NULL, the output file is fmorphgenlow.[n].c,
+ *          where [n] is equal to the 'fileindex' parameter.
+ *      (4) If filename != NULL, the output file is [filename]low.[n].c.
  */
 l_int32
 fmorphautogen2(SELA        *sela,
