@@ -28,6 +28,7 @@
  * \file fpix2.c
  * <pre>
  *
+ *    ------------------------------------------
  *    This file has these FPix utilities:
  *       ~ interconversions with pix, fpix, dpix
  *       ~ min and max values
@@ -37,6 +38,7 @@
  *       ~ border functions
  *       ~ simple rasterop (source --> dest)
  *       ~ geometric transforms
+ *    ------------------------------------------
  *
  *    Interconversions between Pix, FPix and DPix
  *          FPIX          *pixConvertToFPix()
@@ -99,6 +101,10 @@
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include "allheaders.h"
 
@@ -108,8 +114,8 @@
 /*!
  * \brief   pixConvertToFPix()
  *
- * \param[in]    pixs 1, 2, 4, 8, 16 or 32 bpp
- * \param[in]    ncomps number of components: 3 for RGB, 1 otherwise
+ * \param[in]    pixs      1, 2, 4, 8, 16 or 32 bpp
+ * \param[in]    ncomps    number of components: 3 for RGB, 1 otherwise
  * \return  fpix, or NULL on error
  *
  * <pre>
@@ -201,8 +207,8 @@ FPIX       *fpixd;
 /*!
  * \brief   pixConvertToDPix()
  *
- * \param[in]    pixs 1, 2, 4, 8, 16 or 32 bpp
- * \param[in]    ncomps number of components: 3 for RGB, 1 otherwise
+ * \param[in]    pixs      1, 2, 4, 8, 16 or 32 bpp
+ * \param[in]    ncomps    number of components: 3 for RGB, 1 otherwise
  * \return  dpix, or NULL on error
  *
  * <pre>
@@ -295,9 +301,9 @@ DPIX       *dpixd;
  * \brief   fpixConvertToPix()
  *
  * \param[in]    fpixs
- * \param[in]    outdepth 0, 8, 16 or 32 bpp
- * \param[in]    negvals L_CLIP_TO_ZERO, L_TAKE_ABSVAL
- * \param[in]    errorflag 1 to output error stats; 0 otherwise
+ * \param[in]    outdepth     0, 8, 16 or 32 bpp
+ * \param[in]    negvals      L_CLIP_TO_ZERO, L_TAKE_ABSVAL
+ * \param[in]    errorflag    1 to output error stats; 0 otherwise
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -514,9 +520,9 @@ DPIX       *dpix;
  * \brief   dpixConvertToPix()
  *
  * \param[in]    dpixs
- * \param[in]    outdepth 0, 8, 16 or 32 bpp
- * \param[in]    negvals L_CLIP_TO_ZERO, L_TAKE_ABSVAL
- * \param[in]    errorflag 1 to output error stats; 0 otherwise
+ * \param[in]    outdepth     0, 8, 16 or 32 bpp
+ * \param[in]    negvals      L_CLIP_TO_ZERO, L_TAKE_ABSVAL
+ * \param[in]    errorflag    1 to output error stats; 0 otherwise
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -680,12 +686,12 @@ FPIX       *fpix;
  * \brief   fpixGetMin()
  *
  * \param[in]    fpix
- * \param[out]   pminval [optional] min value
- * \param[out]   pxminloc [optional] x location of min
- * \param[out]   pyminloc [optional] y location of min
+ * \param[out]   pminval    [optional] min value
+ * \param[out]   pxminloc   [optional] x location of min
+ * \param[out]   pyminloc   [optional] y location of min
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 fpixGetMin(FPIX       *fpix,
            l_float32  *pminval,
            l_int32    *pxminloc,
@@ -733,12 +739,12 @@ l_float32   minval;
  * \brief   fpixGetMax()
  *
  * \param[in]    fpix
- * \param[out]   pmaxval [optional] max value
- * \param[out]   pxmaxloc [optional] x location of max
- * \param[out]   pymaxloc [optional] y location of max
+ * \param[out]   pmaxval    [optional] max value
+ * \param[out]   pxmaxloc   [optional] x location of max
+ * \param[out]   pymaxloc   [optional] y location of max
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 fpixGetMax(FPIX       *fpix,
            l_float32  *pmaxval,
            l_int32    *pxmaxloc,
@@ -786,12 +792,12 @@ l_float32   maxval;
  * \brief   dpixGetMin()
  *
  * \param[in]    dpix
- * \param[out]   pminval [optional] min value
- * \param[out]   pxminloc [optional] x location of min
- * \param[out]   pyminloc [optional] y location of min
+ * \param[out]   pminval    [optional] min value
+ * \param[out]   pxminloc   [optional] x location of min
+ * \param[out]   pyminloc   [optional] y location of min
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 dpixGetMin(DPIX       *dpix,
            l_float64  *pminval,
            l_int32    *pxminloc,
@@ -839,12 +845,12 @@ l_float64   minval;
  * \brief   dpixGetMax()
  *
  * \param[in]    dpix
- * \param[out]   pmaxval [optional] max value
- * \param[out]   pxmaxloc [optional] x location of max
- * \param[out]   pymaxloc [optional] y location of max
+ * \param[out]   pmaxval    [optional] max value
+ * \param[out]   pxmaxloc   [optional] x location of max
+ * \param[out]   pymaxloc   [optional] y location of max
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 dpixGetMax(DPIX       *dpix,
            l_float64  *pmaxval,
            l_int32    *pxmaxloc,
@@ -894,8 +900,8 @@ l_float64   maxval;
 /*!
  * \brief   fpixScaleByInteger()
  *
- * \param[in]    fpixs low resolution, subsampled
- * \param[in]    factor scaling factor
+ * \param[in]    fpixs     typically low resolution
+ * \param[in]    factor    integer scaling factor
  * \return  fpixd interpolated result, or NULL on error
  *
  * <pre>
@@ -986,8 +992,8 @@ FPIX       *fpixd;
 /*!
  * \brief   dpixScaleByInteger()
  *
- * \param[in]    dpixs low resolution, subsampled
- * \param[in]    factor scaling factor
+ * \param[in]    dpixs     typically low resolution
+ * \param[in]    factor    integer scaling factor
  * \return  dpixd interpolated result, or NULL on error
  *
  * <pre>
@@ -1081,11 +1087,11 @@ DPIX       *dpixd;
 /*!
  * \brief   fpixLinearCombination()
  *
- * \param[in]    fpixd [optional]; this can be null, equal to fpixs1, or
- *                     different from fpixs1
- * \param[in]    fpixs1 can be == to fpixd
+ * \param[in]    fpixd    [optional] this can be null, equal to fpixs1, or
+ *                        different from fpixs1
+ * \param[in]    fpixs1   can be equal to fpixd
  * \param[in]    fpixs2
- * \param[in]    a, b multiplication factors on fpixs1 and fpixs2, rsp.
+ * \param[in]    a, b     multiplication factors on fpixs1 and fpixs2, rsp.
  * \return  fpixd always
  *
  * <pre>
@@ -1147,8 +1153,8 @@ l_float32  *datas, *datad, *lines, *lined;
  * \brief   fpixAddMultConstant()
  *
  * \param[in]    fpix
- * \param[in]    addc  use 0.0 to skip the operation
- * \param[in]    multc use 1.0 to skip the operation
+ * \param[in]    addc     use 0.0 to skip the operation
+ * \param[in]    multc    use 1.0 to skip the operation
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1159,7 +1165,7 @@ l_float32  *datas, *datad, *lines, *lined;
  *          is done first.
  * </pre>
  */
-l_int32
+l_ok
 fpixAddMultConstant(FPIX      *fpix,
                     l_float32  addc,
                     l_float32  multc)
@@ -1200,11 +1206,11 @@ l_float32  *line, *data;
 /*!
  * \brief   dpixLinearCombination()
  *
- * \param[in]    dpixd [optional]; this can be null, equal to dpixs1, or
- *                     different from dpixs1
- * \param[in]    dpixs1 can be == to dpixd
+ * \param[in]    dpixd    [optional] this can be null, equal to dpixs1, or
+ *                        different from dpixs1
+ * \param[in]    dpixs1   can be equal to dpixd
  * \param[in]    dpixs2
- * \param[in]    a, b multiplication factors on dpixs1 and dpixs2, rsp.
+ * \param[in]    a, b     multiplication factors on dpixs1 and dpixs2, rsp.
  * \return  dpixd always
  *
  * <pre>
@@ -1266,8 +1272,8 @@ l_float64  *datas, *datad, *lines, *lined;
  * \brief   dpixAddMultConstant()
  *
  * \param[in]    dpix
- * \param[in]    addc  use 0.0 to skip the operation
- * \param[in]    multc use 1.0 to skip the operation
+ * \param[in]    addc     use 0.0 to skip the operation
+ * \param[in]    multc    use 1.0 to skip the operation
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1278,7 +1284,7 @@ l_float64  *datas, *datad, *lines, *lined;
  *          is done first.
  * </pre>
  */
-l_int32
+l_ok
 dpixAddMultConstant(DPIX      *dpix,
                     l_float64  addc,
                     l_float64  multc)
@@ -1322,10 +1328,10 @@ l_float64  *line, *data;
  * \brief   fpixSetAllArbitrary()
  *
  * \param[in]    fpix
- * \param[in]    inval to set at each pixel
+ * \param[in]    inval    to set at each pixel
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 fpixSetAllArbitrary(FPIX      *fpix,
                     l_float32  inval)
 {
@@ -1353,10 +1359,10 @@ l_float32  *data, *line;
  * \brief   dpixSetAllArbitrary()
  *
  * \param[in]    dpix
- * \param[in]    inval to set at each pixel
+ * \param[in]    inval    to set at each pixel
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 dpixSetAllArbitrary(DPIX      *dpix,
                     l_float64  inval)
 {
@@ -1387,7 +1393,7 @@ l_float64  *data, *line;
  * \brief   fpixAddBorder()
  *
  * \param[in]    fpixs
- * \param[in]    left, right, top, bot pixels on each side to be added
+ * \param[in]    left, right, top, bot     pixels on each side to be added
  * \return  fpixd, or NULL on error
  *
  * <pre>
@@ -1428,7 +1434,7 @@ FPIX    *fpixd;
  * \brief   fpixRemoveBorder()
  *
  * \param[in]    fpixs
- * \param[in]    left, right, top, bot pixels on each side to be removed
+ * \param[in]    left, right, top, bot     pixels on each side to be removed
  * \return  fpixd, or NULL on error
  */
 FPIX *
@@ -1467,7 +1473,7 @@ FPIX    *fpixd;
  * \brief   fpixAddMirroredBorder()
  *
  * \param[in]    fpixs
- * \param[in]    left, right, top, bot pixels on each side to be added
+ * \param[in]    left, right, top, bot      pixels on each side to be added
  * \return  fpixd, or NULL on error
  *
  * <pre>
@@ -1513,7 +1519,7 @@ FPIX    *fpixd;
  * \brief   fpixAddContinuedBorder()
  *
  * \param[in]    fpixs
- * \param[in]    left, right, top, bot pixels on each side to be added
+ * \param[in]    left, right, top, bot     pixels on each side to be added
  * \return  fpixd, or NULL on error
  *
  * <pre>
@@ -1557,7 +1563,7 @@ FPIX    *fpixd;
  * \brief   fpixAddSlopeBorder()
  *
  * \param[in]    fpixs
- * \param[in]    left, right, top, bot pixels on each side to be added
+ * \param[in]    left, right, top, bot     pixels on each side to be added
  * \return  fpixd, or NULL on error
  *
  * <pre>
@@ -1634,14 +1640,14 @@ FPIX      *fpixd;
 /*!
  * \brief   fpixRasterop()
  *
- * \param[in]    fpixd  dest fpix
- * \param[in]    dx     x val of UL corner of dest rectangle
- * \param[in]    dy     y val of UL corner of dest rectangle
- * \param[in]    dw     width of dest rectangle
- * \param[in]    dh     height of dest rectangle
- * \param[in]    fpixs  src fpix
- * \param[in]    sx     x val of UL corner of src rectangle
- * \param[in]    sy     y val of UL corner of src rectangle
+ * \param[in]    fpixd    dest fpix
+ * \param[in]    dx       x val of UL corner of dest rectangle
+ * \param[in]    dy       y val of UL corner of dest rectangle
+ * \param[in]    dw       width of dest rectangle
+ * \param[in]    dh       height of dest rectangle
+ * \param[in]    fpixs    src fpix
+ * \param[in]    sx       x val of UL corner of src rectangle
+ * \param[in]    sy       y val of UL corner of src rectangle
  * \return  0 if OK; 1 on error.
  *
  * <pre>
@@ -1656,7 +1662,7 @@ FPIX      *fpixd;
  *          the result is valid:  use for in-place with caution!
  * </pre>
  */
-l_int32
+l_ok
 fpixRasterop(FPIX    *fpixd,
              l_int32  dx,
              l_int32  dy,
@@ -1753,7 +1759,7 @@ l_float32  *datas, *datad, *lines, *lined;
  * \brief   fpixRotateOrth()
  *
  * \param[in]    fpixs
- * \param[in]    quads 0-3; number of 90 degree cw rotations
+ * \param[in]    quads    0-3; number of 90 degree cw rotations
  * \return  fpixd, or NULL on error
  */
 FPIX *
@@ -1781,8 +1787,8 @@ fpixRotateOrth(FPIX     *fpixs,
 /*!
  * \brief   fpixRotate180()
  *
- * \param[in]    fpixd  [optional]; can be null, equal to fpixs,
- *                      or different from fpixs
+ * \param[in]    fpixd    [optional] can be null, equal to fpixs,
+ *                        or different from fpixs
  * \param[in]    fpixs
  * \return  fpixd, or NULL on error
  *
@@ -1825,7 +1831,7 @@ fpixRotate180(FPIX  *fpixd,
  * \brief   fpixRotate90()
  *
  * \param[in]    fpixs
- * \param[in]    direction 1 = clockwise,  -1 = counter-clockwise
+ * \param[in]    direction     1 = clockwise; -1 = counter-clockwise
  * \return  fpixd, or NULL on error
  *
  * <pre>
@@ -1886,8 +1892,8 @@ FPIX       *fpixd;
 /*!
  * \brief   pixFlipLR()
  *
- * \param[in]    fpixd [optional]; can be null, equal to fpixs,
- *                     or different from fpixs
+ * \param[in]    fpixd    [optional] can be null, equal to fpixs,
+ *                        or different from fpixs
  * \param[in]    fpixs
  * \return  fpixd, or NULL on error
  *
@@ -1947,8 +1953,8 @@ l_float32  *line, *data, *buffer;
 /*!
  * \brief   fpixFlipTB()
  *
- * \param[in]    fpixd [optional]; can be null, equal to fpixs,
- *                     or different from fpixs
+ * \param[in]    fpixd    [optional] can be null, equal to fpixs,
+ *                        or different from fpixs
  * \param[in]    fpixs
  * \return  fpixd, or NULL on error
  *
@@ -2012,11 +2018,11 @@ l_float32  *linet, *lineb, *data, *buffer;
 /*!
  * \brief   fpixAffinePta()
  *
- * \param[in]    fpixs 8 bpp
- * \param[in]    ptad  4 pts of final coordinate space
- * \param[in]    ptas  4 pts of initial coordinate space
- * \param[in]    border size of extension with constant normal derivative
- * \param[in]    inval value brought in; typ. 0
+ * \param[in]    fpixs     8 bpp
+ * \param[in]    ptad      4 pts of final coordinate space
+ * \param[in]    ptas      4 pts of initial coordinate space
+ * \param[in]    border    size of extension with constant normal derivative
+ * \param[in]    inval     value brought in; typ. 0
  * \return  fpixd, or NULL on error
  *
  * <pre>
@@ -2083,9 +2089,9 @@ FPIX       *fpixs2, *fpixd, *fpixd2;
 /*!
  * \brief   fpixAffine()
  *
- * \param[in]    fpixs 8 bpp
- * \param[in]    vc  vector of 8 coefficients for projective transformation
- * \param[in]    inval value brought in; typ. 0
+ * \param[in]    fpixs    8 bpp
+ * \param[in]    vc       vector of 8 coefficients for projective transformation
+ * \param[in]    inval    value brought in; typ. 0
  * \return  fpixd, or NULL on error
  */
 FPIX *
@@ -2131,11 +2137,11 @@ FPIX       *fpixd;
 /*!
  * \brief   fpixProjectivePta()
  *
- * \param[in]    fpixs 8 bpp
- * \param[in]    ptad  4 pts of final coordinate space
- * \param[in]    ptas  4 pts of initial coordinate space
- * \param[in]    border size of extension with constant normal derivative
- * \param[in]    inval value brought in; typ. 0
+ * \param[in]    fpixs     8 bpp
+ * \param[in]    ptad      4 pts of final coordinate space
+ * \param[in]    ptas      4 pts of initial coordinate space
+ * \param[in]    border    size of extension with constant normal derivative
+ * \param[in]    inval     value brought in; typ. 0
  * \return  fpixd, or NULL on error
  *
  * <pre>
@@ -2202,9 +2208,9 @@ FPIX       *fpixs2, *fpixd, *fpixd2;
 /*!
  * \brief   fpixProjective()
  *
- * \param[in]    fpixs 8 bpp
- * \param[in]    vc  vector of 8 coefficients for projective transformation
- * \param[in]    inval value brought in; typ. 0
+ * \param[in]    fpixs     8 bpp
+ * \param[in]    vc        vector of 8 coefficients for projective transform
+ * \param[in]    inval     value brought in; typ. 0
  * \return  fpixd, or NULL on error
  */
 FPIX *
@@ -2250,12 +2256,12 @@ FPIX       *fpixd;
 /*!
  * \brief   linearInterpolatePixelFloat()
  *
- * \param[in]    datas ptr to beginning of float image data
- * \param[in]    w, h of image
- * \param[in]    x, y floating pt location for evaluation
- * \param[in]    inval float value brought in from the outside when the
- *                     input x,y location is outside the image
- * \param[out]   pval interpolated float value
+ * \param[in]    datas     ptr to beginning of float image data
+ * \param[in]    w, h      dimensions of image
+ * \param[in]    x, y      floating pt location for evaluation
+ * \param[in]    inval     float value brought in from the outside when the
+ *                         input x,y location is outside the image
+ * \param[out]   pval      interpolated float value
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -2265,7 +2271,7 @@ FPIX       *fpixd;
  *          avoids "jaggies" when rendering sharp edges.
  * </pre>
  */
-l_int32
+l_ok
 linearInterpolatePixelFloat(l_float32  *datas,
                             l_int32     w,
                             l_int32     h,
@@ -2299,7 +2305,7 @@ l_float32  *lines;
 
 #if  DEBUG
     if (xf < 0 || yf < 0)
-        fprintf(stderr, "xp = %d, yp = %d, xf = %d, yf = %d\n", xp, yp, xf, yf);
+        lept_stderr("xp = %d, yp = %d, xf = %d, yf = %d\n", xp, yp, xf, yf);
 #endif  /* DEBUG */
 
         /* Interpolate by area weighting. */
@@ -2368,9 +2374,9 @@ PIX        *pixd;
 /*!
  * \brief   pixComponentFunction()
  *
- * \param[in]    pix 32 bpp rgb
- * \param[in]    rnum, gnum, bnum coefficients for numerator
- * \param[in]    rdenom, gdenom, bdenom coefficients for denominator
+ * \param[in]    pix                       32 bpp rgb
+ * \param[in]    rnum, gnum, bnum          coefficients for numerator
+ * \param[in]    rdenom, gdenom, bdenom    coefficients for denominator
  * \return  fpixd, or NULL on error
  *
  * <pre>
