@@ -42,6 +42,10 @@
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include <math.h>
 #include "allheaders.h"
@@ -66,12 +70,12 @@ static const l_int32  MAX_JP2K_HEIGHT = 100000;
  *
  * \param[in]    filename
  * \param[out]   pw [optional]
- *           [out]   ph ([optional]
- *           [out]   pbps ([optional]  bits/sample
+ * \param[out]   ph [optional]
+ * \param[out]   pbps [optional]  bits/sample
  * \param[out]   pspp [optional]  samples/pixel
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 readHeaderJp2k(const char *filename,
                l_int32    *pw,
                l_int32    *ph,
@@ -83,10 +87,6 @@ FILE    *fp;
 
     PROCNAME("readHeaderJp2k");
 
-    if (pw) *pw = 0;
-    if (ph) *ph = 0;
-    if (pbps) *pbps = 0;
-    if (pspp) *pspp = 0;
     if (!filename)
         return ERROR_INT("filename not defined", procName, 1);
 
@@ -103,12 +103,12 @@ FILE    *fp;
  *
  * \param[in]    fp file stream opened for read
  * \param[out]   pw [optional]
- *           [out]   ph ([optional]
- *           [out]   pbps ([optional]  bits/sample
+ * \param[out]   ph [optional]
+ * \param[out]   pbps [optional]  bits/sample
  * \param[out]   pspp [optional]  samples/pixel
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 freadHeaderJp2k(FILE     *fp,
                 l_int32  *pw,
                 l_int32  *ph,
@@ -116,14 +116,10 @@ freadHeaderJp2k(FILE     *fp,
                 l_int32  *pspp)
 {
 l_uint8  buf[80];  /* just need the first 80 bytes */
-l_int32  nread;
+l_int32  nread, ret;
 
     PROCNAME("freadHeaderJp2k");
 
-    if (pw) *pw = 0;
-    if (ph) *ph = 0;
-    if (pbps) *pbps = 0;
-    if (pspp) *pspp = 0;
     if (!fp)
         return ERROR_INT("fp not defined", procName, 1);
 
@@ -132,9 +128,9 @@ l_int32  nread;
     if (nread != sizeof(buf))
         return ERROR_INT("read failure", procName, 1);
 
-    readHeaderMemJp2k(buf, sizeof(buf), pw, ph, pbps, pspp);
+    ret = readHeaderMemJp2k(buf, sizeof(buf), pw, ph, pbps, pspp);
     rewind(fp);
-    return 0;
+    return ret;
 }
 
 
@@ -144,8 +140,8 @@ l_int32  nread;
  * \param[in]    data
  * \param[in]    size at least 80
  * \param[out]   pw [optional]
- *           [out]   ph ([optional]
- *           [out]   pbps ([optional]  bits/sample
+ * \param[out]   ph [optional]
+ * \param[out]   pbps [optional]  bits/sample
  * \param[out]   pspp [optional]  samples/pixel
  * \return  0 if OK, 1 on error
  *
@@ -163,7 +159,7 @@ l_int32  nread;
  *               bps:  1 byte   (contains bps - 1)
  * </pre>
  */
-l_int32
+l_ok
 readHeaderMemJp2k(const l_uint8  *data,
                   size_t          size,
                   l_int32        *pw,
@@ -198,6 +194,9 @@ l_uint8  ihdr[4] = {0x69, 0x68, 0x64, 0x72};  /* 'ihdr' */
 #endif  /* DEBUG_IHDR */
 
     windex = loc / 4 + 1;
+    if (4 * (windex + 2) + 2 >= size)
+        return ERROR_INT("image parameters end are outside of header",
+                         procName, 1);
     val = *((l_uint32 *)data + windex);
     h = convertOnLittleEnd32(val);
     val = *((l_uint32 *)data + windex + 1);
@@ -205,12 +204,18 @@ l_uint8  ihdr[4] = {0x69, 0x68, 0x64, 0x72};  /* 'ihdr' */
     val = *((l_uint16 *)data + 2 * (windex + 2));
     spp = convertOnLittleEnd16(val);
     bps = *(data + 4 * (windex + 2) + 2) + 1;
+    if (w < 1 || h < 1)
+        return ERROR_INT("w and h must both be > 0", procName, 1);
     if (w > MAX_JP2K_WIDTH || h > MAX_JP2K_HEIGHT)
         return ERROR_INT("unrealistically large sizes", procName, 1);
+    if (spp != 1 && spp != 3 && spp != 4)
+        return ERROR_INT("spp must be in 1, 3 or 4", procName, 1);
+    if (bps != 8 && bps != 16)
+        return ERROR_INT("bps must be 8 or 16", procName, 1);
     if (pw) *pw = w;
     if (ph) *ph = h;
-    if (pbps) *pbps = bps;
     if (pspp) *pspp = spp;
+    if (pbps) *pbps = bps;
     return 0;
 }
 
@@ -247,7 +252,7 @@ l_uint16   xnum, ynum, xdenom, ydenom;  /* these jp2k fields are 2-byte */
 l_int32    loc, found;
 l_uint8    resc[4] = {0x72, 0x65, 0x73, 0x63};  /* 'resc' */
 size_t     nbytes;
-l_float64  xres, yres;
+l_float64  xres, yres, maxres;
 
     PROCNAME("fgetJp2kResolution");
 
@@ -269,6 +274,11 @@ l_float64  xres, yres;
         LEPT_FREE(data);
         return 1;
     }
+    if (nbytes < 80 || loc >= nbytes - 13) {
+        L_WARNING("image resolution found without enough space\n", procName);
+        LEPT_FREE(data);
+        return 1;
+    }
 
         /* Extract the fields and calculate the resolution in pixels/meter.
          * See section 1.5.3.7.1 of JPEG 2000 ISO/IEC 15444-1 spec.  */
@@ -288,8 +298,15 @@ l_float64  xres, yres;
         /* Convert from pixels/meter to ppi */
     yres *= (300.0 / 11811.0);
     xres *= (300.0 / 11811.0);
-    *pyres = (l_int32)(yres + 0.5);
-    *pxres = (l_int32)(xres + 0.5);
+
+        /* Sanity check for bad data */
+    maxres = 100000.0;  /* ppi */
+    if (xres > maxres || yres > maxres) {
+        L_WARNING("ridiculously large resolution\n", procName);
+    } else {
+        *pyres = (l_int32)(yres + 0.5);
+        *pxres = (l_int32)(xres + 0.5);
+    }
 
     LEPT_FREE(data);
     return 0;
