@@ -45,6 +45,7 @@
  *    Read/write to memory
  *          PIX             *pixReadMemJpeg()
  *          l_int32          readHeaderMemJpeg()
+ *          l_int32          readResolutionMemJpeg()
  *          l_int32          pixWriteMemJpeg()
  *
  *    Setting special flag for chroma sampling on write
@@ -127,7 +128,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config_auto.h"
+#include <config_auto.h>
 #endif  /* HAVE_CONFIG_H */
 
 #include <string.h>
@@ -177,12 +178,12 @@ struct callback_data {
  * \brief   pixReadJpeg()
  *
  * \param[in]    filename
- * \param[in]    cmapflag 0 for no colormap in returned pix;
- *                        1 to return an 8 bpp cmapped pix if spp = 3 or 4
- * \param[in]    reduction scaling factor: 1, 2, 4 or 8
- * \param[out]   pnwarn [optional] number of warnings about
- *                       corrupted data
- * \param[in]    hint a bitwise OR of L_JPEG_* values; 0 for default
+ * \param[in]    cmapflag   0 for no colormap in returned pix;
+ *                          1 to return an 8 bpp cmapped pix if spp = 3 or 4
+ * \param[in]    reduction  scaling factor: 1, 2, 4 or 8
+ * \param[out]   pnwarn     [optional] number of warnings about
+ *                          corrupted data
+ * \param[in]    hint       a bitwise OR of L_JPEG_* values; 0 for default
  * \return  pix, or NULL on error
  *
  * <pre>
@@ -251,12 +252,12 @@ PIX      *pix;
 /*!
  * \brief   pixReadStreamJpeg()
  *
- * \param[in]    fp file stream
- * \param[in]    cmapflag 0 for no colormap in returned pix;
- *                        1 to return an 8 bpp cmapped pix if spp = 3 or 4
- * \param[in]    reduction scaling factor: 1, 2, 4 or 8
- * \param[out]   pnwarn [optional] number of warnings
- * \param[in]    hint a bitwise OR of L_JPEG_* values; 0 for default
+ * \param[in]    fp         file stream
+ * \param[in]    cmapflag   0 for no colormap in returned pix;
+ *                          1 to return an 8 bpp cmapped pix if spp = 3 or 4
+ * \param[in]    reduction  scaling factor: 1, 2, 4 or 8
+ * \param[out]   pnwarn     [optional] number of warnings
+ * \param[in]    hint       a bitwise OR of L_JPEG_* values; 0 for default
  * \return  pix, or NULL on error
  *
  *  Usage: see pixReadJpeg
@@ -332,11 +333,12 @@ jmp_buf                        jmpbuf;  /* must be local to the function */
     ycck = (cinfo.jpeg_color_space == JCS_YCCK && spp == 4 && cmapflag == 0);
     cmyk = (cinfo.jpeg_color_space == JCS_CMYK && spp == 4 && cmapflag == 0);
     if (spp != 1 && spp != 3 && !ycck && !cmyk) {
+        jpeg_destroy_decompress(&cinfo);
         return (PIX *)ERROR_PTR("spp must be 1 or 3, or YCCK or CMYK",
                                 procName, NULL);
     }
     if ((spp == 3 && cmapflag == 0) || ycck || cmyk) {  /* rgb or 4 bpp color */
-        rowbuffer = (JSAMPROW)LEPT_CALLOC(sizeof(JSAMPLE), spp * w);
+        rowbuffer = (JSAMPROW)LEPT_CALLOC(sizeof(JSAMPLE), (size_t)spp * w);
         pix = pixCreate(w, h, 32);
     } else {  /* 8 bpp gray or colormapped */
         rowbuffer = (JSAMPROW)LEPT_CALLOC(sizeof(JSAMPLE), w);
@@ -346,6 +348,7 @@ jmp_buf                        jmpbuf;  /* must be local to the function */
     if (!rowbuffer || !pix) {
         LEPT_FREE(rowbuffer);
         pixDestroy(&pix);
+        jpeg_destroy_decompress(&cinfo);
         return (PIX *)ERROR_PTR("rowbuffer or pix not made", procName, NULL);
     }
 
@@ -468,8 +471,8 @@ jmp_buf                        jmpbuf;  /* must be local to the function */
     }
 
     if (cinfo.output_components != spp)
-        fprintf(stderr, "output spp = %d, spp = %d\n",
-                cinfo.output_components, spp);
+        lept_stderr("output spp = %d, spp = %d\n",
+                    cinfo.output_components, spp);
 
     jpeg_finish_decompress(&cinfo);
     jpeg_destroy_decompress(&cinfo);
@@ -495,14 +498,14 @@ jmp_buf                        jmpbuf;  /* must be local to the function */
  * \brief   readHeaderJpeg()
  *
  * \param[in]    filename
- * \param[out]   pw [optional]
- *           [out]   ph ([optional]
- *           [out]   pspp ([optional]  samples/pixel
- * \param[out]   pycck [optional]  1 if ycck color space; 0 otherwise
- * \param[out]   pcmyk [optional]  1 if cmyk color space; 0 otherwise
+ * \param[out]   pw     [optional]
+ * \param[out]   ph     [optional]
+ * \param[out]   pspp   [optional] samples/pixel
+ * \param[out]   pycck  [optional] 1 if ycck color space; 0 otherwise
+ * \param[out]   pcmyk  [optional] 1 if cmyk color space; 0 otherwise
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 readHeaderJpeg(const char  *filename,
                l_int32     *pw,
                l_int32     *ph,
@@ -536,15 +539,15 @@ FILE    *fp;
 /*!
  * \brief   freadHeaderJpeg()
  *
- * \param[in]    fp file stream
- * \param[out]   pw [optional]
- *           [out]   ph ([optional]
- *           [out]   pspp ([optional]  samples/pixel
- * \param[out]   pycck [optional]  1 if ycck color space; 0 otherwise
- * \param[out]   pcmyk [optional]  1 if cmyk color space; 0 otherwise
+ * \param[in]    fp     file stream
+ * \param[out]   pw     [optional]
+ * \param[out]   ph     [optional]
+ * \param[out]   pspp   [optional]  samples/pixel
+ * \param[out]   pycck  [optional]  1 if ycck color space; 0 otherwise
+ * \param[out]   pcmyk  [optional]  1 if cmyk color space; 0 otherwise
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 freadHeaderJpeg(FILE     *fp,
                 l_int32  *pw,
                 l_int32  *ph,
@@ -552,7 +555,7 @@ freadHeaderJpeg(FILE     *fp,
                 l_int32  *pycck,
                 l_int32  *pcmyk)
 {
-l_int32                        spp;
+l_int32                        spp, w, h;
 struct jpeg_decompress_struct  cinfo;
 struct jpeg_error_mgr          jerr;
 jmp_buf                        jmpbuf;  /* must be local to the function */
@@ -583,8 +586,15 @@ jmp_buf                        jmpbuf;  /* must be local to the function */
     jpeg_stdio_src(&cinfo, fp);
     jpeg_read_header(&cinfo, TRUE);
     jpeg_calc_output_dimensions(&cinfo);
-
     spp = cinfo.out_color_components;
+    w = cinfo.output_width;
+    h = cinfo.output_height;
+    if (w < 1 || h < 1 || spp < 1 || spp > 4) {
+        jpeg_destroy_decompress(&cinfo);
+        rewind(fp);
+        return ERROR_INT("bad jpeg image parameters", procName, 1);
+    }
+
     if (pspp) *pspp = spp;
     if (pw) *pw = cinfo.output_width;
     if (ph) *ph = cinfo.output_height;
@@ -600,16 +610,18 @@ jmp_buf                        jmpbuf;  /* must be local to the function */
 
 
 /*
- *  fgetJpegResolution()
+ * \brief   fgetJpegResolution()
  *
- *      Input:  fp (file stream opened for read)
- *              &xres, &yres (<return> resolution in ppi)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    fp             file stream
+ * \param[out]   pxres, pyres   resolutions
+ * \return   0 if OK; 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) If neither resolution field is set, this is not an error;
  *          the returned resolution values are 0 (designating 'unknown').
  *      (2) Side-effect: this rewinds the stream.
+ * </pre>
  */
 l_int32
 fgetJpegResolution(FILE     *fp,
@@ -660,14 +672,16 @@ jmp_buf                        jmpbuf;  /* must be local to the function */
 
 
 /*
- *  fgetJpegComment()
+ * \brief   fgetJpegComment()
  *
- *      Input:  fp (file stream opened for read)
- *              &comment (<return> comment)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    fp        file stream opened for read
+ * \param[out]   pcomment  comment
+ * \return   0 if OK; 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Side-effect: this rewinds the stream.
+ * </pre>
  */
 l_int32
 fgetJpegComment(FILE      *fp,
@@ -718,12 +732,12 @@ struct callback_data           cb_data;  /* contains local jmp_buf */
  * \brief   pixWriteJpeg()
  *
  * \param[in]    filename
- * \param[in]    pix  any depth; cmap is OK
- * \param[in]    quality 1 - 100; 75 is default
- * \param[in]    progressive 0 for baseline sequential; 1 for progressive
+ * \param[in]    pix           any depth; cmap is OK
+ * \param[in]    quality       1 - 100; 75 is default
+ * \param[in]    progressive   0 for baseline sequential; 1 for progressive
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 pixWriteJpeg(const char  *filename,
              PIX         *pix,
              l_int32      quality,
@@ -754,10 +768,10 @@ FILE  *fp;
 /*!
  * \brief   pixWriteStreamJpeg()
  *
- * \param[in]    fp file stream
- * \param[in]    pixs  any depth; cmap is OK
- * \param[in]    quality  1 - 100; 75 is default value; 0 is also default
- * \param[in]    progressive 0 for baseline sequential; 1 for progressive
+ * \param[in]    fp           file stream
+ * \param[in]    pixs         any depth; cmap is OK
+ * \param[in]    quality      1 - 100; 75 is default value; 0 is also default
+ * \param[in]    progressive  0 for baseline sequential; 1 for progressive
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -784,7 +798,7 @@ FILE  *fp;
  *          for luminosity and a lower resolution one for the chromas.
  * </pre>
  */
-l_int32
+l_ok
 pixWriteStreamJpeg(FILE    *fp,
                    PIX     *pixs,
                    l_int32  quality,
@@ -807,8 +821,11 @@ jmp_buf                      jmpbuf;  /* must be local to the function */
         return ERROR_INT("stream not open", procName, 1);
     if (!pixs)
         return ERROR_INT("pixs not defined", procName, 1);
-    if (quality <= 0)
-        quality = 75;  /* default */
+    if (quality <= 0) quality = 75;  /* default */
+    if (quality > 100) {
+        L_ERROR("invalid jpeg quality; setting to 75\n", procName);
+        quality = 75;
+    }
 
         /* If necessary, convert the pix so that it can be jpeg compressed.
          * The colormap is removed based on the source, so if the colormap
@@ -906,8 +923,8 @@ jmp_buf                      jmpbuf;  /* must be local to the function */
          * Just to be safe, subtract 100 to cover the Adobe name space.  */
     if ((text = pixGetText(pix)) != NULL) {
         if (strlen(text) > 65433) {
-            L_WARNING("text is %lu bytes; clipping to 65433\n",
-                   procName, (unsigned long)strlen(text));
+            L_WARNING("text is %zu bytes; clipping to 65433\n",
+                   procName, strlen(text));
             text[65433] = '\0';
         }
         jpeg_write_marker(&cinfo, JPEG_COM, (const JOCTET *)text, strlen(text));
@@ -961,14 +978,14 @@ jmp_buf                      jmpbuf;  /* must be local to the function */
 /*!
  * \brief   pixReadMemJpeg()
  *
- * \param[in]    data const; jpeg-encoded
- * \param[in]    size of data
- * \param[in]    cmflag colormap flag 0 means return RGB image if color;
- *                      1 means create a colormap and return
- *                      an 8 bpp colormapped image if color
- * \param[in]    reduction scaling factor: 1, 2, 4 or 8
- * \param[out]   pnwarn [optional] number of warnings
- * \param[in]    hint a bitwise OR of L_JPEG_* values; 0 for default
+ * \param[in]    data       const; jpeg-encoded
+ * \param[in]    size       of data
+ * \param[in]    cmflag     colormap flag 0 means return RGB image if color;
+ *                          1 means create a colormap and return
+ *                          an 8 bpp colormapped image if color
+ * \param[in]    reduction  scaling factor: 1, 2, 4 or 8
+ * \param[out]   pnwarn     [optional] number of warnings
+ * \param[in]    hint       a bitwise OR of L_JPEG_* values; 0 for default
  * \return  pix, or NULL on error
  *
  * <pre>
@@ -1017,16 +1034,16 @@ PIX      *pix;
 /*!
  * \brief   readHeaderMemJpeg()
  *
- * \param[in]    data const; jpeg-encoded
- * \param[in]    size of data
- * \param[out]   pw [optional]
- *           [out]   ph ([optional]
- *           [out]   pspp ([optional]  samples/pixel
- * \param[out]   pycck [optional]  1 if ycck color space; 0 otherwise
- * \param[out]   pcmyk [optional]  1 if cmyk color space; 0 otherwise
+ * \param[in]    data    const; jpeg-encoded
+ * \param[in]    size    of data
+ * \param[out]   pw      [optional] width
+ * \param[out]   ph      [optional] height
+ * \param[out]   pspp    [optional] samples/pixel
+ * \param[out]   pycck   [optional] 1 if ycck color space; 0 otherwise
+ * \param[out]   pcmyk   [optional] 1 if cmyk color space; 0 otherwise
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 readHeaderMemJpeg(const l_uint8  *data,
                   size_t          size,
                   l_int32        *pw,
@@ -1058,14 +1075,49 @@ FILE    *fp;
 }
 
 
+/*!
+ * \brief   readResolutionMemJpeg()
+ *
+ * \param[in]   data    const; jpeg-encoded
+ * \param[in]   size    of data
+ * \param[out]  pxres   [optional]
+ * \param[out]  pyres   [optional]
+ * \return  0 if OK, 1 on error
+ */
+l_ok
+readResolutionMemJpeg(const l_uint8  *data,
+                      size_t          size,
+                      l_int32        *pxres,
+                      l_int32        *pyres)
+{
+l_int32  ret;
+FILE    *fp;
+
+    PROCNAME("readResolutionMemJpeg");
+
+    if (pxres) *pxres = 0;
+    if (pyres) *pyres = 0;
+    if (!data)
+        return ERROR_INT("data not defined", procName, 1);
+    if (!pxres && !pyres)
+        return ERROR_INT("no results requested", procName, 1);
+
+    if ((fp = fopenReadFromMemory(data, size)) == NULL)
+        return ERROR_INT("stream not opened", procName, 1);
+    ret = fgetJpegResolution(fp, pxres, pyres);
+    fclose(fp);
+    return ret;
+}
+
+
 /*!
  * \brief   pixWriteMemJpeg()
  *
- * \param[out]   pdata data of jpeg compressed image
- * \param[out]   psize size of returned data
- * \param[in]    pix  any depth; cmap is OK
- * \param[in]    quality  1 - 100; 75 is default value; 0 is also default
- * \param[in]    progressive 0 for baseline sequential; 1 for progressive
+ * \param[out]   pdata        data of jpeg compressed image
+ * \param[out]   psize        size of returned data
+ * \param[in]    pix          any depth; cmap is OK
+ * \param[in]    quality      1 - 100; 75 is default value; 0 is also default
+ * \param[in]    progressive  0 for baseline sequential; 1 for progressive
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1074,7 +1126,7 @@ FILE    *fp;
  *          memory instead of to a file stream.
  * </pre>
  */
-l_int32
+l_ok
 pixWriteMemJpeg(l_uint8  **pdata,
                 size_t    *psize,
                 PIX       *pix,
@@ -1124,7 +1176,7 @@ FILE    *fp;
  * \brief   pixSetChromaSampling()
  *
  * \param[in]    pix
- * \param[in]    sampling 1 for subsampling; 0 for no subsampling
+ * \param[in]    sampling    1 for subsampling; 0 for no subsampling
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1135,7 +1187,7 @@ FILE    *fp;
  *          jpeg writing, call this with %sampling == 0.
  * </pre>
  */
-l_int32
+l_ok
 pixSetChromaSampling(PIX     *pix,
                      l_int32  sampling)
 {
@@ -1239,6 +1291,10 @@ struct callback_data  *pcb_data;
 
         /* Save the comment and return */
     pcb_data = (struct callback_data *)cinfo->client_data;
+    if (pcb_data->comment) {  /* clear before overwriting previous comment */
+        LEPT_FREE(pcb_data->comment);
+        pcb_data->comment = NULL;
+    }
     pcb_data->comment = comment;
     return 1;
 }
