@@ -80,10 +80,16 @@
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include <math.h>
 #include "allheaders.h"
 
+    /* Array size must be > 0 and not larger than this */
+static const l_uint32  MaxArraySize = 100000;
 
 /*------------------------------------------------------------------------*
  *                           Create / Destroy                             *
@@ -99,25 +105,37 @@
  *      (1) kernelCreate() initializes all values to 0.
  *      (2) After this call, (cy,cx) and nonzero data values must be
  *          assigned.
+ *      (2) The number of kernel elements must be less than 2^29.
  * </pre>
  */
 L_KERNEL *
 kernelCreate(l_int32  height,
              l_int32  width)
 {
+l_uint64   size64;
 L_KERNEL  *kel;
 
     PROCNAME("kernelCreate");
 
-    if ((kel = (L_KERNEL *)LEPT_CALLOC(1, sizeof(L_KERNEL))) == NULL)
-        return (L_KERNEL *)ERROR_PTR("kel not made", procName, NULL);
+    if (width <= 0)
+        return (L_KERNEL *)ERROR_PTR("width must be > 0", procName, NULL);
+    if (height <= 0)
+        return (L_KERNEL *)ERROR_PTR("height must be > 0", procName, NULL);
+
+        /* Avoid overflow in malloc arg */
+    size64 = (l_uint64)width * (l_uint64)height;
+    if (size64 >= (1LL << 29)) {
+        L_ERROR("requested width = %d, height = %d\n", procName, width, height);
+        return (L_KERNEL *)ERROR_PTR("size >= 2^29", procName, NULL);
+    }
+
+    kel = (L_KERNEL *)LEPT_CALLOC(1, sizeof(L_KERNEL));
     kel->sy = height;
     kel->sx = width;
     if ((kel->data = create2dFloatArray(height, width)) == NULL) {
         LEPT_FREE(kel);
         return (L_KERNEL *)ERROR_PTR("data not allocated", procName, NULL);
     }
-
     return kel;
 }
 
@@ -125,7 +143,7 @@ L_KERNEL  *kel;
 /*!
  * \brief   kernelDestroy()
  *
- * \param[in,out]   pkel to be nulled
+ * \param[in,out]   pkel    will be set to null before returning
  * \return  void
  */
 void
@@ -156,8 +174,8 @@ L_KERNEL  *kel;
 /*!
  * \brief   kernelCopy()
  *
- * \param[in]    kels source kernel
- * \return  keld copy of kels, or NULL on error
+ * \param[in]    kels    source kernel
+ * \return  keld   copy of kels, or NULL on error
  */
 L_KERNEL *
 kernelCopy(L_KERNEL  *kels)
@@ -195,7 +213,7 @@ L_KERNEL  *keld;
  * \param[out]   pval
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 kernelGetElement(L_KERNEL   *kel,
                  l_int32     row,
                  l_int32     col,
@@ -227,7 +245,7 @@ kernelGetElement(L_KERNEL   *kel,
  * \param[in]    val
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 kernelSetElement(L_KERNEL  *kel,
                  l_int32    row,
                  l_int32    col,
@@ -250,11 +268,11 @@ kernelSetElement(L_KERNEL  *kel,
 /*!
  * \brief   kernelGetParameters()
  *
- * \param[in]    kel  kernel
- * \param[out]   psy, psx, pcy, pcx [optional]  each can be null
+ * \param[in]    kel                  kernel
+ * \param[out]   psy, psx, pcy, pcx   [optional] each can be null
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 kernelGetParameters(L_KERNEL  *kel,
                     l_int32   *psy,
                     l_int32   *psx,
@@ -280,11 +298,11 @@ kernelGetParameters(L_KERNEL  *kel,
 /*!
  * \brief   kernelSetOrigin()
  *
- * \param[in]    kel  kernel
+ * \param[in]    kel       kernel
  * \param[in]    cy, cx
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 kernelSetOrigin(L_KERNEL  *kel,
                 l_int32    cy,
                 l_int32    cx)
@@ -302,11 +320,11 @@ kernelSetOrigin(L_KERNEL  *kel,
 /*!
  * \brief   kernelGetSum()
  *
- * \param[in]    kel  kernel
- * \param[out]   psum sum of all kernel values
+ * \param[in]    kel      kernel
+ * \param[out]   psum     sum of all kernel values
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 kernelGetSum(L_KERNEL   *kel,
              l_float32  *psum)
 {
@@ -333,12 +351,12 @@ l_int32    sx, sy, i, j;
 /*!
  * \brief   kernelGetMinMax()
  *
- * \param[in]    kel  kernel
- * \param[out]   pmin [optional] minimum value
- * \param[out]   pmax [optional] maximum value
+ * \param[in]    kel      kernel
+ * \param[out]   pmin     [optional] minimum value
+ * \param[out]   pmax     [optional] maximum value
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 kernelGetMinMax(L_KERNEL   *kel,
                 l_float32  *pmin,
                 l_float32  *pmax)
@@ -382,10 +400,10 @@ l_float32  val, minval, maxval;
 /*!
  * \brief   kernelNormalize()
  *
- * \param[in]    kels source kel, to be normalized
- * \param[in]    normsum desired sum of elements in keld
- * \return  keld normalized version of kels, or NULL on error
- *                   or if sum of elements is very close to 0)
+ * \param[in]    kels      source kel, to be normalized
+ * \param[in]    normsum   desired sum of elements in keld
+ * \return  keld   normalized version of kels, or NULL on error
+ *                 or if sum of elements is very close to 0)
  *
  * <pre>
  * Notes:
@@ -431,8 +449,8 @@ L_KERNEL  *keld;
 /*!
  * \brief   kernelInvert()
  *
- * \param[in]    kels source kel, to be inverted
- * \return  keld spatially inverted, about the origin, or NULL on error
+ * \param[in]    kels   source kel, to be inverted
+ * \return  keld   spatially inverted, about the origin, or NULL on error
  *
  * <pre>
  * Notes:
@@ -471,15 +489,16 @@ L_KERNEL  *keld;
 /*!
  * \brief   create2dFloatArray()
  *
- * \param[in]    sy rows == height
- * \param[in]    sx columns == width
+ * \param[in]    sy   rows == height
+ * \param[in]    sx   columns == width
  * \return  doubly indexed array i.e., an array of sy row pointers,
  *              each of which points to an array of sx floats
  *
  * <pre>
  * Notes:
- *      (1) The array[sy][sx] is indexed in standard "matrix notation",
+ *      (1) The array[%sy][%sx] is indexed in standard "matrix notation",
  *          with the row index first.
+ *      (2) The caller kernelCreate() limits the size to < 2^29 pixels.
  * </pre>
  */
 l_float32 **
@@ -491,9 +510,13 @@ l_float32  **array;
 
     PROCNAME("create2dFloatArray");
 
+    if (sx <= 0 || sx > MaxArraySize)
+        return (l_float32 **)ERROR_PTR("sx out of bounds", procName, NULL);
+    if (sy <= 0 || sy > MaxArraySize)
+        return (l_float32 **)ERROR_PTR("sy out of bounds", procName, NULL);
+
     if ((array = (l_float32 **)LEPT_CALLOC(sy, sizeof(l_float32 *))) == NULL)
         return (l_float32 **)ERROR_PTR("ptr array not made", procName, NULL);
-
     for (i = 0; i < sy; i++)
         array[i] = (l_float32 *)LEPT_CALLOC(sx, sizeof(l_float32));
     return array;
@@ -506,7 +529,7 @@ l_float32  **array;
 /*!
  * \brief   kernelRead()
  *
- * \param[in]    fname filename
+ * \param[in]    fname    filename
  * \return  kernel, or NULL on error
  */
 L_KERNEL *
@@ -535,7 +558,7 @@ L_KERNEL  *kel;
 /*!
  * \brief   kernelReadStream()
  *
- * \param[in]    fp file stream
+ * \param[in]    fp    file stream
  * \return  kernel, or NULL on error
  */
 L_KERNEL *
@@ -558,7 +581,10 @@ L_KERNEL  *kel;
     if (fscanf(fp, "  sy = %d, sx = %d, cy = %d, cx = %d\n",
             &sy, &sx, &cy, &cx) != 4)
         return (L_KERNEL *)ERROR_PTR("dimensions not read", procName, NULL);
-
+    if (sx > MaxArraySize || sy > MaxArraySize) {
+        L_ERROR("sx = %d or sy = %d > %d\n", procName, sx, sy, MaxArraySize);
+        return NULL;
+    }
     if ((kel = kernelCreate(sy, sx)) == NULL)
         return (L_KERNEL *)ERROR_PTR("kel not made", procName, NULL);
     kernelSetOrigin(kel, cy, cx);
@@ -577,11 +603,11 @@ L_KERNEL  *kel;
 /*!
  * \brief   kernelWrite()
  *
- * \param[in]    fname output file
- * \param[in]    kel kernel
+ * \param[in]    fname    output file
+ * \param[in]    kel      kernel
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 kernelWrite(const char  *fname,
             L_KERNEL    *kel)
 {
@@ -606,11 +632,11 @@ FILE  *fp;
 /*!
  * \brief   kernelWriteStream()
  *
- * \param[in]    fp file stream
+ * \param[in]    fp    file stream
  * \param[in]    kel
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 kernelWriteStream(FILE      *fp,
                   L_KERNEL  *kel)
 {
@@ -692,7 +718,7 @@ NUMA      *na;
     if (n != w * h) {
         kernelDestroy(&kel);
         numaDestroy(&na);
-        fprintf(stderr, "w = %d, h = %d, num ints = %d\n", w, h, n);
+        lept_stderr("w = %d, h = %d, num ints = %d\n", w, h, n);
         return (L_KERNEL *)ERROR_PTR("invalid integer data", procName, NULL);
     }
 
@@ -790,6 +816,11 @@ L_KERNEL  *kel;
         sarrayDestroy(&sa);
         return (L_KERNEL *)ERROR_PTR("error reading h,w", procName, NULL);
     }
+    if (h > MaxArraySize || w > MaxArraySize) {
+        L_ERROR("h = %d or w = %d > %d\n", procName, h, w, MaxArraySize);
+        sarrayDestroy(&sa);
+        return NULL;
+    }
     line = sarrayGetString(sa, first + 1, L_NOCOPY);
     if (sscanf(line, "%d %d", &cy, &cx) != 2) {
         sarrayDestroy(&sa);
@@ -813,7 +844,7 @@ L_KERNEL  *kel;
     n = numaGetCount(na);
     if (n != w * h) {
         numaDestroy(&na);
-        fprintf(stderr, "w = %d, h = %d, num ints = %d\n", w, h, n);
+        lept_stderr("w = %d, h = %d, num ints = %d\n", w, h, n);
         return (L_KERNEL *)ERROR_PTR("invalid integer data", procName, NULL);
     }
 
@@ -840,7 +871,7 @@ L_KERNEL  *kel;
  * \brief   kernelCreateFromPix()
  *
  * \param[in]    pix
- * \param[in]    cy, cx origin of kernel
+ * \param[in]    cy, cx    origin of kernel
  * \return  kernel, or NULL on error
  *
  * <pre>
@@ -886,12 +917,12 @@ L_KERNEL  *kel;
 /*!
  * \brief   kernelDisplayInPix()
  *
- * \param[in]    kel kernel
- * \param[in]    size of grid interiors; odd; either 1 or a minimum size
- *                    of 17 is enforced
- * \param[in]    gthick grid thickness; either 0 or a minimum size of 2
- *                      is enforced
- * \return  pix display of kernel, or NULL on error
+ * \param[in]    kel       kernel
+ * \param[in]    size      of grid interiors; odd; either 1 or a minimum size
+ *                         of 17 is enforced
+ * \param[in]    gthick    grid thickness; either 0 or a minimum size of 2
+ *                         is enforced
+ * \return  pix   display of kernel, or NULL on error
  *
  * <pre>
  * Notes:
@@ -1014,9 +1045,9 @@ PIX       *pixd, *pixt0, *pixt1;
 /*!
  * \brief   parseStringForNumbers()
  *
- * \param[in]    str string containing numbers; not changed
- * \param[in]    seps string of characters that can be used between ints
- * \return  numa of numbers found, or NULL on error
+ * \param[in]    str     string containing numbers; not changed
+ * \param[in]    seps    string of characters that can be used between ints
+ * \return  numa   of numbers found, or NULL on error
  *
  * <pre>
  * Notes:
@@ -1061,7 +1092,7 @@ NUMA      *na;
  * \brief   makeFlatKernel()
  *
  * \param[in]    height, width
- * \param[in]    cy, cx origin of kernel
+ * \param[in]    cy, cx          origin of kernel
  * \return  kernel, or NULL on error
  *
  * <pre>
@@ -1070,8 +1101,8 @@ NUMA      *na;
  *          in the block convolution functions.
  *      (2) The kernel origin (%cy, %cx) is typically placed as near
  *          the center of the kernel as possible.  If height and
- *          width are odd, then using cy = height / 2 and
- *          cx = width / 2 places the origin at the exact center.
+ *          width are odd, then using %cy = height / 2 and
+ *          %cx = width / 2 places the origin at the exact center.
  *      (3) This returns a normalized kernel.
  * </pre>
  */
@@ -1104,16 +1135,17 @@ L_KERNEL  *kel;
 /*!
  * \brief   makeGaussianKernel()
  *
- * \param[in]    halfheight, halfwidth sx = 2 * halfwidth + 1, etc
- * \param[in]    stdev standard deviation
- * \param[in]    max value at (cx,cy)
+ * \param[in]    halfh     sy = 2 * halfh + 1
+ * \param[in]    halfw     sx = 2 * halfw + 1
+ * \param[in]    stdev     standard deviation
+ * \param[in]    max       value at (cx,cy)
  * \return  kernel, or NULL on error
  *
  * <pre>
  * Notes:
- *      (1) The kernel size (sx, sy) = (2 * halfwidth + 1, 2 * halfheight + 1).
- *      (2) The kernel center (cx, cy) = (halfwidth, halfheight).
- *      (3) The halfwidth and halfheight are typically equal, and
+ *      (1) The kernel size (sx, sy) = (2 * %halfw + 1, 2 * %halfh + 1)
+ *      (2) The kernel center (cx, cy) = (%halfw, %halfh).
+ *      (3) %halfw and %halfh are typically equal, and
  *          are typically several times larger than the standard deviation.
  *      (4) If pixConvolve() is invoked with normalization (the sum of
  *          kernel elements = 1.0), use 1.0 for max (or any number that's
@@ -1121,8 +1153,8 @@ L_KERNEL  *kel;
  * </pre>
  */
 L_KERNEL *
-makeGaussianKernel(l_int32    halfheight,
-                   l_int32    halfwidth,
+makeGaussianKernel(l_int32    halfh,
+                   l_int32    halfw,
                    l_float32  stdev,
                    l_float32  max)
 {
@@ -1132,15 +1164,15 @@ L_KERNEL  *kel;
 
     PROCNAME("makeGaussianKernel");
 
-    sx = 2 * halfwidth + 1;
-    sy = 2 * halfheight + 1;
+    sx = 2 * halfw + 1;
+    sy = 2 * halfh + 1;
     if ((kel = kernelCreate(sy, sx)) == NULL)
         return (L_KERNEL *)ERROR_PTR("kel not made", procName, NULL);
-    kernelSetOrigin(kel, halfheight, halfwidth);
+    kernelSetOrigin(kel, halfh, halfw);
     for (i = 0; i < sy; i++) {
         for (j = 0; j < sx; j++) {
-            val = expf(-(l_float32)((i - halfheight) * (i - halfheight) +
-                                    (j - halfwidth) * (j - halfwidth)) /
+            val = expf(-(l_float32)((i - halfh) * (i - halfh) +
+                                    (j - halfw) * (j - halfw)) /
                         (2. * stdev * stdev));
             kernelSetElement(kel, i, j, max * val);
         }
@@ -1153,11 +1185,12 @@ L_KERNEL  *kel;
 /*!
  * \brief   makeGaussianKernelSep()
  *
- * \param[in]    halfheight, halfwidth sx = 2 * halfwidth + 1, etc
- * \param[in]    stdev standard deviation
- * \param[in]    max value at (cx,cy)
- * \param[out]   pkelx x part of kernel
- * \param[out]   pkely y part of kernel
+ * \param[in]    halfh     sy = 2 * halfh + 1
+ * \param[in]    halfw     sx = 2 * halfw + 1
+ * \param[in]    stdev     standard deviation
+ * \param[in]    max       value at (cx,cy)
+ * \param[out]   pkelx     x part of kernel
+ * \param[out]   pkely     y part of kernel
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1167,16 +1200,16 @@ L_KERNEL  *kel;
  *          normalized and un-normalized convolution will be the same
  *          as when convolving with pixConvolve() using the full kernel.
  *      (3) The trick for the un-normalized convolution is to have the
- *          product of the two kernel elemets at (cx,cy) be equal to max,
- *          not max**2.  That's why the max for kely is 1.0.  If instead
- *          we use sqrt(max) for both, the results are slightly less
+ *          product of the two kernel elemets at (cx,cy) be equal to %max,
+ *          not max**2.  That's why %max for kely is 1.0.  If instead
+ *          we use sqrt(%max) for both, the results are slightly less
  *          accurate, when compared to using the full kernel in
  *          makeGaussianKernel().
  * </pre>
  */
-l_int32
-makeGaussianKernelSep(l_int32    halfheight,
-                      l_int32    halfwidth,
+l_ok
+makeGaussianKernelSep(l_int32    halfh,
+                      l_int32    halfw,
                       l_float32  stdev,
                       l_float32  max,
                       L_KERNEL **pkelx,
@@ -1187,8 +1220,8 @@ makeGaussianKernelSep(l_int32    halfheight,
     if (!pkelx || !pkely)
         return ERROR_INT("&kelx and &kely not defined", procName, 1);
 
-    *pkelx = makeGaussianKernel(0, halfwidth, stdev, max);
-    *pkely = makeGaussianKernel(halfheight, 0, stdev, 1.0);
+    *pkelx = makeGaussianKernel(0, halfw, stdev, max);
+    *pkely = makeGaussianKernel(halfh, 0, stdev, 1.0);
     return 0;
 }
 
@@ -1196,9 +1229,10 @@ makeGaussianKernelSep(l_int32    halfheight,
 /*!
  * \brief   makeDoGKernel()
  *
- * \param[in]    halfheight, halfwidth sx = 2 * halfwidth + 1, etc
- * \param[in]    stdev standard deviation of narrower gaussian
- * \param[in]    ratio of stdev for wide filter to stdev for narrow one
+ * \param[in]    halfh     sy = 2 * halfh + 1
+ * \param[in]    halfw     sx = 2 * halfw + 1
+ * \param[in]    stdev     standard deviation of narrower gaussian
+ * \param[in]    ratio     of stdev for wide filter to stdev for narrow one
  * \return  kernel, or NULL on error
  *
  * <pre>
@@ -1209,19 +1243,19 @@ makeGaussianKernelSep(l_int32    halfheight,
  *          frequencies passed by the narrow gaussian but stopped
  *          by the wide one.See:
  *               http://en.wikipedia.org/wiki/Difference_of_Gaussians
- *      (2) The kernel size (sx, sy) = (2 * halfwidth + 1, 2 * halfheight + 1).
- *      (3) The kernel center (cx, cy) = (halfwidth, halfheight).
- *      (4) The halfwidth and halfheight are typically equal, and
- *          are typically several times larger than the standard deviation.
- *      (5) The ratio is the ratio of standard deviations of the wide
+ *      (2) The kernel size (sx, sy) = (2 * halfw + 1, 2 * halfh + 1).
+ *      (3) The kernel center (cx, cy) = (halfw, halfh).
+ *      (4) %halfw and %halfh are typically equal, and are typically
+ *          several times larger than the standard deviation.
+ *      (5) %ratio is the ratio of standard deviations of the wide
  *          to narrow gaussian.  It must be >= 1.0; 1.0 is a no-op.
  *      (6) Because the kernel is a null sum, it must be invoked without
  *          normalization in pixConvolve().
  * </pre>
  */
 L_KERNEL *
-makeDoGKernel(l_int32    halfheight,
-              l_int32    halfwidth,
+makeDoGKernel(l_int32    halfh,
+              l_int32    halfw,
               l_float32  stdev,
               l_float32  ratio)
 {
@@ -1231,17 +1265,17 @@ L_KERNEL  *kel;
 
     PROCNAME("makeDoGKernel");
 
-    sx = 2 * halfwidth + 1;
-    sy = 2 * halfheight + 1;
+    sx = 2 * halfw + 1;
+    sy = 2 * halfh + 1;
     if ((kel = kernelCreate(sy, sx)) == NULL)
         return (L_KERNEL *)ERROR_PTR("kel not made", procName, NULL);
-    kernelSetOrigin(kel, halfheight, halfwidth);
+    kernelSetOrigin(kel, halfh, halfw);
 
     pi = 3.1415926535;
     for (i = 0; i < sy; i++) {
         for (j = 0; j < sx; j++) {
-            squaredist = (l_float32)((i - halfheight) * (i - halfheight) +
-                                     (j - halfwidth) * (j - halfwidth));
+            squaredist = (l_float32)((i - halfh) * (i - halfh) +
+                                     (j - halfw) * (j - halfw));
             highnorm = 1. / (2 * stdev * stdev);
             lownorm = highnorm / (ratio * ratio);
             val = (highnorm / pi) * expf(-(highnorm * squaredist))
