@@ -162,6 +162,10 @@
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <math.h>
 #include "allheaders.h"
 
@@ -185,9 +189,9 @@ static PIX * processMorphArgs2(PIX *pixd, PIX *pixs, SEL *sel);
 /*!
  * \brief   pixDilate()
  *
- * \param[in]    pixd  [optional]; this can be null, equal to pixs,
- *                     or different from pixs
- * \param[in]    pixs 1 bpp
+ * \param[in]    pixd    [optional]; this can be null, equal to pixs,
+ *                       or different from pixs
+ * \param[in]    pixs    1 bpp
  * \param[in]    sel
  * \return  pixd
  *
@@ -239,9 +243,9 @@ PIX     *pixt;
 /*!
  * \brief   pixErode()
  *
- * \param[in]    pixd  [optional]; this can be null, equal to pixs,
- *                     or different from pixs
- * \param[in]    pixs 1 bpp
+ * \param[in]    pixd    [optional]; this can be null, equal to pixs,
+ *                       or different from pixs
+ * \param[in]    pixs    1 bpp
  * \param[in]    sel
  * \return  pixd
  *
@@ -312,9 +316,9 @@ PIX     *pixt;
 /*!
  * \brief   pixHMT()
  *
- * \param[in]    pixd [optional]; this can be null, equal to pixs,
- *                    or different from pixs
- * \param[in]    pixs 1 bpp
+ * \param[in]    pixd   [optional]; this can be null, equal to pixs,
+ *                      or different from pixs
+ * \param[in]    pixs   1 bpp
  * \param[in]    sel
  * \return  pixd
  *
@@ -398,9 +402,9 @@ PIX     *pixt;
 /*!
  * \brief   pixOpen()
  *
- * \param[in]    pixd  [optional]; this can be null, equal to pixs,
- *                     or different from pixs
- * \param[in]    pixs 1 bpp
+ * \param[in]    pixd    [optional]; this can be null, equal to pixs,
+ *                       or different from pixs
+ * \param[in]    pixs    1 bpp
  * \param[in]    sel
  * \return  pixd
  *
@@ -489,9 +493,9 @@ PIX  *pixt;
 /*!
  * \brief   pixCloseSafe()
  *
- * \param[in]    pixd [optional]; this can be null, equal to pixs,
- *                    or different from pixs
- * \param[in]    pixs 1 bpp
+ * \param[in]    pixd   [optional]; this can be null, equal to pixs,
+ *                      or different from pixs
+ * \param[in]    pixs   1 bpp
  * \param[in]    sel
  * \return  pixd
  *
@@ -560,9 +564,9 @@ PIX     *pixt1, *pixt2;
 /*!
  * \brief   pixOpenGeneralized()
  *
- * \param[in]    pixd [optional]; this can be null, equal to pixs,
- *                    or different from pixs
- * \param[in]    pixs 1 bpp
+ * \param[in]    pixd   [optional]; this can be null, equal to pixs,
+ *                      or different from pixs
+ * \param[in]    pixs   1 bpp
  * \param[in]    sel
  * \return  pixd
  *
@@ -606,9 +610,9 @@ PIX  *pixt;
 /*!
  * \brief   pixCloseGeneralized()
  *
- * \param[in]    pixd [optional]; this can be null, equal to pixs,
- *                    or different from pixs
- * \param[in]    pixs 1 bpp
+ * \param[in]    pixd   [optional]; this can be null, equal to pixs,
+ *                      or different from pixs
+ * \param[in]    pixs   1 bpp
  * \param[in]    sel
  * \return  pixd
  *
@@ -657,11 +661,11 @@ PIX  *pixt;
 /*!
  * \brief   pixDilateBrick()
  *
- * \param[in]    pixd  [optional]; this can be null, equal to pixs,
- *                     or different from pixs
- * \param[in]    pixs 1 bpp
- * \param[in]    hsize width of brick Sel
- * \param[in]    vsize height of brick Sel
+ * \param[in]    pixd    [optional]; this can be null, equal to pixs,
+ *                       or different from pixs
+ * \param[in]    pixs    1 bpp
+ * \param[in]    hsize   width of brick Sel
+ * \param[in]    vsize   height of brick Sel
  * \return  pixd
  *
  * <pre>
@@ -702,11 +706,17 @@ SEL  *sel, *selh, *selv;
         return pixCopy(pixd, pixs);
     if (hsize == 1 || vsize == 1) {  /* no intermediate result */
         sel = selCreateBrick(vsize, hsize, vsize / 2, hsize / 2, SEL_HIT);
+        if (!sel)
+            return (PIX *)ERROR_PTR("sel not made", procName, pixd);
         pixd = pixDilate(pixd, pixs, sel);
         selDestroy(&sel);
     } else {
-        selh = selCreateBrick(1, hsize, 0, hsize / 2, SEL_HIT);
-        selv = selCreateBrick(vsize, 1, vsize / 2, 0, SEL_HIT);
+        if ((selh = selCreateBrick(1, hsize, 0, hsize / 2, SEL_HIT)) == NULL)
+            return (PIX *)ERROR_PTR("selh not made", procName, pixd);
+        if ((selv = selCreateBrick(vsize, 1, vsize / 2, 0, SEL_HIT)) == NULL) {
+            selDestroy(&selh);
+            return (PIX *)ERROR_PTR("selv not made", procName, pixd);
+        }
         pixt = pixDilate(NULL, pixs, selh);
         pixd = pixDilate(pixd, pixt, selv);
         pixDestroy(&pixt);
@@ -721,11 +731,11 @@ SEL  *sel, *selh, *selv;
 /*!
  * \brief   pixErodeBrick()
  *
- * \param[in]    pixd  [optional]; this can be null, equal to pixs,
- *                     or different from pixs
- * \param[in]    pixs 1 bpp
- * \param[in]    hsize width of brick Sel
- * \param[in]    vsize height of brick Sel
+ * \param[in]    pixd    [optional]; this can be null, equal to pixs,
+ *                       or different from pixs
+ * \param[in]    pixs    1 bpp
+ * \param[in]    hsize   width of brick Sel
+ * \param[in]    vsize   height of brick Sel
  * \return  pixd
  *
  * <pre>
@@ -766,11 +776,17 @@ SEL  *sel, *selh, *selv;
         return pixCopy(pixd, pixs);
     if (hsize == 1 || vsize == 1) {  /* no intermediate result */
         sel = selCreateBrick(vsize, hsize, vsize / 2, hsize / 2, SEL_HIT);
+        if (!sel)
+            return (PIX *)ERROR_PTR("sel not made", procName, pixd);
         pixd = pixErode(pixd, pixs, sel);
         selDestroy(&sel);
     } else {
-        selh = selCreateBrick(1, hsize, 0, hsize / 2, SEL_HIT);
-        selv = selCreateBrick(vsize, 1, vsize / 2, 0, SEL_HIT);
+        if ((selh = selCreateBrick(1, hsize, 0, hsize / 2, SEL_HIT)) == NULL)
+            return (PIX *)ERROR_PTR("selh not made", procName, pixd);
+        if ((selv = selCreateBrick(vsize, 1, vsize / 2, 0, SEL_HIT)) == NULL) {
+            selDestroy(&selh);
+            return (PIX *)ERROR_PTR("selv not made", procName, pixd);
+        }
         pixt = pixErode(NULL, pixs, selh);
         pixd = pixErode(pixd, pixt, selv);
         pixDestroy(&pixt);
@@ -785,11 +801,11 @@ SEL  *sel, *selh, *selv;
 /*!
  * \brief   pixOpenBrick()
  *
- * \param[in]    pixd  [optional]; this can be null, equal to pixs,
- *                     or different from pixs
- * \param[in]    pixs 1 bpp
- * \param[in]    hsize width of brick Sel
- * \param[in]    vsize height of brick Sel
+ * \param[in]    pixd    [optional]; this can be null, equal to pixs,
+ *                       or different from pixs
+ * \param[in]    pixs    1 bpp
+ * \param[in]    hsize   width of brick Sel
+ * \param[in]    vsize   height of brick Sel
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -830,11 +846,17 @@ SEL  *sel, *selh, *selv;
         return pixCopy(pixd, pixs);
     if (hsize == 1 || vsize == 1) {  /* no intermediate result */
         sel = selCreateBrick(vsize, hsize, vsize / 2, hsize / 2, SEL_HIT);
+        if (!sel)
+            return (PIX *)ERROR_PTR("sel not made", procName, pixd);
         pixd = pixOpen(pixd, pixs, sel);
         selDestroy(&sel);
     } else {  /* do separably */
-        selh = selCreateBrick(1, hsize, 0, hsize / 2, SEL_HIT);
-        selv = selCreateBrick(vsize, 1, vsize / 2, 0, SEL_HIT);
+        if ((selh = selCreateBrick(1, hsize, 0, hsize / 2, SEL_HIT)) == NULL)
+            return (PIX *)ERROR_PTR("selh not made", procName, pixd);
+        if ((selv = selCreateBrick(vsize, 1, vsize / 2, 0, SEL_HIT)) == NULL) {
+            selDestroy(&selh);
+            return (PIX *)ERROR_PTR("selv not made", procName, pixd);
+        }
         pixt = pixErode(NULL, pixs, selh);
         pixd = pixErode(pixd, pixt, selv);
         pixDilate(pixt, pixd, selh);
@@ -851,11 +873,11 @@ SEL  *sel, *selh, *selv;
 /*!
  * \brief   pixCloseBrick()
  *
- * \param[in]    pixd  [optional]; this can be null, equal to pixs,
- *                     or different from pixs
- * \param[in]    pixs 1 bpp
- * \param[in]    hsize width of brick Sel
- * \param[in]    vsize height of brick Sel
+ * \param[in]    pixd    [optional]; this can be null, equal to pixs,
+ *                       or different from pixs
+ * \param[in]    pixs    1 bpp
+ * \param[in]    hsize   width of brick Sel
+ * \param[in]    vsize   height of brick Sel
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -896,11 +918,17 @@ SEL  *sel, *selh, *selv;
         return pixCopy(pixd, pixs);
     if (hsize == 1 || vsize == 1) {  /* no intermediate result */
         sel = selCreateBrick(vsize, hsize, vsize / 2, hsize / 2, SEL_HIT);
+        if (!sel)
+            return (PIX *)ERROR_PTR("sel not made", procName, pixd);
         pixd = pixClose(pixd, pixs, sel);
         selDestroy(&sel);
     } else {  /* do separably */
-        selh = selCreateBrick(1, hsize, 0, hsize / 2, SEL_HIT);
-        selv = selCreateBrick(vsize, 1, vsize / 2, 0, SEL_HIT);
+        if ((selh = selCreateBrick(1, hsize, 0, hsize / 2, SEL_HIT)) == NULL)
+            return (PIX *)ERROR_PTR("selh not made", procName, pixd);
+        if ((selv = selCreateBrick(vsize, 1, vsize / 2, 0, SEL_HIT)) == NULL) {
+            selDestroy(&selh);
+            return (PIX *)ERROR_PTR("selv not made", procName, pixd);
+        }
         pixt = pixDilate(NULL, pixs, selh);
         pixd = pixDilate(pixd, pixt, selv);
         pixErode(pixt, pixd, selh);
@@ -917,11 +945,11 @@ SEL  *sel, *selh, *selv;
 /*!
  * \brief   pixCloseSafeBrick()
  *
- * \param[in]    pixd  [optional]; this can be null, equal to pixs,
- *                     or different from pixs
- * \param[in]    pixs 1 bpp
- * \param[in]    hsize width of brick Sel
- * \param[in]    vsize height of brick Sel
+ * \param[in]    pixd    [optional]; this can be null, equal to pixs,
+ *                       or different from pixs
+ * \param[in]    pixs    1 bpp
+ * \param[in]    hsize   width of brick Sel
+ * \param[in]    vsize   height of brick Sel
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -977,11 +1005,22 @@ SEL     *sel, *selh, *selv;
 
     if (hsize == 1 || vsize == 1) {  /* no intermediate result */
         sel = selCreateBrick(vsize, hsize, vsize / 2, hsize / 2, SEL_HIT);
+        if (!sel) {
+            pixDestroy(&pixsb);
+            return (PIX *)ERROR_PTR("sel not made", procName, pixd);
+        }
         pixdb = pixClose(NULL, pixsb, sel);
         selDestroy(&sel);
     } else {  /* do separably */
         selh = selCreateBrick(1, hsize, 0, hsize / 2, SEL_HIT);
         selv = selCreateBrick(vsize, 1, vsize / 2, 0, SEL_HIT);
+        if (!selh || !selv) {
+            selDestroy(&selh);
+            selDestroy(&selh);
+            pixDestroy(&pixsb);
+            return (PIX *)ERROR_PTR("selh and selv not both made",
+                                    procName, pixd);
+        }
         pixt = pixDilate(NULL, pixsb, selh);
         pixdb = pixDilate(NULL, pixt, selv);
         pixErode(pixt, pixdb, selh);
@@ -1001,7 +1040,6 @@ SEL     *sel, *selh, *selv;
         pixCopy(pixd, pixt);
         pixDestroy(&pixt);
     }
-
     return pixd;
 }
 
@@ -1009,15 +1047,16 @@ SEL     *sel, *selh, *selv;
 /*-----------------------------------------------------------------*
  *     Binary composed morphological (raster) ops with brick Sels  *
  *-----------------------------------------------------------------*/
-/*  selectComposableSels()
+/* \brief   selectComposableSels()
  *
- *      Input:  size (of composed sel)
- *              direction (L_HORIZ, L_VERT)
- *              &sel1 (<optional return> contiguous sel; can be null)
- *              &sel2 (<optional return> comb sel; can be null)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    size         of composed sel
+ * \param[in]    direction    L_HORIZ, L_VERT
+ * \param[out]   psel1        [optional] contiguous sel; can be null
+ * \param[out]   psel2        [optional] comb sel; can be null
+ * \return   0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) When using composable Sels, where the original Sel is
  *          decomposed into two, the best you can do in terms
  *          of reducing the computation is by a factor:
@@ -1032,6 +1071,7 @@ SEL     *sel, *selh, *selv;
  *                64        |          1/4
  *               144        |          1/6
  *               256        |          1/8
+ * </pre>
  */
 l_int32
 selectComposableSels(l_int32  size,
@@ -1047,8 +1087,8 @@ l_int32  factor1, factor2;
         return ERROR_INT("neither &sel1 nor &sel2 are defined", procName, 1);
     if (psel1) *psel1 = NULL;
     if (psel2) *psel2 = NULL;
-    if (size < 1 || size > 250 * 250)
-        return ERROR_INT("size < 1", procName, 1);
+    if (size < 1 || size > 10000)
+        return ERROR_INT("size < 1 or size > 10000", procName, 1);
     if (direction != L_HORIZ && direction != L_VERT)
         return ERROR_INT("invalid direction", procName, 1);
 
@@ -1070,14 +1110,14 @@ l_int32  factor1, factor2;
 /*!
  * \brief   selectComposableSizes()
  *
- * \param[in]    size of sel to be decomposed
- * \param[out]   pfactor1 larger factor
- * \param[out]   pfactor2 smaller factor
+ * \param[in]    size       of sel to be decomposed
+ * \param[out]   pfactor1   larger factor
+ * \param[out]   pfactor2   smaller factor
  * \return  0 if OK, 1 on error
  *
  * <pre>
  * Notes:
- *      (1) This works for Sel sizes up to 62500, which seems sufficient.
+ *      (1) This works for Sel sizes up to 10000, which seems sufficient.
  *      (2) The composable sel size is typically within +- 1 of
  *          the requested size.  Up to size = 300, the maximum difference
  *          is +- 2.
@@ -1088,7 +1128,7 @@ l_int32  factor1, factor2;
  *          If size > 1, then factor1 > 1.
  * </pre>
  */
-l_int32
+l_ok
 selectComposableSizes(l_int32   size,
                       l_int32  *pfactor1,
                       l_int32  *pfactor2)
@@ -1103,8 +1143,8 @@ l_int32  diff[256];  /* diff between product (sel size) and input size */
 
     PROCNAME("selectComposableSizes");
 
-    if (size < 1 || size > 250 * 250)
-        return ERROR_INT("size < 1", procName, 1);
+    if (size < 1 || size > 10000)
+        return ERROR_INT("size < 1 or size > 10000", procName, 1);
     if (!pfactor1 || !pfactor2)
         return ERROR_INT("&factor1 or &factor2 not defined", procName, 1);
 
@@ -1163,11 +1203,11 @@ l_int32  diff[256];  /* diff between product (sel size) and input size */
 /*!
  * \brief   pixDilateCompBrick()
  *
- * \param[in]    pixd  [optional]; this can be null, equal to pixs,
- *                     or different from pixs
- * \param[in]    pixs 1 bpp
- * \param[in]    hsize width of brick Sel
- * \param[in]    vsize height of brick Sel
+ * \param[in]    pixd    [optional]; this can be null, equal to pixs,
+ *                       or different from pixs
+ * \param[in]    pixs    1 bpp
+ * \param[in]    hsize   width of brick Sel
+ * \param[in]    vsize   height of brick Sel
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -1207,7 +1247,10 @@ pixDilateCompBrick(PIX     *pixd,
                    l_int32  vsize)
 {
 PIX  *pix1, *pix2, *pix3;
-SEL  *selh1, *selh2, *selv1, *selv2;
+SEL  *selh1 = NULL;
+SEL  *selh2 = NULL;
+SEL  *selv1 = NULL;
+SEL  *selv2 = NULL;
 
     PROCNAME("pixDilateCompBrick");
 
@@ -1220,10 +1263,22 @@ SEL  *selh1, *selh2, *selv1, *selv2;
 
     if (hsize == 1 && vsize == 1)
         return pixCopy(pixd, pixs);
-    if (hsize > 1)
-        selectComposableSels(hsize, L_HORIZ, &selh1, &selh2);
-    if (vsize > 1)
-        selectComposableSels(vsize, L_VERT, &selv1, &selv2);
+    if (hsize > 1) {
+        if (selectComposableSels(hsize, L_HORIZ, &selh1, &selh2)) {
+            selDestroy(&selh1);
+            selDestroy(&selh2);
+            return (PIX *)ERROR_PTR("horiz sels not made", procName, pixd);
+        }
+    }
+    if (vsize > 1) {
+        if (selectComposableSels(vsize, L_VERT, &selv1, &selv2)) {
+            selDestroy(&selh1);
+            selDestroy(&selh2);
+            selDestroy(&selv1);
+            selDestroy(&selv2);
+            return (PIX *)ERROR_PTR("vert sels not made", procName, pixd);
+        }
+    }
 
     pix1 = pixAddBorder(pixs, 32, 0);
     if (vsize == 1) {
@@ -1241,14 +1296,10 @@ SEL  *selh1, *selh2, *selv1, *selv2;
     pixDestroy(&pix1);
     pixDestroy(&pix2);
 
-    if (hsize > 1) {
-        selDestroy(&selh1);
-        selDestroy(&selh2);
-    }
-    if (vsize > 1) {
-        selDestroy(&selv1);
-        selDestroy(&selv2);
-    }
+    selDestroy(&selh1);
+    selDestroy(&selh2);
+    selDestroy(&selv1);
+    selDestroy(&selv2);
 
     pix1 = pixRemoveBorder(pix3, 32);
     pixDestroy(&pix3);
@@ -1263,11 +1314,11 @@ SEL  *selh1, *selh2, *selv1, *selv2;
 /*!
  * \brief   pixErodeCompBrick()
  *
- * \param[in]    pixd  [optional]; this can be null, equal to pixs,
- *                     or different from pixs
- * \param[in]    pixs 1 bpp
- * \param[in]    hsize width of brick Sel
- * \param[in]    vsize height of brick Sel
+ * \param[in]    pixd    [optional]; this can be null, equal to pixs,
+ *                       or different from pixs
+ * \param[in]    pixs    1 bpp
+ * \param[in]    hsize   width of brick Sel
+ * \param[in]    vsize   height of brick Sel
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -1307,7 +1358,10 @@ pixErodeCompBrick(PIX     *pixd,
                   l_int32  vsize)
 {
 PIX  *pixt;
-SEL  *selh1, *selh2, *selv1, *selv2;
+SEL  *selh1 = NULL;
+SEL  *selh2 = NULL;
+SEL  *selv1 = NULL;
+SEL  *selv2 = NULL;
 
     PROCNAME("pixErodeCompBrick");
 
@@ -1320,10 +1374,23 @@ SEL  *selh1, *selh2, *selv1, *selv2;
 
     if (hsize == 1 && vsize == 1)
         return pixCopy(pixd, pixs);
-    if (hsize > 1)
-        selectComposableSels(hsize, L_HORIZ, &selh1, &selh2);
-    if (vsize > 1)
-        selectComposableSels(vsize, L_VERT, &selv1, &selv2);
+    if (hsize > 1) {
+        if (selectComposableSels(hsize, L_HORIZ, &selh1, &selh2)) {
+            selDestroy(&selh1);
+            selDestroy(&selh2);
+            return (PIX *)ERROR_PTR("horiz sels not made", procName, pixd);
+        }
+    }
+    if (vsize > 1) {
+        if (selectComposableSels(vsize, L_VERT, &selv1, &selv2)) {
+            selDestroy(&selh1);
+            selDestroy(&selh2);
+            selDestroy(&selv1);
+            selDestroy(&selv2);
+            return (PIX *)ERROR_PTR("vert sels not made", procName, pixd);
+        }
+    }
+
     if (vsize == 1) {
         pixt = pixErode(NULL, pixs, selh1);
         pixd = pixErode(pixd, pixt, selh2);
@@ -1338,15 +1405,10 @@ SEL  *selh1, *selh2, *selv1, *selv2;
     }
     pixDestroy(&pixt);
 
-    if (hsize > 1) {
-        selDestroy(&selh1);
-        selDestroy(&selh2);
-    }
-    if (vsize > 1) {
-        selDestroy(&selv1);
-        selDestroy(&selv2);
-    }
-
+    selDestroy(&selh1);
+    selDestroy(&selh2);
+    selDestroy(&selv1);
+    selDestroy(&selv2);
     return pixd;
 }
 
@@ -1354,11 +1416,11 @@ SEL  *selh1, *selh2, *selv1, *selv2;
 /*!
  * \brief   pixOpenCompBrick()
  *
- * \param[in]    pixd  [optional]; this can be null, equal to pixs,
- *                     or different from pixs
- * \param[in]    pixs 1 bpp
- * \param[in]    hsize width of brick Sel
- * \param[in]    vsize height of brick Sel
+ * \param[in]    pixd    [optional]; this can be null, equal to pixs,
+ *                       or different from pixs
+ * \param[in]    pixs    1 bpp
+ * \param[in]    hsize   width of brick Sel
+ * \param[in]    vsize   height of brick Sel
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -1398,7 +1460,10 @@ pixOpenCompBrick(PIX     *pixd,
                  l_int32  vsize)
 {
 PIX  *pixt;
-SEL  *selh1, *selh2, *selv1, *selv2;
+SEL  *selh1 = NULL;
+SEL  *selh2 = NULL;
+SEL  *selv1 = NULL;
+SEL  *selv2 = NULL;
 
     PROCNAME("pixOpenCompBrick");
 
@@ -1411,10 +1476,23 @@ SEL  *selh1, *selh2, *selv1, *selv2;
 
     if (hsize == 1 && vsize == 1)
         return pixCopy(pixd, pixs);
-    if (hsize > 1)
-        selectComposableSels(hsize, L_HORIZ, &selh1, &selh2);
-    if (vsize > 1)
-        selectComposableSels(vsize, L_VERT, &selv1, &selv2);
+    if (hsize > 1) {
+        if (selectComposableSels(hsize, L_HORIZ, &selh1, &selh2)) {
+            selDestroy(&selh1);
+            selDestroy(&selh2);
+            return (PIX *)ERROR_PTR("horiz sels not made", procName, pixd);
+        }
+    }
+    if (vsize > 1) {
+        if (selectComposableSels(vsize, L_VERT, &selv1, &selv2)) {
+            selDestroy(&selh1);
+            selDestroy(&selh2);
+            selDestroy(&selv1);
+            selDestroy(&selv2);
+            return (PIX *)ERROR_PTR("vert sels not made", procName, pixd);
+        }
+    }
+
     if (vsize == 1) {
         pixt = pixErode(NULL, pixs, selh1);
         pixd = pixErode(pixd, pixt, selh2);
@@ -1437,15 +1515,10 @@ SEL  *selh1, *selh2, *selv1, *selv2;
     }
     pixDestroy(&pixt);
 
-    if (hsize > 1) {
-        selDestroy(&selh1);
-        selDestroy(&selh2);
-    }
-    if (vsize > 1) {
-        selDestroy(&selv1);
-        selDestroy(&selv2);
-    }
-
+    selDestroy(&selh1);
+    selDestroy(&selh2);
+    selDestroy(&selv1);
+    selDestroy(&selv2);
     return pixd;
 }
 
@@ -1453,11 +1526,11 @@ SEL  *selh1, *selh2, *selv1, *selv2;
 /*!
  * \brief   pixCloseCompBrick()
  *
- * \param[in]    pixd  [optional]; this can be null, equal to pixs,
- *                     or different from pixs
- * \param[in]    pixs 1 bpp
- * \param[in]    hsize width of brick Sel
- * \param[in]    vsize height of brick Sel
+ * \param[in]    pixd    [optional]; this can be null, equal to pixs,
+ *                       or different from pixs
+ * \param[in]    pixs    1 bpp
+ * \param[in]    hsize   width of brick Sel
+ * \param[in]    vsize   height of brick Sel
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -1497,7 +1570,10 @@ pixCloseCompBrick(PIX     *pixd,
                   l_int32  vsize)
 {
 PIX  *pixt;
-SEL  *selh1, *selh2, *selv1, *selv2;
+SEL  *selh1 = NULL;
+SEL  *selh2 = NULL;
+SEL  *selv1 = NULL;
+SEL  *selv2 = NULL;
 
     PROCNAME("pixCloseCompBrick");
 
@@ -1510,10 +1586,23 @@ SEL  *selh1, *selh2, *selv1, *selv2;
 
     if (hsize == 1 && vsize == 1)
         return pixCopy(pixd, pixs);
-    if (hsize > 1)
-        selectComposableSels(hsize, L_HORIZ, &selh1, &selh2);
-    if (vsize > 1)
-        selectComposableSels(vsize, L_VERT, &selv1, &selv2);
+    if (hsize > 1) {
+        if (selectComposableSels(hsize, L_HORIZ, &selh1, &selh2)) {
+            selDestroy(&selh1);
+            selDestroy(&selh2);
+            return (PIX *)ERROR_PTR("horiz sels not made", procName, pixd);
+        }
+    }
+    if (vsize > 1) {
+        if (selectComposableSels(vsize, L_VERT, &selv1, &selv2)) {
+            selDestroy(&selh1);
+            selDestroy(&selh2);
+            selDestroy(&selv1);
+            selDestroy(&selv2);
+            return (PIX *)ERROR_PTR("vert sels not made", procName, pixd);
+        }
+    }
+
     if (vsize == 1) {
         pixt = pixDilate(NULL, pixs, selh1);
         pixd = pixDilate(pixd, pixt, selh2);
@@ -1536,15 +1625,10 @@ SEL  *selh1, *selh2, *selv1, *selv2;
     }
     pixDestroy(&pixt);
 
-    if (hsize > 1) {
-        selDestroy(&selh1);
-        selDestroy(&selh2);
-    }
-    if (vsize > 1) {
-        selDestroy(&selv1);
-        selDestroy(&selv2);
-    }
-
+    selDestroy(&selh1);
+    selDestroy(&selh2);
+    selDestroy(&selv1);
+    selDestroy(&selv2);
     return pixd;
 }
 
@@ -1552,11 +1636,11 @@ SEL  *selh1, *selh2, *selv1, *selv2;
 /*!
  * \brief   pixCloseSafeCompBrick()
  *
- * \param[in]    pixd  [optional]; this can be null, equal to pixs,
- *                     or different from pixs
- * \param[in]    pixs 1 bpp
- * \param[in]    hsize width of brick Sel
- * \param[in]    vsize height of brick Sel
+ * \param[in]    pixd    [optional]; this can be null, equal to pixs,
+ *                       or different from pixs
+ * \param[in]    pixs    1 bpp
+ * \param[in]    hsize   width of brick Sel
+ * \param[in]    vsize   height of brick Sel
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -1602,7 +1686,10 @@ pixCloseSafeCompBrick(PIX     *pixd,
 {
 l_int32  maxtrans, bordsize;
 PIX     *pixsb, *pixt, *pixdb;
-SEL     *selh1, *selh2, *selv1, *selv2;
+SEL     *selh1 = NULL;
+SEL     *selh2 = NULL;
+SEL     *selv1 = NULL;
+SEL     *selv2 = NULL;
 
     PROCNAME("pixCloseSafeCompBrick");
 
@@ -1620,14 +1707,27 @@ SEL     *selh1, *selh2, *selv1, *selv2;
     if (MORPH_BC == SYMMETRIC_MORPH_BC)
         return pixCloseCompBrick(pixd, pixs, hsize, vsize);
 
+    if (hsize > 1) {
+        if (selectComposableSels(hsize, L_HORIZ, &selh1, &selh2)) {
+            selDestroy(&selh1);
+            selDestroy(&selh2);
+            return (PIX *)ERROR_PTR("horiz sels not made", procName, pixd);
+        }
+    }
+    if (vsize > 1) {
+        if (selectComposableSels(vsize, L_VERT, &selv1, &selv2)) {
+            selDestroy(&selh1);
+            selDestroy(&selh2);
+            selDestroy(&selv1);
+            selDestroy(&selv2);
+            return (PIX *)ERROR_PTR("vert sels not made", procName, pixd);
+        }
+    }
+
     maxtrans = L_MAX(hsize / 2, vsize / 2);
     bordsize = 32 * ((maxtrans + 31) / 32);  /* full 32 bit words */
     pixsb = pixAddBorder(pixs, bordsize, 0);
 
-    if (hsize > 1)
-        selectComposableSels(hsize, L_HORIZ, &selh1, &selh2);
-    if (vsize > 1)
-        selectComposableSels(vsize, L_VERT, &selv1, &selv2);
     if (vsize == 1) {
         pixt = pixDilate(NULL, pixsb, selh1);
         pixdb = pixDilate(NULL, pixt, selh2);
@@ -1661,15 +1761,10 @@ SEL     *selh1, *selh2, *selv1, *selv2;
         pixDestroy(&pixt);
     }
 
-    if (hsize > 1) {
-        selDestroy(&selh1);
-        selDestroy(&selh2);
-    }
-    if (vsize > 1) {
-        selDestroy(&selv1);
-        selDestroy(&selv2);
-    }
-
+    selDestroy(&selh1);
+    selDestroy(&selh2);
+    selDestroy(&selv1);
+    selDestroy(&selv2);
     return pixd;
 }
 
@@ -1680,7 +1775,7 @@ SEL     *selh1, *selh2, *selv1, *selv2;
 /*!
  * \brief   resetMorphBoundaryCondition()
  *
- * \param[in]    bc SYMMETRIC_MORPH_BC, ASYMMETRIC_MORPH_BC
+ * \param[in]    bc    SYMMETRIC_MORPH_BC, ASYMMETRIC_MORPH_BC
  * \return  void
  */
 void
@@ -1733,11 +1828,11 @@ getMorphBorderPixelColor(l_int32  type,
 /*!
  * \brief   processMorphArgs1()
  *
- * \param[in]    pixd [optional]; this can be null, equal to pixs,
- *                    or different from pixs
- * \param[in]    pixs 1 bpp
- * \param[in]    sel
- * \param[out]   ppixt ptr to PIX*
+ * \param[in]       pixd   [optional]; this can be null, equal to pixs,
+ *                         or different from pixs
+ * \param[in]       pixs   1 bpp
+ * \param[in]       sel
+ * \param[out]      ppixt  copy or clone of %pixs
  * \return  pixd, or NULL on error.
  *
  * <pre>
