@@ -197,6 +197,10 @@
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include <math.h>
 #include "allheaders.h"
@@ -206,8 +210,7 @@
  /* --------------------------------------------*/
 
     /* Typical scan resolution in ppi (pixels/inch) */
-static const l_int32  DEFAULT_INPUT_RES = 300;
-
+static const l_int32  DefaultInputRes = 300;
 
 /*---------------------------------------------------------------------*
  *    Convert specified image files to pdf (one image file per page)   *
@@ -221,7 +224,8 @@ static const l_int32  DEFAULT_INPUT_RES = 300;
  * \param[in]    res           input resolution of all images
  * \param[in]    scalefactor   scaling factor applied to each image; > 0.0
  * \param[in]    type          encoding type (L_JPEG_ENCODE, L_G4_ENCODE,
- *                             L_FLATE_ENCODE, L_JP2K_ENCODE or 0 for default
+ *                             L_FLATE_ENCODE, L_JP2K_ENCODE or
+ *                             L_DEFAULT_ENCODE for default)
  * \param[in]    quality       for jpeg: 1-100; 0 for default (75)
  *                             for jp2k: 27-45; 0 for default (34)
  * \param[in]    title         [optional] pdf title; if null, taken from
@@ -281,7 +285,8 @@ SARRAY  *sa;
  * \param[in]    res           input resolution of all images
  * \param[in]    scalefactor   scaling factor applied to each image; > 0.0
  * \param[in]    type          encoding type (L_JPEG_ENCODE, L_G4_ENCODE,
- *                             L_FLATE_ENCODE, L_JP2K_ENCODE or 0 for default
+ *                             L_FLATE_ENCODE, L_JP2K_ENCODE or
+ *                             L_DEFAULT_ENCODE for default)
  * \param[in]    quality       for jpeg: 1-100; 0 for default (75)
  *                             for jp2k: 27-45; 0 for default (34)
  * \param[in]    title         [optional] pdf title; if null, taken from
@@ -334,7 +339,8 @@ size_t    nbytes;
  * \param[in]    res           input resolution of all images
  * \param[in]    scalefactor   scaling factor applied to each image; > 0.0
  * \param[in]    type          encoding type (L_JPEG_ENCODE, L_G4_ENCODE,
- *                             L_FLATE_ENCODE, L_JP2K_ENCODE or 0 for default
+ *                             L_FLATE_ENCODE, L_JP2K_ENCODE or
+ *                             L_DEFAULT_ENCODE for default)
  * \param[in]    quality       for jpeg: 1-100; 0 for default (75)
  *                             for jp2k: 27-45; 0 for default (34)
  * \param[in]    title         [optional] pdf title; if null, taken from
@@ -380,9 +386,7 @@ L_PTRA      *pa_data;
     if (scalefactor <= 0.0) scalefactor = 1.0;
     if (type != L_JPEG_ENCODE && type != L_G4_ENCODE &&
         type != L_FLATE_ENCODE && type != L_JP2K_ENCODE) {
-        L_WARNING("invalid compression type; using per-page default\n",
-                  procName);
-        type = 0;
+        type = L_DEFAULT_ENCODE;
     }
 
         /* Generate all the encoded pdf strings */
@@ -390,7 +394,7 @@ L_PTRA      *pa_data;
     pa_data = ptraCreate(n);
     pdftitle = NULL;
     for (i = 0; i < n; i++) {
-        if (i && (i % 10 == 0)) fprintf(stderr, ".. %d ", i);
+        if (i && (i % 10 == 0)) lept_stderr(".. %d ", i);
         fname = sarrayGetString(sa, i, L_NOCOPY);
         if ((pixs = pixRead(fname)) == NULL) {
             L_ERROR("image not readable from file %s\n", procName, fname);
@@ -404,7 +408,9 @@ L_PTRA      *pa_data;
             pix = pixClone(pixs);
         pixDestroy(&pixs);
         scaledres = (l_int32)(res * scalefactor);
-        if (type != 0) {
+
+            /* Select the encoding type */
+        if (type != L_DEFAULT_ENCODE) {
             pagetype = type;
         } else if (selectDefaultPdfEncoding(pix, &pagetype) != 0) {
             pixDestroy(&pix);
@@ -412,6 +418,7 @@ L_PTRA      *pa_data;
                     procName, fname);
             continue;
         }
+
         ret = pixConvertToPdfData(pix, pagetype, quality, &imdata, &imbytes,
                                   0, 0, scaledres, pdftitle, NULL, 0);
         pixDestroy(&pix);
@@ -432,9 +439,9 @@ L_PTRA      *pa_data;
     }
 
         /* Concatenate them */
-    fprintf(stderr, "\nconcatenating ... ");
+    lept_stderr("\nconcatenating ... ");
     ret = ptraConcatenatePdfToData(pa_data, NULL, pdata, pnbytes);
-    fprintf(stderr, "done\n");
+    lept_stderr("done\n");
 
     ptraGetActualCount(pa_data, &npages);  /* recalculate in case it changes */
     for (i = 0; i < npages; i++) {
@@ -475,11 +482,11 @@ PIXCMAP  *cmap;
 
     PROCNAME("selectDefaultPdfEncoding");
 
-    if (!pix)
-        return ERROR_INT("pix not defined", procName, 1);
     if (!ptype)
         return ERROR_INT("&type not defined", procName, 1);
     *ptype = L_FLATE_ENCODE;  /* default universal encoding */
+    if (!pix)
+        return ERROR_INT("pix not defined", procName, 1);
     pixGetDimensions(pix, &w, &h, &d);
     cmap = pixGetColormap(pix);
     if (d == 8 && !cmap) {
@@ -640,7 +647,7 @@ L_PTRA       *pa_data;
     n = sarrayGetCount(sa);
     pa_data = ptraCreate(n);
     for (i = 0; i < n; i++) {
-        if (i && (i % 10 == 0)) fprintf(stderr, ".. %d ", i);
+        if (i && (i % 10 == 0)) lept_stderr(".. %d ", i);
         fname = sarrayGetString(sa, i, L_NOCOPY);
 
             /* Generate the pdf data */
@@ -660,9 +667,9 @@ L_PTRA       *pa_data;
     }
 
         /* Concatenate to generate a multipage pdf */
-    fprintf(stderr, "\nconcatenating ... ");
+    lept_stderr("\nconcatenating ... ");
     ret = ptraConcatenatePdfToData(pa_data, NULL, pdata, pnbytes);
-    fprintf(stderr, "done\n");
+    lept_stderr("done\n");
 
         /* Clean up */
     ptraGetActualCount(pa_data, &npages);  /* maybe failed to read some files */
@@ -760,7 +767,8 @@ L_COMP_DATA  *cid;
  *                             embedded in the input images
  * \param[in]    scalefactor   scaling factor applied to each image; > 0.0
  * \param[in]    type          encoding type (L_JPEG_ENCODE, L_G4_ENCODE,
- *                             L_FLATE_ENCODE, L_JP2K_ENCODE or 0 for default
+ *                             L_FLATE_ENCODE, L_JP2K_ENCODE, or
+ *                             L_DEFAULT_ENCODE for default)
  * \param[in]    quality       for jpeg: 1-100; 0 for default (75)
  *                             for jp2k: 27-45; 0 for default (34)
  * \param[in]    title         [optional] pdf title
@@ -818,7 +826,8 @@ size_t    nbytes;
  * \param[in]    res            input resolution of all images
  * \param[in]    scalefactor    scaling factor applied to each image; > 0.0
  * \param[in]    type           encoding type (L_JPEG_ENCODE, L_G4_ENCODE,
- *                              L_FLATE_ENCODE, L_JP2K_ENCODE or 0 for default
+ *                              L_FLATE_ENCODE, L_JP2K_ENCODE, or
+ *                              L_DEFAULT_ENCODE for default)
  * \param[in]    quality        for jpeg: 1-100; 0 for default (75)
  *                              for jp2k: 27-45; 0 for default (34)
  * \param[in]    title          [optional] pdf title
@@ -859,11 +868,12 @@ L_PTRA   *pa_data;
     if (!pixa)
         return ERROR_INT("pixa not defined", procName, 1);
     if (scalefactor <= 0.0) scalefactor = 1.0;
-    if (type != L_JPEG_ENCODE && type != L_G4_ENCODE &&
-        type != L_FLATE_ENCODE && type != L_JP2K_ENCODE) {
+    if (type != L_DEFAULT_ENCODE && type != L_JPEG_ENCODE &&
+        type != L_G4_ENCODE && type != L_FLATE_ENCODE &&
+        type != L_JP2K_ENCODE) {
         L_WARNING("invalid compression type; using per-page default\n",
                   procName);
-        type = 0;
+        type = L_DEFAULT_ENCODE;
     }
 
         /* Generate all the encoded pdf strings */
@@ -880,7 +890,9 @@ L_PTRA   *pa_data;
             pix = pixClone(pixs);
         pixDestroy(&pixs);
         scaledres = (l_int32)(res * scalefactor);
-        if (type != 0) {
+
+            /* Select the encoding type */
+        if (type != L_DEFAULT_ENCODE) {
             pagetype = type;
         } else if (selectDefaultPdfEncoding(pix, &pagetype) != 0) {
             L_ERROR("encoding type selection failed for pix[%d]\n",
@@ -888,6 +900,7 @@ L_PTRA   *pa_data;
             pixDestroy(&pix);
             continue;
         }
+
         ret = pixConvertToPdfData(pix, pagetype, quality, &imdata, &imbytes,
                                   0, 0, scaledres, title, NULL, 0);
         pixDestroy(&pix);
@@ -928,7 +941,7 @@ L_PTRA   *pa_data;
  *
  * \param[in]      filein       input image file -- any format
  * \param[in]      type         encoding type (L_JPEG_ENCODE, L_G4_ENCODE,
- *                              L_FLATE_ENCODE, L_JP2K_ENCODE or 0 for default
+ *                              L_FLATE_ENCODE, or L_JP2K_ENCODE)
  * \param[in]      quality      for jpeg: 1-100; 0 for default (75)
  *                              for jp2k: 27-45; 0 for default (34)
  * \param[in]      fileout      output pdf file; only required on last
@@ -1006,9 +1019,6 @@ size_t    nbytes;
         if (!fileout)
             return ERROR_INT("fileout not defined", procName, 1);
     }
-    if (type != L_JPEG_ENCODE && type != L_G4_ENCODE &&
-        type != L_FLATE_ENCODE && type != L_JP2K_ENCODE)
-        return ERROR_INT("invalid conversion type", procName, 1);
 
     if (convertToPdfData(filein, type, quality, &data, &nbytes, x, y,
                          res, title, plpd, position))
@@ -1031,7 +1041,7 @@ size_t    nbytes;
  * \param[in]      imdata       array of formatted image data; e.g., png, jpeg
  * \param[in]      size         size of image data
  * \param[in]      type         encoding type (L_JPEG_ENCODE, L_G4_ENCODE,
- *                              L_FLATE_ENCODE, L_JP2K_ENCODE or 0 for default
+ *                              L_FLATE_ENCODE, or L_JP2K_ENCODE)
  * \param[in]      quality      for jpeg: 1-100; 0 for default (75)
  *                              for jp2k: 27-45; 0 for default (34)
  * \param[in]      fileout      output pdf file; only required on last
@@ -1053,7 +1063,7 @@ size_t    nbytes;
  * <pre>
  * Notes:
  *      (1) If %res == 0 and the input resolution field is 0,
- *          this will use DEFAULT_INPUT_RES.
+ *          this will use DefaultInputRes.
  *      (2) See comments in convertToPdf().
  * </pre>
  */
@@ -1077,9 +1087,6 @@ PIX     *pix;
 
     if (!imdata)
         return ERROR_INT("image data not defined", procName, 1);
-    if (type != L_JPEG_ENCODE && type != L_G4_ENCODE &&
-        type != L_FLATE_ENCODE && type != L_JP2K_ENCODE)
-        return ERROR_INT("invalid conversion type", procName, 1);
     if (!plpd || (position == L_LAST_IMAGE)) {
         if (!fileout)
             return ERROR_INT("fileout not defined", procName, 1);
@@ -1087,6 +1094,10 @@ PIX     *pix;
 
     if ((pix = pixReadMem(imdata, size)) == NULL)
         return ERROR_INT("pix not read", procName, 1);
+    if (type != L_JPEG_ENCODE && type != L_G4_ENCODE &&
+        type != L_FLATE_ENCODE && type != L_JP2K_ENCODE) {
+        selectDefaultPdfEncoding(pix, &type);
+    }
     ret = pixConvertToPdf(pix, type, quality, fileout, x, y, res,
                           title, plpd, position);
     pixDestroy(&pix);
@@ -1099,7 +1110,7 @@ PIX     *pix;
  *
  * \param[in]      filein       input image file -- any format
  * \param[in]      type         encoding type (L_JPEG_ENCODE, L_G4_ENCODE,
- *                              L_FLATE_ENCODE, L_JP2K_ENCODE or 0 for default
+ *                              L_FLATE_ENCODE, or L_JP2K_ENCODE)
  * \param[in]      quality      for jpeg: 1-100; 0 for default (75)
  *                              for jp2k: 27-45; 0 for default (34)
  * \param[out]     pdata        pdf data in memory
@@ -1121,7 +1132,7 @@ PIX     *pix;
  * <pre>
  * Notes:
  *      (1) If %res == 0 and the input resolution field is 0,
- *          this will use DEFAULT_INPUT_RES.
+ *          this will use DefaultInputRes.
  *      (2) See comments in convertToPdf().
  * </pre>
  */
@@ -1150,9 +1161,6 @@ PIX  *pix;
     *pnbytes = 0;
     if (!filein)
         return ERROR_INT("filein not defined", procName, 1);
-    if (type != L_JPEG_ENCODE && type != L_G4_ENCODE &&
-        type != L_FLATE_ENCODE && type != L_JP2K_ENCODE)
-        return ERROR_INT("invalid conversion type", procName, 1);
 
     if ((pix = pixRead(filein)) == NULL)
         return ERROR_INT("pix not made", procName, 1);
@@ -1170,7 +1178,7 @@ PIX  *pix;
  * \param[in]    imdata       array of formatted image data; e.g., png, jpeg
  * \param[in]    size         size of image data
  * \param[in]    type         encoding type (L_JPEG_ENCODE, L_G4_ENCODE,
- *                            L_FLATE_ENCODE, L_JP2K_ENCODE or 0 for default
+ *                            L_FLATE_ENCODE, or L_JP2K_ENCODE)
  * \param[in]    quality      for jpeg: 1-100; 0 for default (75)
  *                            for jp2k: 27-45; 0 for default (34)
  * \param[out]   pdata        pdf data in memory
@@ -1192,7 +1200,7 @@ PIX  *pix;
  * <pre>
  * Notes:
  *      (1) If %res == 0 and the input resolution field is 0,
- *          this will use DEFAULT_INPUT_RES.
+ *          this will use DefaultInputRes.
  *      (2) See comments in convertToPdf().
  * </pre>
  */
@@ -1230,6 +1238,10 @@ PIX     *pix;
 
     if ((pix = pixReadMem(imdata, size)) == NULL)
         return ERROR_INT("pix not read", procName, 1);
+    if (type != L_JPEG_ENCODE && type != L_G4_ENCODE &&
+        type != L_FLATE_ENCODE && type != L_JP2K_ENCODE) {
+        selectDefaultPdfEncoding(pix, &type);
+    }
     ret = pixConvertToPdfData(pix, type, quality, pdata, pnbytes,
                               x, y, res, title, plpd, position);
     pixDestroy(&pix);
@@ -1242,7 +1254,7 @@ PIX     *pix;
  *
  * \param[in]      pix
  * \param[in]      type         encoding type (L_JPEG_ENCODE, L_G4_ENCODE,
- *                              L_FLATE_ENCODE, L_JP2K_ENCODE or 0 for default
+ *                              L_FLATE_ENCODE, L_JP2K_ENCODE)
  * \param[in]      quality      for jpeg: 1-100; 0 for default (75)
  *                              for jp2k: 27-45; 0 for default (34)
  * \param[in]      fileout      output pdf file; only required on last
@@ -1264,7 +1276,7 @@ PIX     *pix;
  * <pre>
  * Notes:
  *      (1) If %res == 0 and the input resolution field is 0,
- *          this will use DEFAULT_INPUT_RES.
+ *          this will use DefaultInputRes.
  *      (2) This only writes data to fileout if it is the last
  *          image to be written on the page.
  *      (3) See comments in convertToPdf().
@@ -1290,9 +1302,6 @@ size_t    nbytes;
 
     if (!pix)
         return ERROR_INT("pix not defined", procName, 1);
-    if (type != L_JPEG_ENCODE && type != L_G4_ENCODE &&
-        type != L_FLATE_ENCODE && type != L_JP2K_ENCODE)
-        return ERROR_INT("invalid conversion type", procName, 1);
     if (!plpd || (position == L_LAST_IMAGE)) {
         if (!fileout)
             return ERROR_INT("fileout not defined", procName, 1);
@@ -1378,8 +1387,8 @@ size_t    nbytes, nbytes_written;
  * Notes:
  *      (1) This is the simplest interface for writing a single image
  *          with pdf encoding to memory.  It uses G4 encoding for 1 bpp,
- *          JPEG encoding for 8 bpp (no cmap) and 32 bpp, and FLATE
- *          encoding for everything else.
+ *          and makes a guess whether to use JPEG or FLATE encoding for
+ *          everything else.
  * </pre>
  */
 l_ok
@@ -1389,8 +1398,7 @@ pixWriteMemPdf(l_uint8    **pdata,
                l_int32      res,
                const char  *title)
 {
-l_int32   ret, d, type;
-PIXCMAP  *cmap;
+l_int32  ret, type;
 
     PROCNAME("pixWriteMemPdf");
 
@@ -1401,15 +1409,7 @@ PIXCMAP  *cmap;
     if (!pix)
         return ERROR_INT("pix not defined", procName, 1);
 
-    d = pixGetDepth(pix);
-    cmap = pixGetColormap(pix);
-    if (d == 1)
-        type = L_G4_ENCODE;
-    else if (cmap || d == 2 || d == 4 || d == 16)
-        type = L_FLATE_ENCODE;
-    else  /* d == 8 (no cmap) or d == 32 */
-        type = L_JPEG_ENCODE;
-
+    selectDefaultPdfEncoding(pix, &type);
     ret = pixConvertToPdfData(pix, type, 75, pdata, pnbytes,
                               0, 0, res, title, NULL, 0);
     if (ret)
@@ -1926,7 +1926,7 @@ L_PDF_DATA  *lpd;
 
         /* Adjust scalefactor so that the product with res gives an integer */
     if (res <= 0)
-        res = DEFAULT_INPUT_RES;
+        res = DefaultInputRes;
     scale = (l_float32)((l_int32)(scalefactor * res + 0.5)) / (l_float32)res;
     cmap = pixGetColormap(pixs);
 
