@@ -148,14 +148,23 @@
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include "allheaders.h"
 
-static const l_int32  INITIAL_PTR_ARRAYSIZE = 20;   /* n'import quoi */
+    /* Bounds on initial array size */
+static const l_uint32  MaxPtrArraySize = 1000000;
+static const l_int32  InitialPtrArraySize = 20;      /*!< n'importe quoi */
+
+    /* Bound on data size */
+static const size_t  MaxDataSize = 1000000000;
 
     /* These two globals are defined in writefile.c */
-extern l_int32 NumImageFileFormatExtensions;
-extern const char *ImageFileFormatExtensions[];
+extern l_int32  NumImageFileFormatExtensions;
+extern const char  *ImageFileFormatExtensions[];
 
     /* Static functions */
 static l_int32 pixacompExtendArray(PIXAC *pixac);
@@ -199,8 +208,7 @@ PIXC     *pixc;
         comptype != IFF_PNG && comptype != IFF_JFIF_JPEG)
         return (PIXC *)ERROR_PTR("invalid comptype", procName, NULL);
 
-    if ((pixc = (PIXC *)LEPT_CALLOC(1, sizeof(PIXC))) == NULL)
-        return (PIXC *)ERROR_PTR("pixc not made", procName, NULL);
+    pixc = (PIXC *)LEPT_CALLOC(1, sizeof(PIXC));
     pixGetDimensions(pix, &pixc->w, &pixc->h, &pixc->d);
     pixGetResolution(pix, &pixc->xres, &pixc->yres);
     if (pixGetColormap(pix))
@@ -255,8 +263,7 @@ PIXC    *pixc;
 
     if (pixReadHeaderMem(data, size, &format, &w, &h, &bps, &spp, &iscmap) == 1)
         return (PIXC *)ERROR_PTR("header data not read", procName, NULL);
-    if ((pixc = (PIXC *)LEPT_CALLOC(1, sizeof(PIXC))) == NULL)
-        return (PIXC *)ERROR_PTR("pixc not made", procName, NULL);
+    pixc = (PIXC *)LEPT_CALLOC(1, sizeof(PIXC));
     d = (spp == 3) ? 32 : bps * spp;
     pixc->w = w;
     pixc->h = h;
@@ -392,8 +399,7 @@ PIXC     *pixcd;
     if (!pixcs)
         return (PIXC *)ERROR_PTR("pixcs not defined", procName, NULL);
 
-    if ((pixcd = (PIXC *)LEPT_CALLOC(1, sizeof(PIXC))) == NULL)
-        return (PIXC *)ERROR_PTR("pixcd not made", procName, NULL);
+    pixcd = (PIXC *)LEPT_CALLOC(1, sizeof(PIXC));
     pixcd->w = pixcs->w;
     pixcd->h = pixcs->h;
     pixcd->d = pixcs->d;
@@ -407,7 +413,10 @@ PIXC     *pixcd;
         /* Copy image data */
     size = pixcs->size;
     datas = pixcs->data;
-    datad = (l_uint8 *)LEPT_CALLOC(size, sizeof(l_int8));
+    if ((datad = (l_uint8 *)LEPT_CALLOC(size, sizeof(l_int8))) == NULL) {
+        pixcompDestroy(&pixcd);
+        return (PIXC *)ERROR_PTR("pixcd not made", procName, NULL);
+    }
     memcpy(datad, datas, size);
     pixcd->data = datad;
     pixcd->size = size;
@@ -597,15 +606,13 @@ PIXAC  *pixac;
 
     PROCNAME("pixacompCreate");
 
-    if (n <= 0)
-        n = INITIAL_PTR_ARRAYSIZE;
+    if (n <= 0 || n > MaxPtrArraySize)
+        n = InitialPtrArraySize;
 
-    if ((pixac = (PIXAC *)LEPT_CALLOC(1, sizeof(PIXAC))) == NULL)
-        return (PIXAC *)ERROR_PTR("pixac not made", procName, NULL);
+    pixac = (PIXAC *)LEPT_CALLOC(1, sizeof(PIXAC));
     pixac->n = 0;
     pixac->nalloc = n;
     pixac->offset = 0;
-
     if ((pixac->pixc = (PIXC **)LEPT_CALLOC(n, sizeof(PIXC *))) == NULL) {
         pixacompDestroy(&pixac);
         return (PIXAC *)ERROR_PTR("pixc ptrs not made", procName, NULL);
@@ -670,8 +677,8 @@ PIXAC   *pixac;
 
     PROCNAME("pixacompCreateWithInit");
 
-    if (n <= 0)
-        return (PIXAC *)ERROR_PTR("n must be > 0", procName, NULL);
+    if (n <= 0 || n > MaxPtrArraySize)
+        return (PIXAC *)ERROR_PTR("n out of valid bounds", procName, NULL);
     if (pix) {
         if (comptype != IFF_DEFAULT && comptype != IFF_TIFF_G4 &&
             comptype != IFF_PNG && comptype != IFF_JFIF_JPEG)
@@ -1675,7 +1682,8 @@ pixacompReadStream(FILE  *fp)
 char      buf[256];
 l_uint8  *data;
 l_int32   n, offset, i, w, h, d, ignore;
-l_int32   comptype, size, cmapflag, version, xres, yres;
+l_int32   comptype, cmapflag, version, xres, yres;
+size_t    size;
 BOXA     *boxa;
 PIXC     *pixc;
 PIXAC    *pixac;
@@ -1708,13 +1716,18 @@ PIXAC    *pixac;
         if (fscanf(fp, "\nPixcomp[%d]: w = %d, h = %d, d = %d\n",
                    &ignore, &w, &h, &d) != 4) {
             pixacompDestroy(&pixac);
-            return (PIXAC *)ERROR_PTR("size reading", procName, NULL);
+            return (PIXAC *)ERROR_PTR("dimension reading", procName, NULL);
         }
-        if (fscanf(fp, "  comptype = %d, size = %d, cmapflag = %d\n",
+        if (fscanf(fp, "  comptype = %d, size = %zu, cmapflag = %d\n",
                    &comptype, &size, &cmapflag) != 3) {
             pixacompDestroy(&pixac);
             return (PIXAC *)ERROR_PTR("comptype/size reading", procName, NULL);
         }
+        if (size > MaxDataSize) {
+            pixacompDestroy(&pixac);
+            L_ERROR("data size = %zu is too big", procName, size);
+            return NULL;
+        }
 
            /* Use fgets() and sscanf(); not fscanf(), for the last
              * bit of header data before the binary data.  The reason is
@@ -1858,8 +1871,8 @@ PIXC    *pixc;
             return ERROR_INT("pixc not found", procName, 1);
         fprintf(fp, "\nPixcomp[%d]: w = %d, h = %d, d = %d\n",
                 i, pixc->w, pixc->h, pixc->d);
-        fprintf(fp, "  comptype = %d, size = %lu, cmapflag = %d\n",
-                pixc->comptype, (unsigned long)pixc->size, pixc->cmapflag);
+        fprintf(fp, "  comptype = %d, size = %zu, cmapflag = %d\n",
+                pixc->comptype, pixc->size, pixc->cmapflag);
         fprintf(fp, "  xres = %d, yres = %d\n", pixc->xres, pixc->yres);
         fwrite(pixc->data, 1, pixc->size, fp);
         fprintf(fp, "\n");
@@ -1934,7 +1947,8 @@ FILE    *fp;
  *                             in the input
  * \param[in]    scalefactor   scaling factor applied to each image; > 0.0
  * \param[in]    type          encoding type (L_JPEG_ENCODE, L_G4_ENCODE,
- *                             L_FLATE_ENCODE, or L_DEFAULT_ENCODE for default
+ *                             L_FLATE_ENCODE, L_JP2K_ENCODE, or
+ *                             L_DEFAULT_ENCODE for default)
  * \param[in]    quality       used for JPEG only; 0 for default (75)
  * \param[in]    title         [optional] pdf title
  * \param[in]    fileout       pdf file of all images
@@ -1996,7 +2010,8 @@ size_t    nbytes;
  * \param[in]    res           input resolution of all images
  * \param[in]    scalefactor   scaling factor applied to each image; > 0.0
  * \param[in]    type          encoding type (L_JPEG_ENCODE, L_G4_ENCODE,
- *                             L_FLATE_ENCODE, or L_DEFAULT_ENCODE for default
+ *                             L_FLATE_ENCODE, L_JP2K_ENCODE, or
+ *                             L_DEFAULT_ENCODE for default)
  * \param[in]    quality       used for JPEG only; 0 for default (75)
  * \param[in]    title         [optional] pdf title
  * \param[out]   pdata         output pdf data (of all images
@@ -2036,7 +2051,9 @@ L_PTRA   *pa_data;
     if (!pixac)
         return ERROR_INT("pixac not defined", procName, 1);
     if (scalefactor <= 0.0) scalefactor = 1.0;
-    if (type < L_DEFAULT_ENCODE || type > L_FLATE_ENCODE) {
+    if (type != L_DEFAULT_ENCODE && type != L_JPEG_ENCODE &&
+        type != L_G4_ENCODE && type != L_FLATE_ENCODE &&
+        type != L_JP2K_ENCODE) {
         L_WARNING("invalid compression type; using per-page default\n",
                   procName);
         type = L_DEFAULT_ENCODE;
@@ -2062,6 +2079,8 @@ L_PTRA   *pa_data;
             pix = pixClone(pixs);
         pixDestroy(&pixs);
         scaledres = (l_int32)(res * scalefactor);
+
+            /* Select the encoding type */
         if (type != L_DEFAULT_ENCODE) {
             pagetype = type;
         } else if (selectDefaultPdfEncoding(pix, &pagetype) != 0) {
@@ -2070,6 +2089,7 @@ L_PTRA   *pa_data;
             pixDestroy(&pix);
             continue;
         }
+
         ret = pixConvertToPdfData(pix, pagetype, quality, &imdata, &imbytes,
                                   0, 0, scaledres, title, NULL, 0);
         pixDestroy(&pix);
@@ -2306,8 +2326,8 @@ pixcompWriteStreamInfo(FILE        *fp,
         fprintf(fp, "  Pixcomp Info:");
     fprintf(fp, " width = %d, height = %d, depth = %d\n",
             pixc->w, pixc->h, pixc->d);
-    fprintf(fp, "    xres = %d, yres = %d, size in bytes = %lu\n",
-            pixc->xres, pixc->yres, (unsigned long)pixc->size);
+    fprintf(fp, "    xres = %d, yres = %d, size in bytes = %zu\n",
+            pixc->xres, pixc->yres, pixc->size);
     if (pixc->cmapflag)
         fprintf(fp, "    has colormap\n");
     else
