@@ -118,7 +118,7 @@
  */
 
 #ifdef  HAVE_CONFIG_H
-#include "config_auto.h"
+#include <config_auto.h>
 #endif  /* HAVE_CONFIG_H */
 
 #include <string.h>
@@ -233,10 +233,11 @@ PIXCMAP     *cmap;
     png_init_io(png_ptr, fp);
 
         /* ---------------------------------------------------------- *
-         *  Set the transforms flags.  Whatever happens here,
-         *  NEVER invert 1 bpp using PNG_TRANSFORM_INVERT_MONO.
-         *  Also, do not use PNG_TRANSFORM_EXPAND, which would
-         *  expand all images with bpp < 8 to 8 bpp.
+         *  - Set the transforms flags.  Whatever happens here,
+         *    NEVER invert 1 bpp using PNG_TRANSFORM_INVERT_MONO.
+         *  - Do not use PNG_TRANSFORM_EXPAND, which would
+         *    expand all images with bpp < 8 to 8 bpp.
+         *  - Strip 16 --> 8 if reading 16-bit gray+alpha
          * ---------------------------------------------------------- */
         /* To strip 16 --> 8 bit depth, use PNG_TRANSFORM_STRIP_16 */
     if (var_PNG_STRIP_16_TO_8 == 1) {  /* our default */
@@ -266,10 +267,11 @@ PIXCMAP     *cmap;
     }
 
         /* Remove if/when this is implemented for all bit_depths */
-    if (spp == 3 && bit_depth != 8) {
-        fprintf(stderr, "Help: spp = 3 and depth = %d != 8\n!!", bit_depth);
+    if (spp != 1 && bit_depth != 8) {
+        L_ERROR("spp = %d and bps = %d != 8\n"
+                "turn on 16 --> 8 stripping\n", procName, spp, bit_depth);
         png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
-        return (PIX *)ERROR_PTR("not implemented for this depth",
+        return (PIX *)ERROR_PTR("not implemented for this image",
             procName, NULL);
     }
 
@@ -328,7 +330,9 @@ PIXCMAP     *cmap;
                 SET_DATA_BYTE(ppixel, COLOR_RED, rowptr[k++]);
                 SET_DATA_BYTE(ppixel, COLOR_GREEN, rowptr[k++]);
                 SET_DATA_BYTE(ppixel, COLOR_BLUE, rowptr[k++]);
-                if (spp == 4)
+                if (spp == 3)  /* set to opaque; some readers are buggy */
+                    SET_DATA_BYTE(ppixel, L_ALPHA_CHANNEL, 255);
+                else  /* spp == 4 */
                     SET_DATA_BYTE(ppixel, L_ALPHA_CHANNEL, rowptr[k++]);
                 ppixel++;
             }
@@ -373,16 +377,16 @@ PIXCMAP     *cmap;
             pixSetSpp(pix, 4);
 
 #if DEBUG_READ
-            fprintf(stderr, "ncolors = %d, num_trans = %d\n",
-                    ncolors, num_trans);
+            lept_stderr("ncolors = %d, num_trans = %d\n",
+                        ncolors, num_trans);
             for (i = 0; i < ncolors; i++) {
                 pixcmapGetColor(cmap, i, &rval, &gval, &bval);
                 if (i < num_trans) {
-                    fprintf(stderr, "(r,g,b,a) = (%d,%d,%d,%d)\n",
-                            rval, gval, bval, trans[i]);
+                    lept_stderr("(r,g,b,a) = (%d,%d,%d,%d)\n",
+                                rval, gval, bval, trans[i]);
                 } else {
-                    fprintf(stderr, "(r,g,b,a) = (%d,%d,%d,<<255>>)\n",
-                            rval, gval, bval);
+                    lept_stderr("(r,g,b,a) = (%d,%d,%d,<<255>>)\n",
+                                rval, gval, bval);
                 }
             }
 #endif  /* DEBUG_READ */
@@ -438,8 +442,7 @@ PIXCMAP     *cmap;
 #if  DEBUG_READ
     if (cmap) {
         for (i = 0; i < 16; i++) {
-            fprintf(stderr, "[%d] = %d\n", i,
-                   ((l_uint8 *)(cmap->array))[i]);
+            lept_stderr("[%d] = %d\n", i, ((l_uint8 *)(cmap->array))[i]);
         }
     }
 #endif  /* DEBUG_READ */
@@ -470,6 +473,7 @@ PIXCMAP     *cmap;
         if (!cmap) {
             pixInvert(pix, pix);
         } else {
+            L_INFO("removing opaque cmap from 1 bpp\n", procName);
             pix1 = pixRemoveColormap(pix, REMOVE_CMAP_BASED_ON_SRC);
             pixDestroy(&pix);
             pix = pix1;
@@ -498,10 +502,10 @@ PIXCMAP     *cmap;
  * \brief   readHeaderPng()
  *
  * \param[in]    filename
- * \param[out]   pw [optional]
- *           [out]   ph ([optional]
- *           [out]   pbps ([optional]  bits/sample
- * \param[out]   pspp [optional]  samples/pixel
+ * \param[out]   pw      [optional]
+ * \param[out]   ph      [optional]
+ * \param[out]   pbps    [optional]  bits/sample
+ * \param[out]   pspp    [optional]  samples/pixel
  * \param[out]   piscmap [optional]
  * \return  0 if OK, 1 on error
  *
@@ -513,7 +517,7 @@ PIXCMAP     *cmap;
  *          When a gray+alpha is read, it is converted to 32 bpp RGBA.
  * </pre>
  */
-l_int32
+l_ok
 readHeaderPng(const char *filename,
               l_int32    *pw,
               l_int32    *ph,
@@ -544,12 +548,12 @@ FILE    *fp;
 /*!
  * \brief   freadHeaderPng()
  *
- * \param[in]    fp file stream
- * \param[out]   pw [optional]
- *           [out]   ph ([optional]
- *           [out]   pbps ([optional]  bits/sample
- * \param[out]   pspp [optional]  samples/pixel
- * \param[out]   piscmap [optional]
+ * \param[in]    fp       file stream
+ * \param[out]   pw       [optional]
+ * \param[out]   ph       [optional]
+ * \param[out]   pbps     [optional]  bits/sample
+ * \param[out]   pspp     [optional]  samples/pixel
+ * \param[out]   piscmap  [optional]
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -557,7 +561,7 @@ FILE    *fp;
  *      (1) See readHeaderPng().  We only need the first 40 bytes in the file.
  * </pre>
  */
-l_int32
+l_ok
 freadHeaderPng(FILE     *fp,
                l_int32  *pw,
                l_int32  *ph,
@@ -592,11 +596,11 @@ l_uint8  data[40];
  * \brief   readHeaderMemPng()
  *
  * \param[in]    data
- * \param[in]    size 40 bytes is sufficient
- * \param[out]   pw [optional]
- *           [out]   ph ([optional]
- *           [out]   pbps ([optional]  bits/sample
- * \param[out]   pspp [optional]  samples/pixel
+ * \param[in]    size    40 bytes is sufficient
+ * \param[out]   pw      [optional]
+ * \param[out]   ph      [optional]
+ * \param[out]   pbps    [optional]  bits/sample
+ * \param[out]   pspp    [optional]  samples/pixel
  * \param[out]   piscmap [optional]  input NULL to ignore
  * \return  0 if OK, 1 on error
  *
@@ -614,7 +618,7 @@ l_uint8  data[40];
  *            4 and 6 have separate alpha samples with each pixel.
  * </pre>
  */
-l_int32
+l_ok
 readHeaderMemPng(const l_uint8  *data,
                  size_t          size,
                  l_int32        *pw,
@@ -625,7 +629,7 @@ readHeaderMemPng(const l_uint8  *data,
 {
 l_uint16   twobytes;
 l_uint16  *pshort;
-l_int32    colortype, bps, spp;
+l_int32    colortype, w, h, bps, spp;
 l_uint32  *pword;
 
     PROCNAME("readHeaderMemPng");
@@ -648,8 +652,10 @@ l_uint32  *pword;
 
     pword = (l_uint32 *)data;
     pshort = (l_uint16 *)data;
-    if (pw) *pw = convertOnLittleEnd32(pword[4]);
-    if (ph) *ph = convertOnLittleEnd32(pword[5]);
+    w = convertOnLittleEnd32(pword[4]);
+    h = convertOnLittleEnd32(pword[5]);
+    if (w < 1 || h < 1)
+        return ERROR_INT("invalid w or h", procName, 1);
     twobytes = convertOnLittleEnd16(pshort[12]); /* contains depth/sample  */
                                                  /* and the color type     */
     colortype = twobytes & 0xff;  /* color type */
@@ -672,6 +678,12 @@ l_uint32  *pword;
     } else {  /* gray (0) or cmap (3) or cmap+alpha (3) */
         spp = 1;
     }
+    if (bps < 1 || bps > 16) {
+        L_ERROR("invalid bps = %d\n", procName, bps);
+        return 1;
+    }
+    if (pw) *pw = w;
+    if (ph) *ph = h;
     if (pbps) *pbps = bps;
     if (pspp) *pspp = spp;
     if (piscmap) {
@@ -757,7 +769,7 @@ png_infop    info_ptr;
  * \param[out]   pinterlaced 1 if interlaced png; 0 otherwise
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 isPngInterlaced(const char *filename,
                 l_int32    *pinterlaced)
 {
@@ -786,13 +798,13 @@ FILE    *fp;
 
 
 /*
- *  fgetPngColormapInfo()
+ * \brief   fgetPngColormapInfo()
  *
- *      Input:  fp (file stream opened for read)
- *              &cmap (optional <return>; use NULL to skip)
- *              &transparency (optional <return> 1 if colormapped with
- *                transparency, 0 otherwise; use NULL to skip)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    fp     file stream opened for read
+ * \param[out]   pcmap  optional; use NULL to skip
+ * \param[out]   ptransparency   optional; 1 if colormapped with
+ *                      transparency, 0 otherwise; use NULL to skip
+ * \return  0 if OK, 1 on error
  *
  *  Notes:
  *      (1) The transparency information in a png is in the tRNA array,
@@ -801,7 +813,7 @@ FILE    *fp;
  *          transparency.
  *      (2) Side-effect: this rewinds the stream.
  */
-l_int32
+l_ok
 fgetPngColormapInfo(FILE      *fp,
                     PIXCMAP  **pcmap,
                     l_int32   *ptransparency)
@@ -902,7 +914,7 @@ png_infop    info_ptr;
  *          When using pixWrite(), no field is given for gamma.
  * </pre>
  */
-l_int32
+l_ok
 pixWritePng(const char  *filename,
             PIX         *pix,
             l_float32    gamma)
@@ -1002,7 +1014,7 @@ FILE  *fp;
  *          to a 4 spp rgba image.
  * </pre>
  */
-l_int32
+l_ok
 pixWriteStreamPng(FILE      *fp,
                   PIX       *pix,
                   l_float32  gamma)
@@ -1088,8 +1100,8 @@ char        *text;
         color_type = PNG_COLOR_TYPE_PALETTE;  /* 3 */
 
 #if  DEBUG_WRITE
-    fprintf(stderr, "cmflag = %d, bit_depth = %d, color_type = %d\n",
-            cmflag, bit_depth, color_type);
+    lept_stderr("cmflag = %d, bit_depth = %d, color_type = %d\n",
+                cmflag, bit_depth, color_type);
 #endif  /* DEBUG_WRITE */
 
     png_set_IHDR(png_ptr, info_ptr, w, h, bit_depth, color_type,
@@ -1246,7 +1258,7 @@ char        *text;
  *          compression integers given above, you must include zlib.h.
  * </pre>
  */
-l_int32
+l_ok
 pixSetZlibCompression(PIX     *pix,
                       l_int32  compval)
 {
@@ -1461,6 +1473,9 @@ memio_png_read_data(png_structp  png_ptr,
 MEMIODATA  *thing;
 
     thing = (MEMIODATA *)png_get_io_ptr(png_ptr);
+    if (byteCountToRead > (thing->m_Size - thing->m_Count)) {
+        png_error(png_ptr, "read error in memio_png_read_data");
+    }
     memcpy(outBytes, thing->m_Buffer + thing->m_Count, byteCountToRead);
     thing->m_Count += byteCountToRead;
 }
@@ -1609,7 +1624,7 @@ MEMIODATA    state;
 
         /* Remove if/when this is implemented for all bit_depths */
     if (spp == 3 && bit_depth != 8) {
-        fprintf(stderr, "Help: spp = 3 and depth = %d != 8\n!!", bit_depth);
+        lept_stderr("Help: spp = 3 and depth = %d != 8\n!!", bit_depth);
         png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
         return (PIX *)ERROR_PTR("not implemented for this depth",
             procName, NULL);
@@ -1716,16 +1731,16 @@ MEMIODATA    state;
             pixSetSpp(pix, 4);
 
 #if DEBUG_READ
-            fprintf(stderr, "ncolors = %d, num_trans = %d\n",
-                    ncolors, num_trans);
+            lept_stderr("ncolors = %d, num_trans = %d\n",
+                        ncolors, num_trans);
             for (i = 0; i < ncolors; i++) {
                 pixcmapGetColor(cmap, i, &rval, &gval, &bval);
                 if (i < num_trans) {
-                    fprintf(stderr, "(r,g,b,a) = (%d,%d,%d,%d)\n",
-                            rval, gval, bval, trans[i]);
+                    lept_stderr("(r,g,b,a) = (%d,%d,%d,%d)\n",
+                                rval, gval, bval, trans[i]);
                 } else {
-                    fprintf(stderr, "(r,g,b,a) = (%d,%d,%d,<<255>>)\n",
-                            rval, gval, bval);
+                    lept_stderr("(r,g,b,a) = (%d,%d,%d,<<255>>)\n",
+                                rval, gval, bval);
                 }
             }
 #endif  /* DEBUG_READ */
@@ -1781,8 +1796,7 @@ MEMIODATA    state;
 #if  DEBUG_READ
     if (cmap) {
         for (i = 0; i < 16; i++) {
-            fprintf(stderr, "[%d] = %d\n", i,
-                   ((l_uint8 *)(cmap->array))[i]);
+            lept_stderr("[%d] = %d\n", i, ((l_uint8 *)(cmap->array))[i]);
         }
     }
 #endif  /* DEBUG_READ */
@@ -1851,7 +1865,7 @@ MEMIODATA    state;
  *      (1) See pixWriteStreamPng()
  * </pre>
  */
-l_int32
+l_ok
 pixWriteMemPng(l_uint8  **pfiledata,
                size_t    *pfilesize,
                PIX       *pix,
@@ -1949,8 +1963,8 @@ MEMIODATA    state;
         color_type = PNG_COLOR_TYPE_PALETTE;  /* 3 */
 
 #if  DEBUG_WRITE
-    fprintf(stderr, "cmflag = %d, bit_depth = %d, color_type = %d\n",
-            cmflag, bit_depth, color_type);
+    lept_stderr("cmflag = %d, bit_depth = %d, color_type = %d\n",
+                cmflag, bit_depth, color_type);
 #endif  /* DEBUG_WRITE */
 
     png_set_IHDR(png_ptr, info_ptr, w, h, bit_depth, color_type,
@@ -2101,4 +2115,3 @@ MEMIODATA    state;
 /* --------------------------------------------*/
 #endif  /* HAVE_LIBPNG */
 /* --------------------------------------------*/
-
