@@ -111,7 +111,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config_auto.h"
+#include <config_auto.h>
 #endif  /* HAVE_CONFIG_H */
 
 #include <string.h>
@@ -138,14 +138,14 @@ static const l_int32  MAX_PNM_HEIGHT = 100000;
 /*!
  * \brief   pixReadStreamPnm()
  *
- * \param[in]    fp file stream opened for read
+ * \param[in]    fp   file stream opened for read
  * \return  pix, or NULL on error
  */
 PIX *
 pixReadStreamPnm(FILE  *fp)
 {
 l_uint8    val8, rval8, gval8, bval8, aval8, mask8;
-l_uint16   val16, rval16, gval16, bval16, aval16, mask16;
+l_uint16   val16, rval16, gval16, bval16, aval16;
 l_int32    w, h, d, bps, spp, bpl, wpl, i, j, type;
 l_int32    val, rval, gval, bval;
 l_uint32   rgbval;
@@ -158,25 +158,32 @@ PIX       *pix;
         return (PIX *)ERROR_PTR("fp not defined", procName, NULL);
 
     if (freadHeaderPnm(fp, &w, &h, &d, &type, &bps, &spp))
-        return (PIX *)ERROR_PTR( "header read failed", procName, NULL);
+        return (PIX *)ERROR_PTR("header read failed", procName, NULL);
     if (bps < 1 || bps > 16)
-        return (PIX *)ERROR_PTR( "invalid bps", procName, NULL);
+        return (PIX *)ERROR_PTR("invalid bps", procName, NULL);
     if (spp < 1 || spp > 4)
-        return (PIX *)ERROR_PTR( "invalid spp", procName, NULL);
+        return (PIX *)ERROR_PTR("invalid spp", procName, NULL);
     if ((pix = pixCreate(w, h, d)) == NULL)
-        return (PIX *)ERROR_PTR( "pix not made", procName, NULL);
+        return (PIX *)ERROR_PTR("pix not made", procName, NULL);
     pixSetInputFormat(pix, IFF_PNM);
     data = pixGetData(pix);
     wpl = pixGetWpl(pix);
 
+        /* If type == 6 and bps == 16, we use the code in type 7
+         * to read 6 bytes/pixel from the input file. */
+    if (type == 6 && bps == 16)
+        type = 7;
+
     switch (type) {
     case 1:
     case 2:
         /* Old "ASCII" binary or gray format */
         for (i = 0; i < h; i++) {
             for (j = 0; j < w; j++) {
-                if (pnmReadNextAsciiValue(fp, &val))
-                    return (PIX *)ERROR_PTR( "read abend", procName, pix);
+                if (pnmReadNextAsciiValue(fp, &val)) {
+                    pixDestroy(&pix);
+                    return (PIX *)ERROR_PTR("read abend", procName, NULL);
+                }
                 pixSetPixel(pix, j, i, val);
             }
         }
@@ -186,12 +193,18 @@ PIX       *pix;
         /* Old "ASCII" rgb format */
         for (i = 0; i < h; i++) {
             for (j = 0; j < w; j++) {
-                if (pnmReadNextAsciiValue(fp, &rval))
-                    return (PIX *)ERROR_PTR("read abend", procName, pix);
-                if (pnmReadNextAsciiValue(fp, &gval))
-                    return (PIX *)ERROR_PTR("read abend", procName, pix);
-                if (pnmReadNextAsciiValue(fp, &bval))
-                    return (PIX *)ERROR_PTR("read abend", procName, pix);
+                if (pnmReadNextAsciiValue(fp, &rval)) {
+                    pixDestroy(&pix);
+                    return (PIX *)ERROR_PTR("read abend", procName, NULL);
+                }
+                if (pnmReadNextAsciiValue(fp, &gval)) {
+                    pixDestroy(&pix);
+                    return (PIX *)ERROR_PTR("read abend", procName, NULL);
+                }
+                if (pnmReadNextAsciiValue(fp, &bval)) {
+                    pixDestroy(&pix);
+                    return (PIX *)ERROR_PTR("read abend", procName, NULL);
+                }
                 composeRGBPixel(rval, gval, bval, &rgbval);
                 pixSetPixel(pix, j, i, rgbval);
             }
@@ -204,8 +217,10 @@ PIX       *pix;
         for (i = 0; i < h; i++) {
             line = data + i * wpl;
             for (j = 0; j < bpl; j++) {
-                if (fread(&val8, 1, 1, fp) != 1)
-                    return (PIX *)ERROR_PTR("read error in 4", procName, pix);
+                if (fread(&val8, 1, 1, fp) != 1) {
+                    pixDestroy(&pix);
+                    return (PIX *)ERROR_PTR("read error in 4", procName, NULL);
+                }
                 SET_DATA_BYTE(line, j, val8);
             }
         }
@@ -217,8 +232,10 @@ PIX       *pix;
             line = data + i * wpl;
             if (d != 16) {
                 for (j = 0; j < w; j++) {
-                    if (fread(&val8, 1, 1, fp) != 1)
-                        return (PIX *)ERROR_PTR("error in 5", procName, pix);
+                    if (fread(&val8, 1, 1, fp) != 1) {
+                        pixDestroy(&pix);
+                        return (PIX *)ERROR_PTR("error in 5", procName, NULL);
+                    }
                     if (d == 2)
                         SET_DATA_DIBIT(line, j, val8);
                     else if (d == 4)
@@ -228,8 +245,10 @@ PIX       *pix;
                 }
             } else {  /* d == 16 */
                 for (j = 0; j < w; j++) {
-                    if (fread(&val16, 2, 1, fp) != 1)
-                        return (PIX *)ERROR_PTR("16 bpp error", procName, pix);
+                    if (fread(&val16, 2, 1, fp) != 1) {
+                        pixDestroy(&pix);
+                        return (PIX *)ERROR_PTR("16 bpp error", procName, NULL);
+                    }
                     SET_DATA_TWO_BYTES(line, j, val16);
                 }
             }
@@ -237,24 +256,29 @@ PIX       *pix;
         break;
 
     case 6:
-        /* "raw" format, type == 6; rgb */
+        /* "raw" format, type == 6; 8 bps, rgb */
         for (i = 0; i < h; i++) {
             line = data + i * wpl;
             for (j = 0; j < wpl; j++) {
-                if (fread(&rval8, 1, 1, fp) != 1)
+                if (fread(&rval8, 1, 1, fp) != 1) {
+                    pixDestroy(&pix);
                     return (PIX *)ERROR_PTR("read error type 6",
-                                            procName, pix);
-                if (fread(&gval8, 1, 1, fp) != 1)
+                                            procName, NULL);
+                }
+                if (fread(&gval8, 1, 1, fp) != 1) {
+                    pixDestroy(&pix);
                     return (PIX *)ERROR_PTR("read error type 6",
-                                            procName, pix);
-                if (fread(&bval8, 1, 1, fp) != 1)
+                                            procName, NULL);
+                }
+                if (fread(&bval8, 1, 1, fp) != 1) {
+                    pixDestroy(&pix);
                     return (PIX *)ERROR_PTR("read error type 6",
-                                            procName, pix);
+                                            procName, NULL);
+                }
                 composeRGBPixel(rval8, gval8, bval8, &rgbval);
                 line[j] = rgbval;
             }
         }
-        pixSetSpp(pix, 4);
         break;
 
     case 7:
@@ -265,9 +289,11 @@ PIX       *pix;
             case 1: /* 1, 2, 4, 8 bpp grayscale */
                 for (i = 0; i < h; i++) {
                     for (j = 0; j < w; j++) {
-                        if (fread(&val8, 1, 1, fp) != 1)
+                        if (fread(&val8, 1, 1, fp) != 1) {
+                            pixDestroy(&pix);
                             return (PIX *)ERROR_PTR("read error type 7",
-                                                    procName, pix);
+                                                    procName, NULL);
+                        }
                         val8 = val8 & mask8;
                         if (bps == 1) val8 ^= 1;  /* white-is-1 photometry */
                         pixSetPixel(pix, j, i, val8);
@@ -278,12 +304,16 @@ PIX       *pix;
             case 2: /* 1, 2, 4, 8 bpp grayscale + alpha */
                 for (i = 0; i < h; i++) {
                     for (j = 0; j < w; j++) {
-                        if (fread(&val8, 1, 1, fp) != 1)
+                        if (fread(&val8, 1, 1, fp) != 1) {
+                            pixDestroy(&pix);
                             return (PIX *)ERROR_PTR("read error type 7",
-                                                    procName, pix);
-                        if (fread(&aval8, 1, 1, fp) != 1)
+                                                    procName, NULL);
+                        }
+                        if (fread(&aval8, 1, 1, fp) != 1) {
+                            pixDestroy(&pix);
                             return (PIX *)ERROR_PTR("read error type 7",
-                                                    procName, pix);
+                                                    procName, NULL);
+                        }
                         val8 = val8 & mask8;
                         aval8 = aval8 & mask8;
                         composeRGBAPixel(val8, val8, val8, aval8, &rgbval);
@@ -297,15 +327,21 @@ PIX       *pix;
                 for (i = 0; i < h; i++) {
                     line = data + i * wpl;
                     for (j = 0; j < wpl; j++) {
-                        if (fread(&rval8, 1, 1, fp) != 1)
+                        if (fread(&rval8, 1, 1, fp) != 1) {
+                            pixDestroy(&pix);
                             return (PIX *)ERROR_PTR("read error type 7",
-                                                    procName, pix);
-                        if (fread(&gval8, 1, 1, fp) != 1)
+                                                    procName, NULL);
+                        }
+                        if (fread(&gval8, 1, 1, fp) != 1) {
+                            pixDestroy(&pix);
                             return (PIX *)ERROR_PTR("read error type 7",
-                                                    procName, pix);
-                        if (fread(&bval8, 1, 1, fp) != 1)
+                                                    procName, NULL);
+                        }
+                        if (fread(&bval8, 1, 1, fp) != 1) {
+                            pixDestroy(&pix);
                             return (PIX *)ERROR_PTR("read error type 7",
-                                                    procName, pix);
+                                                    procName, NULL);
+                        }
                         rval8 = rval8 & mask8;
                         gval8 = gval8 & mask8;
                         bval8 = bval8 & mask8;
@@ -319,18 +355,26 @@ PIX       *pix;
                 for (i = 0; i < h; i++) {
                     line = data + i * wpl;
                     for (j = 0; j < wpl; j++) {
-                        if (fread(&rval8, 1, 1, fp) != 1)
+                        if (fread(&rval8, 1, 1, fp) != 1) {
+                            pixDestroy(&pix);
                             return (PIX *)ERROR_PTR("read error type 7",
-                                                    procName, pix);
-                        if (fread(&gval8, 1, 1, fp) != 1)
+                                                    procName, NULL);
+                        }
+                        if (fread(&gval8, 1, 1, fp) != 1) {
+                            pixDestroy(&pix);
                             return (PIX *)ERROR_PTR("read error type 7",
-                                                    procName, pix);
-                        if (fread(&bval8, 1, 1, fp) != 1)
+                                                    procName, NULL);
+                        }
+                        if (fread(&bval8, 1, 1, fp) != 1) {
+                            pixDestroy(&pix);
                             return (PIX *)ERROR_PTR("read error type 7",
-                                                    procName, pix);
-                        if (fread(&aval8, 1, 1, fp) != 1)
+                                                    procName, NULL);
+                        }
+                        if (fread(&aval8, 1, 1, fp) != 1) {
+                            pixDestroy(&pix);
                             return (PIX *)ERROR_PTR("read error type 7",
-                                                    procName, pix);
+                                                    procName, NULL);
+                        }
                         rval8 = rval8 & mask8;
                         gval8 = gval8 & mask8;
                         bval8 = bval8 & mask8;
@@ -343,31 +387,41 @@ PIX       *pix;
                 break;
             }
         } else {  /* bps == 16 */
-            mask16 = (1 << 16) - 1;
+                /* I have only seen one example that is type 6, 16 bps.
+                 * It was 3 spp (rgb), and the 8 bps of real data was stored
+                 * in the second byte.  In the following, I make the wild
+                 * assumption that for all 16 bpp pnm/pam files, we can
+                 * take the second byte. */
             switch (spp) {
-            case 1: /* 16 bpp grayscale */
+            case 1: /* 16 bps grayscale */
                 for (i = 0; i < h; i++) {
                     for (j = 0; j < w; j++) {
-                        if (fread(&val16, 2, 1, fp) != 1)
+                        if (fread(&val16, 2, 1, fp) != 1) {
+                            pixDestroy(&pix);
                             return (PIX *)ERROR_PTR("read error type 7",
-                                                    procName, pix);
-                        val8 = (val16 & mask16) >> 8;
+                                                    procName, NULL);
+                        }
+                        val8 = val16 & 0xff;
                         pixSetPixel(pix, j, i, val8);
                     }
                 }
                 break;
 
-            case 2: /* 16 bpp grayscale + alpha */
+            case 2: /* 16 bps grayscale + alpha */
                 for (i = 0; i < h; i++) {
                     for (j = 0; j < w; j++) {
-                        if (fread(&val16, 2, 1, fp) != 1)
+                        if (fread(&val16, 2, 1, fp) != 1) {
+                            pixDestroy(&pix);
                             return (PIX *)ERROR_PTR("read error type 7",
-                                                    procName, pix);
-                        if (fread(&aval16, 2, 1, fp) != 1)
+                                                    procName, NULL);
+                        }
+                        if (fread(&aval16, 2, 1, fp) != 1) {
+                            pixDestroy(&pix);
                             return (PIX *)ERROR_PTR("read error type 7",
-                                                    procName, pix);
-                        val8 = (val16 & mask16) >> 8;
-                        aval8 = (aval16 & mask16) >> 8;
+                                                    procName, NULL);
+                        }
+                        val8 = val16 & 0xff;
+                        aval8 = aval16 & 0xff;
                         composeRGBAPixel(val8, val8, val8, aval8, &rgbval);
                         pixSetPixel(pix, j, i, rgbval);
                     }
@@ -375,48 +429,62 @@ PIX       *pix;
                 pixSetSpp(pix, 4);
                 break;
 
-            case 3: /* 16bpp rgb */
+            case 3: /* 16bps rgb */
                 for (i = 0; i < h; i++) {
                     line = data + i * wpl;
                     for (j = 0; j < wpl; j++) {
-                        if (fread(&rval16, 2, 1, fp) != 1)
+                        if (fread(&rval16, 2, 1, fp) != 1) {
+                            pixDestroy(&pix);
                             return (PIX *)ERROR_PTR("read error type 7",
-                                                    procName, pix);
-                        if (fread(&gval16, 2, 1, fp) != 1)
+                                                    procName, NULL);
+                        }
+                        if (fread(&gval16, 2, 1, fp) != 1) {
+                            pixDestroy(&pix);
                             return (PIX *)ERROR_PTR("read error type 7",
-                                                    procName, pix);
-                        if (fread(&bval16, 2, 1, fp) != 1)
+                                                    procName, NULL);
+                        }
+                        if (fread(&bval16, 2, 1, fp) != 1) {
+                            pixDestroy(&pix);
                             return (PIX *)ERROR_PTR("read error type 7",
-                                                    procName, pix);
-                        rval8 = (rval16 & mask16) >> 8;
-                        gval8 = (gval16 & mask16) >> 8;
-                        bval8 = (bval16 & mask16) >> 8;
+                                                    procName, NULL);
+                        }
+                        rval8 = rval16 & 0xff;
+                        gval8 = gval16 & 0xff;
+                        bval8 = bval16 & 0xff;
                         composeRGBPixel(rval8, gval8, bval8, &rgbval);
                         line[j] = rgbval;
                     }
                 }
                 break;
 
-            case 4: /* 16bpp rgba */
+            case 4: /* 16bps rgba */
                 for (i = 0; i < h; i++) {
                     line = data + i * wpl;
                     for (j = 0; j < wpl; j++) {
-                        if (fread(&rval16, 2, 1, fp) != 1)
+                        if (fread(&rval16, 2, 1, fp) != 1) {
+                            pixDestroy(&pix);
                             return (PIX *)ERROR_PTR("read error type 7",
-                                                    procName, pix);
-                        if (fread(&gval16, 2, 1, fp) != 1)
+                                                    procName, NULL);
+                        }
+                        if (fread(&gval16, 2, 1, fp) != 1) {
+                            pixDestroy(&pix);
                             return (PIX *)ERROR_PTR("read error type 7",
-                                                    procName, pix);
-                        if (fread(&bval16, 2, 1, fp) != 1)
+                                                    procName, NULL);
+                        }
+                        if (fread(&bval16, 2, 1, fp) != 1) {
+                            pixDestroy(&pix);
                             return (PIX *)ERROR_PTR("read error type 7",
-                                                    procName, pix);
-                        if (fread(&aval16, 2, 1, fp) != 1)
+                                                    procName, NULL);
+                        }
+                        if (fread(&aval16, 2, 1, fp) != 1) {
+                            pixDestroy(&pix);
                             return (PIX *)ERROR_PTR("read error type 7",
-                                                    procName, pix);
-                        rval8 = (rval16 & mask16) >> 8;
-                        gval8 = (gval16 & mask16) >> 8;
-                        bval8 = (bval16 & mask16) >> 8;
-                        aval8 = (aval16 & mask16) >> 8;
+                                                    procName, NULL);
+                        }
+                        rval8 = rval16 & 0xff;
+                        gval8 = gval16 & 0xff;
+                        bval8 = bval16 & 0xff;
+                        aval8 = aval16 & 0xff;
                         composeRGBAPixel(rval8, gval8, bval8, aval8, &rgbval);
                         line[j] = rgbval;
                     }
@@ -435,15 +503,15 @@ PIX       *pix;
  * \brief   readHeaderPnm()
  *
  * \param[in]    filename
- * \param[out]   pw [optional]
- * \param[out]   ph [optional]
- * \param[out]   pd [optional]
- * \param[out]   ptype [optional] pnm type
- * \param[out]   pbps [optional]  bits/sample
- * \param[out]   pspp [optional]  samples/pixel
+ * \param[out]   pw       [optional]
+ * \param[out]   ph       [optional]
+ * \param[out]   pd       [optional]
+ * \param[out]   ptype    [optional] pnm type
+ * \param[out]   pbps     [optional] bits/sample
+ * \param[out]   pspp     [optional] samples/pixel
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 readHeaderPnm(const char *filename,
               l_int32    *pw,
               l_int32    *ph,
@@ -477,16 +545,16 @@ FILE    *fp;
 /*!
  * \brief   freadHeaderPnm()
  *
- * \param[in]    fp file stream opened for read
- * \param[out]   pw [optional]
- * \param[out]   ph [optional]
- * \param[out]   pd [optional]
- * \param[out]   ptype [optional] pnm type
- * \param[out]   pbps [optional]  bits/sample
- * \param[out]   pspp [optional]  samples/pixel
+ * \param[in]    fp     file stream opened for read
+ * \param[out]   pw     [optional]
+ * \param[out]   ph     [optional]
+ * \param[out]   pd     [optional]
+ * \param[out]   ptype  [optional] pnm type
+ * \param[out]   pbps   [optional]  bits/sample
+ * \param[out]   pspp   [optional]  samples/pixel
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 freadHeaderPnm(FILE     *fp,
                l_int32  *pw,
                l_int32  *ph,
@@ -621,7 +689,7 @@ l_int32  ch;
             } else if (maxval == 0xffff) {
                 d = 16;
             } else {
-                fprintf(stderr, "maxval = %d\n", maxval);
+                lept_stderr("maxval = %d\n", maxval);
                 return ERROR_INT("invalid maxval", procName, 1);
             }
             bps = d;
@@ -629,11 +697,13 @@ l_int32  ch;
         } else {  /* type == 3 || type == 6; this is rgb  */
             if (pnmReadNextNumber(fp, &maxval))
                 return ERROR_INT("invalid read for maxval (3,6)", procName, 1);
-            if (maxval != 255)
-                L_WARNING("unexpected maxval = %d\n", procName, maxval);
+            if (maxval != 255 && maxval != 0xffff) {
+                L_ERROR("unexpected maxval = %d\n", procName, maxval);
+                return 1;
+            }
+            bps = (maxval == 255) ? 8 : 16;
             d = 32;
             spp = 3;
-            bps = 8;
         }
     }
     if (pw) *pw = w;
@@ -649,8 +719,8 @@ l_int32  ch;
 /*!
  * \brief   pixWriteStreamPnm()
  *
- * \param[in]    fp file stream opened for write
- * \param[in]    pix
+ * \param[in]   fp    file stream opened for write
+ * \param[in]   pix
  * \return  0 if OK; 1 on error
  *
  * <pre>
@@ -663,7 +733,7 @@ l_int32  ch;
  *          write them out as a packed array of bytes (3 to a pixel).
  * </pre>
  */
-l_int32
+l_ok
 pixWriteStreamPnm(FILE  *fp,
                   PIX   *pix)
 {
@@ -773,8 +843,8 @@ PIX       *pixs;
 /*!
  * \brief   pixWriteStreamAsciiPnm()
  *
- * \param[in]    fp file stream opened for write
- * \param[in]    pix
+ * \param[in]   fp    file stream opened for write
+ * \param[in]   pix
  * \return  0 if OK; 1 on error
  *
  *  Writes "ASCII" format only:
@@ -782,7 +852,7 @@ PIX       *pixs;
  *      2, 4, 8, 16 bpp, no colormap or grayscale colormap --> pgm P2
  *      2, 4, 8 bpp with color-valued colormap, or rgb --> rgb ppm P3
  */
-l_int32
+l_ok
 pixWriteStreamAsciiPnm(FILE  *fp,
                        PIX   *pix)
 {
@@ -893,8 +963,8 @@ PIX       *pixs;
 /*!
  * \brief   pixWriteStreamPam()
  *
- * \param[in]    fp file stream opened for write
- * \param[in]    pix
+ * \param[in]   fp    file stream opened for write
+ * \param[in]   pix
  * \return  0 if OK; 1 on error
  *
  * <pre>
@@ -904,7 +974,7 @@ PIX       *pixs;
  *          write them out as a packed array of bytes (3 to a pixel).
  * </pre>
  */
-l_int32
+l_ok
 pixWriteStreamPam(FILE  *fp,
                   PIX   *pix)
 {
@@ -1071,8 +1141,8 @@ PIX       *pixs;
 /*!
  * \brief   pixReadMemPnm()
  *
- * \param[in]    data const; pnm-encoded
- * \param[in]    size of data
+ * \param[in]   data   const; pnm-encoded
+ * \param[in]   size   of data
  * \return  pix, or NULL on error
  *
  * <pre>
@@ -1103,17 +1173,17 @@ PIX   *pix;
 /*!
  * \brief   readHeaderMemPnm()
  *
- * \param[in]    data const; pnm-encoded
- * \param[in]    size of data
- * \param[out]   pw [optional]
- * \param[out]   ph [optional]
- * \param[out]   pd [optional]
- * \param[out]   ptype [optional] pnm type
- * \param[out]   pbps [optional]  bits/sample
- * \param[out]   pspp [optional]  samples/pixel
+ * \param[in]    data    const; pnm-encoded
+ * \param[in]    size    of data
+ * \param[out]   pw      [optional]
+ * \param[out]   ph      [optional]
+ * \param[out]   pd      [optional]
+ * \param[out]   ptype   [optional] pnm type
+ * \param[out]   pbps    [optional] bits/sample
+ * \param[out]   pspp    [optional] samples/pixel
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 readHeaderMemPnm(const l_uint8  *data,
                  size_t          size,
                  l_int32        *pw,
@@ -1144,8 +1214,8 @@ FILE    *fp;
 /*!
  * \brief   pixWriteMemPnm()
  *
- * \param[out]   pdata data of PNM image
- * \param[out]   psize size of returned data
+ * \param[out]   pdata   data of PNM image
+ * \param[out]   psize   size of returned data
  * \param[in]    pix
  * \return  0 if OK, 1 on error
  *
@@ -1155,7 +1225,7 @@ FILE    *fp;
  *          memory instead of to a file stream.
  * </pre>
  */
-l_int32
+l_ok
 pixWriteMemPnm(l_uint8  **pdata,
                size_t    *psize,
                PIX       *pix)
@@ -1199,8 +1269,8 @@ FILE    *fp;
 /*!
  * \brief   pixWriteMemPam()
  *
- * \param[out]   pdata data of PAM image
- * \param[out]   psize size of returned data
+ * \param[out]   pdata   data of PAM image
+ * \param[out]   psize   size of returned data
  * \param[in]    pix
  * \return  0 if OK, 1 on error
  *
@@ -1210,7 +1280,7 @@ FILE    *fp;
  *          memory instead of to a file stream.
  * </pre>
  */
-l_int32
+l_ok
 pixWriteMemPam(l_uint8  **pdata,
                size_t    *psize,
                PIX       *pix)
@@ -1290,8 +1360,8 @@ l_int32   c, ignore;
 /*!
  * \brief   pnmReadNextNumber()
  *
- * \param[in]    fp file stream
- * \param[out]   pval value as an integer
+ * \param[in]    fp    file stream
+ * \param[out]   pval  value as an integer
  * \return  0 if OK, 1 on error or EOF.
  *
  * <pre>
@@ -1344,9 +1414,9 @@ l_int32   i, c, foundws;
 /*!
  * \brief   pnmReadNextString()
  *
- * \param[in]    fp file stream
- * \param[out]   buff pointer to the string buffer
- * \param[in]    size max. number of charactes in buffer
+ * \param[in]    fp    file stream
+ * \param[out]   buff  pointer to the string buffer
+ * \param[in]    size  max. number of charactes in buffer
  * \return  0 if OK, 1 on error or EOF.
  *
  * <pre>
