@@ -24,7 +24,6 @@
  -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
-
 /*!
  * \file  ptabasic.c
  * <pre>
@@ -93,23 +92,27 @@
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include "allheaders.h"
 
-static const l_int32  INITIAL_PTR_ARRAYSIZE = 20;   /* n'import quoi */
+static const l_uint32  MaxPtrArraySize = 10000000;
+static const l_int32 InitialPtrArraySize = 50;      /*!< n'importe quoi */
 
     /* Static functions */
 static l_int32 ptaExtendArrays(PTA *pta);
 static l_int32 ptaaExtendArray(PTAA *ptaa);
 
-
 /*---------------------------------------------------------------------*
  *                Pta creation, destruction, copy, clone               *
  *---------------------------------------------------------------------*/
 /*!
  * \brief   ptaCreate()
  *
- * \param[in]    n  initial array sizes
+ * \param[in]    n    initial array sizes
  * \return  pta, or NULL on error.
  */
 PTA *
@@ -119,14 +122,13 @@ PTA  *pta;
 
     PROCNAME("ptaCreate");
 
-    if (n <= 0)
-        n = INITIAL_PTR_ARRAYSIZE;
+    if (n <= 0 || n > MaxPtrArraySize)
+        n = InitialPtrArraySize;
 
     pta = (PTA *)LEPT_CALLOC(1, sizeof(PTA));
     pta->n = 0;
     pta->nalloc = n;
     ptaChangeRefcount(pta, 1);  /* sets to 1 */
-
     pta->x = (l_float32 *)LEPT_CALLOC(n, sizeof(l_float32));
     pta->y = (l_float32 *)LEPT_CALLOC(n, sizeof(l_float32));
     if (!pta->x || !pta->y) {
@@ -141,7 +143,7 @@ PTA  *pta;
 /*!
  * \brief   ptaCreateFromNuma()
  *
- * \param[in]    nax [optional] can be null
+ * \param[in]    nax   [optional] can be null
  * \param[in]    nay
  * \return  pta, or NULL on error.
  */
@@ -179,7 +181,7 @@ PTA       *pta;
 /*!
  * \brief   ptaDestroy()
  *
- * \param[in,out]   ppta to be nulled
+ * \param[in,out]   ppta   will be set to null before returning
  * \return  void
  *
  * <pre>
@@ -249,8 +251,8 @@ PTA       *npta;
  * \brief   ptaCopyRange()
  *
  * \param[in]    ptas
- * \param[in]    istart  starting index in ptas
- * \param[in]    iend  ending index in ptas; use 0 to copy to end
+ * \param[in]    istart    starting index in ptas
+ * \param[in]    iend      ending index in ptas; use 0 to copy to end
  * \return  0 if OK, 1 on error
  */
 PTA *
@@ -316,7 +318,7 @@ ptaClone(PTA  *pta)
  *      This only resets the Pta::n field, for reuse
  * </pre>
  */
-l_int32
+l_ok
 ptaEmpty(PTA  *pta)
 {
     PROCNAME("ptaEmpty");
@@ -338,7 +340,7 @@ ptaEmpty(PTA  *pta)
  * \param[in]    x, y
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 ptaAddPt(PTA       *pta,
          l_float32  x,
          l_float32  y)
@@ -396,11 +398,11 @@ ptaExtendArrays(PTA  *pta)
  * \brief   ptaInsertPt()
  *
  * \param[in]    pta
- * \param[in]    index at which pt is to be inserted
- * \param[in]    x, y point values
+ * \param[in]    index   at which pt is to be inserted
+ * \param[in]    x, y    point values
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 ptaInsertPt(PTA     *pta,
             l_int32  index,
             l_int32  x,
@@ -433,7 +435,7 @@ l_int32  i, n;
  * \brief   ptaRemovePt()
  *
  * \param[in]    pta
- * \param[in]    index of point to be removed
+ * \param[in]    index    of point to be removed
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -443,7 +445,7 @@ l_int32  i, n;
  *          because the function is O(n).
  * </pre>
  */
-l_int32
+l_ok
 ptaRemovePt(PTA     *pta,
             l_int32  index)
 {
@@ -516,12 +518,12 @@ ptaGetCount(PTA  *pta)
  * \brief   ptaGetPt()
  *
  * \param[in]    pta
- * \param[in]    index  into arrays
- * \param[out]   px [optional] float x value
- * \param[out]   py [optional] float y value
+ * \param[in]    index    into arrays
+ * \param[out]   px       [optional] float x value
+ * \param[out]   py       [optional] float y value
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 ptaGetPt(PTA        *pta,
          l_int32     index,
          l_float32  *px,
@@ -546,12 +548,12 @@ ptaGetPt(PTA        *pta,
  * \brief   ptaGetIPt()
  *
  * \param[in]    pta
- * \param[in]    index  into arrays
- * \param[out]   px [optional] integer x value
- * \param[out]   py [optional] integer y value
+ * \param[in]    index    into arrays
+ * \param[out]   px       [optional] integer x value
+ * \param[out]   py       [optional] integer y value
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 ptaGetIPt(PTA      *pta,
           l_int32   index,
           l_int32  *px,
@@ -576,11 +578,11 @@ ptaGetIPt(PTA      *pta,
  * \brief   ptaSetPt()
  *
  * \param[in]    pta
- * \param[in]    index  into arrays
+ * \param[in]    index    into arrays
  * \param[in]    x, y
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 ptaSetPt(PTA       *pta,
          l_int32    index,
          l_float32  x,
@@ -603,8 +605,8 @@ ptaSetPt(PTA       *pta,
  * \brief   ptaGetArrays()
  *
  * \param[in]    pta
- * \param[out]   pnax [optional] numa of x array
- * \param[out]   pnay [optional] numa of y array
+ * \param[out]   pnax    [optional] numa of x array
+ * \param[out]   pnay    [optional] numa of y array
  * \return  0 if OK; 1 on error or if pta is empty
  *
  * <pre>
@@ -612,7 +614,7 @@ ptaSetPt(PTA       *pta,
  *      (1) This copies the internal arrays into new Numas.
  * </pre>
  */
-l_int32
+l_ok
 ptaGetArrays(PTA    *pta,
              NUMA  **pnax,
              NUMA  **pnay)
@@ -684,7 +686,7 @@ PTA   *pta;
 /*!
  * \brief   ptaReadStream()
  *
- * \param[in]    fp file stream
+ * \param[in]    fp    file stream
  * \return  pta, or NULL on error
  */
 PTA *
@@ -736,8 +738,8 @@ PTA       *pta;
 /*!
  * \brief   ptaReadMem()
  *
- * \param[in]    data  serialization in ascii
- * \param[in]    size  of data in bytes; can use strlen to get it
+ * \param[in]    data    serialization in ascii
+ * \param[in]    size    of data in bytes; can use strlen to get it
  * \return  pta, or NULL on error
  */
 PTA *
@@ -766,7 +768,7 @@ PTA   *pta;
  *
  * \param[in]    filename
  * \param[in]    pta
- * \param[in]    type  0 for float values; 1 for integer values
+ * \param[in]    type       0 for float values; 1 for integer values
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -778,7 +780,7 @@ PTA   *pta;
  *          or cleared by the function setLeptDebugOK().
  * </pre>
  */
-l_int32
+l_ok
 ptaWriteDebug(const char  *filename,
               PTA         *pta,
               l_int32      type)
@@ -799,10 +801,10 @@ ptaWriteDebug(const char  *filename,
  *
  * \param[in]    filename
  * \param[in]    pta
- * \param[in]    type  0 for float values; 1 for integer values
+ * \param[in]    type       0 for float values; 1 for integer values
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 ptaWrite(const char  *filename,
          PTA         *pta,
          l_int32      type)
@@ -830,12 +832,12 @@ FILE    *fp;
 /*!
  * \brief   ptaWriteStream()
  *
- * \param[in]    fp file stream
+ * \param[in]    fp      file stream
  * \param[in]    pta
- * \param[in]    type  0 for float values; 1 for integer values
+ * \param[in]    type    0 for float values; 1 for integer values
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 ptaWriteStream(FILE    *fp,
                PTA     *pta,
                l_int32  type)
@@ -873,10 +875,10 @@ l_float32  x, y;
 /*!
  * \brief   ptaWriteMem()
  *
- * \param[out]   pdata data of serialized pta; ascii
- * \param[out]   psize size of returned data
+ * \param[out]   pdata    data of serialized pta; ascii
+ * \param[out]   psize    size of returned data
  * \param[in]    pta
- * \param[in]    type  0 for float values; 1 for integer values
+ * \param[in]    type     0 for float values; 1 for integer values
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -884,7 +886,7 @@ l_float32  x, y;
  *      (1) Serializes a pta in memory and puts the result in a buffer.
  * </pre>
  */
-l_int32
+l_ok
 ptaWriteMem(l_uint8  **pdata,
             size_t    *psize,
             PTA       *pta,
@@ -932,7 +934,7 @@ FILE    *fp;
 /*!
  * \brief   ptaaCreate()
  *
- * \param[in]    n  initial number of ptrs
+ * \param[in]    n    initial number of ptrs
  * \return  ptaa, or NULL on error
  */
 PTAA *
@@ -942,11 +944,10 @@ PTAA  *ptaa;
 
     PROCNAME("ptaaCreate");
 
-    if (n <= 0)
-        n = INITIAL_PTR_ARRAYSIZE;
+    if (n <= 0 || n > MaxPtrArraySize)
+        n = InitialPtrArraySize;
 
-    if ((ptaa = (PTAA *)LEPT_CALLOC(1, sizeof(PTAA))) == NULL)
-        return (PTAA *)ERROR_PTR("ptaa not made", procName, NULL);
+    ptaa = (PTAA *)LEPT_CALLOC(1, sizeof(PTAA));
     ptaa->n = 0;
     ptaa->nalloc = n;
     if ((ptaa->pta = (PTA **)LEPT_CALLOC(n, sizeof(PTA *))) == NULL) {
@@ -960,7 +961,7 @@ PTAA  *ptaa;
 /*!
  * \brief   ptaaDestroy()
  *
- * \param[in,out]   pptaa to be nulled
+ * \param[in,out]   pptaa   will be set to null before returning
  * \return  void
  */
 void
@@ -996,11 +997,11 @@ PTAA    *ptaa;
  * \brief   ptaaAddPta()
  *
  * \param[in]    ptaa
- * \param[in]    pta  to be added
- * \param[in]    copyflag  L_INSERT, L_COPY, L_CLONE
+ * \param[in]    pta         to be added
+ * \param[in]    copyflag    L_INSERT, L_COPY, L_CLONE
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 ptaaAddPta(PTAA    *ptaa,
            PTA     *pta,
            l_int32  copyflag)
@@ -1086,8 +1087,8 @@ ptaaGetCount(PTAA  *ptaa)
  * \brief   ptaaGetPta()
  *
  * \param[in]    ptaa
- * \param[in]    index  to the i-th pta
- * \param[in]    accessflag  L_COPY or L_CLONE
+ * \param[in]    index         to the i-th pta
+ * \param[in]    accessflag    L_COPY or L_CLONE
  * \return  pta, or NULL on error
  */
 PTA *
@@ -1115,13 +1116,13 @@ ptaaGetPta(PTAA    *ptaa,
  * \brief   ptaaGetPt()
  *
  * \param[in]    ptaa
- * \param[in]    ipta  to the i-th pta
- * \param[in]    jpt index to the j-th pt in the pta
- * \param[out]   px [optional] float x value
- * \param[out]   py [optional] float y value
+ * \param[in]    ipta   to the i-th pta
+ * \param[in]    jpt    index to the j-th pt in the pta
+ * \param[out]   px     [optional] float x value
+ * \param[out]   py     [optional] float y value
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 ptaaGetPt(PTAA       *ptaa,
            l_int32     ipta,
            l_int32     jpt,
@@ -1157,11 +1158,11 @@ PTA  *pta;
 /*!
  * \brief   ptaaInitFull()
  *
- * \param[in]    ptaa can have non-null ptrs in the ptr array
- * \param[in]    pta to be replicated into the entire ptr array
+ * \param[in]    ptaa    can have non-null ptrs in the ptr array
+ * \param[in]    pta     to be replicated into the entire ptr array
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 ptaaInitFull(PTAA  *ptaa,
              PTA   *pta)
 {
@@ -1189,18 +1190,18 @@ PTA     *ptat;
  * \brief   ptaaReplacePta()
  *
  * \param[in]    ptaa
- * \param[in]    index  to the index-th pta
- * \param[in]    pta insert and replace any existing one
+ * \param[in]    index   to the index-th pta
+ * \param[in]    pta     insert and replace any existing one
  * \return  0 if OK, 1 on error
  *
  * <pre>
  * Notes:
  *      (1) Any existing pta is destroyed, and the input one
  *          is inserted in its place.
- *      (2) If the index is invalid, return 1 (error)
+ *      (2) If %index is invalid, return 1 (error)
  * </pre>
  */
-l_int32
+l_ok
 ptaaReplacePta(PTAA    *ptaa,
                l_int32  index,
                PTA     *pta)
@@ -1227,11 +1228,11 @@ l_int32  n;
  * \brief   ptaaAddPt()
  *
  * \param[in]    ptaa
- * \param[in]    ipta  to the i-th pta
- * \param[in]    x,y point coordinates
+ * \param[in]    ipta   to the i-th pta
+ * \param[in]    x,y    point coordinates
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 ptaaAddPt(PTAA      *ptaa,
           l_int32    ipta,
           l_float32  x,
@@ -1266,7 +1267,7 @@ PTA  *pta;
  *          and resets the count.
  * </pre>
  */
-l_int32
+l_ok
 ptaaTruncate(PTAA  *ptaa)
 {
 l_int32  i, n, np;
@@ -1330,7 +1331,7 @@ PTAA  *ptaa;
 /*!
  * \brief   ptaaReadStream()
  *
- * \param[in]    fp file stream
+ * \param[in]    fp    file stream
  * \return  ptaa, or NULL on error
  */
 PTAA *
@@ -1369,8 +1370,8 @@ PTAA    *ptaa;
 /*!
  * \brief   ptaaReadMem()
  *
- * \param[in]    data  serialization in ascii
- * \param[in]    size  of data in bytes; can use strlen to get it
+ * \param[in]    data    serialization in ascii
+ * \param[in]    size    of data in bytes; can use strlen to get it
  * \return  ptaa, or NULL on error
  */
 PTAA *
@@ -1399,7 +1400,7 @@ PTAA  *ptaa;
  *
  * \param[in]    filename
  * \param[in]    ptaa
- * \param[in]    type  0 for float values; 1 for integer values
+ * \param[in]    type      0 for float values; 1 for integer values
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1411,7 +1412,7 @@ PTAA  *ptaa;
  *          or cleared by the function setLeptDebugOK().
  * </pre>
  */
-l_int32
+l_ok
 ptaaWriteDebug(const char  *filename,
                PTAA        *ptaa,
                l_int32      type)
@@ -1432,10 +1433,10 @@ ptaaWriteDebug(const char  *filename,
  *
  * \param[in]    filename
  * \param[in]    ptaa
- * \param[in]    type  0 for float values; 1 for integer values
+ * \param[in]    type      0 for float values; 1 for integer values
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 ptaaWrite(const char  *filename,
           PTAA        *ptaa,
           l_int32      type)
@@ -1463,12 +1464,12 @@ FILE    *fp;
 /*!
  * \brief   ptaaWriteStream()
  *
- * \param[in]    fp file stream
+ * \param[in]    fp      file stream
  * \param[in]    ptaa
- * \param[in]    type  0 for float values; 1 for integer values
+ * \param[in]    type    0 for float values; 1 for integer values
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 ptaaWriteStream(FILE    *fp,
                 PTAA    *ptaa,
                 l_int32  type)
@@ -1499,18 +1500,18 @@ PTA     *pta;
 /*!
  * \brief   ptaaWriteMem()
  *
- * \param[out]   pdata data of serialized ptaa; ascii
- * \param[out]   psize size of returned data
+ * \param[out]   pdata    data of serialized ptaa; ascii
+ * \param[out]   psize    size of returned data
  * \param[in]    ptaa
- * \param[in]    type  0 for float values; 1 for integer values
+ * \param[in]    type     0 for float values; 1 for integer values
  * \return  0 if OK, 1 on error
  *
  * <pre>
  * Notes:
- *      (1) Serializes a ptaa in memory and puts the result in a buffer.
+ *      (1) Serializes %ptaa in memory and puts the result in a buffer.
  * </pre>
  */
-l_int32
+l_ok
 ptaaWriteMem(l_uint8  **pdata,
              size_t    *psize,
              PTAA      *ptaa,
@@ -1550,4 +1551,3 @@ FILE    *fp;
     fclose(fp);
     return ret;
 }
-
