@@ -118,21 +118,26 @@
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
-static const l_int32 INITIAL_PTR_ARRAYSIZE = 20;      /* n'importe quoi */
+    /* Bounds on initial array size */
+static const l_uint32  MaxPtrArraySize = 100000;
+static const l_int32 InitialPtrArraySize = 20;      /*!< n'importe quoi */
 
     /* Static function */
 static l_int32 ptraExtendArray(L_PTRA *pa);
 
-
 /*--------------------------------------------------------------------------*
  *                       Ptra creation and destruction                      *
  *--------------------------------------------------------------------------*/
 /*!
  * \brief   ptraCreate()
  *
- * \param[in]    n size of ptr array to be alloc'd 0 for default
+ * \param[in]    n     size of ptr array to be alloc'd; use 0 for default
  * \return  pa, or NULL on error
  */
 L_PTRA *
@@ -142,8 +147,8 @@ L_PTRA  *pa;
 
     PROCNAME("ptraCreate");
 
-    if (n <= 0)
-        n = INITIAL_PTR_ARRAYSIZE;
+    if (n <= 0 || n > MaxPtrArraySize)
+        n = InitialPtrArraySize;
 
     pa = (L_PTRA *)LEPT_CALLOC(1, sizeof(L_PTRA));
     if ((pa->array = (void **)LEPT_CALLOC(n, sizeof(void *))) == NULL) {
@@ -160,9 +165,10 @@ L_PTRA  *pa;
 /*!
  * \brief   ptraDestroy()
  *
- * \param[in,out]   ppa ptra to be nulled
- * \param[in]       freeflag TRUE to free each remaining item in the array
- * \param[in]       warnflag TRUE to warn if any remaining items are not destroyed
+ * \param[in,out]   ppa        will be set to null before returning
+ * \param[in]       freeflag   TRUE to free each remaining item in the array
+ * \param[in]       warnflag   TRUE to warn if any remaining items
+ *                             are not destroyed
  * \return  void
  *
  * <pre>
@@ -225,8 +231,8 @@ L_PTRA  *pa;
 /*!
  * \brief   ptraAdd()
  *
- * \param[in]    pa ptra
- * \param[in]    item  generic ptr to a struct
+ * \param[in]    pa      ptra
+ * \param[in]    item    generic ptr to a struct
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -238,7 +244,7 @@ L_PTRA  *pa;
  *          items in the array is entirely arbitrary.
  * </pre>
  */
-l_int32
+l_ok
 ptraAdd(L_PTRA  *pa,
         void    *item)
 {
@@ -288,10 +294,10 @@ ptraExtendArray(L_PTRA  *pa)
 /*!
  * \brief   ptraInsert()
  *
- * \param[in]    pa ptra
- * \param[in]    index location in ptra to insert new value
- * \param[in]    item  generic ptr to a struct; can be null
- * \param[in]    shiftflag L_AUTO_DOWNSHIFT, L_MIN_DOWNSHIFT, L_FULL_DOWNSHIFT
+ * \param[in]    pa          ptra
+ * \param[in]    index       location in ptra to insert new value
+ * \param[in]    item        generic ptr to a struct; can be null
+ * \param[in]    shiftflag   L_AUTO_DOWNSHIFT, L_MIN_DOWNSHIFT, L_FULL_DOWNSHIFT
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -332,7 +338,7 @@ ptraExtendArray(L_PTRA  *pa)
  *          randomly to the ptr array.
  * </pre>
  */
-l_int32
+l_ok
 ptraInsert(L_PTRA  *pa,
            l_int32  index,
            void    *item,
@@ -413,9 +419,9 @@ l_float32  nexpected;
 /*!
  * \brief   ptraRemove()
  *
- * \param[in]    pa ptra
- * \param[in]    index element to be removed
- * \param[in]    flag L_NO_COMPACTION, L_COMPACTION
+ * \param[in]    pa       ptra
+ * \param[in]    index    element to be removed
+ * \param[in]    flag     L_NO_COMPACTION, L_COMPACTION
  * \return  item, or NULL on error
  *
  * <pre>
@@ -476,7 +482,7 @@ void    *item;
 /*!
  * \brief   ptraRemoveLast()
  *
- * \param[in]    pa ptra
+ * \param[in]    pa    ptra
  * \return  item, or NULL on error or if the array is empty
  */
 void *
@@ -501,10 +507,10 @@ l_int32  imax;
 /*!
  * \brief   ptraReplace()
  *
- * \param[in]    pa ptra
- * \param[in]    index element to be replaced
- * \param[in]    item  new generic ptr to a struct; can be null
- * \param[in]    freeflag TRUE to free old item; FALSE to return it
+ * \param[in]    pa          ptra
+ * \param[in]    index       element to be replaced
+ * \param[in]    item        new generic ptr to a struct; can be null
+ * \param[in]    freeflag    TRUE to free old item; FALSE to return it
  * \return  item  old item, if it exists and is not freed,
  *                     or NULL on error
  */
@@ -549,7 +555,7 @@ void    *olditem;
  * \param[in]    index2
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 ptraSwap(L_PTRA  *pa,
          l_int32  index1,
          l_int32  index2)
@@ -586,7 +592,7 @@ void    *item;
  *      (2) This does not change the size of the array of ptrs.
  * </pre>
  */
-l_int32
+l_ok
 ptraCompactArray(L_PTRA  *pa)
 {
 l_int32  i, imax, nactual, index;
@@ -618,10 +624,10 @@ l_int32  i, imax, nactual, index;
 /*!
  * \brief   ptraReverse()
  *
- * \param[in]    pa ptra
+ * \param[in]    pa     ptra
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 ptraReverse(L_PTRA  *pa)
 {
 l_int32  i, imax;
@@ -641,11 +647,11 @@ l_int32  i, imax;
 /*!
  * \brief   ptraJoin()
  *
- * \param[in]    pa1 add to this one
- * \param[in]    pa2 appended to pa1, and emptied of items; can be null
+ * \param[in]    pa1    add to this one
+ * \param[in]    pa2    appended to pa1, and emptied of items; can be null
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 ptraJoin(L_PTRA  *pa1,
          L_PTRA  *pa2)
 {
@@ -676,8 +682,8 @@ void    *item;
 /*!
  * \brief   ptraGetMaxIndex()
  *
- * \param[in]    pa ptra
- * \param[out]   pmaxindex index of last item in the array;
+ * \param[in]    pa          ptra
+ * \param[out]   pmaxindex   index of last item in the array;
  * \return  0 if OK; 1 on error
  *
  * <pre>
@@ -695,7 +701,7 @@ void    *item;
  *      (3) If there are no items in the array, this returns %maxindex = -1.
  * </pre>
  */
-l_int32
+l_ok
 ptraGetMaxIndex(L_PTRA   *pa,
                 l_int32  *pmaxindex)
 {
@@ -713,8 +719,8 @@ ptraGetMaxIndex(L_PTRA   *pa,
 /*!
  * \brief   ptraGetActualCount()
  *
- * \param[in]    pa ptra
- * \param[out]   pcount actual number of items on the ptr array
+ * \param[in]    pa        ptra
+ * \param[out]   pcount    actual number of items on the ptr array
  * \return  0 if OK; 1 on error
  *
  * <pre>
@@ -723,7 +729,7 @@ ptraGetMaxIndex(L_PTRA   *pa,
  *          will be smaller than pa->n if the array is not compacted.
  * </pre>
  */
-l_int32
+l_ok
 ptraGetActualCount(L_PTRA   *pa,
                    l_int32  *pcount)
 {
@@ -742,8 +748,8 @@ ptraGetActualCount(L_PTRA   *pa,
 /*!
  * \brief   ptraGetPtrToItem()
  *
- * \param[in]    pa ptra
- * \param[in]    index of element to be retrieved
+ * \param[in]    pa       ptra
+ * \param[in]    index    of element to be retrieved
  * \return  a ptr to the element, or NULL on error
  *
  * <pre>
@@ -777,7 +783,7 @@ ptraGetPtrToItem(L_PTRA  *pa,
 /*!
  * \brief   ptraaCreate()
  *
- * \param[in]    n size of ptr array to be alloc'd
+ * \param[in]    n    size of ptr array to be alloc'd
  * \return  paa, or NULL on error
  *
  * <pre>
@@ -796,8 +802,7 @@ L_PTRAA  *paa;
     if (n <= 0)
         return (L_PTRAA *)ERROR_PTR("n must be > 0", procName, NULL);
 
-    if ((paa = (L_PTRAA *)LEPT_CALLOC(1, sizeof(L_PTRAA))) == NULL)
-        return (L_PTRAA *)ERROR_PTR("paa not made", procName, NULL);
+    paa = (L_PTRAA *)LEPT_CALLOC(1, sizeof(L_PTRAA));
     if ((paa->ptra = (L_PTRA **)LEPT_CALLOC(n, sizeof(L_PTRA *))) == NULL) {
         ptraaDestroy(&paa, 0, 0);
         return (L_PTRAA *)ERROR_PTR("ptr array not made", procName, NULL);
@@ -810,9 +815,10 @@ L_PTRAA  *paa;
 /*!
  * \brief   ptraaDestroy()
  *
- * \param[in,out]   ppaa to be nulled
- * \param[in]    freeflag TRUE to free each remaining item in each ptra
- * \param[in]    warnflag TRUE to warn if any remaining items are not destroyed
+ * \param[in,out]   ppaa       will be set to null before returning
+ * \param[in]       freeflag   TRUE to free each remaining item in each ptra
+ * \param[in]       warnflag   TRUE to warn if any remaining items
+ *                             are not destroyed
  * \return  void
  *
  * <pre>
@@ -860,10 +866,10 @@ L_PTRAA  *paa;
  * \brief   ptraaGetSize()
  *
  * \param[in]    paa
- * \param[out]   psize size of ptr array
+ * \param[out]   psize    size of ptr array
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 ptraaGetSize(L_PTRAA  *paa,
              l_int32  *psize)
 {
@@ -882,9 +888,9 @@ ptraaGetSize(L_PTRAA  *paa,
 /*!
  * \brief   ptraaInsertPtra()
  *
- * \param[in]    paa ptraa
- * \param[in]    index location in array for insertion
- * \param[in]    pa to be inserted
+ * \param[in]    paa      ptraa
+ * \param[in]    index    location in array for insertion
+ * \param[in]    pa       to be inserted
  * \return  0 if OK; 1 on error
  *
  * <pre>
@@ -894,7 +900,7 @@ ptraaGetSize(L_PTRAA  *paa,
  *          on error, the Ptra remains owned by the caller.
  * </pre>
  */
-l_int32
+l_ok
 ptraaInsertPtra(L_PTRAA  *paa,
                 l_int32   index,
                 L_PTRA   *pa)
@@ -911,7 +917,7 @@ l_int32  n;
     if (index < 0 || index >= n)
         return ERROR_INT("invalid index", procName, 1);
     if (paa->ptra[index] != NULL)
-        return ERROR_INT("ptra alread stored at index", procName, 1);
+        return ERROR_INT("ptra already stored at index", procName, 1);
 
     paa->ptra[index] = pa;
     return 0;
@@ -921,9 +927,9 @@ l_int32  n;
 /*!
  * \brief   ptraaGetPtra()
  *
- * \param[in]    paa ptraa
- * \param[in]    index location in array
- * \param[in]    accessflag L_HANDLE_ONLY, L_REMOVE
+ * \param[in]    paa          ptraa
+ * \param[in]    index        location in array
+ * \param[in]    accessflag   L_HANDLE_ONLY, L_REMOVE
  * \return  ptra at index location, or NULL on error or if there
  *              is no ptra there.
  *
@@ -968,7 +974,7 @@ L_PTRA  *pa;
 /*!
  * \brief   ptraaFlattenToPtra()
  *
- * \param[in]    paa ptraa
+ * \param[in]    paa    ptraa
  * \return  ptra, or NULL on error
  *
  * <pre>
