@@ -71,6 +71,10 @@
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
     /* The node color enum is only needed in the rbtree implementation */
@@ -118,7 +122,6 @@ static void verify_properties(L_RBTREE *t);
 #define  VERIFY_RBTREE     0   /* only for debugging */
 #endif  /* ~NO_CONSOLE_IO */
 
-
 /* ------------------------------------------------------------- *
  *                   Interface to Red-black Tree                 *
  * ------------------------------------------------------------- */
@@ -147,7 +150,7 @@ l_rbtreeCreate(l_int32  keytype)
  * \brief   l_rbtreeLookup()
  *
  * \param[in]   t        rbtree, including root node
- * \param[in    key      find a node with this key
+ * \param[in]   key      find a node with this key
  * \return    &value     a pointer to a union, if the node exists; else NULL
  */
 RB_TYPE *
@@ -228,7 +231,7 @@ node  *n, *inserted_node;
  * \brief   l_rbtreeDelete()
  *
  * \param[in]   t     rbtree, including root node
- * \param[in]   key  (delete the node with this key
+ * \param[in]   key   delete the node with this key
  * \return      void
  */
 void
@@ -271,7 +274,7 @@ node  *n, *child;
 /*!
  * \brief   l_rbtreeDestroy()
  *
- * \param[in]   &t     ptr to rbtree
+ * \param[in]   pt     pointer to tree; will be wet to null before returning
  * \return      void
  *
  * <pre>
@@ -446,7 +449,7 @@ l_rbtreeGetPrev(L_RBTREE_NODE  *n)
 /*!
  * \brief   l_rbtreeGetCount()
  *
- * \param[in   t      rbtree
+ * \param[in]  t      rbtree
  * \return     count  the number of nodes in the tree, or 0 on error
  */
 l_int32
