@@ -99,9 +99,32 @@
  *     center wanders -- it seems to be doing a 1 pixel translation
  *     in addition to the rotation.
  *
+ *     Consider again the comparison of image quality between sampling
+ *     and area mapping.  With sampling, sharp edges such as found in
+ *     text images remain sharp.  However, sampling artifacts such as
+ *     characters randomly bouncing up and down by one pixel, or
+ *     one pixel horizontal shear lines going through a line of text
+ *     (causing the characters to look like badly rendered italic),
+ *     are highly visible.  It does not help to sample the source pixel
+ *     with the largest area covering each dest pixel; the result has
+ *     the same ugly sampling artifacts.
+ *
+ *     With area mapping, these annoying artifacts are avoided, but the
+ *     blurring of edges makes small text a bit more difficult to read.
+ *     However, if you are willing to do more computation, you can have
+ *     the best of both worlds: no sampling artifacts and sharp edges.
+ *     Use area mapping to avoid sampling issues, and follow it with
+ *     unsharp masking.  Experiment with the sharpening parameters.
+ *     I have found that a small amount of sharpening is sufficient to
+ *     restore the sharp edges in text; e.g.,
+ *         pix2 = pixUnsharpMasking(pix1, 1, 0.3);
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include <math.h>   /* required for sin and tan */
 #include "allheaders.h"
@@ -124,7 +147,7 @@ static void rotateAMColorFastLow(l_uint32 *datad, l_int32 w, l_int32 h,
                                  l_int32 wpld, l_uint32 *datas, l_int32 wpls,
                                  l_float32 angle, l_uint32 colorval);
 
-static const l_float32  MIN_ANGLE_TO_ROTATE = 0.001;  /* radians; ~0.06 deg */
+static const l_float32  MinAngleToRotate = 0.001;  /* radians; ~0.06 deg */
 
 
 /*------------------------------------------------------------------*
@@ -161,7 +184,7 @@ PIX      *pixt1, *pixt2, *pixd;
     if (pixGetDepth(pixs) == 1)
         return (PIX *)ERROR_PTR("pixs is 1 bpp", procName, NULL);
 
-    if (L_ABS(angle) < MIN_ANGLE_TO_ROTATE)
+    if (L_ABS(angle) < MinAngleToRotate)
         return pixClone(pixs);
 
         /* Remove cmap if it exists, and unpack to 8 bpp if necessary */
@@ -224,7 +247,7 @@ PIX       *pix1, *pix2, *pixd;
     if (pixGetDepth(pixs) != 32)
         return (PIX *)ERROR_PTR("pixs must be 32 bpp", procName, NULL);
 
-    if (L_ABS(angle) < MIN_ANGLE_TO_ROTATE)
+    if (L_ABS(angle) < MinAngleToRotate)
         return pixClone(pixs);
 
     pixGetDimensions(pixs, &w, &h, NULL);
@@ -278,7 +301,7 @@ PIX        *pixd;
     if (pixGetDepth(pixs) != 8)
         return (PIX *)ERROR_PTR("pixs must be 8 bpp", procName, NULL);
 
-    if (L_ABS(angle) < MIN_ANGLE_TO_ROTATE)
+    if (L_ABS(angle) < MinAngleToRotate)
         return pixClone(pixs);
 
     pixGetDimensions(pixs, &w, &h, NULL);
@@ -457,7 +480,7 @@ PIX      *pixt1, *pixt2, *pixd;
     if (!pixs)
         return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
 
-    if (L_ABS(angle) < MIN_ANGLE_TO_ROTATE)
+    if (L_ABS(angle) < MinAngleToRotate)
         return pixClone(pixs);
 
         /* Remove cmap if it exists, and unpack to 8 bpp if necessary */
@@ -520,7 +543,7 @@ PIX       *pix1, *pix2, *pixd;
     if (pixGetDepth(pixs) != 32)
         return (PIX *)ERROR_PTR("pixs must be 32 bpp", procName, NULL);
 
-    if (L_ABS(angle) < MIN_ANGLE_TO_ROTATE)
+    if (L_ABS(angle) < MinAngleToRotate)
         return pixClone(pixs);
 
     pixGetDimensions(pixs, &w, &h, NULL);
@@ -574,7 +597,7 @@ PIX       *pixd;
     if (pixGetDepth(pixs) != 8)
         return (PIX *)ERROR_PTR("pixs must be 8 bpp", procName, NULL);
 
-    if (L_ABS(angle) < MIN_ANGLE_TO_ROTATE)
+    if (L_ABS(angle) < MinAngleToRotate)
         return pixClone(pixs);
 
     pixGetDimensions(pixs, &w, &h, NULL);
@@ -753,7 +776,7 @@ PIX       *pixd;
     if (pixGetDepth(pixs) != 32)
         return (PIX *)ERROR_PTR("pixs must be 32 bpp", procName, NULL);
 
-    if (L_ABS(angle) < MIN_ANGLE_TO_ROTATE)
+    if (L_ABS(angle) < MinAngleToRotate)
         return pixClone(pixs);
 
     pixGetDimensions(pixs, &w, &h, NULL);
@@ -1101,7 +1124,7 @@ l_float32  sina, cosa;
                                ((blue << 4) & 0x0000ff00);
                 break;
             default:
-                fprintf(stderr, "shouldn't get here\n");
+                lept_stderr("shouldn't get here\n");
                 break;
             }
         }
