@@ -95,6 +95,7 @@
  *         Making hit-miss sels from Pix and image files
  *            SEL       *selReadFromColorImage()
  *            SEL       *selCreateFromColorPix()
+              SELA      *selaCreateFromColorPixa()
  *
  *         Printable display of sel
  *            PIX       *selDisplayInPix()
@@ -138,12 +139,19 @@
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include "allheaders.h"
 
-static const l_int32  L_BUFSIZE = 256;  /* hardcoded below in sscanf */
-static const l_int32  INITIAL_PTR_ARRAYSIZE = 50;  /* n'import quoi */
-static const l_int32  MANY_SELS = 1000;
+    /* Bounds on sel ptr array size */
+static const l_uint32  MaxPtrArraySize = 10000;
+static const l_int32 InitialPtrArraySize = 50;      /*!< n'importe quoi */
+
+    /* Bounds on kernel size */
+static const l_uint32  MaxKernelSize = 10000;
 
     /* Static functions */
 static l_int32 selaExtendArray(SELA *sela);
@@ -242,18 +250,13 @@ SELA  *sela;
 
     PROCNAME("selaCreate");
 
-    if (n <= 0)
-        n = INITIAL_PTR_ARRAYSIZE;
-    if (n > MANY_SELS)
-        L_WARNING("%d sels\n", procName, n);
-
-    if ((sela = (SELA *)LEPT_CALLOC(1, sizeof(SELA))) == NULL)
-        return (SELA *)ERROR_PTR("sela not made", procName, NULL);
+    if (n <= 0 || n > MaxPtrArraySize)
+        n = InitialPtrArraySize;
 
+        /* Make array of sel ptrs */
+    sela = (SELA *)LEPT_CALLOC(1, sizeof(SELA));
     sela->nalloc = n;
     sela->n = 0;
-
-        /* make array of se ptrs */
     if ((sela->sel = (SEL **)LEPT_CALLOC(n, sizeof(SEL *))) == NULL) {
         LEPT_FREE(sela);
         return (SELA *)ERROR_PTR("sel ptrs not made", procName, NULL);
@@ -476,18 +479,20 @@ SEL     *sel;
 
     size = factor1 * factor2;
     if (direction == L_HORIZ) {
-        sel = selCreate(1, size, NULL);
+        if ((sel = selCreate(1, size, NULL)) == NULL)
+            return (SEL *)ERROR_PTR("horiz sel not made", procName, NULL);
         selSetOrigin(sel, 0, size / 2);
     } else {
-        sel = selCreate(size, 1, NULL);
+        if ((sel = selCreate(size, 1, NULL)) == NULL)
+            return (SEL *)ERROR_PTR("vert sel not made", procName, NULL);
         selSetOrigin(sel, size / 2, 0);
     }
 
         /* Lay down the elements of the comb */
     for (i = 0; i < factor2; i++) {
         z = factor1 / 2 + i * factor1;
-/*        fprintf(stderr, "i = %d, factor1 = %d, factor2 = %d, z = %d\n",
-                        i, factor1, factor2, z); */
+/*        lept_stderr("i = %d, factor1 = %d, factor2 = %d, z = %d\n",
+                      i, factor1, factor2, z); */
         if (direction == L_HORIZ)
             selSetElement(sel, 0, z, SEL_HIT);
         else
@@ -521,9 +526,13 @@ l_int32  **array;
 
     PROCNAME("create2dIntArray");
 
+    if (sx <= 0 || sx > MaxKernelSize)
+        return (l_int32 **)ERROR_PTR("sx out of bounds", procName, NULL);
+    if (sy <= 0 || sy > MaxKernelSize)
+        return (l_int32 **)ERROR_PTR("sy out of bounds", procName, NULL);
+
     if ((array = (l_int32 **)LEPT_CALLOC(sy, sizeof(l_int32 *))) == NULL)
         return (l_int32 **)ERROR_PTR("ptr array not made", procName, NULL);
-
     success = TRUE;
     for (i = 0; i < sy; i++) {
         if ((array[i] = (l_int32 *)LEPT_CALLOC(sx, sizeof(l_int32))) == NULL) {
@@ -979,7 +988,7 @@ selaGetCombName(SELA    *sela,
                 l_int32  direction)
 {
 char    *selname;
-char     combname[L_BUFSIZE];
+char     combname[256];
 l_int32  i, nsels, sx, sy, found;
 SEL     *sel;
 
@@ -992,9 +1001,9 @@ SEL     *sel;
 
         /* Derive the comb name we're looking for */
     if (direction == L_HORIZ)
-        snprintf(combname, L_BUFSIZE, "sel_comb_%dh", size);
+        snprintf(combname, sizeof(combname), "sel_comb_%dh", size);
     else  /* direction == L_VERT */
-        snprintf(combname, L_BUFSIZE, "sel_comb_%dv", size);
+        snprintf(combname, sizeof(combname), "sel_comb_%dv", size);
 
     found = FALSE;
     nsels = selaGetCount(sela);
@@ -1042,7 +1051,7 @@ static void
 selaComputeCompositeParameters(const char  *fileout)
 {
 char    *str, *nameh1, *nameh2, *namev1, *namev2;
-char     buf[L_BUFSIZE];
+char     buf[256];
 l_int32  size, size1, size2, len;
 SARRAY  *sa;
 SELA    *selabasic, *selacomb;
@@ -1061,7 +1070,7 @@ SELA    *selabasic, *selacomb;
             nameh2 = stringNew("");
             namev2 = stringNew("");
         }
-        snprintf(buf, L_BUFSIZE,
+        snprintf(buf, sizeof(buf),
                  "      { %d, %d, %d, \"%s\", \"%s\", \"%s\", \"%s\" },",
                  size, size1, size2, nameh1, nameh2, namev1, namev2);
         sarrayAddString(sa, buf, L_COPY);
@@ -1410,7 +1419,7 @@ SEL  *
 selReadStream(FILE  *fp)
 {
 char    *selname;
-char     linebuf[L_BUFSIZE];
+char     linebuf[256];
 l_int32  sy, sx, cy, cx, i, j, version, ignore;
 SEL     *sel;
 
@@ -1424,7 +1433,7 @@ SEL     *sel;
     if (version != SEL_VERSION_NUMBER)
         return (SEL *)ERROR_PTR("invalid sel version", procName, NULL);
 
-    if (fgets(linebuf, L_BUFSIZE, fp) == NULL)
+    if (fgets(linebuf, sizeof(linebuf), fp) == NULL)
         return (SEL *)ERROR_PTR("error reading into linebuf", procName, NULL);
     selname = stringNew(linebuf);
     sscanf(linebuf, "  ------  %200s  ------", selname);
@@ -1644,6 +1653,7 @@ char     ch;
                 case 'X':
                     norig++;
                     selSetOrigin(sel, y, x);
+                    /* fall through */
                 case 'x':
                     selSetElement(sel, y, x, SEL_HIT);
                     break;
@@ -1651,6 +1661,7 @@ char     ch;
                 case 'O':
                     norig++;
                     selSetOrigin(sel, y, x);
+                    /* fall through */
                 case 'o':
                     selSetElement(sel, y, x, SEL_MISS);
                     break;
@@ -1658,6 +1669,7 @@ char     ch;
                 case 'C':
                     norig++;
                     selSetOrigin(sel, y, x);
+                    /* fall through */
                 case ' ':
                     selSetElement(sel, y, x, SEL_DONT_CARE);
                     break;
@@ -1830,7 +1842,7 @@ SELA    *sela;
         numaGetIValue(nafirst, i, &first);
         numaGetIValue(nalast, i, &last);
         if ((sel = selCreateFromSArray(sa, first, last)) == NULL) {
-            fprintf(stderr, "Error reading sel from %d to %d\n", first, last);
+            lept_stderr("Error reading sel from %d to %d\n", first, last);
             selaDestroy(&sela);
             sarrayDestroy(&sa);
             numaDestroy(&nafirst);
@@ -1913,18 +1925,21 @@ SEL     *sel;
             {
                 case 'X':
                     selSetOrigin(sel, y, x);  /* set origin and hit */
+                    /* fall through */
                 case 'x':
                     selSetElement(sel, y, x, SEL_HIT);
                     break;
 
                 case 'O':
                     selSetOrigin(sel, y, x);  /* set origin and miss */
+                    /* fall through */
                 case 'o':
                     selSetElement(sel, y, x, SEL_MISS);
                     break;
 
                 case 'C':
                     selSetOrigin(sel, y, x);  /* set origin and don't-care */
+                    /* fall through */
                 case ' ':
                     selSetElement(sel, y, x, SEL_DONT_CARE);
                     break;
@@ -2112,7 +2127,7 @@ selCreateFromColorPix(PIX         *pixs,
 {
 PIXCMAP  *cmap;
 SEL      *sel;
-l_int32   hascolor, hasorigin, nohits;
+l_int32   hascolor, num_origins, nohits;
 l_int32   w, h, d, i, j, red, green, blue;
 l_uint32  pixval;
 
@@ -2131,10 +2146,10 @@ l_uint32  pixval;
 
     if ((sel = selCreate (h, w, NULL)) == NULL)
         return (SEL *)ERROR_PTR ("sel not made", procName, NULL);
-    selSetOrigin (sel, h / 2, w / 2);
+    selSetOrigin (sel, h / 2, w / 2);  /* default */
     selSetName(sel, selname);
 
-    hasorigin = FALSE;
+    num_origins = 0;
     nohits = TRUE;
     for (i = 0; i < h; i++) {
         for (j = 0; j < w; j++) {
@@ -2149,10 +2164,11 @@ l_uint32  pixval;
             }
 
             if (red < 255 && green < 255 && blue < 255) {
-                if (hasorigin)
+                num_origins++;
+                if (num_origins == 1)  /* first one found */
+                    selSetOrigin (sel, i, j);
+                if (num_origins == 2)
                     L_WARNING("multiple origins in sel image\n", procName);
-                selSetOrigin (sel, i, j);
-                hasorigin = TRUE;
             }
             if (!red && green && !blue) {
                 nohits = FALSE;
@@ -2176,6 +2192,52 @@ l_uint32  pixval;
 }
 
 
+/*!
+ *
+ *  selaCreateFromColorPixa()
+ *
+ * \param[in]    pixa      color pixa representing the sels
+ * \param[in]    sa        sarray of sel names
+ * \return  sel if OK, NULL on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) See notes in selCreateFromColorPix()
+ *      (2) sa is required because all sels that are put in a sela
+ *          must have a name.
+ * </pre>
+ */
+SELA *
+selaCreateFromColorPixa(PIXA    *pixa,
+                        SARRAY  *sa)
+{
+char    *str;
+l_int32  i, n;
+PIX     *pix;
+SEL     *sel;
+SELA    *sela;
+
+    PROCNAME("selaCreateFromColorPixa");
+
+    if (!pixa)
+        return (SELA *)ERROR_PTR("pixa not defined", procName, NULL);
+    if (!sa)
+        return (SELA *)ERROR_PTR("sa of sel names not defined", procName, NULL);
+
+    n = pixaGetCount(pixa);
+    if ((sela = selaCreate(n)) == NULL)
+        return (SELA *)ERROR_PTR("sela not allocated", procName, NULL);
+    for (i = 0; i < n; i++) {
+        pix = pixaGetPix(pixa, i, L_CLONE);
+        str = sarrayGetString(sa, i, L_NOCOPY);
+        sel = selCreateFromColorPix(pix, str);
+        selaAddSel(sela, sel, NULL, L_INSERT);
+        pixDestroy(&pix);
+    }
+    return sela;
+}
+
+
 /*----------------------------------------------------------------------*
  *                     Printable display of sel                         *
  *----------------------------------------------------------------------*/
