@@ -24,9 +24,9 @@
  -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
-
-/*
- *  stack.c
+/*!
+ * \file stack.c
+ * <pre>
  *
  *      Generic stack
  *
@@ -53,64 +53,72 @@
  *
  *      Text description
  *           l_int32         lstackPrint()
+ * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include "allheaders.h"
 
-static const l_int32  INITIAL_PTR_ARRAYSIZE = 20;
+    /* Bounds on initial array size */
+static const l_uint32  MaxPtrArraySize = 100000;
+static const l_int32 InitialPtrArraySize = 20;      /*!< n'importe quoi */
 
     /* Static function */
 static l_int32 lstackExtendArray(L_STACK *lstack);
 
-
 /*---------------------------------------------------------------------*
  *                          Create/Destroy                             *
  *---------------------------------------------------------------------*/
 /*!
- *  lstackCreate()
+ * \brief   lstackCreate()
  *
- *      Input:  nalloc (initial ptr array size; use 0 for default)
- *      Return: lstack, or null on error
+ * \param[in]    n   initial ptr array size; use 0 for default
+ * \return  lstack, or NULL on error
  */
 L_STACK *
-lstackCreate(l_int32  nalloc)
+lstackCreate(l_int32  n)
 {
 L_STACK  *lstack;
 
     PROCNAME("lstackCreate");
 
-    if (nalloc <= 0)
-        nalloc = INITIAL_PTR_ARRAYSIZE;
+    if (n <= 0 || n > MaxPtrArraySize)
+        n = InitialPtrArraySize;
 
-    if ((lstack = (L_STACK *)CALLOC(1, sizeof(L_STACK))) == NULL)
-        return (L_STACK *)ERROR_PTR("lstack not made", procName, NULL);
-
-    if ((lstack->array = (void **)CALLOC(nalloc, sizeof(void *))) == NULL)
+    lstack = (L_STACK *)LEPT_CALLOC(1, sizeof(L_STACK));
+    lstack->array = (void **)LEPT_CALLOC(n, sizeof(void *));
+    if (!lstack->array) {
+        lstackDestroy(&lstack, FALSE);
         return (L_STACK *)ERROR_PTR("lstack array not made", procName, NULL);
+    }
 
-    lstack->nalloc = nalloc;
+    lstack->nalloc = n;
     lstack->n = 0;
-
     return lstack;
 }
 
 
 /*!
- *  lstackDestroy()
+ * \brief   lstackDestroy()
  *
- *      Input:  &lstack (<to be nulled>)
- *              freeflag (TRUE to free each remaining struct in the array)
- *      Return: void
+ * \param[in,out]   plstack    will be set to null before returning
+ * \param[in]    freeflag TRUE to free each remaining struct in the array
+ * \return  void
  *
- *  Notes:
- *      (1) If freeflag is TRUE, frees each struct in the array.
- *      (2) If freeflag is FALSE but there are elements on the array,
+ * <pre>
+ * Notes:
+ *      (1) If %freeflag is TRUE, frees each struct in the array.
+ *      (2) If %freeflag is FALSE but there are elements on the array,
  *          gives a warning and destroys the array.  This will
  *          cause a memory leak of all the items that were on the lstack.
  *          So if the items require their own destroy function, they
  *          must be destroyed before the lstack.
  *      (3) To destroy the lstack, we destroy the ptr array, then
  *          the lstack, and then null the contents of the input ptr.
+ * </pre>
  */
 void
 lstackDestroy(L_STACK  **plstack,
@@ -131,7 +139,7 @@ L_STACK  *lstack;
     if (freeflag) {
         while(lstack->n > 0) {
             item = lstackRemove(lstack);
-            FREE(item);
+            LEPT_FREE(item);
         }
     } else if (lstack->n > 0) {
         L_WARNING("memory leak of %d items in lstack\n", procName, lstack->n);
@@ -141,8 +149,8 @@ L_STACK  *lstack;
         lstackDestroy(&lstack->auxstack, freeflag);
 
     if (lstack->array)
-        FREE(lstack->array);
-    FREE(lstack);
+        LEPT_FREE(lstack->array);
+    LEPT_FREE(lstack);
     *plstack = NULL;
 }
 
@@ -152,13 +160,13 @@ L_STACK  *lstack;
  *                               Accessors                             *
  *---------------------------------------------------------------------*/
 /*!
- *  lstackAdd()
+ * \brief   lstackAdd()
  *
- *      Input:  lstack
- *              item to be added to the lstack
- *      Return: 0 if OK; 1 on error.
+ * \param[in]    lstack
+ * \param[in]    item      to be added to the lstack
+ * \return  0 if OK; 1 on error.
  */
-l_int32
+l_ok
 lstackAdd(L_STACK  *lstack,
           void     *item)
 {
@@ -182,11 +190,11 @@ lstackAdd(L_STACK  *lstack,
 
 
 /*!
- *  lstackRemove()
+ * \brief   lstackRemove()
  *
- *      Input:  lstack
- *      Return: ptr to item popped from the top of the lstack,
- *              or null if the lstack is empty or on error
+ * \param[in]    lstack
+ * \return  ptr to item popped from the top of the lstack,
+ *              or NULL if the lstack is empty or on error
  */
 void *
 lstackRemove(L_STACK  *lstack)
@@ -209,10 +217,10 @@ void  *item;
 
 
 /*!
- *  lstackExtendArray()
+ * \brief   lstackExtendArray()
  *
- *      Input:  lstack
- *      Return: 0 if OK; 1 on error
+ * \param[in]    lstack
+ * \return  0 if OK; 1 on error
  */
 static l_int32
 lstackExtendArray(L_STACK  *lstack)
@@ -233,10 +241,10 @@ lstackExtendArray(L_STACK  *lstack)
 
 
 /*!
- *  lstackGetCount()
+ * \brief   lstackGetCount()
  *
- *      Input:  lstack
- *      Return: count, or 0 on error
+ * \param[in]    lstack
+ * \return  count, or 0 on error
  */
 l_int32
 lstackGetCount(L_STACK  *lstack)
@@ -255,13 +263,13 @@ lstackGetCount(L_STACK  *lstack)
  *                            Debug output                             *
  *---------------------------------------------------------------------*/
 /*!
- *  lstackPrint()
+ * \brief   lstackPrint()
  *
- *      Input:  stream
- *              lstack
- *      Return: 0 if OK; 1 on error
+ * \param[in]    fp       file stream
+ * \param[in]    lstack
+ * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 lstackPrint(FILE     *fp,
             L_STACK  *lstack)
 {
